// *********************************************************************
// *                                                                   *
// *       Konstruktion von Isomeren der Alkane (C_nH_2n+2)            *
// *       ================================================            *
// *                                                                   *
// *       Autor:        Sascha Kurz                                   *
// *       modifiert im Rahmen eines Softwarepraktikums durch:         *
// *                     Sabine Boehm und Florian Haeberlein           *
// *       Email:        sascha.kurz@uni-bayreuth.de                   *
// *       Homepage:     www.mathe2.uni-bayreuth.de/sascha             *
// *                                                                   *
// *       Filename:           konstruiere_alkane.cpp                  *
// *       zuletzt geaendert:  11.01.2005                              *
// *       Betriebssystem:     Suse Linux 9.1                          *
// *       Compiler:           gcc3.3.3                                *
// *       Compiler-Optionen:  g++ -O2 -Wall -pedantic -ansi           *
// *                           konstruiere_alkane.cpp                  *
// *                           -o konstruiere_alkane                   *
// *                                                                   *
// *       Algorithmus                                                 *
// *       -----------                                                 *
// *       Basierend auf dem Artikel "Anzahl von Strukturisomeren      *
// *       der Alkane". Es werden rekursiv Wurzelbaeume zusammenge-    *
// *       setzt, die wiederum zum Zusammensetzen von Alkanen ver-     *
// *       wendet werden.                                              *
// *                                                                   *
// *       + Wurzelbaeume                                              *
// *       -------------                                               *
// *       Ebene 3        o o o o o      Der Knoten in Ebene 1         *
// *                       \|  \|/       heisst Wurzel. Jeder Kno-     *
// *       Ebene 2          o o o        ten, ausser der Wurzel, hat   *
// *                         \|/         genau eine Kante zu einem     *
// *       Ebene 1            o          Knoten auf einer um 1 nie-    *
// *                                     drigeren Ebene.               *
// *                                                                   *
// *       Abgespeichert werden Wurzelbaeume als Array. Hierzu num-    *
// *       meriert man die Knoten, und speichert zu jedem Knoten       *
// *       die Nummer des Knotens auf der um eines niedrigeren Ebene   *
// *       zu dem er eine Kante besitzt. Gibt man der Wurzel die       *
// *       Nummer 0, so kann man als Konvention im Array an der        *
// *       Position 0 ebenfalls die 0 speichern.                       *
// *                                                                   *
// *       Ebene 3        4 5 6 7 8      Nummerierte Version des       *
// *                       \|  \|/       obigen Wurzelbaums. Das       *
// *       Ebene 2          1 2 3        entsprechende Array ist       *
// *                         \|/         in diesem Fall:               *
// *       Ebene 1            0          [0,0,0,0,1,1,3,3,3]           *
// *                                                                   *
// *       + Zusammensetzen von Wurzelbaeumen                          *
// *       ---------------------------------                           *
// *       Die Anzahl der Ebenen eines Wurzelbaums bezeichnet man      *
// *       als seine Hoehe. Wurzelbaeume der Hoehe h kann man folgt    *
// *       aus Wurzelbaeumen kleinerer Hoehe zusammensetzen:           *
// *                                                                   *
// *       4 5 6 7 8   2 3 4              5 6  7 8 9 12 13 14          *
// *        \|  \|/     \|/                \|   \|/    \|/             *
// *         1 2 3       1     1   --->     2 3 4      11 16           *
// *          \|/        |     |             \|/        | |            *
// *           0         0     0              1        10 15           *
// *                                           \        |/             *
// *                                            --------0              *
// *                                                                   *
// *       Das Zusammensetzen und Umnummerieren von Wurzelbaeumen,     *
// *       wird von der Funktion "Fuege_an()" erledigt. Diese fuegt    *
// *       den neu konstruierten Wurzelbaum gleichzeitig noch an       *
// *       eine Liste L von Wurzelbaeumen an. Die Wurzelbaeume         *
// *       kleinerer Hoehe werden als Positionen in der Liste L        *
// *       uebergeben. Da in unserer Anwendung jeder Knoten nur        *
// *       hoechstens 4 Nachbarn hat, brauchen nur 3 int-Werte a,b,c   *
// *       uebergeben werden.                                          *
// *                                                                   *
// *       + Erzeugen von Wurzelbaeumen                                *
// *       ---------------------------                                 *
// *       Im vorhergehenden Abschnitt haben wir erklaert, wie         *
// *       Wurzelbaeume mit kleinerer Hoehe zu Wurzelbaeumen mit       *
// *       groesserer Hoehe zusammengesetzt werden. Hierzu startet     *
// *       man mit einer Liste von Wurzelbaeumen der Hoehe kleiner-    *
// *       gleich h-1 und konstruieren daraus eine Liste von Wurzel-   *
// *       baeumen der Hoehe kleinergleich h. Fuer den Start initiali- *
// *       siert man die Liste mit dem leeren Wurzelbaum und dem       *
// *       Wurzelbaum der Hoehe 1.Nun waehlt man 3 Wurzelbaeume aus der*
// *       Liste der Wurzelbaeume der Hoehe kleinergleich h-1 aus,     *
// *       wobei mindestens einer davon exakt die Hoehe h-1 besitzen   *
// *       muss, und wendet die Funktion "Fuege_an()" an, um aus den   *
// *       3 Wurzelbaeumen einen neuen Wurzelbaum der Hoehe h zu       *
// *       konstruieren. Erledigt wird diese Arbeit von der Funktion   *
// *       "konstruiere_Wurzelbaeume()". Aus Geschwindigkeitsgruenden  *
// *       wird nur mit einer Liste von Wurzelbaeumen gearbeitet, und  *
// *       die Auswahl der 3 Wurzelbaeume wird vorausschauend durchge- *
// *       fuehrt, so dass keine Wurzelbaeume mit "zu vielen" Knoten   *
// *       konstruiert werden.                                         *
// *                                                                   *
// *       + Zusammensetzen von Wurzelbaeumen zu Alkanen               *
// *       --------------------------------------------                *
// *       Um Wurzelbaeume zu Alkanen zusammenzusetzen, muss man die   *
// *       laengste gerade Kette eines Alkans betrachten. U.U. kann es *
// *       mehrere Ketten mit der Maximallaenge k im Molekuel geben.   *
// *       Ist nun k gerade, so kann man das Alkan darstellen als zwei *
// *       Wurzelbaeume W_1 und W_2 der Hoehe k/2, deren Wurzeln durch *
// *       eine Kante verbunden werden: W_1-W_2.                       *
// *       Ist dagegen k ungerade, so besteht das Alkanmolekuel aus 4  *
// *       Wurzelbaeumen, wovon mindestens 2 die Hoehe (k-1)/2 haben   *
// *       muessen und die restlichen maximal diese Hoehe besitzen,    *
// *       deren Wurzeln mit einem zusaetzlichen Knoten durch je       *
// *       eine Kante verbunden sind:                                  *
// *                                                                   *
// *          W_2                                                      *
// *           |                                                       *
// *       W_1-o-W_3                                                   *
// *           |                                                       *
// *          W_4                                                      *
// *                                                                   *
// *       Somit koennen wir ein Alkan speichern, als eine Menge von   *
// *       4 Wurzelbaeumen und der zusaetzlichen Information, ob die   *
// *       Laenge der laengsten geraden Kette gerade oder ungerade ist.*
// *       Die Konstruktion der Alkane aus Wurzelbaeumen uebernimmt die*
// *       Funktion "konstruiere_Alkane()". Das eigentlich zusammen-   *
// *       setzen zu einem Alkan wird von der Funktion "Fuege_an()"    *
// *       erledigt, die das Alkan auch gleichzeitig an eine Liste     *
// *       aus Alkanen anhaengt.                                       *
// *                                                                   *
// *       + Ausgeben von Alkanen auf dem Bildschirm                   *
// *       -----------------------------------------                   *
// *       Mit dem soeben beschriebenen Algorithmus werden die Alkane  *
// *       nicht nur gezaehlt, sondern sogar konstruiert. Da noch      *
// *       keine Anwendung fuer diese konstruierten Objekte            *
// *       implementiert ist, waere es vielleicht sinnvoll sie sich    *
// *       zumindest anschauen zu koennen. Allerdings ist die Ausgabe  *
// *       der Datenstruktur selber dabei wenig sinnvoll. Deswegen     *
// *       sind zwei Funktionen implementiert, die die Alkane entweder *
// *       im XML-Format oder in "Ascii-Art" ausgeben koennen.         *
// *                                                                   *
// *       Die Alkane koennen im XML-Format ausgegeben werden, und     *
// *       dann z.B. mit dem Graphviewer betrachtet werden. Die        *
// *       angegebenen Koordinaten haben wenig mit der Realitaet       *
// *       zu tun und stellen auch nicht unbedingt eine schoene        *
// *       Zeichnung des Graphens dar, doch sind sie immerhin so       *
// *       berechnet, dass die Kanten sich nicht ueberschneiden, und   *
// *       man ein ungefaehres Bild von dem Molekuel bekommen kann.    *
// *                                                                   *
// *       Wahlweise kann man die konstruierten Alkane auch noch       *
// *       Hilfe der Funktion "ausgabe_ascii()" auf dem Bildschirm     *
// *       in "Ascii-Art" ausgeben. Hierbei ist das Ziel ein Alkan     *
// *       ueberschneidungsfrei mit den Zeichen o, /, |, - und \       *
// *       darzustellen. Mit dem realen Aussehen des entsprechenden    *
// *       Alkans wird dies in der Regel wenig zu tun haben, gibt aber *
// *       zumindest eine erste Vorstellung.                           *
// *                                                                   *
// *                                                                   *
// *       Schlussbemerkung                                            *
// *       ===============                                             *
// *       Bei Fragen, Anmerkungen, Kritik oder sonstigen Kommentaren  *
// *       wende man sich bitte per email an den oben genannten Autor  *
// *       dieses Programms. Der Quellcode ist fuer nicht-kommerzielle *
// *       Anwendungen frei verfuegbar und darf beliebig veraendert    *
// *       werden. Bei Veraenderungen ist geeignet darauf hinzuweisen, *
// *       von wem sie vorgenommen wurden. Es wuerde mich freuen ueber *
// *       Verwendungen dieses Quellcodes eine kurze Nachricht per     *
// *       email zu erhalten.                                          *
// *                                                                   *
// *       Sascha Kurz                                                 *
// *                                                                   *
// *********************************************************************



#include<vector>              // Headerfile fuer die Klasse Vector
#include<iostream>            // Headerfile fuer die Ein- und Ausgabe
#include<fstream>             // Headerfile fuer die Ein- und Ausgabe in Dateien

#define BILDSCHIRMBREITE 122  // Breite des Bildschirms in Ascii-Zeichen

#define ASCII false           // false -> Ausgabe im XML-Format
                              // true ->  Ausgabe in "Ascii-art"

using namespace std;

struct Alkan                               // Datenstruktur fuer ein Alkan
{
  vector <unsigned char> Wurzelbaum_1;     // Vektor fuer den ersten Wurzelbaum
  vector <unsigned char> Wurzelbaum_2;     // Vektor fuer den zweiten Wurzelbaum
  vector <unsigned char> Wurzelbaum_3;     // Vektor fuer den dritten Wurzelbaum
  vector <unsigned char> Wurzelbaum_4;     // Vektor fuer den vierten Wurzelbaum
  bool laengste_gerade_Kette_ist_gerade;   // Ist die laengste gerade Kette des Alkans eine gerade Zahl?
};

struct ascii_platzierung                   // Datenstruktur fuer eine Platzierung eines Alkans
{                                          // in "Ascii-Art" auf dem Bildschirm
  vector< vector<char> > Matrix;           // Matrix der auf dem Bildschirm dartzstellenden Zeichen
  unsigned int anzahl_zeilen;              // Anzahl der Spalten der Matrix
  unsigned int anzahl_spalten;             // Anzahl der Zeilen der Matrix
  unsigned int position_wurzel;            // Spalten-Nr. der Wurzel in der untersten (0.) Zeile
};

// *********************************************************************
// *                  Prototypdeklarationen                            *
// *********************************************************************

int ausgabe_ascii(Alkan& A);
// Gibt eine zweidimensionale Platzierung eines Alkanmolekuels in "Ascii-Art"
// auf dem Bildschirm aus, sofern es nicht zu breit ist

int ausgabe(Alkan& A);
// Gibt das Alkanmolekuel im XML-Format in der Datei graph aus

void Fuege_an(vector< vector <unsigned char> >& L,int a,int b,int c);
// Konstruiert aus 3 Wurzelbaeumen einen neuen Wurzelbaum mit groesserer
// Hoehe und fuegt ihn der Liste L an

void Fuege_an(vector<Alkan>&,const vector< vector <unsigned char> >& L,
unsigned int a,unsigned int b,bool ausgeben,int ausgabe_bei_anzahl_c_atomen);
// Konstruiert aus 2 Wurzelbaeumen der Hoehe kleinergleich h ein Alkan mit
// laengster gerader Kette 2h und fuegt es der Liste Alkane an


void Fuege_an(vector<Alkan>&,const vector< vector <unsigned char> >& L,
unsigned int a,unsigned int b,unsigned int c,unsigned d,bool ausgeben,int ausgabe_bei_anzahl_c_atomen);
// Konstruiert aus 4 Wurzelbaeumen der Hoehe kleinergleich h ein Alkan mit
// laengster gerader Kette 2h+1 und fuegt es der Liste Alkane an

void sortiere_Wurzelbaum_Liste(vector< vector <unsigned char> >& L,int p);
// Sortiert eine Liste L von Wurzelbaeumen, so dass die ersten p Wurzelbaeumen
// und die restlichen Wurzelbaeumen jeweils nach der Anzahl der Knoten
// aufsteigend angeordnet sind

void konstruiere_Wurzelbaeume(vector<Alkan>& Alkane,vector< vector <unsigned char> >& L,int n,bool ausgeben,int ausgabe_bei_anzahl_c_atomen);
// Konstuiert eine "vollstaendige" Liste von Wurzelbaeumen, so dass damit
// eine vollstaendige Liste aller Alkane mit bis zu n C-Atomen konstruiert
// werden kann.

void konstruiere_Alkane(vector<Alkan>& Alkane,const vector< vector <unsigned char> >& L,
unsigned int p,unsigned int h,unsigned int n,bool ausgeben,int ausgabe_bei_anzahl_c_atomen);
// Erweitert eine Liste von Alkanen mit denen solchen die eine laengste gerade
// Kette der Laenge 2h oder 2h+1 und maximal n Knoten haben. Die dafuer notwendigen
// Wurzelbaeume der Hoehe h stehen in der Liste L ab Position p



// ********************************************************************
// *         Implementierungen der benoetigten Funktionen              *
// ********************************************************************

void Fuege_an(vector< vector <unsigned char> >& L,int a,int b,int c)
{
  // L:   Liste von Wurzelbaeumen
  // a:   Position eines Wurzelbaums in der Liste L
  // b:   Position eines Wurzelbaums in der Liste L
  // c:   Position eines Wurzelbaums in der Liste L
  //
  // Funktion:
  // ---------
  // Konstruiert einen neuen Wurzelbaum W, indem die Wurzelbaeume Nr. a,b
  // und c vereinigt werden, und die Wurzeln von a,b und c mit einer
  // neuen Wurzel verbunden werden.
  //
  //      a b c
  //  W=   \|/
  //        o
  //
  // Der neu konstruierte Wurzelbaum W wird an die Liste L angehaengt

  int p=L.size();               // die alte Laenge der Liste der Wurzelbaeume
  unsigned int i;               // Schleifenvariable
  int a_s=L[a].size();          // aus wie vielen Knoten besteht Wurzelbaum Nr. a
  int b_s=L[b].size();          // aus wie vielen Knoten besteht Wurzelbaum Nr. b
  int c_s=L[c].size();          // aus wie vielen Knoten besteht Wurzelbaum Nr. c
  L.resize(p+1);                // Verlaengere die Liste um ein Element
  L[p].resize(a_s+b_s+c_s+1);   // Der neue Wurzelbaum setzt sich aus Wurzelbaum Nr. a,
                                // Wurzelbaum Nr. b, Wurzelbaum Nr. c und einem zusaetzlichen
                                // Knoten, der neuen Wurzel, zusammen
  L[p][0]=0;                    // neue Wurzel
  L[p][1]=0;                    // alte Wurzel von Wurzelbaum Nr. a zeigt auf neue Wurzel
  for(i=1;i<L[a].size();i++)    // durchlaufe die restlichen Knoten von Wurzelbaum Nr. a
    L[p][i+1]=L[a][i]+1;        // verschiebe die alte Nummerierung
  L[p][1+a_s]=0;                // alte Wurzel von Wurzelbaum Nr. b zeigt auf neue Wurzel
  for(i=1;i<L[b].size();i++)    // durchlaufe die restlichen Knoten von Wurzelbaum Nr. a
    L[p][i+1+a_s]=L[b][i]+1+a_s;// verschiebe die alte Nummerierung
  L[p][1+a_s+b_s]=0;            // alte Wurzel von Wurzelbaum Nr. c zeigt auf neue Wurzel
  for(i=1;i<L[c].size();i++)    // durchlaufe die restlichen Knoten von Wurzelbaum Nr. a
    L[p][i+1+a_s+b_s]=L[c][i]+1+a_s+b_s; // verschiebe die alte Nummerierung
}

void Fuege_an(vector<Alkan>& Alkane,const vector< vector <unsigned char> >& L,unsigned int a,
unsigned int b,bool ausgeben,int ausgabe_bei_anzahl_c_atomen)
{
  // Alkane : Liste von Alkanen
  // L:       Liste von Wurzelbaeumen
  // a:       Position eines Wurzelbaums in der Liste L
  // b:       Position eines Wurzelbaums in der Liste L
  //
  // Funktion:
  // ---------
  // Konstruiert aus zwei Wurzelbaeumen W_1 und W_2 der  Hoehe h ein Alkan A mit laengster gerader Kette 2h:
  //
  //  A=  W_1-W_2 (die Wurzeln von W_1 und W_2 werden mit einer Kante verbunden)
  //
  // Das neu konstruierte Alkan A wird an die Liste Alkane angehaengt

  unsigned int i;               // Schleifenvariable;
  int p=Alkane.size();          // die alte Laenge der Liste der Alkane
  Alkane.resize(p+1);           // verlaengere die Liste um ein Element

  Alkane[p].laengste_gerade_Kette_ist_gerade=true;
  Alkane[p].Wurzelbaum_1.resize(L[a].size());   // Reserviere Speicherplatz fuer den Wurzelbaum Nr. a
  for(i=0;i<L[a].size();i++)                    // Kopiere Wurzelbaum Nr. a
    Alkane[p].Wurzelbaum_1[i]=L[a][i];
  Alkane[p].Wurzelbaum_2.resize(L[b].size());   // Reserviere Speicherplatz fuer den Wurzelbaum Nr. b
  for(i=0;i<L[b].size();i++)                    // Kopiere Wurzelbaum Nr. b
    Alkane[p].Wurzelbaum_2[i]=L[b][i];
  Alkane[p].Wurzelbaum_3.resize(0);             // Da die Laenge der laensten geraden Kette eine gerade
  Alkane[p].Wurzelbaum_4.resize(0);             // Zahl ist, werden nur 2 Wurzelbaeume benutzt

  if(ausgeben==true)
  {
    if(ausgabe_bei_anzahl_c_atomen==-1|| (int)(Alkane[p].Wurzelbaum_1.size()+Alkane[p].Wurzelbaum_2.size())==ausgabe_bei_anzahl_c_atomen)
      if(ASCII==true)     
        ausgabe_ascii(Alkane[p]);
      else
        ausgabe(Alkane[p]);
  }
}

void Fuege_an(vector<Alkan>& Alkane,const vector< vector <unsigned char> >& L,unsigned int a,
unsigned int b,unsigned int c,unsigned int d,bool ausgeben,int ausgabe_bei_anzahl_c_atomen)
{
  // Alkane : Liste von Alkanen
  // L:       Liste von Wurzelbaeumen
  // a:       Position eines Wurzelbaums in der Liste L
  // b:       Position eines Wurzelbaums in der Liste L
  // c:       Position eines Wurzelbaums in der Liste L
  // d:       Position eines Wurzelbaums in der Liste L
  //
  // Funktion:
  // ---------
  // Konstruiert aus vier Wurzelbaeumen W_1, W_2, W_3 und W_4 der Hoehe kleinergleich h ein Alkan A mit
  // laengster gerader Kette 2h+1:
  //
  //         W_2
  //          |
  //  A=  W_1-o-W_3          (die Wurzeln von W_1, W_2, W_3 und W_4 werden mit dem
  //          |              zusaetzlichen Knoten durch je eine Kante verbunden)
  //         W_4
  //
  // Das neu konstruierte Alkan A wird an die Liste Alkane angehaengt

  unsigned int i;               // Schleifenvariable;
  int p=Alkane.size();          // die alte Laenge der Liste der Alkane
  Alkane.resize(p+1);           // verlaengere die Liste um ein Element

  Alkane[p].laengste_gerade_Kette_ist_gerade=false;
  Alkane[p].Wurzelbaum_1.resize(L[a].size());   // Reserviere Speicherplatz fuer den Wurzelbaum Nr. a
  for(i=0;i<L[a].size();i++)                    // Kopiere Wurzelbaum Nr. a
    Alkane[p].Wurzelbaum_1[i]=L[a][i];
  Alkane[p].Wurzelbaum_2.resize(L[b].size());   // Reserviere Speicherplatz fuer den Wurzelbaum Nr. b
  for(i=0;i<L[b].size();i++)                    // Kopiere Wurzelbaum Nr. b
    Alkane[p].Wurzelbaum_2[i]=L[b][i];
  Alkane[p].Wurzelbaum_3.resize(L[c].size());   // Reserviere Speicherplatz fuer den Wurzelbaum Nr. c
  for(i=0;i<L[c].size();i++)                    // Kopiere Wurzelbaum Nr. c
    Alkane[p].Wurzelbaum_3[i]=L[c][i];
  Alkane[p].Wurzelbaum_4.resize(L[d].size());   // Reserviere Speicherplatz fuer den Wurzelbaum Nr. d
  for(i=0;i<L[d].size();i++)                    // Kopiere Wurzelbaum Nr. d
    Alkane[p].Wurzelbaum_4[i]=L[d][i];
  if(ausgeben==true)
  {
    if(ausgabe_bei_anzahl_c_atomen==-1|| (int)(Alkane[p].Wurzelbaum_1.size()+Alkane[p].Wurzelbaum_2.size()+Alkane[p].Wurzelbaum_3.size()+Alkane[p].Wurzelbaum_4.size()+1)==ausgabe_bei_anzahl_c_atomen)
      if(ASCII==true)
        ausgabe_ascii(Alkane[p]);
      else
        ausgabe(Alkane[p]);
  }
}


void sortiere_Wurzelbaum_Liste(vector< vector <unsigned char> >& L,int laenge,int n)
{
  // L:       Liste von Wurzelbaeumen
  // laenge:  Position in Liste L, siehe "Funktion"
  // n:       maximale Anzahl der Knoten
  //
  // Funktion:
  // ---------
  // Sortiert die Positionen 0 bis laenge-1 der Liste L so um, dass die Anzahl der
  // Knoten der Wurzelbaeume aufsteigend ist. Die Positionen laenge bis L.size()-1
  // werden entsprechend sortiert.
  // Da die maximale Anzahl der Knoten (n) gegenueber der Laenge der Liste eine sehr
  // kleine Zahl ist, werden anstatt einem Sortieralgorithmus wie Mergesort oder
  // Quicksort einfach n+1 Listen angelegt, und jeder Wurzelbaum wird in die Liste
  // mit der Nummer entsprechend der Anzahl seiner Knoten gelegt. Anschliessend
  // werden diese Listen wieder zu einer zusammengefuegt. Der Aufwand betraegt somit
  // O(n+L.size())

  int i,j;            // Hilfsvariablen
  unsigned int k,a,b; // Hilfsvariablen

  vector< vector< vector<unsigned char> > > sort;     // Hilfsvektor zum Sortieren

  sort.resize(n+1);                    // Anlegen der n+1 Hilfslisten
  for(i=0;i<=n;i++)                    // durchlaufe die Hilfslisten
    sort[i].resize(0);                 // Jede Liste ist am Anfang leer

  // Trage die Wurzelbaeume, der Positionen 0,...,laenge-1, in die Hilfslisten ein
  for(i=0;i<laenge;i++)                // durchlaufe die Positionen 0 bis laenge-1 von L
  {
    k=L[i].size();                     // Wie viele Knoten besitzt der gerade betrachtete Wurzelbaum?
    a=sort[k].size();                  // Wie lang ist die entsprechende Hilfsliste
    sort[k].resize(a+1);               // Verlaengere die Hilfsliste um ein Element
    sort[k][a].resize(k);              // Der neue Wurzelbaum besteht aus k Knoten
    for(b=0;b<k;b++)                   // Fuege den betrachteten Wurzelbaum in die
      sort[k][a][b]=L[i][b];           // Hilfsliste ein
  }

  // Trage die Wurzelbaeume aus den Hilfslisten wieder zurueck in L ein
  j=0;                                 // Aktuelle Position in L
  for(i=0;i<=n;i++)                    // durchlaufe die unterschiedlichen Hilfslisten
  {
    for(a=0;a<sort[i].size();a++)      // durchlaufe Hilfsliste i
    {
      L[j].resize(i);                  // Speichere an Position j in L einen Wurzelbaum bestehend aus i Knoten
      for(b=0;b<(unsigned int)i;b++)
        L[j][b]=sort[i][a][b];
      j++;                             // Verschiebe die aktuelle Position in L um eins nach rechts
    }
  }

  sort.resize(n+1);                    // "Loesche" die Hilfslisten wieder
  for(i=0;i<=n;i++)
  {
    for(a=0;a<sort[i].size();a++)
      sort[i][a].resize(0);
    sort[i].resize(0);
  }

  // Trage die Wurzelbaeume, der Positionen laenge,...,L.size()-1, in die Hilfslisten ein
  for(i=laenge;i<(int)L.size();i++)         // durchlaufe die Positionen laenge bis L.size()-1 von L
  {
    k=L[i].size();                     // Wie viele Knoten besitzt der gerade betrachtete Wurzelbaum?
    a=sort[k].size();                  // Wie lang ist die entsprechende Hilfsliste
    sort[k].resize(a+1);               // Verlaengere die Hilfsliste um ein Element
    sort[k][a].resize(k);              // Der neue Wurzelbaum besteht aus k Knoten
    for(b=0;b<k;b++)                   // Fuege den betrachteten Wurzelbaum in die
      sort[k][a][b]=L[i][b];           // Hilfsliste ein
  }

  // Trage die Wurzelbaeume aus den Hilfslisten wieder zurueck in L ein
  for(i=0;i<=n;i++)                    // durchlaufe die unterschiedlichen Hilfslisten
  {
    for(a=0;a<sort[i].size();a++)      // durchlaufe Hilfsliste i
    {
      L[j].resize(i);                  // Speichere an Position j in L einen Wurzelbaum bestehend aus i Knoten
      for(b=0;b<(unsigned int)i;b++)
        L[j][b]=sort[i][a][b];
      j++;                             // Verschiebe die aktuelle Position in L um eins nach rechts
    }
  }

  sort.resize(n+1);                    // "Loesche" die Hilfslisten wieder
  for(i=0;i<=n;i++)
  {
    for(a=0;a<sort[i].size();a++)
      sort[i][a].resize(0);
    sort[i].resize(0);
  }
}


void konstruiere_Wurzelbaeume(vector<Alkan>& Alkane,vector< vector <unsigned char> >& L,int n,bool ausgeben,int ausgabe_bei_anzahl_c_atomen)
{
  // L: Liste zum Speichern von Wurzelbaeumen
  // n: maximale Anzahl von C-Atomen in den spaeter zu konstruierenden Alkanen
  //
  // Funktion:
  // ---------
  // Ziel ist es eine vollstaendige Liste aller Wurzelbaeume der Hoehe h mit maximal n-h Knoten
  // zu konstruieren. Hierbei laeuft h von 0 bis [n/2].
  // Dabei werden Wurzelbaeume der Hoehe h aus Wurzelbaeumen der Hoehe kleinergleich h-1 zusammen-
  // gesetzt. Fuer weitere Details lese den Artikel:"Anzahl von Strukturisomeren der Alkane"



  // Startbelegung fuer die Liste der Wurzelbaeume
  L.resize(2);                              // Am Anfang sind nur 2 Wurzelbaeume in der Liste eingetragen
  L[0].resize(0);                           // Leerer Wurzelbaum aus 0 Knoten
  L[1].resize(1);
  L[1][0]=0;                                // Wurzelbaum aus einem Knoten
  int p=1;                                  // Position des Wurzelbaums der Hoehe 1
                                            // Allgemein: Position des ersten Wurzelbaums der Hoehe h in der
                                            // Liste L

  // Startbelegung fuer die Liste der Alkane
  Alkane.resize(2);
  Alkane[0].laengste_gerade_Kette_ist_gerade=true;   // Leeres Alkan
  Alkane[0].Wurzelbaum_1.resize(0);
  Alkane[0].Wurzelbaum_2.resize(0);
  Alkane[0].Wurzelbaum_3.resize(0);
  Alkane[0].Wurzelbaum_4.resize(0);
  Alkane[1].laengste_gerade_Kette_ist_gerade=false;  // Alkan aus einem C-Atom (Methan)
  Alkane[1].Wurzelbaum_1.resize(0);
  Alkane[1].Wurzelbaum_2.resize(0);
  Alkane[1].Wurzelbaum_3.resize(0);
  Alkane[1].Wurzelbaum_4.resize(0);

  cout << endl;
  if(ausgeben==true)
    if(ausgabe_bei_anzahl_c_atomen==-1 || ausgabe_bei_anzahl_c_atomen==1)
      cout << "o" << endl << endl;

  if(ausgeben==false)
  {
    cout << endl;
    cout << "Konstruiere Wurzelbaeume der Hoehe 1 ";
    if(n>=20) cout << " ";
    cout << ": ." << endl;
  }
  // Konstruiere Alkane mit laengster gerader Kette 2 und 3
  konstruiere_Alkane(Alkane,L,p,1,n,ausgeben,ausgabe_bei_anzahl_c_atomen);


  int a,b,c;                                // Position der drei kleineren Wurzelbaeume in der Liste L
  int laenge;                               // Hilfsgroesse, um "p" bzw. die Laenge der Liste auch bei
                                            // Änderungen an der Liste verfuegbar zu haben

  for(int h=2;h<=int(n/2);h++)              // Durchlaufe die moeglichen Hoehen der Wurzelbaeume. Die Wurzelbaeume
  {                                         // der Hoehe 0 und 1 wurden mit der Startbelegung behandelt.
    // Forstschrittsausgabe, da Berechnung lange dauern kann
    if(ausgeben==false)
    {
      cout << "Konstruiere Wurzelbaeume der Hoehe " << h << " ";
      if(n>=20 && h<10) cout << " ";
      cout << ": ";
    }

    laenge=L.size();                        // Wie  lang ist die Liste L bisher

    // In der Liste L stehen an den Positionen 0 bis p-1 Wurzelbaeume der Hoehe kleiner h-1
    // geordnet nach der Anzahl der Knoten. An den Positionen p bis laenge-1 stehen
    // Wurzelbaeume der Hoehe h-1, ebenfalls nach der Anzahl der Knoten geordnet.
    // Fuer die Kombination zu einem Wurzelbaum der Hoehe h gibt es nun 3 Moeglichkeiten:
    // (i)   zwei Wurzelbaeume der Hoehe kleiner h-1 und ein Wurzelbaum der Hoehe h-1
    // (ii)  ein Wurzelbaum der Hoehe kleiner h-1 und zwei Wurzelbaeume der Hoehe h-1
    // (iii) drei Wurzelbaeume der Hoehe h-1
    //
    // Die Positionen der Wurzelbaeume in L werden mit a,b und c bezeichnet. Um die Konstruktion
    // zu beschleunigen, wird bei jedem Schritt ueberprueft, ob ueberhaupt ein Alkan mit hoechstens
    // n C-Atomen entstehen kann.


    // Fall (i)
    for(a=0;a<p;a++)                                // a ist ein Wurzelbaum der Hoehe kleiner h-1
    {
      if(ausgeben==false)
        cout << "." << flush;                       // Forstschrittsausgabe, da Berechnung lange dauern kann
      if(1+(int)L[a].size()*2+(h-1)+h<=n)           // 1:               die neue Wurzel
      {                                             // 2*L[a].size()*2: b besitzt mindestens genau so viele
                                                    //                  Knoten wie a, also fuer a und b zusammen
                                                    //                  2*L[a].size()*2
                                                    // (h-1):           Wurzelbaum c besitzt Hoehe h-1, also mindestens
                                                    //                  h-1 Knoten
                                                    // h:               Es werden mindestens zwei Wurzelbaeume der Hoehe
                                                    //                  h zu einem Alkan zusammengesetzt
        for(b=a;b<p;b++)                            // b ist ein Wurzelbaum der Hoehe kleiner h-1
        {
          if(1+(int)L[a].size()+(int)L[b].size()+(h-1)+h<=n)  // 1:           die neue Wurzel
          {                                         // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                    // L[b].size(): Anzahl der Knoten von Wurzelbaum b
                                                    // (h-1):       Wurzelbaum c besitzt Hoehe h-1, also mindestens
                                                    //              h-1 Knoten
                                                    // h:               Es werden mindestens zwei Wurzelbaeume der Hoehe
                                                    //                  h zu einem Alkan zusammengesetzt
            for(c=p;c<laenge;c++)                   // c ist ein Wurzelbaum der Hoehe h-1
            {
              if(1+(int)L[a].size()+(int)L[b].size()+(int)L[c].size()+h<=n) // 1:           die neue Wurzel
              {                                              // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                             // L[b].size(): Anzahl der Knoten von Wurzelbaum b
                                                             // L[c].size(): Anzahl der Knoten von Wurzelbaum b
                                                             // h:           Es werden mindestens zwei Wurzelbaeume der
                                                             //              Hoehe h zu einem Alkan zusammengesetzt
                Fuege_an(L,a,b,c);                  // Fuege die Wurzelbaeume a,b und c zusammen und haenge sie an
              }                                     // die Liste L an
              else                                  // Der entehende Wurzelbaum kann nicht zur Konstruktion eines
                c=laenge;                           // Alkans mit maximal n C-Atomen verwendet werden, also gehe mit
            }                                       // b eins weiter und starte mit c wieder bei Position p
          }
          else                                      // Die entehenden Wurzelbaeume koennen nicht zur Konstruktion eines
          {                                         // Alkans mit maximal n C-Atomen verwendet werden, also gehe mit
            b=p;                                    // a eins weiter und starte mit b wieder bei Position a
          }
        }
      }
      else                                          // Die entehenden Wurzelbaeume koennen nicht zur Konstruktion eines
      {                                             // Alkans mit maximal n C-Atomen verwendet werden, also beende die
        a=p;                                        // Schleife
      }
    }


    // Fall (ii)
    for(a=0;a<p;a++)                                // a ist ein Wurzelbaum der Hoehe kleiner h-1
    {
      if(ausgeben==false)
        cout << "." << flush;                       // Forstschrittsausgabe, da Berechnung lange dauern kann
      if(1+(int)L[a].size()+2*(h-1)+h<=n)           // 1:           die neue Wurzel
      {                                             // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                    // 2*(h-1):     Die Wurzelbaeume b und c besitzen die Hoehe
                                                    //              h-1, bestehen also aus mindestens h-1 Knoten
                                                    // h:           Es werden mindestens zwei Wurzelbaeume der
                                                    //              Hoehe h zu einem Alkan zusammengesetzt
        for(b=p;b<laenge;b++)                       // b ist ein Wurzelbaum der Hoehe h-1
        {
          if(1+(int)L[a].size()+2*((int)L[b].size())+h<=n)      // 1:           die neue Wurzel
          {                                         // L[a].size(): Anzahl der Knoten von Wurzelbaum a

                                                    // h:           Es werden mindestens zwei Wurzelbaeume der
                                                    //              Hoehe h zu einem Alkan zusammengesetzt
            for(c=b;c<laenge;c++)                   // c ist ein Wurzelbaum der Hoehe h-1
              if(1+(int)L[a].size()+(int)L[b].size()+(int)L[c].size()+h<=n) // 1:           die neue Wurzel
              {                                              // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                             // L[b].size(): Anzahl der Knoten von Wurzelbaum b
                                                             // L[c].size(): Anzahl der Knoten von Wurzelbaum b
                                                             // h:           Es werden mindestens zwei Wurzelbaeume der
                                                             //              Hoehe h zu einem Alkan zusammengesetzt
                Fuege_an(L,a,b,c);                  // Fuege die Wurzelbaeume a,b und c zusammen und haenge sie an
              }                                     // die Liste L an
              else                                  // Der entehende Wurzelbaum kann nicht zur Konstruktion eines
                c=laenge;                           // Alkans mit maximal n C-Atomen verwendet werden, also gehe mit
          }                                         // b eins weiter und starte mit c wieder bei Position b
          else                                      // Die entehenden Wurzelbaeume koennen nicht zur Konstruktion eines
          {                                         // Alkans mit maximal n C-Atomen verwendet werden, also gehe mit
            b=laenge;                               // a eins weiter und starte mit b wieder bei Position p
          }
        }
      }
      else                                          // Die entehenden Wurzelbaeume koennen nicht zur Konstruktion eines
      {                                             // Alkans mit maximal n C-Atomen verwendet werden, also beende die
        a=p;                                        // Schleife
      }
    }


    // Fall (iii)
    for(a=p;a<laenge;a++)                           // a ist ein Wurzelbaum der Hoehe h-1
    {
      if(ausgeben==false)
        cout << "." << flush;                       // Forstschrittsausgabe, da Berechnung lange dauern kann
      if(1+3*((int)L[a].size())+h<=n)               // 1:             die neue Wurzel
      {                                             // 3*L[a].size(): Die Wurzelbaeume b und c besitzen
                                                    //                mindestens so viele Knoten wie der
                                                    //                Wurzelbauma
                                                    // h:             Es werden mindestens zwei Wurzelbaeume der
                                                    //                Hoehe h zu einem Alkan zusammengesetzt
        for(b=a;b<laenge;b++)                       // b ist ein Wurzelbaum der Hoehe h-1
        {
          if(1+(int)L[a].size()+2*((int)L[b].size())+h<=n)      // 1:             die neue Wurzel
          {                                         // L[a].size():   Anzahl der Knoten von Wurzelbaum a
                                                    // 2*L[b].size(): Der Wurzelbaum c besitzt mindestens
                                                    //                so viele Knoten wie Wurzelbaum b
                                                    // h:             Es werden mindestens zwei Wurzelbaeume der
                                                    //                Hoehe h zu einem Alkan zusammengesetzt
            for(c=b;c<laenge;c++)                   // c ist ein Wurzelbaum der Hoehe h-1
              if(1+(int)L[a].size()+(int)L[b].size()+(int)L[c].size()+h<=n) // 1:           die neue Wurzel
              {                                              // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                             // L[b].size(): Anzahl der Knoten von Wurzelbaum b
                                                             // L[c].size(): Anzahl der Knoten von Wurzelbaum b
                                                             // h:           Es werden mindestens zwei Wurzelbaeume der
                                                             //              Hoehe h zu einem Alkan zusammengesetzt
                Fuege_an(L,a,b,c);                  // Fuege die Wurzelbaeume a,b und c zusammen und haenge sie an
              }                                     // die Liste L an
              else                                  // Der entehende Wurzelbaum kann nicht zur Konstruktion eines
                c=laenge;                           // Alkans mit maximal n C-Atomen verwendet werden, also gehe mit
          }                                         // b eins weiter und starte mit c wieder bei Position b
          else                                      // Die entehenden Wurzelbaeume koennen nicht zur Konstruktion eines
          {                                         // Alkans mit maximal n C-Atomen verwendet werden, also gehe mit
            b=laenge;                               // a eins weiter und starte mit b wieder bei Position a
          }
        }
      }
      else                                          // Die entehenden Wurzelbaeume koennen nicht zur Konstruktion eines
      {                                             // Alkans mit maximal n C-Atomen verwendet werden, also beende die
        a=laenge;                                   // Schleife
      }
    }
    if(ausgeben==false)
      cout << endl;


    sortiere_Wurzelbaum_Liste(L,laenge,n);          // Stelle die "richtige" Sortierung der Liste L wieder her, so
                                                    // wie sie fuer diesen Algorithmus gebraucht wird
    p=laenge;                                       // Start fuer Wurzelbaeume der Laenge h

    // Konstruiere Alkane mit laengster gerader Kette der Laenge 2h und 2h+1 aus den bereits erzeugten Wurzelbaeumen
    konstruiere_Alkane(Alkane,L,p,h,n,ausgeben,ausgabe_bei_anzahl_c_atomen);

  }
}

void konstruiere_Alkane(vector<Alkan>& Alkane,const vector< vector <unsigned char> >& L,
unsigned int p,unsigned int h,unsigned int n,bool ausgeben,int ausgabe_bei_anzahl_c_atomen)
{
  // Alkane:  eine Liste von Alkanen
  // L:       eine Liste von Wurzelbaeumen
  // p:       Position in L, ab der die Wurzelbaeume der Hoehe h stehen
  // h:       Hoehe der Wurzelbaeume, die aktuell fuer die Konstruktion verwendet werden sollen
  // n:       maximale Anzahl der C-Atome, die die Alkane besitzen sollen
  //
  // Funktion:
  // ---------
  // Erweitert eine Liste von Alkanen mit denen solchen die eine laengste gerade
  // Kette der Laenge 2h oder 2h+1 und maximal n Knoten haben. Die dafuer notwendigen
  // Wurzelbaeume der Hoehe h stehen in der Liste L ab Position p

  if(ausgeben==false)
  {
    cout << "Konstruiere zugehoerige Alkane      ";// Fortschrittsanzeige
    if(n>=20) cout << " ";
    cout << ": " << flush;
  }

  unsigned int a,b,c,d;                           // Nummer der bis zu 4 Wurzelbaeume, die aus der Liste L ausgewaehlt werden

  if(2*h>n) return;                               // Es koennen keine Alkane mit hinreichend wenig C-Atomen
                                                  // konstruiert werden

  // In der Liste L stehen an den Positionen 0 bis p-1 Wurzelbaeume der Hoehe kleiner h
  // geordnet nach der Anzahl der Knoten. An den Positionen p bis laenge-1 stehen
  // Wurzelbaeume der Hoehe h, ebenfalls nach der Anzahl der Knoten geordnet.
  // Fuer die Kombination zu einem Alkan mit laengster gerader Kette der Laenge 2h gibt
  // es nun 1 Moeglichkeit:
  // (i)   zwei Wurzelbaeume der Hoehe h, deren Wurzeln mit einer Kante verbunden werden
  // Die Positionen der Wurzelbaeume in L werden mit a und b bezeichnet. Um die Konstruktion
  // zu beschleunigen, wird bei jedem Schritt ueberprueft, ob ueberhaupt ein Alkan mit hoechstens
  // n C-Atomen entstehen kann.


  for(a=p;a<L.size();a++)                         // a ist ein Wurzelbaum der Hoehe h
  {
    if(ausgeben==false)
      cout << "." << flush;                       // Fortschrittsanzeige
    if(2*L[a].size()<=n)                          // der Wurzelbaum b besitzt mindestens so viele Knoten wie a
    {
      for(b=a;b<L.size();b++)                     // b ist ein Wurzelbaum der Hoehe h
        if(L[a].size()+L[b].size()<=n)            // L[a].size(): Anzahl der Knoten vom Wurzelbaum a
        {                                         // L[b].size(): Anzahl der Knoten vom Wurzelbaum b
          Fuege_an(Alkane,L,a,b,ausgeben,ausgabe_bei_anzahl_c_atomen); // Fuege die Wurzelbaeume a und b zusammen und
        }                                         // haenge sie an die Liste Alkane an
        else
          b=L.size();                             // Es kann kein Alkan mehr mit hoechstens n C-Atomen konstruiert
    }                                             // werden. Erhoehe a um eins und setze b zurueck
    else                                          // Es kann kein Alkan mehr mit hoechstens n C-Atomen konstruiert
      a=L.size();                                 // werden. Beende die Schleife.
  }


  // Fuer die Kombination zu einem Alkan mit laengster gerader Kette der Laenge 2h+1 gibt
  // es nun 3 Moeglichkeiten:
  // (i)   zwei Wurzelbaeume der Hoehe kleiner h und 2 Wurzelbaeume der Hoehe h
  // (ii)  ein Wurzelbaum der Hoehe kleiner h und 3 Wurzelbaeume der Hoehe h
  // (iii) vier Wurzelbaeume der Hoehe h
  // Die Positionen der Wurzelbaeume in L werden mit a,b,c und d bezeichnet. Um die Konstruktion
  // zu beschleunigen, wird bei jedem Schritt ueberprueft, ob ueberhaupt ein Alkan mit hoechstens
  // n C-Atomen entstehen kann.

  if(2*h+1<=n)                                    // Falls es ueberhaupt moeglich ist, dass ein Alkan mit
  {                                               // hoechstens n C-Atomen entstehen kann
    // Fall (i)
    for(a=0;a<p;a++)                              // a ist ein Wurzelbaum der Hoehe kleiner h
    {
      if(ausgeben==false)
        cout << "." << flush;                     // Fortschrittsanzeige
      if(1+2*L[a].size()+2*h<=n)                  // 1:             zusaetzlicher Knoten
      {                                           // 2*L[a].size(): der Wurzelbaum b besitzt mindestens so
                                                  //                viele Knoten wie der Wurzelbaum a
                                                  // 2*h:           die Wurzelbaeume c und d besitzen die Hoehe
                                                  //                h, bestehen also aus mindestens h Knoten
        for(b=a;b<p;b++)                          // b ist ein Wurzelbaum der Hoehe kleiner h
          if(1+L[a].size()+L[b].size()+2*h<=n)    // 1:           zusaetzlicher Knoten
          {                                       // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                  // L[b].size(): Anzahl der Knoten von Wurzelbaum b
                                                  // 2*h:           die Wurzelbaeume c und d besitzen die Hoehe
                                                  //                h, bestehen also aus mindestens h Knoten
            for(c=p;c<L.size();c++)               // c ist ein Wurzelbaum der Hoehe h
              if(1+L[a].size()+L[b].size()+2*L[c].size()<=n)
              {                                   // 1:             zusaetzlicher Knoten
                                                  // L[a].size():   Anzahl der Knoten von Wurzelbaum a
                                                  // L[b].size():   Anzahl der Knoten von Wurzelbaum b
                                                  // 2*L[c].size(): der Wurzelbaum d besitzt mindestens so
                                                  //                viele Knoten wie der Wurzelbaum c
                for(d=c;d<L.size();d++)           // d ist ein Wurzelbaum der Hoehe h
                  if(1+L[a].size()+L[b].size()+L[c].size()+L[d].size()<=n)
                  {                               // 1:           zusaetzlicher Knoten
                                                  // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                  // L[b].size(): Anzahl der Knoten von Wurzelbaum b
                                                  // L[c].size(): Anzahl der Knoten von Wurzelbaum c
                                                  // L[d].size(): Anzahl der Knoten von Wurzelbaum d
                    Fuege_an(Alkane,L,a,b,c,d,ausgeben,ausgabe_bei_anzahl_c_atomen);
                                                  // Fuege die Wurzelbaeume a, b, c und d mit einem zusaetzlichen Knoten
                  }                               // zusammen und haenge das entstehende Alkan an die Liste an
                  else                            // Es kann kein Alkan mehr mit hoechstens n C-Atomen konstruiert
                    d=L.size();                   // werden. Erhoehe c um eins und setze d zurueck
              }
              else                                // Es kann kein Alkan mehr mit hoechstens n C-Atomen konstruiert
                c=L.size();                       // werden. Erhoehe b um eins und setze c zurueck
          }
          else                                    // Es kann kein Alkan mehr mit hoechstens n C-Atomen konstruiert
            b=p;                                  // werden. Erhoehe a um eins und setze b zurueck
      }
      else                                        // Es kann kein Alkan mehr mit hoechstens n C-Atomen konstruiert
        a=p;                                      // werden. Beende die Schleife.
    }

    // Fall (ii)
    for(a=0;a<p;a++)                              // a ist ein Wurzelbaum der Hoehe kleiner h
    {
      if(ausgeben==false)
        cout << "." << flush;                     // Fortschrittsanzeige
      if(1+L[a].size()+3*h<=n)                    // 1:             zusaetzlicher Knoten
      {                                           // L[a].size():   Anzahl der Knoten von Wurzelbaum a
                                                  // 3*h:           die Wurzelbaeume b, c und d besitzen die Hoehe
                                                  //                h, bestehen also aus mindestens h Knoten
        for(b=p;b<L.size();b++)                   // b ist ein Wurzelbaum der Hoehe h
          if(1+L[a].size()+3*L[b].size()<=n)      // 1:             zusaetzlicher Knoten
          {                                       // L[a].size():   Anzahl der Knoten von Wurzelbaum a
                                                  // 3*L[b].size(): die Wurzelbaeume c und d besitzten mindestens
                                                  //                so viele Knoten wie der Wurzelbaum b
            for(c=b;c<L.size();c++)               // c ist ein Wurzelbaum der Hoehe h
              if(1+L[a].size()+L[b].size()+2*L[c].size()<=n)
              {                                   // 1:             zusaetzlicher Knoten
                                                  // L[a].size():   Anzahl der Knoten von Wurzelbaum a
                                                  // L[b].size():   Anzahl der Knoten von Wurzelbaum b
                                                  // 2*L[c].size(): der Wurzelbaum d besitzt mindestens so
                                                  //                viele Knoten wie der Wurzelbaum c
                for(d=c;d<L.size();d++)           // d ist ein Wurzelbaum der Hoehe h
                  if(1+L[a].size()+L[b].size()+L[c].size()+L[d].size()<=n)
                  {                               // 1:           zusaetzlicher Knoten
                                                  // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                  // L[b].size(): Anzahl der Knoten von Wurzelbaum b
                                                  // L[c].size(): Anzahl der Knoten von Wurzelbaum c
                                                  // L[d].size(): Anzahl der Knoten von Wurzelbaum d
                    Fuege_an(Alkane,L,a,b,c,d,ausgeben,ausgabe_bei_anzahl_c_atomen);
                                                  // Fuege die Wurzelbaeume a, b, c und d mit einem zusaetzlichen Knoten
                  }                               // zusammen und haenge das entstehende Alkan an die Liste an
                  else                            // Es kann kein Alkan mehr mit hoechstens n C-Atomen konstruiert
                    d=L.size();                   // werden. Erhoehe c um eins und setze d zurueck
              }
              else                                // Es kann kein Alkan mehr mit hoechstens n C-Atomen konstruiert
                c=L.size();                       // werden. Erhoehe b um eins und setze c zurueck
          }
          else                                    // Es kann kein Alkan mehr mit hoechstens n C-Atomen konstruiert
            b=L.size();                           // werden. Erhoehe a um eins und setze b zurueck
      }
      else                                        // Es kann kein Alkan mehr mit hoechstens n C-Atomen konstruiert
        a=p;                                      // werden. Beende die Schleife.
    }

    // Fall (iii)
    for(a=p;a<L.size();a++)                       // a ist ein Wurzelbaum der Hoehe h
    {
      if(ausgeben==false)
        cout << "." << flush;                     // Fortschrittsanzeige
      if(1+L[a].size()+3*h<=n)                    // 1:             zusaetzlicher Knoten
      {                                           // L[a].size():   Anzahl der Knoten von Wurzelbaum a
                                                  // 3*h:           die Wurzelbaeume b, c und d besitzen die Hoehe
                                                  //                h, bestehen also aus mindestens h Knoten
        for(b=a;b<L.size();b++)                   // b ist ein Wurzelbaum der Hoehe h
          if(1+L[a].size()+3*L[b].size()<=n)      // 1:             zusaetzlicher Knoten
          {                                       // L[a].size():   Anzahl der Knoten von Wurzelbaum a
                                                  // 3*L[b].size(): die Wurzelbaeume c und d besitzten mindestens
                                                  //                so viele Knoten wie der Wurzelbaum b
            for(c=b;c<L.size();c++)               // c ist ein Wurzelbaum der Hoehe h
              if(1+L[a].size()+L[b].size()+2*L[c].size()<=n)
              {                                   // 1:             zusaetzlicher Knoten
                                                  // L[a].size():   Anzahl der Knoten von Wurzelbaum a
                                                  // L[b].size():   Anzahl der Knoten von Wurzelbaum b
                                                  // 2*L[c].size(): der Wurzelbaum d besitzt mindestens so
                                                  //                viele Knoten wie der Wurzelbaum c
                for(d=c;d<L.size();d++)           // d ist ein Wurzelbaum der Hoehe h
                  if(1+L[a].size()+L[b].size()+L[c].size()+L[d].size()<=n)
                  {                               // 1:           zusaetzlicher Knoten
                                                  // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                  // L[b].size(): Anzahl der Knoten von Wurzelbaum b
                                                  // L[c].size(): Anzahl der Knoten von Wurzelbaum c
                                                  // L[d].size(): Anzahl der Knoten von Wurzelbaum d
                    Fuege_an(Alkane,L,a,b,c,d,ausgeben,ausgabe_bei_anzahl_c_atomen);
                                                  // Fuege die Wurzelbaeume a, b, c und d mit einem zusaetzlichen Knoten
                  }                               // zusammen und haenge das entstehende Alkan an die Liste an
                  else                            // Es kann kein Alkan mehr mit hoechstens n C-Atomen konstruiert
                    d=L.size();                   // werden. Erhoehe c um eins und setze d zurueck
              }
              else                                // Es kann kein Alkan mehr mit hoechstens n C-Atomen konstruiert
                c=L.size();                       // werden. Erhoehe b um eins und setze c zurueck
          }
          else                                    // Es kann kein Alkan mehr mit hoechstens n C-Atomen konstruiert
            b=L.size();                           // werden. Erhoehe a um eins und setze b zurueck
      }
      else                                        // Es kann kein Alkan mehr mit hoechstens n C-Atomen konstruiert
        a=L.size();                               // werden. Beende die Schleife.
    }
  }
  if(ausgeben==false)
    cout << endl;                                 // Fortschrittsanzeiger
}


// ********************************************************************
// *                       Hauptprogramm                              *
// ********************************************************************

int main(void)
{
  unsigned int n;                           // maximale Anzahl der Knoten bzw C-Atome
  bool ausgeben;                            // sollen die konstruierten Alkane auf dem Bildschirm ausgegeben werden
  int eingabe;                              // Variable zum Einlesen von Eingaben
  int ausgabe_bei_anzahl_c_atomen;          // geben nur ALkane mit dieser Anzahl an C-Atomen auf dem Bildschirm aus

  cout << "Konstruktion von Isomeren der Alkane (Summenformel: C_nH_2n+2)" << endl;
  cout << "==============================================================" << endl;
  cout << endl;
  cout << "Bitte gebe die maximale Anzahl der C-Atome ein: ";
  cin  >> n;
  cout << "Sollen die konstruierten Alkane auf dem Bildschirm ausgegeben werden? (Nein=0, Ja=1) ";
  cin  >> eingabe;
  if(eingabe==1)
    ausgeben=true;
  else
    ausgeben=false;
  if(ausgeben==true)
  {
    cout << "Gebe bitte die Anzahl der C-Atome an, bei denen die konstruierten Alkane ausgegeben werden sollen. ";
    cout << "Moechtest du alle Alkane auf dem Bildschirm ausgegeben haben, so gebe bitte -1 ein. ";
    cin  >> ausgabe_bei_anzahl_c_atomen;
  }

  vector<Alkan> Alkane;                     // Liste von Alkanen
  vector< vector <unsigned char> > L;       // Liste von Wurzelbaeumen

  // Konstruiere eine vollstaendige Liste von Wurzelbaeumen und damit alle ALkane aus bis zu n C-Atomen
  konstruiere_Wurzelbaeume(Alkane,L,n,ausgeben,ausgabe_bei_anzahl_c_atomen);


  // Zur Kontrolle gebe die Anzahl der Alkane aus
  cout << endl;
  vector<long> anz;
  unsigned int i;
  unsigned int sp;
  anz.resize(n+1);
  for(i=0;i<=n;i++)
    anz[i]=0;
  for(i=0;i<Alkane.size();i++)
  {
    sp=Alkane[i].Wurzelbaum_1.size()+Alkane[i].Wurzelbaum_2.size();
    if(Alkane[i].laengste_gerade_Kette_ist_gerade==true)
      anz[sp]++;
    else
      anz[sp+Alkane[i].Wurzelbaum_3.size()+Alkane[i].Wurzelbaum_4.size()+1]++;
  }
  cout << "Anzahl der Alkane mit n C-Atomen" << endl;
  cout << "--------------------------------" << endl;
  for(i=0;i<=n;i++)
  {
    cout << i;
    if(i<10 && n>=10) cout << " ";
    cout << " : " << anz[i] << endl;
  }
  return 0;
};

// ****************************************************************************
// *              Subroutinen zur Ausgabe eines Alkans                        *
// ****************************************************************************

void umnummerieren(vector<unsigned char>& b,vector<unsigned char>& wurzel_baum); // Nummeriert einen Wurzelbaum um
// Platziere ein Alkan aus 2 Wurzelbaeumen in "Ascii-Art"
void ascii_platzierung_alkan_aus_2_wurzelbaeumen(ascii_platzierung& p1,ascii_platzierung& p2,ascii_platzierung& p);
// Platziere ein Alkan aus 4 Wurzelbaeumen in "Ascii-Art"
void ascii_platzierung_alkan_aus_4_wurzelbaeumen(ascii_platzierung& p1,ascii_platzierung& p2,
     ascii_platzierung& p3,ascii_platzierung& p4,ascii_platzierung& p);
// Platziere einen Wurzelbaum in "Ascii-Art"
void bestimme_ascii_platzierung(vector<unsigned char>& b,ascii_platzierung& p);
inline void swap(unsigned int& a,unsigned int& b);                               // Vertauscht zwei Werte
// Fuegt Teilplatzierungen eines Wurzelbaums zu einer groesseren Teilplatzierung zusammen
void bestimme_platzierung(const int& i,vector<unsigned int>& sohn,vector<ascii_platzierung>& platzierungen);


void umnummerieren(vector<unsigned char>& b,vector<unsigned char>& wurzel_baum)
{
  // b:            Wurzelbaum
  // wurzel_baum:  Adresse fuer einen Wurzelbaum
  //
  // Funktion:
  // ---------
  // Der Wurzelbaum b soll umnummeriert und in wurzel_baum gespeichert werden. Die
  // Umnummerierung soll folgende Bedingungn erfuellen:
  // (i)   die Wurzel bekommt die Nummer 0
  // (ii)  bei zwei Knoten liegt derjenige mit der kleineren Nummer entweder in der
  //       selben oder einer niedrigeren Ebene
  // (iii) alle Knoten, die einen Knoten v als Vater haben besitzen die Nummerierungen
  //       n,n+1,...,n+i, folgen also direkt aufeinander

  unsigned int i,j,k;                      // Schleifenvariablen
  unsigned int n;                          // aktuell zu vergebene Nummer
  unsigned int anzahl_ebenen;              // Anzahl der Ebenen
  vector<unsigned char> ebene;             // Vektor fuer die Ebene der Knoten
  vector<unsigned char> perm;              // Vektor zur Speicherung der Umnummerierung
  vector<unsigned char> inv_perm;          // Vektor zur Speicherung der Inverse der Umnummerierung
  wurzel_baum.resize(b.size());            // der umnummerierte Wurzelbaum enthaelt natuerlich genauso viele
                                           // Knoten wie der alte Wurzelbaum
  ebene.resize(b.size());                  // Jeder Knoten besitzt eine Ebene
  perm.resize(b.size());
  inv_perm.resize(b.size());
  for(i=0;i<ebene.size();i++)
    ebene[i]=0;
  for(i=0;i<b.size();i++)                  // Bestimme die Ebene der Knoten
    for(j=0;j<b.size();j++)
      if(b[j]!=j)
        ebene[j]=ebene[b[j]]+1;
   anzahl_ebenen=0;

  for(i=0;i<ebene.size();i++)              // Bestimme die Anzahl der Ebenen
    if(ebene[i]>anzahl_ebenen)
      anzahl_ebenen=ebene[i];
  if(b.size()>0) anzahl_ebenen++;

  for(i=0;i<ebene.size();i++)               // gebe der Wurzel die Nummer 0
    if(ebene[i]==0)
    {
      perm[i]=0;
      break;
    }

  n=1;                                      // aktuell noch nicht veregebene Knotennummer
  for(i=0;i+1<anzahl_ebenen;i++)            // durchlaufe die Ebenen von unten nach oben
    for(j=0;j<ebene.size();j++)             // durchlaufe alle Knoten
    {
      if(ebene[j]==i)                       // Falls der Knoten in der gerade betrachteten Ebene liegt
        for(k=0;k<b.size();k++)             // durchlaufe alle Knoten
          if(b[k]==j && k!=j)               // falls k ein Sohn von j ist
          {
            perm[k]=n;                      // der Knoten k heisst jetzt n
            n++;                            // der Name n wurde eben vergeben, ist also nicht mehr frei
          }
    }
  for(i=0;i<perm.size();i++)               // Bestimme die inverse Permutation
    inv_perm[perm[i]]=i;

  for(i=0;i<wurzel_baum.size();i++)        // nummeriere den Wurzelbaum tatsaechlich um
    wurzel_baum[i]=perm[b[inv_perm[i]]];
}

int ausgabe_ascii(Alkan& A)
{
  // A: ein Alkan
  //
  // Funktion:
  // ---------
  // Gibt eine zweidimensionale Platzierung des Alkanmolekuels A in "Ascii-Art"
  // auf dem Bildschirm aus, sofern es nicht zu breit ist (-> wird -1 zurueckgegeben

  ascii_platzierung p1;                    // Speicherplatz fuer eine Platzierung des ersten Wurzelbaums in "Ascii-Art"
  ascii_platzierung p2;                    // Speicherplatz fuer eine Platzierung des zweiten Wurzelbaums in "Ascii-Art"
  ascii_platzierung p3;                    // Speicherplatz fuer eine Platzierung des dritten Wurzelbaums in "Ascii-Art"
  ascii_platzierung p4;                    // Speicherplatz fuer eine Platzierung des vierten Wurzelbaums in "Ascii-Art"
  ascii_platzierung p;                     // Speicherplatz fuer eine Platzierung des gesamten Alkans in "Ascii-Art"

  vector<unsigned char> wurzel_baum;       // Speicherplatz fuer einen umnummerierten Wurzelbaum
  unsigned int i,j;                        // Schleifenvariablen

  umnummerieren(A.Wurzelbaum_1,wurzel_baum);      // nummeriere den ersten Wurzelbaum um
  bestimme_ascii_platzierung(wurzel_baum,p1);     // platziere den ersten Wurzelbaum in "Ascii-Art"
  umnummerieren(A.Wurzelbaum_2,wurzel_baum);      // nummeriere den zweiten Wurzelbaum um
  bestimme_ascii_platzierung(wurzel_baum,p2);     // platziere den zweiten Wurzelbaum in "Ascii-Art"
  if(A.laengste_gerade_Kette_ist_gerade==false)   // falls das Alkan aus 4 Wurzelbaeumen besteht
  {
    umnummerieren(A.Wurzelbaum_3,wurzel_baum);    // nummeriere den dritten Wurzelbaum um
    bestimme_ascii_platzierung(wurzel_baum,p3);   // platziere den dritten Wurzelbaum in "Ascii-Art"
    umnummerieren(A.Wurzelbaum_4,wurzel_baum);    // nummeriere den vierten Wurzelbaum um
    bestimme_ascii_platzierung(wurzel_baum,p4);   // platziere den vierten Wurzelbaum in "Ascii-Art"
  }

  if(A.laengste_gerade_Kette_ist_gerade==true)    // das Alkan besteht aus 2 Wurzelbaeumen
  {
    ascii_platzierung_alkan_aus_2_wurzelbaeumen(p1,p2,p);  // platziere das gesamte Alkan
    if(p.anzahl_spalten>BILDSCHIRMBREITE)         // Zu breit fuer eine Ausgabe auf dem Bildschirm
    {
      cout << "Das Alkan ist leider zu breit fuer eine Ausgabe auf dem Bildschirm." << endl << endl;
      return -1;
    }
    for(j=0;j<p.anzahl_zeilen;j++)                // durchlaufe die Zeilen der Platzierung
    {
      for(i=0;i<p.anzahl_spalten;i++)             // durchlaufe die Spalten der Platzierung
        cout << p.Matrix[i][j];                   // gebe das berechnete Symbol auf dem Bildschirm aus
      cout << endl;
    }
    cout << endl;
  }
  else                                            // das Alkan besteht aus 4 Wurzelbaeumen
  {
    ascii_platzierung_alkan_aus_4_wurzelbaeumen(p1,p2,p3,p4,p); // platziere das gesamte Alkan
    if(p.anzahl_spalten>BILDSCHIRMBREITE)         // Zu breit fuer eine Ausgabe auf dem Bildschirm
    {
      cout << "Das Alkan ist leider zu breit fuer eine Ausgabe auf dem Bildschirm." << endl << endl;
      return -1;
    }
    for(j=0;j<p.anzahl_zeilen;j++)                // durchlaufe die Zeilen der Platzierung
    {
      for(i=0;i<p.anzahl_spalten;i++)             // durchlaufe die Spalten der Platzierung
        cout << p.Matrix[i][j];                   // gebe das berechnete Symbol auf dem Bildschirm aus
      cout << endl;
    }
    cout << endl;
  }

  //char taste;
  //cout << "Bitte beliebige Taste druecken: ";
  //cin >> taste;

  return 0;
}

void ascii_platzierung_alkan_aus_2_wurzelbaeumen(ascii_platzierung& p1,ascii_platzierung& p2,ascii_platzierung& p)
{
  // p1: Platzierung eines Wurzelbaums in "Ascii-Art"
  // p2: Platzierung eines Wurzelbaums in "Ascii-Art"
  // p:  Speicherplatz fuer Platzierung eines Alkans in "Ascii-Art"
  //
  // Funktion:
  // ---------
  // Kombiniert die zweidimensionalen Platzierungen p1 und p2 zweier Wurzelbaeume zu einer
  // Platzierung des Alkans aus diesen beiden Wurzelbaeumen in "Ascii-Art"

  unsigned int i,j;   // Schleifenvariablen
  unsigned int m;     // Position der neuen Wurzel

  m=max(p1.position_wurzel,p2.position_wurzel);

  p.anzahl_zeilen=p1.anzahl_zeilen+1+p2.anzahl_zeilen; // das Alkan entsteht durch das aneinander setzen
                                                       // dieser 2 Wurzelbaeume
  p.anzahl_spalten=m+1;                                // benoetigter Platz links des Zentrums + Platz fuer
                                                       // das Zentrum
  p.anzahl_spalten+=max(p1.anzahl_spalten-1-p1.position_wurzel,p2.anzahl_spalten-1-p2.position_wurzel); // Platz rechts
                                                       // des Zentrums
  p.position_wurzel=max(p1.position_wurzel,p2.position_wurzel); // Position der neuen "Wurzel"
  p.Matrix.resize(p.anzahl_spalten);                   // Reserviere Speicherplatz
  for(i=0;i<p.anzahl_spalten;i++)                      // durchlaufe die Spalten der Platzierung
  {
    p.Matrix[i].resize(p.anzahl_zeilen);               // Reserviere Speicherplatz
    for(j=0;j<p.anzahl_zeilen;j++)                     // durchlaufe die Zeilen der Platzierung
      p.Matrix[i][j]=' ';                              // beginne zunaechst mit einem leeren Bildschirm
  }
  for(i=p1.anzahl_zeilen;i>0;i--)                      // durchlaufe die Zeilen des ersten Wurzelbaums
    for(j=0;j<p1.anzahl_spalten;j++)                   // durchlaufe die Spalten des ersten Wurzelbaums
      p.Matrix[j+m-p1.position_wurzel][p1.anzahl_zeilen-i]=p1.Matrix[j][i-1]; // uebernehme die Teilplatzierung
  p.Matrix[p.position_wurzel][p1.anzahl_zeilen]='|';   // gebe Symbol fuer das Zentrum aus
  for(i=0;i<p2.anzahl_zeilen;i++)                      // durchlaufe die Zeilen des zweiten Wurzelbaums
    for(j=0;j<p2.anzahl_spalten;j++)                   // durchlaufe die Spalten des zweiten Wurzelbaums
      switch(p2.Matrix[j][i])                          // uebersetze die Symbole der Teilplatzierung
      {
        case (char)92: p.Matrix[j+m-p2.position_wurzel][p1.anzahl_zeilen+1+i]='/';      // Orientierung aendert sich
                       break;
        case '/':      p.Matrix[j+m-p2.position_wurzel][p1.anzahl_zeilen+1+i]=(char)92; // Orientierung aendert sich
                       break;
        default:       p.Matrix[j+m-p2.position_wurzel][p1.anzahl_zeilen+1+i]=p2.Matrix[j][i];
      }
}

void ascii_platzierung_alkan_aus_4_wurzelbaeumen(ascii_platzierung& p1,ascii_platzierung& p2,
     ascii_platzierung& p3,ascii_platzierung& p4,ascii_platzierung& p)
{
  // p1: Platzierung eines Wurzelbaums in "Ascii-Art"
  // p2: Platzierung eines Wurzelbaums in "Ascii-Art"
  // p3: Platzierung eines Wurzelbaums in "Ascii-Art"
  // p4: Platzierung eines Wurzelbaums in "Ascii-Art"
  // p:  Speicherplatz fuer Platzierung eines Alkans in "Ascii-Art"
  //
  // Funktion:
  // ---------
  // Kombiniert die zweidimensionalen Platzierungen p1, p2, p3 und p4 von Wurzelbaeume zu einer
  // Platzierung des Alkans aus diesen Wurzelbaeumen in "Ascii-Art"
  //
  //        p3
  //         |
  //      p1-o-p2
  //         |
  //        p4

  unsigned int i,j;                    // Schleifenvariablen
  unsigned int w1,w2,w3,w4;            // Abkuerzungen fuer die Positionen der 4 Wurzeln
  unsigned int s1,s2,s3,s4;            // Abkuerzungen fuer die 4 Anzahlen der Zeilen
  unsigned int z1,z2,z3,z4;            // Abkuerzungen fuer die 4 Anzahlen der Zeilen
  w1=p1.position_wurzel;               // Position der Wurzel der Platzierung des ersten Wurzelbaums
  w2=p2.position_wurzel;               // Position der Wurzel der Platzierung des zweiten Wurzelbaums
  w3=p3.position_wurzel;               // Position der Wurzel der Platzierung des dritten Wurzelbaums
  w4=p4.position_wurzel;               // Position der Wurzel der Platzierung des vierten Wurzelbaums
  s1=p1.anzahl_spalten;                // Anzahl der Spalten der Platzierung des ersten Wurzelbaums
  s2=p2.anzahl_spalten;                // Anzahl der Spalten der Platzierung des zweiten Wurzelbaums
  s3=p3.anzahl_spalten;                // Anzahl der Spalten der Platzierung des dritten Wurzelbaums
  s4=p4.anzahl_spalten;                // Anzahl der Spalten der Platzierung des vierten Wurzelbaums
  z1=p1.anzahl_zeilen;                 // Anzahl der Zeilen der Platzierung des ersten Wurzelbaums
  z2=p2.anzahl_zeilen;                 // Anzahl der Zeilen der Platzierung des zweiten Wurzelbaums
  z3=p3.anzahl_zeilen;                 // Anzahl der Zeilen der Platzierung des dritten Wurzelbaums
  z4=p4.anzahl_zeilen;                 // Anzahl der Zeilen der Platzierung des vierten Wurzelbaums

  unsigned int links,rechts,oben,unten;         // Platzbedarf links,rechts,oberhalb und unterhalb des zusaetzlichen Knotens

  oben=z3+1;                                    // Die Platzierung des dritten Wurzelbaums muss ueber
                                                // den zusaetzlichen Knoten passen, inklusive "|"
  if(s1-w1-1>oben && s1+w1>0) oben=s1-w1-1;     // ein Teil der Platzierung des ersten Wurzelbaums reicht nach oben
  if(s2-w2-1>oben && s2+w2>0) oben=s2-w2-1;     // ein Teil der Platzierung des zweiten Wurzelbaums reicht nach oben
  unten=z4+1;                                   // Die Platzierung des dritten Wurzelbaums muss ueber
                                                // den zusaetzlichen Knoten passen, inklusive "|"
  if(w1>unten) unten=w1;                        // ein Teil der Platzierung des ersten Wurzelbaums reicht nach unten
  if(w2>unten) unten=w2;                        // ein Teil der Platzierung des zweiten Wurzelbaums reicht nach unten
  links=0;
  if(z1>0) links=z1+1;                          // falls der ersten Wurzelbaum aus mindestens einem Knoten besteht,
                                                // benoetigt er Platz + "-"
  if(w3>links) links=w3;                        // ein Teil der Platzierung des dritten Wurzelbaums reicht nach links
  if(w4>links) links=w4;                        // ein Teil der Platzierung des vierten Wurzelbaums reicht nach links
  rechts=0;
  if(z2>0) rechts=z2+1;                         // falls der zweite Wurzelbaum aus mindestens einem Knoten besteht,
                                                // benoetigt er Platz + "-"
  if(s3-w3-1>rechts) rechts=s3-w3-1;            // ein Teil der Platzierung des dritten Wurzelbaums reicht nach rechts
  if(s4-w4-1>rechts) rechts=s4-w4-1;            // ein Teil der Platzierung des vierten Wurzelbaums reicht nach rechts

  p.anzahl_zeilen=unten+1+oben;                 // Teil oberhalb des zusaetzlichen Knoten + zusaetzlicher Knoten + Teil
                                                // unterhalb des zusaetzlichen Knotens
  p.anzahl_spalten=links+1+rechts;              // Teil links vom zusaetzlichen Knoten + zusaetzlicher Knoten + Teil
                                                // rechts vom zusaetzlichen Knoten
  p.position_wurzel=links;

  p.Matrix.resize(p.anzahl_spalten);            // Reserviere Speicherplatz
  for(i=0;i<p.anzahl_spalten;i++)               // durchlaufe die Spalten der Platzierung
  {
    p.Matrix[i].resize(p.anzahl_zeilen);        // Reserviere Speicherplatz
    for(j=0;j<p.anzahl_zeilen;j++)              // durchlaufe die Zeilen der Platzierung
      p.Matrix[i][j]=' ';                       // starte mit einem leeren Bildschirm
  }
  for(i=z3;i>0;i--)                             // durchlaufe die Zeilen der Platzierung des dritten Wurzelbaums
    for(j=0;j<s3;j++)                           // durchlaufe die Spalten der Platzierung des dritten Wurzelbaums
      p.Matrix[links-w3+j][(oben-z3-1)+(z3-i)]=p3.Matrix[j][i-1];
  p.Matrix[p.position_wurzel][oben-1]='|';
  p.Matrix[p.position_wurzel][oben]='o';
  p.Matrix[p.position_wurzel][oben+1]='|';
  for(i=0;i<z4;i++)                             // durchlaufe die Zeilen der Platzierung des vierten Wurzelbaums
    for(j=0;j<s4;j++)                           // durchlaufe die Spalten der Platzierung des vierten Wurzelbaums
      switch(p4.Matrix[j][i])                   // uebersetze die Symbole der Teilplatzierung
      {
        case '|':      p.Matrix[links-w4+j][oben+2+i]='|';       // immer noch hochkant
                       break;
        case '/':      p.Matrix[links-w4+j][oben+2+i]=(char)92;  // Orientierung aendert sich
                       break;
        case (char)92: p.Matrix[links-w4+j][oben+2+i]='/';       // Orientierung aendert sich
                       break;
        case 'o':      p.Matrix[links-w4+j][oben+2+i]='o';       // ein Knoten bleibt ein Knoten
                       break;
      }
  if(p1.anzahl_zeilen>0)
  {
    p.Matrix[p.position_wurzel-1][oben]='-';
    for(i=0;i<z1;i++)                           // durchlaufe die Zeilen der Platzierung des ersten Wurzelbaums
      for(j=0;j<s1;j++)                         // durchlaufe die Spalten der Platzierung des ersten Wurzelbaums
        switch(p1.Matrix[j][i])                 // uebersetze die Symbole der Teilplatzierung
        {
          case '|':      p.Matrix[(links-z1-1)+(z1-i-1)][oben+w1+(-j)]='-';       // Orientierung aendert sich
                         break;
          case 'o':      p.Matrix[(links-z1-1)+(z1-i-1)][oben+w1+(-j)]='o';       // ein Knoten bleibt ein Knoten
                         break;
          case '/':      p.Matrix[(links-z1-1)+(z1-i-1)][oben+w1+(-j)]=(char)92;  // Orientierung aendert sich
                         break;
          case (char)92: p.Matrix[(links-z1-1)+(z1-i-1)][oben+w1+(-j)]='/';       // Orientierung aendert sich
                         break;
        }
  }
  if(p2.anzahl_zeilen>0)
  {
    p.Matrix[p.position_wurzel+1][oben]='-';
    for(i=0;i<z2;i++)                           // durchlaufe die Zeilen der Platzierung des zweiten Wurzelbaums
      for(j=0;j<s2;j++)                         // durchlaufe die Spalten der Platzierung des zweiten Wurzelbaums
        switch(p2.Matrix[j][i])                 // uebersetze die Symbole der Teilplatzierung
        {
          case '|':      p.Matrix[p.position_wurzel+2+i][oben+w2+(-j)]='-';         // Orientierung aendert sich
                         break;
          case 'o':      p.Matrix[p.position_wurzel+2+i][oben+w2+(-j)]='o';         // ein Knoten bleibt ein Knoten
                         break;
          case '/':      p.Matrix[p.position_wurzel+2+i][oben+w2+(-j)]='/';
                         break;
          case (char)92: p.Matrix[p.position_wurzel+2+i][oben+w2+(-j)]=(char)92;
                         break;
        }
  }

}

void bestimme_ascii_platzierung(vector<unsigned char>& b,ascii_platzierung& p)
{
  // b: ein Wurzelbaum, der bereits mit der Funktion "umnummerieren" bearbeitet wurde
  // p: Speicherplatz fuer eine Platzierung eines Wurzelbaums in "Ascii-Art"
  //
  // Funktion:
  // ---------
  // Platziert einen Wurzelbaum in "Ascii-Art" und speichert das Ergebnis in p.

  unsigned int i,j;                        // Schleifenvariablen
  unsigned int pos;                        // Position des zu bearbeitenden Knotens
  vector<unsigned char> ebene;             // Vektor fuer die Ebene der Knoten
  ebene.resize(b.size());                  // Jeder Knoten besitzt eine Ebene
  vector<ascii_platzierung> platzierungen; // eine Liste von einzelnen Platzierungen, man kann jeden
  platzierungen.resize(b.size());          // Knoten einzeln platzieren und die einzelnen Platzierungen
                                           // dann Stueck fuer Stueck zu groesseren Platzierungen zusammenfassen
  vector<unsigned int> sohn;               // Liste von Soehnen eines Knotens

  for(i=0;i<ebene.size();i++)
    ebene[i]=0;
  for(i=0;i<b.size();i++)                  // Bestimme die Ebene der Knoten
    for(j=0;j<b.size();j++)
      if(b[j]!=j)
        ebene[j]=ebene[b[j]]+1;

  pos=0;
  for(i=b.size();i>0;i--)                    // durchlaufe die Knoten der obersten Ebene
    if(ebene[i-1]==ebene[b.size()-1])        // falls der Knoten noch zur obersten Ebene gehoert
    {
      platzierungen[i-1].anzahl_zeilen=1;    // belege Matrix der Platzierung mit dem Symbol
      platzierungen[i-1].anzahl_spalten=1;   // fuer einen Knoten: "o"
      platzierungen[i-1].position_wurzel=0;
      platzierungen[i-1].Matrix.resize(1);
      platzierungen[i-1].Matrix[0].resize(1);
      platzierungen[i-1].Matrix[0][0]='o';
    }
    else
    {
      pos=i-1;                             // merke den Start der neuen Ebene
      break;                               // oberste Ebene ist durchlaufen
    }

  if(b.size()>0) pos++;
  if(b.size()==1) pos=0;
  while(pos>0)                             // solange es noch Knoten zu bearbeiten gibt
  {
    for(i=pos;i>0;i--)                     // durchlaufe die Knoten der aktuell zu bearbeitenden Ebene
    {
      if(ebene[i-1]==ebene[pos-1])         // falls der Knoten noch zu dieser Ebene gehoert
      {
        sohn.resize(0);                    // bestimme die Liste der Soehne des Knoten i
        for(j=pos;j<b.size();j++)
        {
          if(ebene[j]>ebene[pos])          // es koennen keine Soehne mehr kommen, da die Knoten der Ebene
          {                                // nach ansteigend sortiert sind
            break;
          }
          else
          {
           if((unsigned int)b[j]==i-1)
            {
              sohn.resize(sohn.size()+1);  // es gibt einen Sohn mehr
              sohn[sohn.size()-1]=j;       // speichere die Nummer des Sohns
            }
          }
        }
        bestimme_platzierung(i-1,sohn,platzierungen);  // Fuege die Platzierungen der Soehne zu einer Platzierung vom
                                                       // Knoten i-1 zusammen
        if(i==1) pos=0;                                // Abbruchbedingung
      }
      else
      {
        pos=i;                             // merke den Start der neuen Ebene
        break;                             // aktuelle Ebene ist bereits fertig durchlaufen
      }
    }
  }

  if(b.size()>0)                           // kopiere die Platzierung der Wurzel
  {
    p.anzahl_zeilen=platzierungen[0].anzahl_zeilen;
    p.anzahl_spalten=platzierungen[0].anzahl_spalten;
    p.position_wurzel=platzierungen[0].position_wurzel;
    p.Matrix.resize(p.anzahl_spalten);
    for(i=0;i<p.anzahl_spalten;i++)
    {
      p.Matrix[i].resize(p.anzahl_zeilen);
      for(j=0;j<p.anzahl_zeilen;j++)
        p.Matrix[i][j]=platzierungen[0].Matrix[i][j];
    }
  }
  else                      // leere Platzierung
  {
    p.anzahl_zeilen=0;
    p.anzahl_spalten=0;
    p.position_wurzel=0;
    p.Matrix.resize(0);
  }

}

inline void swap(unsigned int& a,unsigned int& b)
{
  // Vertauscht zwei Werte
  unsigned int c=a;
  a=b;
  b=c;
}

void bestimme_platzierung(const int& i,vector<unsigned int>& sohn,vector<ascii_platzierung>& platzierungen)
{
  // i:             Nummer eines Knotens
  // sohn:          Liste der Nummern der Soehne von i
  // platzierungen: Liste von Platzierungen der Wurzelbaeume oberhalb eines Knotens
  //
  // Funktion:
  // ---------
  // Es wird die Platzierung des Wurzelbaums oberhalb eines Knotens i bestimmt. Hierzu werden die
  // bereits vorhanden derartigen Platzierungen der Soehne von i benutzt.

  unsigned int h,j;                  // Schleifenvariablen
  unsigned int links,mitte,rechts;   // Welcher der 3 Teilwurzelbaeume kommt in die Mitte, nach links oder nach rechts?
  unsigned int s1,s2,s3;             // Abkuerzung fuer die Anzahl der Spalten der Platzierung der 3 Teilwurzelbaeume
  unsigned int w1,w2,w3;             // Abkuerzung fuer die Position der Wurzel der Platzierung der 3 Teilwurzelbaeume
  unsigned int z1,z2,z3;             // Abkuerzung fuer die Anzahl der Zeilen der Platzierung der 3 Teilwurzelbaeume
  unsigned int a;

  switch(sohn.size())                // Wieviele Teilwurzelbaeume muessen haengen amKnoten i
  {
    // Es haengt kein Teilwurzelbaum am Knoten i, also platziere nur den Knoten i
    case 0: platzierungen[i].anzahl_zeilen=1;    // belege Matrix der Platzierung mit dem Symbol
            platzierungen[i].anzahl_spalten=1;   // fuer einen Knoten: "o"
            platzierungen[i].position_wurzel=0;
            platzierungen[i].Matrix.resize(1);
            platzierungen[i].Matrix[0].resize(1);
            platzierungen[i].Matrix[0][0]='o';
            break;
    // Es haengt ein Teilwurzelbaum am Knoten i, also haenge die Platzierung an "o-" an
    case 1: platzierungen[i].anzahl_zeilen=platzierungen[sohn[0]].anzahl_zeilen+2;   // 2 hoeher, wegen "o-"
            platzierungen[i].anzahl_spalten=platzierungen[sohn[0]].anzahl_spalten;   // genauso breit
            platzierungen[i].position_wurzel=platzierungen[sohn[0]].position_wurzel; // selbe Position der Wurzel
            platzierungen[i].Matrix.resize(platzierungen[i].anzahl_spalten);         // Reserviere Speicherplatz
            for(j=0;j<platzierungen[i].anzahl_spalten;j++)
              platzierungen[i].Matrix[j].resize(platzierungen[i].anzahl_zeilen);     // Reserviere Speicherplatz
            for(j=0;j<platzierungen[sohn[0]].anzahl_spalten;j++)                     // Übernehme die Platzierung
              for(h=0;h<platzierungen[sohn[0]].anzahl_zeilen;h++)                    // vomTeilwurzelbaum
              {
                platzierungen[i].Matrix[j][h+2]=platzierungen[sohn[0]].Matrix[j][h];
              }
            for(j=0;j<platzierungen[sohn[0]].anzahl_spalten;j++)                     // Zwei Leerzeilen
            {
              platzierungen[i].Matrix[j][0]=' ';
              platzierungen[i].Matrix[j][1]=' ';
            }
            platzierungen[i].Matrix[platzierungen[sohn[0]].position_wurzel][0]='o';  // Knoten i
            platzierungen[i].Matrix[platzierungen[sohn[0]].position_wurzel][1]='|';
            // Verbindung zur Wurzel vom Teilwurzelbaum
            break;
    // Es haengen zwei Teilwurzelbaeume am Knoten i
    case 2: s1=platzierungen[sohn[0]].anzahl_spalten;        // Anzahl der Spalten der Platzierung des ersten Wurzelbaums
            s2=platzierungen[sohn[1]].anzahl_spalten;        // Anzahl der Spalten der Platzierung des zweiten Wurzelbaums
            z1=platzierungen[sohn[0]].anzahl_zeilen;         // Anzahl der Zeilen der Platzierung des ersten Wurzelbaums
            z2=platzierungen[sohn[1]].anzahl_zeilen;         // Anzahl der Zeilen der Platzierung des zweiten Wurzelbaums
            w1=platzierungen[sohn[0]].position_wurzel;       // Position der Wurzel der Platzierung des ersten Wurzelbaums
            w2=platzierungen[sohn[1]].position_wurzel;       // Position der Wurzel der Platzierung des zweiten Wurzelbaums

            // Bestimme die platzsparendere Variante
            if(max(s1-w1-1,w2)<=max(s2-w2-1,w1))
            {
              links=0;
              rechts=1;
              a=max(s1-w1-1,w2);
            }
            else
            {
              links=1;
              rechts=0;
              swap(s1,s2);
              swap(z1,z2);
              swap(w1,w2);
              a=max(s2-w2-1,w1);
            }
            if(a<1) a=1;
            platzierungen[i].anzahl_zeilen=max(z1,z2)+a+1;
            platzierungen[i].anzahl_spalten=w1+1+a+1+a+(s2-w2);
            platzierungen[i].position_wurzel=w1+1+a;
            platzierungen[i].Matrix.resize(platzierungen[i].anzahl_spalten);
            for(j=0;j<platzierungen[i].anzahl_spalten;j++)
              platzierungen[i].Matrix[j].resize(platzierungen[i].anzahl_zeilen);
            for(j=0;j<platzierungen[i].anzahl_spalten;j++)
              for(h=0;h<platzierungen[i].anzahl_zeilen;h++)
                platzierungen[i].Matrix[j][h]=' ';
            for(j=0;j<s1;j++)
              for(h=0;h<z1;h++)
                platzierungen[i].Matrix [j][h+a+1]=platzierungen[sohn[links]].Matrix[j][h];
            for(j=0;j<s2;j++)
              for(h=0;h<z2;h++)
                platzierungen[i].Matrix [j+w1+1+a+1+a-w2][h+a+1]=platzierungen[sohn[rechts]].Matrix[j][h];
            platzierungen[i].Matrix [platzierungen[i].position_wurzel][0]='o';
            for(j=1;j<=a;j++)
            {
              platzierungen[i].Matrix [platzierungen[i].position_wurzel-j][j]=(char)(92);
              platzierungen[i].Matrix [platzierungen[i].position_wurzel+j][j]='/';
            }
            break;
    // Es haengen drei Teilwurzelbaeume am Knoten i
    case 3: s1=platzierungen[sohn[0]].anzahl_spalten;  // Anzahl der Spalten der Platzierung des ersten Wurzelbaums
            s2=platzierungen[sohn[1]].anzahl_spalten;  // Anzahl der Spalten der Platzierung des zweiten Wurzelbaums
            s3=platzierungen[sohn[2]].anzahl_spalten;  // Anzahl der Spalten der Platzierung des dritten Wurzelbaums
            z1=platzierungen[sohn[0]].anzahl_zeilen;   // Anzahl der Zeilen der Platzierung des ersten Wurzelbaums
            z2=platzierungen[sohn[1]].anzahl_zeilen;   // Anzahl der Zeilen der Platzierung des zweiten Wurzelbaums
            z3=platzierungen[sohn[2]].anzahl_zeilen;   // Anzahl der Zeilen der Platzierung des dritten Wurzelbaums
            w1=platzierungen[sohn[0]].position_wurzel; // Position der Wurzel der Platzierung des ersten Wurzelbaums
            w2=platzierungen[sohn[1]].position_wurzel; // Position der Wurzel der Platzierung des zweiten Wurzelbaums
            w3=platzierungen[sohn[2]].position_wurzel; // Position der Wurzel der Platzierung des dritten Wurzelbaums

            // bestimme die platzsparendere Variante der 6 verschiedenen Anordnungen der 3 Wurzelbaeume
            links=0;
            mitte=1;
            rechts=2;
            a=max((s1-w1-1)+1+w2,(s2-w2-1)+1+w3);
            if(a>max((s1-w1-1)+1+w3,(s3-w3-1)+1+w2))
            {
              links=0;
              mitte=2;
              rechts=1;
              a=max((s1-w1-1)+1+w3,(s3-w3-1)+1+w2);
            }
            if(a>max((s2-w2-1)+1+w1,(s1-w1-1)+1+w3))
            {
              links=1;
              mitte=0;
              rechts=2;
              a=max((s2-w2-1)+1+w1,(s1-w1-1)+1+w3);
            }
            if(a>max((s2-w2-1)+1+w3,(s3-w3-1)+1+w1))
            {
              links=1;
              mitte=2;
              rechts=0;
              a=max((s2-w2-1)+1+w3,(s3-w3-1)+1+w1);
            }
            if(a>max((s3-w3-1)+1+w1,(s1-w1-1)+1+w2))
            {
              links=2;
              mitte=0;
              rechts=1;
              a=max((s3-w3-1)+1+w1,(s1-w1-1)+1+w2);
            }
            if(a>max((s3-w3-1)+1+w2,(s2-w2-1)+1+w1))
            {
              links=2;
              mitte=1;
              rechts=0;
              a=max((s3-w3-1)+1+w2,(s2-w2-1)+1+w1);
            }

            s1=platzierungen[sohn[links]].anzahl_spalten;
            s2=platzierungen[sohn[mitte]].anzahl_spalten;
            s3=platzierungen[sohn[rechts]].anzahl_spalten;
            z1=platzierungen[sohn[links]].anzahl_zeilen;
            z2=platzierungen[sohn[mitte]].anzahl_zeilen;
            z3=platzierungen[sohn[rechts]].anzahl_zeilen;
            w1=platzierungen[sohn[links]].position_wurzel;
            w2=platzierungen[sohn[mitte]].position_wurzel;
            w3=platzierungen[sohn[rechts]].position_wurzel;

            platzierungen[i].anzahl_zeilen=max(max(z1,z2),z3)+a+1;
            platzierungen[i].anzahl_spalten=w1+1+a+1+a+1+(s3-w3-1);
            platzierungen[i].position_wurzel=w1+1+a;

            platzierungen[i].Matrix.resize(platzierungen[i].anzahl_spalten);
            for(j=0;j<platzierungen[i].anzahl_spalten;j++)
              platzierungen[i].Matrix[j].resize(platzierungen[i].anzahl_zeilen);
            for(j=0;j<platzierungen[i].anzahl_spalten;j++)
              for(h=0;h<platzierungen[i].anzahl_zeilen;h++)
                platzierungen[i].Matrix[j][h]=' ';
            for(j=0;j<s1;j++)
              for(h=0;h<z1;h++)
                platzierungen[i].Matrix [j][h+a+1]=platzierungen[sohn[links]].Matrix[j][h];
            for(j=0;j<s2;j++)
              for(h=0;h<z2;h++)
                platzierungen[i].Matrix [j+w1+1+a-w2][h+a+1]=platzierungen[sohn[mitte]].Matrix[j][h];
            for(j=0;j<s3;j++)
              for(h=0;h<z3;h++)
                platzierungen[i].Matrix [j+w1+1+a+1+a-w3][h+a+1]=platzierungen[sohn[rechts]].Matrix[j][h];
            platzierungen[i].Matrix [platzierungen[i].position_wurzel][0]='o';
            for(j=1;j<=a;j++)
            {
              platzierungen[i].Matrix [platzierungen[i].position_wurzel-j][j]=(char)(92);
              platzierungen[i].Matrix [platzierungen[i].position_wurzel][j]='|';
              platzierungen[i].Matrix [platzierungen[i].position_wurzel+j][j]='/';
            }
            break;
  }
}

int ausgabe(Alkan& A)
{
  // Gibt ein Alkan im XML-Format in der Datei graph aus
  unsigned int anzahl_c_atome;
  if(A.laengste_gerade_Kette_ist_gerade==true)
    anzahl_c_atome=A.Wurzelbaum_1.size()+A.Wurzelbaum_2.size();
  else
    anzahl_c_atome=1+A.Wurzelbaum_1.size()+A.Wurzelbaum_2.size()+A.Wurzelbaum_3.size()+A.Wurzelbaum_4.size();

  vector<unsigned int> Ebene;     // in welcher Ebene liegt der Knoten im Wurzelbaum
  vector<unsigned int> Position;  // Teile die Knoten eines Wurzelbaums in einer Ebene in 3^Ebene Positionen auf
  vector<unsigned int> Nachfolgervon;
  Ebene.resize(anzahl_c_atome);
  Position.resize(anzahl_c_atome);
  Nachfolgervon.resize(anzahl_c_atome);
  unsigned int w1,w2,w3,w4;
  for(unsigned int i = 0; i<anzahl_c_atome; i++)
  {
    Ebene[i]=0;
    Position[i]=0;
    Nachfolgervon[i]=0;  //Anzahl der Nachfolger von Knoten i
  }  
  unsigned int knoten;
  unsigned int eb;
  unsigned int pos;
  ofstream datei("graph", ios_base::app);
  w1=A.Wurzelbaum_1.size(); 
  w2=A.Wurzelbaum_2.size();
  datei << "<GRAPH NAME=\"C" << anzahl_c_atome << "H" << 2*(anzahl_c_atome)+2 << "\" NUMBER_OF_VERTICES=\"";
  datei << 3*(anzahl_c_atome)+2 << "\" NUMBER_OF_EDGES=\"" << 3*(anzahl_c_atome)+1 << "\" MOLECULAR=\"1\">" << endl;
  datei << "<VERTEXLABELS>" << endl;
    for(unsigned i=0;i<anzahl_c_atome;i++)
      datei << "\"C\"" << endl;
    for(unsigned i=0;i<2*(anzahl_c_atome)+2;i++)
      datei << "\"H\"" << endl;
  datei << "</VERTEXLABELS>" << endl;   
  if(A.laengste_gerade_Kette_ist_gerade==true)
  {
    // in der Mitte ist eine Kante
    for(unsigned int i = 1; i<w1; i++)
    {
      Ebene[i]=Ebene[A.Wurzelbaum_1[i]]+1;
      Position[i]=3*Position[A.Wurzelbaum_1[i]]+Nachfolgervon[A.Wurzelbaum_1[i]];
      Nachfolgervon[A.Wurzelbaum_1[i]]++;
    }
    for(unsigned int i = 1; i<w2; i++)
    {
      Ebene[w1+i]=Ebene[w1+A.Wurzelbaum_2[i]]+1;
      Position[w1+i]=3*Position[w1+A.Wurzelbaum_2[i]]+Nachfolgervon[w1+A.Wurzelbaum_2[i]];
      Nachfolgervon[w1+A.Wurzelbaum_2[i]]++;
    }
    datei << "<EDGELIST>" << endl;
      if(w1+w2>0) datei << "0 " << (int) w1 << endl;
      for(unsigned int i=1; i<w1; i++)
        datei << (int)i << " " << (int)A.Wurzelbaum_1[i] << endl;
      for(unsigned int i=1; i<w2; i++)
        datei << (int)(w1+i) << " " << (int)(w1+A.Wurzelbaum_2[i]) << endl;
      knoten=w1+w2;
      for(unsigned int i=0;i<w1+w2;i++) 
        for(unsigned j=0;j<3-Nachfolgervon[i];j++)
        {
          datei << (int)knoten << " " << (int)i << endl;
          knoten++;
        }
    datei << "</EDGELIST>" << endl;
    datei << "<COORDS2D_FLOAT>" << endl;
      for(unsigned int i=0;i<w1;i++)
      {
        if(Ebene[i]==0)
          datei << "0";
        else
        {
          datei << -((long double)Ebene[i])+Position[i]*2.*Ebene[i]/(pow((long double)3,(int)Ebene[i])-1);
        }
        datei << " " << Ebene[i]+0.5  << endl;
      }
      for(unsigned int i=0;i<w2;i++)
      {
        if(Ebene[w1+i]==0)
          datei << "0";
        else
          datei << -((long double)Ebene[w1+i])+Position[w1+i]*2.*Ebene[w1+i]/(pow((long double)3,(int)Ebene[w1+i])-1);
        datei << " " << -((int)Ebene[w1+i])-0.5  << endl;		
      }
      for(unsigned int i=0;i<w1+w2;i++)
        for(unsigned j=0;j<3-Nachfolgervon[i];j++) 
        {
          pos=3*Position[i]+2-j;         
          eb=Ebene[i]+1;
          datei << -((long double)eb)+pos*2.*eb/(pow((long double)3,(int)eb)-1) << " ";
          if(i<w1)
            datei << ((int)eb)+0.5;
          else
            datei << -((int)eb)-0.5;
          datei << endl; 		
        }    
    datei << "</COORDS2D_FLOAT>" << endl;
    datei << "</GRAPH>" << endl;
  }
  else
  {
    // in der Mitte ist ein zusaetzlicher Knoten
    w3=A.Wurzelbaum_3.size(); 
    w4=A.Wurzelbaum_4.size();
    Nachfolgervon[0]=0;
    if(w1>0) Nachfolgervon[0]++;  
    if(w2>0) Nachfolgervon[0]++;
    if(w3>0) Nachfolgervon[0]++;
    if(w4>0) Nachfolgervon[0]++;
    for(unsigned int i = 1; i<w1; i++)
    {
      Ebene[1+i]=Ebene[1+A.Wurzelbaum_1[i]]+1;
      Position[1+i]=3*Position[1+A.Wurzelbaum_1[i]]+Nachfolgervon[1+A.Wurzelbaum_1[i]];
      Nachfolgervon[1+A.Wurzelbaum_1[i]]++;
    }
    for(unsigned int i = 1; i<w2; i++)
    {
      Ebene[1+w1+i]=Ebene[1+w1+A.Wurzelbaum_2[i]]+1;
      Position[1+w1+i]=3*Position[1+w1+A.Wurzelbaum_2[i]]+Nachfolgervon[1+w1+A.Wurzelbaum_2[i]];
      Nachfolgervon[1+w1+A.Wurzelbaum_2[i]]++;
    } 
    for(unsigned int i = 1; i<w3; i++)
    {
      Ebene[1+w1+w2+i]=Ebene[1+w1+w2+A.Wurzelbaum_3[i]]+1;
      Position[1+w1+w2+i]=3*Position[1+w1+w2+A.Wurzelbaum_3[i]]+Nachfolgervon[1+w1+w2+A.Wurzelbaum_3[i]];
      Nachfolgervon[1+w1+w2+A.Wurzelbaum_3[i]]++;
    } 
    for(unsigned int i = 1; i<w4; i++)
    {
      Ebene[1+w1+w2+w3+i]=Ebene[1+w1+w2+w3+A.Wurzelbaum_4[i]]+1;
      Position[1+w1+w2+w3+i]=3*Position[1+w1+w2+w3+A.Wurzelbaum_4[i]]+Nachfolgervon[1+w1+w2+w3+A.Wurzelbaum_4[i]];
      Nachfolgervon[1+w1+w2+w3+A.Wurzelbaum_4[i]]++;
    }
    datei << "<EDGELIST>" << endl;
      if(w1>0) datei << "0 " << 1 << endl;
      if(w2>0) datei << "0 " << 1+w1 << endl;
      if(w3>0) datei << "0 " << 1+w1+w2 << endl;
      if(w4>0) datei << "0 " << 1+w1+w2+w3 << endl;  
      for(unsigned int i=1; i<w1; i++)
        datei << 1+i << " " << 1+A.Wurzelbaum_1[i] << endl;
      for(unsigned int i=1; i<w2; i++)
        datei << 1+w1+i << " " << 1+w1+A.Wurzelbaum_2[i] << endl;
      for(unsigned int i=1; i<w3; i++)
        datei << 1+w1+w2+i << " " << 1+w1+w2+A.Wurzelbaum_3[i] << endl;
      for(unsigned int i=1; i<w4; i++)
        datei << 1+w1+w2+w3+i << " " << 1+w1+w2+w3+A.Wurzelbaum_4[i] << endl;
      knoten=1+w1+w2+w3+w4;
      for(unsigned j=0;j<4-Nachfolgervon[0];j++)
      {
        datei << "0 " << (int)knoten << endl;
        knoten++;
      }
      for(unsigned int i=0;i<w1+w2+w3+w4;i++) 
        for(unsigned j=0;j<3-Nachfolgervon[1+i];j++)
        {
          datei << knoten << " " << 1+i << endl;
          knoten++;
        }
    datei << "</EDGELIST>" << endl;
    
    datei << "<COORDS2D_FLOAT>" << endl;
      datei << "(0,0)" << endl;
      for(unsigned int i=0;i<w1;i++)
      {
        if(Ebene[1+i]==0)
          datei << "0";
        else
        {
          datei << -((long double)Ebene[1+i])+Position[1+i]*2.*Ebene[1+i]/(pow((long double)3,(int)Ebene[1+i])-1);
        }
        datei << " " << Ebene[1+i]+1 << endl;
      }
      for(unsigned int i=0;i<w2;i++)
      {
        if(Ebene[1+w1+i]==0)
          datei << "0";
        else
          datei << -((long double)Ebene[1+w1+i])+Position[1+w1+i]*2.*Ebene[1+w1+i]/(pow((long double)3,(int)Ebene[1+w1+i])-1);
        datei << " " << -((int)Ebene[1+w1+i])-1  << endl;		
      }
      for(unsigned int i=0;i<w3;i++)
      {
        datei << Ebene[1+w1+w2+i]+1 << " ";
        if(Ebene[1+w1+w2+i]==0)
          datei << "0";
        else
          datei << -((long double)Ebene[1+w1+w2+i])+Position[1+w1+w2+i]*2.*Ebene[1+w1+w2+i]/(pow((long double)3,(int)Ebene[1+w1+w2+i])-1);
        datei << endl;   
      }
      for(unsigned int i=0;i<w4;i++)
      {
        datei << -((int)Ebene[1+w1+w2+w3+i])-1 << " ";
        if(Ebene[1+w1+w2+w3+i]==0)
          datei << "0";  
        else
          datei << -((long double)Ebene[1+w1+w2+w3+i])+Position[1+w1+w2+w3+i]*2.*Ebene[1+w1+w2+w3+i]/(pow((long double)3,(int)Ebene[1+w1+w2+w3+i])-1); 
        datei << endl;
      } 
      if(w1==0) datei << "0 1" << endl;
      if(w2==0) datei << "0 -1" << endl; 
      if(w3==0) datei << "1 0" << endl;
      if(w4==0) datei << "-1 0" << endl;
      for(unsigned i=0;i<w1;i++)
        for(unsigned j=0;j<3-Nachfolgervon[1+i];j++)
        {
          pos=3*Position[1+i]+2-j;         
          eb=Ebene[1+i]+1;   
          datei << -((long double)eb)+pos*2.*eb/(pow((long double)3,(int)eb)-1);
          datei << " " << eb+1 << endl;
        }
      for(unsigned i=0;i<w2;i++)
        for(unsigned j=0;j<3-Nachfolgervon[1+w1+i];j++)
        {
          pos=3*Position[1+w1+i]+2-j;         
          eb=Ebene[1+w1+i]+1;   
          datei << -((long double)eb)+pos*2.*eb/(pow((long double)3,(int)eb)-1);
          datei << " " << -((int)eb)-1 << endl; 
        }
      for(unsigned i=0;i<w3;i++)
        for(unsigned j=0;j<3-Nachfolgervon[1+w1+w2+i];j++)
        {
          pos=3*Position[1+w1+w2+i]+2-j;         
          eb=Ebene[1+w1+w2+i]+1;   
          datei << eb+1 << " ";
          datei << -((long double)eb)+pos*2.*eb/(pow((long double)3,(int)eb)-1) << endl;
        }
      for(unsigned i=0;i<w4;i++)
        for(unsigned j=0;j<3-Nachfolgervon[1+w1+w2+w3+i];j++)
        {
          pos=3*Position[1+w1+w2+w3+i]+2-j;         
          eb=Ebene[1+w1+w2+w3+i]+1;   
          datei << -((int)eb)-1 << " ";
          datei << -((long double)eb)+pos*2.*eb/(pow((long double)3,(int)eb)-1); 
          datei << endl;
        } 
    datei << "</COORDS2D_FLOAT>" << endl;
    datei << "</GRAPH>" << endl;
  }
  return 0;
}
