// *********************************************************************
// *                                                                   *
// *       Konstruktion von Isomeren der Alkane (C_nH_2n+2)            *
// *       ================================================            *
// *                                                                   *
// *       Autor:        Sascha Kurz                                   *
// *       Email:        sascha.kurz@uni-bayreuth.de                   *
// *       Homepage:     www.mathe2.uni-bayreuth.de/sascha             *
// *                                                                   *
// *       Filename:           konstruiere_alkane.cpp                  *
// *       zuletzt geändert:   31.10.2004                              *
// *       Betriebssystem:     Suse Linux 9.1                          *
// *       Compiler:           gcc3.3.3                                *
// *       Compiler-Optionen:  g++ -O2 -Wall -pedantic -ansi           *
// *                           konstruiere_alkane.cpp                  *
// *                           -o konstruktion                         *
// *                                                                   *
// *       Algorithmus                                                 *
// *       -----------                                                 *
// *       Basierend auf dem Artikel "Anzahl von Strukturisomeren      *
// *       der Alkane". Es werden rekursiv Wurzelbäume zusammenge-     *
// *       setzt, die wiederum zum Zusammensetzen von Alkanen ver-     *
// *       wendet werden.                                              *
// *                                                                   *
// *       + Wurzelbäume                                               *  
// *       -------------                                               *                                           
// *       Ebene 3        o o o o o      Der Knoten in Ebene 1         *
// *                       \|  \|/       heißt Wurzel. Jeder Kno-      *
// *       Ebene 2          o o o        ten, außer der Wurzel, hat    *  
// *                         \|/         genau eine Kante zu einem     * 
// *       Ebene 1            o          Knoten auf einer um 1 nie-    *
// *                                     drigeren Ebene.               *
// *                                                                   *
// *       Abgespeichert werden Wurzelbäume als Array. Hierzu num-     *
// *       meriert man die Knoten, und speichert zu jedem Knoten       *
// *       die Nummer des Knotens auf der um eines niedrigeren Ebene   *
// *       zu dem er eine Kante besitzt. Gibt man der Wurzel die       *
// *       Nummer 0, so kann man als Konvention im Array an der        *
// *       Position 0 ebenfalls die 0 speichern.                       *
// *                                                                   *
// *       Ebene 3        4 5 6 7 8      Nummerierte Version des       *
// *                       \|  \|/       obigen Wurzelbaums. Das       *
// *       Ebene 2          1 2 3        entsprechende Array ist       *  
// *                         \|/         in diesem Fall:               * 
// *       Ebene 1            0          [0,0,0,0,1,1,3,3,3]           *
// *                                                                   *
// *       + Zusammensetzen von Wurzelbäumen                           *
// *       ---------------------------------                           *
// *       Die Anzahl der Ebenen eines Wurzelbaums bezeichnet man      *
// *       als seine Höhe. Wurzelbäume der Höhe h kann man folgt       *
// *       aus Wurzelbäumen kleinerer Höhe zusammensetzen:             *
// *                                                                   *    
// *       4 5 6 7 8   2 3 4              5 6  7 8 9 12 13 14          *                        
// *        \|  \|/     \|/                \|   \|/    \|/             *
// *         1 2 3       1     1   --->     2 3 4      11 16           *
// *          \|/        |     |             \|/        | |            *
// *           0         0     0              1        10 15           *           
// *                                           \        |/             *
// *                                            --------0              *
// *                                                                   *
// *       Das Zusammensetzen und Umnummerieren von Wurzelbäumen,      *
// *       wird von der Funktion "Fuege_an()" erledigt. Diese fügt     *
// *       den neu konstruierten Wurzelbaum gleichzeitig noch an       *
// *       eine Liste L von Wurzelbäumen an. Die Wurzelbäume           *
// *       kleinerer Höhe werden als Positionen in der Liste L         *
// *       übergeben. Da in unserer Anwendung jeder Knoten nur         *
// *       höchstens 4 Nachbarn hat, brauchen nur 3 int-Werte a,b,c    *
// *       übergeben werden.                                           *
// *                                                                   *
// *       + Erzeugen von Wurzelbäumen                                 *
// *       ---------------------------                                 *
// *       Im vorhergehenden Abschnitt haben wir erklärt, wie          *
// *       Wurzelbäume mit kleinerer Höhe zu Wurzelbäumen mit          *
// *       größerer Höhe zusammengesetzt werden. Hierzu startet        *
// *       man mit einer Liste von Wurzelbäumen der Höhe kleiner-      *
// *       gleich h-1 und konstruieren daraus eine Liste von Wurzel-   *
// *       bäumen der Höhe kleinergleich h. Für den Start initiali-    *
// *       siert man die Liste mit dem leeren Wurzelbaum und dem       *
// *       Wurzelbaum der Höhe 1.Nun wählt man 3 Wurzelbäume aus der   *
// *       Liste der Wurzelbäume der Höhe kleinergleich h-1 aus,       * 
// *       wobei mindestens einer davon exakt die Höhe h-1 besitzen    *
// *       muss, und wendet die Funktion "Fuege_an()" an, um aus den   *
// *       3 Wurzelbäumen einen neuen Wurzelbaum der Höhe h zu         *
// *       konstruieren. Erledigt wird diese Arbeit von der Funktion   *
// *       "konstruiere_Wurzelbaeume()". Aus Geschwindigkeitsgründen   *
// *       wird nur mit einer Liste von Wurzelbäumen gearbeitet, und   *
// *       die Auswahl der 3 Wurzelbäume wird vorausschauend durchge-  *
// *       führt, so daß keine Wurzelbäume mit "zu vielen" Knoten      *
// *       konstruiert werden.                                         *                 
// *                                                                   *                
// *       + Zusammensetzen von Wurzelbäumen zu Alkanen                *
// *       --------------------------------------------                *
// *       Um Wurzelbäume zu Alkanen zusammenzusetzen, muss man die    *
// *       längste gerade Kette eines Alkans betrachten. U.U. kann es  *
// *       mehrere Ketten mit der Maximallänge k im Molekül geben.     *
// *       Ist nun k gerade, so kann man das Alkan darstellen als zwei *
// *       Wurzelbäume W_1 und W_2 der Höhe k/2, deren Wurzeln durch   *
// *       eine Kante verbunden werden: W_1-W_2.                       *
// *       Ist dagegen k ungerade, so besteht das Alkanmolekül aus 4   *
// *       Wurzelbäumen, wovon mindestens 2 die Höhe (k-1)/2 haben     *
// *       müssen und die restlichen maximal diese Höhe besitzen,      *
// *       deren Wurzeln mit einem zusätzlichen Knoten durch je        *
// *       eine Kante verbunden sind:                                  *
// *                                                                   *
// *          W_2                                                      *
// *           |                                                       * 
// *       W_1-o-W_3                                                   *
// *           |                                                       *
// *          W_4                                                      *
// *                                                                   *
// *       Somit können wir ein Alkan speichern, als eine Menge von    *
// *       4 Wurzelbäumen und der zusätzlichen Information, ob die     *
// *       Länge der längsten geraden Kette gerade oder ungerade ist.  *
// *       Die Konstruktion der Alkane aus Wurzelbäumen übernimmt die  *
// *       Funktion "konstruiere_Alkane()". Das eigentlich zusammen-   *
// *       setzen zu einem Alkan wird von der Funktion "Fuege_an()"    *  
// *       erledigt, die das Alkan auch gleichzeitig an eine Liste     *
// *       aus Alkanen anhängt.                                        *
// *                                                                   *
// *       + Ausgeben von Alkanen auf dem Bildschirm                   *
// *       -----------------------------------------                   *
// *       Mit diesem Algorithmus werden die Isomere der Alkane        *
// *       nur gezählt, sondern auch konstruiert. Also ist es zweck-   *
// *       mässig sie auch auf dem Bildschirm auszugeben. Die Ausgabe  *
// *       der internen Datenstruktur ist nicht sonderlich sinnvoll.   *
// *       Daher gibt es eine Funktion "ausgabe()", die ein Alkan in   *
// *       "Ascii-Art" auf dem Bildschirm ausgibt. Hierbei ist das     *
// *       Ziel, ein Alkan überschneidungsfrei mit den Zeichen         *
// *       o, /, |, - und \ darzustellen. Mit dem realen Aussehen des  *
// *       entsprechenden Alkans wird dies in der Regel wenig zu tun   *
// *       haben, gibt aber zumindest eine erste Vorstellung.          *
// *                                                                   *
// *                                                                   *
// *       Schlußbemerkung                                             *
// *       ===============                                             *
// *       Bei Fragen, Anmerkungen, Kritik oder sonstigen Kommentaren  *
// *       wende man sich bitte per email an den oben genannten Autor  *
// *       dieses Programms. Der Quellcode ist für nicht-kommerzielle  *
// *       Anwendungen frei verfügbar und darf beliebig verändert      *
// *       werden. Bei Veränderungen ist geeignet darauf hinzuweisen,  *
// *       von wem sie vorgenommen wurden. Es würde mich freuen über   *
// *       Verwendungen dieses Quellcodes eine kurze Nachricht per     *
// *       email zu erhalten.                                          *
// *                                                                   *
// *       Sascha Kurz                                                 *    
// *                                                                   *    
// *********************************************************************



#include<vector>              // Headerfile für die Klasse Vector
#include<iostream>            // Headerfile für die Ein- und Ausgabe

#define BILDSCHIRMBREITE 122  // Breite des Bildschirms in Ascii-Zeichen

using namespace std;

struct Alkan                               // Datenstruktur für ein Alkan
{
  vector <unsigned char> Wurzelbaum_1;     // Vektor für den ersten Wurzelbaum
  vector <unsigned char> Wurzelbaum_2;     // Vektor für den zweiten Wurzelbaum
  vector <unsigned char> Wurzelbaum_3;     // Vektor für den dritten Wurzelbaum  
  vector <unsigned char> Wurzelbaum_4;     // Vektor für den vierten Wurzelbaum 
  bool laengste_gerade_Kette_ist_gerade;   // Ist die längste gerade Kette des Alkans eine gerade Zahl?
};

struct ascii_platzierung                   // Datenstruktur für eine Platzierung eines Alkans 
{                                          // in "Ascii-Art" auf dem Bildschirm 
  vector< vector<char> > Matrix;           // Matrix der auf dem Bildschirm dartzstellenden Zeichen
  unsigned int anzahl_zeilen;              // Anzahl der Spalten der Matrix 
  unsigned int anzahl_spalten;             // Anzahl der Zeilen der Matrix  
  unsigned int position_wurzel;            // Spalten-Nr. der Wurzel in der untersten (0.) Zeile 
};

// *********************************************************************
// *                  Prototypdeklarationen                            *
// *********************************************************************

int ausgabe(Alkan& A);
// Gibt eine zweidimensionale Platzierung eines Alkanmoleküls in "Ascii-Art" 
// auf dem Bildschirm aus, sofern es nicht zu breit ist 

void Fuege_an(vector< vector <unsigned char> >& L,int a,int b,int c);
// Konstruiert aus 3 Wurzelbäumen einen neuen Wurzelbaum mit größerer
// Höhe und fügt ihn der Liste L an

void Fuege_an(vector<Alkan>&,const vector< vector <unsigned char> >& L,
unsigned int a,unsigned int b,bool ausgeben,int ausgabe_bei_anzahl_c_atomen);
// Konstruiert aus 2 Wurzelbäumen der Höhe kleinergleich h ein Alkan mit 
// längster gerader Kette 2h und fügt es der Liste Alkane an


void Fuege_an(vector<Alkan>&,const vector< vector <unsigned char> >& L,
unsigned int a,unsigned int b,unsigned int c,unsigned d,bool ausgeben,int ausgabe_bei_anzahl_c_atomen);
// Konstruiert aus 4 Wurzelbäumen der Höhe kleinergleich h ein Alkan mit 
// längster gerader Kette 2h+1 und fügt es der Liste Alkane an

void sortiere_Wurzelbaum_Liste(vector< vector <unsigned char> >& L,int p);
// Sortiert eine Liste L von Wurzelbäumen, so dass die ersten p Wurzelbäumen
// und die restlichen Wurzelbäumen jeweils nach der Anzahl der Knoten 
// aufsteigend angeordnet sind

void konstruiere_Wurzelbaeume(vector<Alkan>& Alkane,vector< vector <unsigned char> >& L,int n,bool ausgeben,int ausgabe_bei_anzahl_c_atomen);
// Konstuiert eine "vollständige" Liste von Wurzelbäumen, so dass damit 
// eine vollständige Liste aller Alkane mit bis zu n C-Atomen konstruiert 
// werden kann.

void konstruiere_Alkane(vector<Alkan>& Alkane,const vector< vector <unsigned char> >& L,
unsigned int p,unsigned int h,unsigned int n,bool ausgeben,int ausgabe_bei_anzahl_c_atomen);
// Erweitert eine Liste von Alkanen mit denen solchen die eine längste gerade
// Kette der Länge 2h oder 2h+1 und maximal n Knoten haben. Die dafür notwendigen
// Wurzelbäume der Höhe h stehen in der Liste L ab Position p



// ********************************************************************
// *         Implementierungen der benötigten Funktionen              *
// ********************************************************************

void Fuege_an(vector< vector <unsigned char> >& L,int a,int b,int c)
{
  // L:   Liste von Wurzelbäumen
  // a:   Position eines Wurzelbaums in der Liste L
  // b:   Position eines Wurzelbaums in der Liste L
  // c:   Position eines Wurzelbaums in der Liste L
  //
  // Funktion: 
  // ---------
  // Konstruiert einen neuen Wurzelbaum W, indem die Wurzelbäume Nr. a,b 
  // und c vereinigt werden, und die Wurzeln von a,b und c mit einer 
  // neuen Wurzel verbunden werden.
  //
  //      a b c
  //  W=   \|/
  //        o
  //
  // Der neu konstruierte Wurzelbaum W wird an die Liste L angehängt

  int p=L.size();               // die alte Länge der Liste der Wurzelbäume
  unsigned int i;               // Schleifenvariable
  int a_s=L[a].size();          // aus wie vielen Knoten besteht Wurzelbaum Nr. a
  int b_s=L[b].size();          // aus wie vielen Knoten besteht Wurzelbaum Nr. b 
  int c_s=L[c].size();          // aus wie vielen Knoten besteht Wurzelbaum Nr. c
  L.resize(p+1);                // Verlängere die Liste um ein Element
  L[p].resize(a_s+b_s+c_s+1);   // Der neue Wurzelbaum setzt sich aus Wurzelbaum Nr. a, 
                                // Wurzelbaum Nr. b, Wurzelbaum Nr. c und einem zusätzlichen 
                                // Knoten, der neuen Wurzel, zusammen
  L[p][0]=0;                    // neue Wurzel
  L[p][1]=0;                    // alte Wurzel von Wurzelbaum Nr. a zeigt auf neue Wurzel
  for(i=1;i<L[a].size();i++)    // durchlaufe die restlichen Knoten von Wurzelbaum Nr. a
    L[p][i+1]=L[a][i]+1;        // verschiebe die alte Nummerierung 
  L[p][1+a_s]=0;                // alte Wurzel von Wurzelbaum Nr. b zeigt auf neue Wurzel
  for(i=1;i<L[b].size();i++)    // durchlaufe die restlichen Knoten von Wurzelbaum Nr. a
    L[p][i+1+a_s]=L[b][i]+1+a_s;// verschiebe die alte Nummerierung
  L[p][1+a_s+b_s]=0;            // alte Wurzel von Wurzelbaum Nr. c zeigt auf neue Wurzel
  for(i=1;i<L[c].size();i++)    // durchlaufe die restlichen Knoten von Wurzelbaum Nr. a
    L[p][i+1+a_s+b_s]=L[c][i]+1+a_s+b_s; // verschiebe die alte Nummerierung
}

void Fuege_an(vector<Alkan>& Alkane,const vector< vector <unsigned char> >& L,unsigned int a,
unsigned int b,bool ausgeben,int ausgabe_bei_anzahl_c_atomen)
{
  // Alkane : Liste von Alkanen
  // L:       Liste von Wurzelbäumen
  // a:       Position eines Wurzelbaums in der Liste L
  // b:       Position eines Wurzelbaums in der Liste L
  //
  // Funktion: 
  // ---------
  // Konstruiert aus zwei Wurzelbäumen W_1 und W_2 der  Höhe h ein Alkan A mit längster gerader Kette 2h:
  //
  //  A=  W_1-W_2 (die Wurzeln von W_1 und W_2 werden mit einer Kante verbunden)
  //
  // Das neu konstruierte Alkan A wird an die Liste Alkane angehängt

  unsigned int i;               // Schleifenvariable; 
  int p=Alkane.size();          // die alte Länge der Liste der Alkane
  Alkane.resize(p+1);           // verlängere die Liste um ein Element

  Alkane[p].laengste_gerade_Kette_ist_gerade=true;
  Alkane[p].Wurzelbaum_1.resize(L[a].size());   // Reserviere Speicherplatz für den Wurzelbaum Nr. a
  for(i=0;i<L[a].size();i++)                    // Kopiere Wurzelbaum Nr. a 
    Alkane[p].Wurzelbaum_1[i]=L[a][i];
  Alkane[p].Wurzelbaum_2.resize(L[b].size());   // Reserviere Speicherplatz für den Wurzelbaum Nr. b
  for(i=0;i<L[b].size();i++)                    // Kopiere Wurzelbaum Nr. b     
    Alkane[p].Wurzelbaum_2[i]=L[b][i];
  Alkane[p].Wurzelbaum_3.resize(0);             // Da die Länge der länsten geraden Kette eine gerade  
  Alkane[p].Wurzelbaum_4.resize(0);             // Zahl ist, werden nur 2 Wurzelbäume benutzt

  if(ausgeben==true)
  {
    if(ausgabe_bei_anzahl_c_atomen==-1|| (int)(Alkane[p].Wurzelbaum_1.size()+Alkane[p].Wurzelbaum_2.size())==ausgabe_bei_anzahl_c_atomen)
    ausgabe(Alkane[p]);
  }
}

void Fuege_an(vector<Alkan>& Alkane,const vector< vector <unsigned char> >& L,unsigned int a,
unsigned int b,unsigned int c,unsigned int d,bool ausgeben,int ausgabe_bei_anzahl_c_atomen)
{
  // Alkane : Liste von Alkanen
  // L:       Liste von Wurzelbäumen
  // a:       Position eines Wurzelbaums in der Liste L
  // b:       Position eines Wurzelbaums in der Liste L
  // c:       Position eines Wurzelbaums in der Liste L
  // d:       Position eines Wurzelbaums in der Liste L
  //
  // Funktion: 
  // ---------
  // Konstruiert aus vier Wurzelbäumen W_1, W_2, W_3 und W_4 der Höhe kleinergleich h ein Alkan A mit 
  // längster gerader Kette 2h+1:
  //
  //         W_2  
  //          |
  //  A=  W_1-o-W_3          (die Wurzeln von W_1, W_2, W_3 und W_4 werden mit dem 
  //          |              zusätzlichen Knoten durch je eine Kante verbunden)
  //         W_4 
  //
  // Das neu konstruierte Alkan A wird an die Liste Alkane angehängt

  unsigned int i;               // Schleifenvariable; 
  int p=Alkane.size();          // die alte Länge der Liste der Alkane
  Alkane.resize(p+1);           // verlängere die Liste um ein Element

  Alkane[p].laengste_gerade_Kette_ist_gerade=false;
  Alkane[p].Wurzelbaum_1.resize(L[a].size());   // Reserviere Speicherplatz für den Wurzelbaum Nr. a
  for(i=0;i<L[a].size();i++)                    // Kopiere Wurzelbaum Nr. a 
    Alkane[p].Wurzelbaum_1[i]=L[a][i];
  Alkane[p].Wurzelbaum_2.resize(L[b].size());   // Reserviere Speicherplatz für den Wurzelbaum Nr. b
  for(i=0;i<L[b].size();i++)                    // Kopiere Wurzelbaum Nr. b     
    Alkane[p].Wurzelbaum_2[i]=L[b][i];
  Alkane[p].Wurzelbaum_3.resize(L[c].size());   // Reserviere Speicherplatz für den Wurzelbaum Nr. c
  for(i=0;i<L[c].size();i++)                    // Kopiere Wurzelbaum Nr. c     
    Alkane[p].Wurzelbaum_3[i]=L[c][i];
  Alkane[p].Wurzelbaum_4.resize(L[d].size());   // Reserviere Speicherplatz für den Wurzelbaum Nr. d
  for(i=0;i<L[d].size();i++)                    // Kopiere Wurzelbaum Nr. d     
    Alkane[p].Wurzelbaum_4[i]=L[d][i];
  if(ausgeben==true)
  {
    if(ausgabe_bei_anzahl_c_atomen==-1|| (int)(Alkane[p].Wurzelbaum_1.size()+Alkane[p].Wurzelbaum_2.size()+Alkane[p].Wurzelbaum_3.size()+Alkane[p].Wurzelbaum_4.size()+1)==ausgabe_bei_anzahl_c_atomen)
    ausgabe(Alkane[p]);
  }
}


void sortiere_Wurzelbaum_Liste(vector< vector <unsigned char> >& L,int laenge,int n)
{
  // L:       Liste von Wurzelbäumen
  // laenge:  Position in Liste L, siehe "Funktion"
  // n:       maximale Anzahl der Knoten
  //
  // Funktion: 
  // ---------
  // Sortiert die Positionen 0 bis laenge-1 der Liste L so um, daß die Anzahl der 
  // Knoten der Wurzelbäume aufsteigend ist. Die Positionen laenge bis L.size()-1
  // werden entsprechend sortiert.
  // Da die maximale Anzahl der Knoten (n) gegenüber der Länge der Liste eine sehr 
  // kleine Zahl ist, werden anstatt einem Sortieralgorithmus wie Mergesort oder 
  // Quicksort einfach n+1 Listen angelegt, und jeder Wurzelbaum wird in die Liste 
  // mit der Nummer entsprechend der Anzahl seiner Knoten gelegt. Anschliessend 
  // werden diese Listen wieder zu einer zusammengefügt. Der Aufwand beträgt somit 
  // O(n+L.size())  
 
  int i,j;            // Hilfsvariablen
  unsigned int k,a,b; // Hilfsvariablen
 
  vector< vector< vector<unsigned char> > > sort;     // Hilfsvektor zum Sortieren

  sort.resize(n+1);                    // Anlegen der n+1 Hilfslisten         
  for(i=0;i<=n;i++)                    // durchlaufe die Hilfslisten
    sort[i].resize(0);                 // Jede Liste ist am Anfang leer

  // Trage die Wurzelbäume, der Positionen 0,...,laenge-1, in die Hilfslisten ein
  for(i=0;i<laenge;i++)                // durchlaufe die Positionen 0 bis laenge-1 von L
  {
    k=L[i].size();                     // Wie viele Knoten besitzt der gerade betrachtete Wurzelbaum?
    a=sort[k].size();                  // Wie lang ist die entsprechende Hilfsliste
    sort[k].resize(a+1);               // Verlängere die Hilfsliste um ein Element
    sort[k][a].resize(k);              // Der neue Wurzelbaum besteht aus k Knoten 
    for(b=0;b<k;b++)                   // Füge den betrachteten Wurzelbaum in die    
      sort[k][a][b]=L[i][b];           // Hilfsliste ein 
  }

  // Trage die Wurzelbäume aus den Hilfslisten wieder zurück in L ein
  j=0;                                 // Aktuelle Position in L
  for(i=0;i<=n;i++)                    // durchlaufe die unterschiedlichen Hilfslisten
  {
    for(a=0;a<sort[i].size();a++)      // durchlaufe Hilfsliste i 
    {
      L[j].resize(i);                  // Speichere an Position j in L einen Wurzelbaum bestehend aus i Knoten
      for(b=0;b<(unsigned int)i;b++)
        L[j][b]=sort[i][a][b];
      j++;                             // Verschiebe die aktuelle Position in L um eins nach rechts 
    } 
  }
  
  sort.resize(n+1);                    // "Lösche" die Hilfslisten wieder
  for(i=0;i<=n;i++)
  {
    for(a=0;a<sort[i].size();a++)
      sort[i][a].resize(0);
    sort[i].resize(0);
  }
  
  // Trage die Wurzelbäume, der Positionen laenge,...,L.size()-1, in die Hilfslisten ein
  for(i=laenge;i<(int)L.size();i++)         // durchlaufe die Positionen laenge bis L.size()-1 von L
  {
    k=L[i].size();                     // Wie viele Knoten besitzt der gerade betrachtete Wurzelbaum?
    a=sort[k].size();                  // Wie lang ist die entsprechende Hilfsliste  
    sort[k].resize(a+1);               // Verlängere die Hilfsliste um ein Element 
    sort[k][a].resize(k);              // Der neue Wurzelbaum besteht aus k Knoten
    for(b=0;b<k;b++)                   // Füge den betrachteten Wurzelbaum in die 
      sort[k][a][b]=L[i][b];           // Hilfsliste ein 
  }

  // Trage die Wurzelbäume aus den Hilfslisten wieder zurück in L ein
  for(i=0;i<=n;i++)                    // durchlaufe die unterschiedlichen Hilfslisten
  {
    for(a=0;a<sort[i].size();a++)      // durchlaufe Hilfsliste i 
    {
      L[j].resize(i);                  // Speichere an Position j in L einen Wurzelbaum bestehend aus i Knoten
      for(b=0;b<(unsigned int)i;b++)
        L[j][b]=sort[i][a][b];
      j++;                             // Verschiebe die aktuelle Position in L um eins nach rechts 
    } 
  }

  sort.resize(n+1);                    // "Lösche" die Hilfslisten wieder
  for(i=0;i<=n;i++)
  {
    for(a=0;a<sort[i].size();a++)
      sort[i][a].resize(0);
    sort[i].resize(0);
  }
}


void konstruiere_Wurzelbaeume(vector<Alkan>& Alkane,vector< vector <unsigned char> >& L,int n,bool ausgeben,int ausgabe_bei_anzahl_c_atomen)
{
  // L: Liste zum Speichern von Wurzelbäumen
  // n: maximale Anzahl von C-Atomen in den später zu konstruierenden Alkanen
  //
  // Funktion:
  // ---------
  // Ziel ist es eine vollständige Liste aller Wurzelbäume der Höhe h mit maximal n-h Knoten
  // zu konstruieren. Hierbei läuft h von 0 bis [n/2].
  // Dabei werden Wurzelbäume der Höhe h aus Wurzelbäumen der Höhe kleinergleich h-1 zusammen-
  // gesetzt. Für weitere Details lese den Artikel:"Anzahl von Strukturisomeren der Alkane"



  // Startbelegung für die Liste der Wurzelbäume
  L.resize(2);                              // Am Anfang sind nur 2 Wurzelbäume in der Liste eingetragen         
  L[0].resize(0);                           // Leerer Wurzelbaum aus 0 Knoten
  L[1].resize(1);
  L[1][0]=0;                                // Wurzelbaum aus einem Knoten
  int p=1;                                  // Position des Wurzelbaums der Höhe 1
                                            // Allgemein: Position des ersten Wurzelbaums der Höhe h in der 
                                            // Liste L

  // Startbelegung für die Liste der Alkane
  Alkane.resize(2);
  Alkane[0].laengste_gerade_Kette_ist_gerade=true;   // Leeres Alkan
  Alkane[0].Wurzelbaum_1.resize(0);
  Alkane[0].Wurzelbaum_2.resize(0);
  Alkane[0].Wurzelbaum_3.resize(0);
  Alkane[0].Wurzelbaum_4.resize(0);
  Alkane[1].laengste_gerade_Kette_ist_gerade=false;  // Alkan aus einem C-Atom (Methan)
  Alkane[1].Wurzelbaum_1.resize(0);
  Alkane[1].Wurzelbaum_2.resize(0);
  Alkane[1].Wurzelbaum_3.resize(0);
  Alkane[1].Wurzelbaum_4.resize(0);
 
  cout << endl;
  if(ausgeben==true)
    if(ausgabe_bei_anzahl_c_atomen==-1 || ausgabe_bei_anzahl_c_atomen==1)
      cout << "o" << endl << endl;
 
  if(ausgeben==false)
  { 
    cout << endl;
    cout << "Konstruiere Wurzelbäume der Höhe 1 ";
    if(n>=20) cout << " ";
    cout << ": ." << endl;
  }
  // Konstruiere Alkane mit längster gerader Kette 2 und 3
  konstruiere_Alkane(Alkane,L,p,1,n,ausgeben,ausgabe_bei_anzahl_c_atomen);
 

  int a,b,c;                                // Position der drei kleineren Wurzelbäume in der Liste L 
  int laenge;                               // Hilfsgröße, um "p" bzw. die Länge der Liste auch bei 
                                            // Änderungen an der Liste verfügbar zu haben  
    
  for(int h=2;h<=int(n/2);h++)              // Durchlaufe die möglichen Höhen der Wurzelbäume. Die Wurzelbäume  
  {                                         // der Höhe 0 und 1 wurden mit der Startbelegung behandelt.     
    // Forstschrittsausgabe, da Berechnung lange dauern kann
    if(ausgeben==false)
    {
      cout << "Konstruiere Wurzelbäume der Höhe " << h << " ";
      if(n>=20 && h<10) cout << " ";
      cout << ": "; 
    }

    laenge=L.size();                        // Wie  lang ist die Liste L bisher

    // In der Liste L stehen an den Positionen 0 bis p-1 Wurzelbäume der Höhe kleiner h-1 
    // geordnet nach der Anzahl der Knoten. An den Positionen p bis laenge-1 stehen 
    // Wurzelbäume der Höhe h-1, ebenfalls nach der Anzahl der Knoten geordnet.
    // Für die Kombination zu einem Wurzelbaum der Höhe h gibt es nun 3 Möglichkeiten:
    // (i)   zwei Wurzelbäume der Höhe kleiner h-1 und ein Wurzelbaum der Höhe h-1 
    // (ii)  ein Wurzelbaum der Höhe kleiner h-1 und zwei Wurzelbäume der Höhe h-1
    // (iii) drei Wurzelbäume der Höhe h-1
    //
    // Die Positionen der Wurzelbäume in L werden mit a,b und c bezeichnet. Um die Konstruktion 
    // zu beschleunigen, wird bei jedem Schritt überprüft, ob überhaupt ein Alkan mit höchstens 
    // n C-Atomen entstehen kann.


    // Fall (i)
    for(a=0;a<p;a++)                                // a ist ein Wurzelbaum der Höhe kleiner h-1
    {
      if(ausgeben==false)
        cout << "." << flush;                       // Forstschrittsausgabe, da Berechnung lange dauern kann
      if(1+(int)L[a].size()*2+(h-1)+h<=n)           // 1:               die neue Wurzel
      {                                             // 2*L[a].size()*2: b besitzt mindestens genau so viele
                                                    //                  Knoten wie a, also für a und b zusammen
                                                    //                  2*L[a].size()*2  
                                                    // (h-1):           Wurzelbaum c besitzt Höhe h-1, also mindestens
                                                    //                  h-1 Knoten
                                                    // h:               Es werden mindestens zwei Wurzelbäume der Höhe 
                                                    //                  h zu einem Alkan zusammengesetzt 
        for(b=a;b<p;b++)                            // b ist ein Wurzelbaum der Höhe kleiner h-1  
        {
          if(1+(int)L[a].size()+(int)L[b].size()+(h-1)+h<=n)  // 1:           die neue Wurzel
          {                                         // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                    // L[b].size(): Anzahl der Knoten von Wurzelbaum b
                                                    // (h-1):       Wurzelbaum c besitzt Höhe h-1, also mindestens
                                                    //              h-1 Knoten  
                                                    // h:               Es werden mindestens zwei Wurzelbäume der Höhe 
                                                    //                  h zu einem Alkan zusammengesetzt    
            for(c=p;c<laenge;c++)                   // c ist ein Wurzelbaum der Höhe h-1 
            {
              if(1+(int)L[a].size()+(int)L[b].size()+(int)L[c].size()+h<=n) // 1:           die neue Wurzel
              {                                              // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                             // L[b].size(): Anzahl der Knoten von Wurzelbaum b 
                                                             // L[c].size(): Anzahl der Knoten von Wurzelbaum b 
                                                             // h:           Es werden mindestens zwei Wurzelbäume der
                                                             //              Höhe h zu einem Alkan zusammengesetzt
                Fuege_an(L,a,b,c);                  // Füge die Wurzelbäume a,b und c zusammen und hänge sie an           
              }                                     // die Liste L an
              else                                  // Der entehende Wurzelbaum kann nicht zur Konstruktion eines 
                c=laenge;                           // Alkans mit maximal n C-Atomen verwendet werden, also gehe mit   
            }                                       // b eins weiter und starte mit c wieder bei Position p 
          }
          else                                      // Die entehenden Wurzelbäume können nicht zur Konstruktion eines
          {                                         // Alkans mit maximal n C-Atomen verwendet werden, also gehe mit
            b=p;                                    // a eins weiter und starte mit b wieder bei Position a 
          }     
        }
      }
      else                                          // Die entehenden Wurzelbäume können nicht zur Konstruktion eines
      {                                             // Alkans mit maximal n C-Atomen verwendet werden, also beende die
        a=p;                                        // Schleife  
      } 
    }


    // Fall (ii)
    for(a=0;a<p;a++)                                // a ist ein Wurzelbaum der Höhe kleiner h-1
    {
      if(ausgeben==false)
        cout << "." << flush;                       // Forstschrittsausgabe, da Berechnung lange dauern kann
      if(1+(int)L[a].size()+2*(h-1)+h<=n)           // 1:           die neue Wurzel
      {                                             // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                    // 2*(h-1):     Die Wurzelbäume b und c besitzen die Höhe
                                                    //              h-1, bestehen also aus mindestens h-1 Knoten
                                                    // h:           Es werden mindestens zwei Wurzelbäume der
                                                    //              Höhe h zu einem Alkan zusammengesetzt 
        for(b=p;b<laenge;b++)                       // b ist ein Wurzelbaum der Höhe h-1   
        {
          if(1+(int)L[a].size()+2*((int)L[b].size())+h<=n)      // 1:           die neue Wurzel
          {                                         // L[a].size(): Anzahl der Knoten von Wurzelbaum a

                                                    // h:           Es werden mindestens zwei Wurzelbäume der
                                                    //              Höhe h zu einem Alkan zusammengesetzt 
            for(c=b;c<laenge;c++)                   // c ist ein Wurzelbaum der Höhe h-1
              if(1+(int)L[a].size()+(int)L[b].size()+(int)L[c].size()+h<=n) // 1:           die neue Wurzel
              {                                              // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                             // L[b].size(): Anzahl der Knoten von Wurzelbaum b 
                                                             // L[c].size(): Anzahl der Knoten von Wurzelbaum b 
                                                             // h:           Es werden mindestens zwei Wurzelbäume der
                                                             //              Höhe h zu einem Alkan zusammengesetzt
                Fuege_an(L,a,b,c);                  // Füge die Wurzelbäume a,b und c zusammen und hänge sie an           
              }                                     // die Liste L an
              else                                  // Der entehende Wurzelbaum kann nicht zur Konstruktion eines  
                c=laenge;                           // Alkans mit maximal n C-Atomen verwendet werden, also gehe mit
          }                                         // b eins weiter und starte mit c wieder bei Position b
          else                                      // Die entehenden Wurzelbäume können nicht zur Konstruktion eines
          {                                         // Alkans mit maximal n C-Atomen verwendet werden, also gehe mit
            b=laenge;                               // a eins weiter und starte mit b wieder bei Position p
          }     
        }
      }
      else                                          // Die entehenden Wurzelbäume können nicht zur Konstruktion eines
      {                                             // Alkans mit maximal n C-Atomen verwendet werden, also beende die  
        a=p;                                        // Schleife 
      } 
    }

    
    // Fall (iii)
    for(a=p;a<laenge;a++)                           // a ist ein Wurzelbaum der Höhe h-1
    {
      if(ausgeben==false)
        cout << "." << flush;                       // Forstschrittsausgabe, da Berechnung lange dauern kann
      if(1+3*((int)L[a].size())+h<=n)               // 1:             die neue Wurzel 
      {                                             // 3*L[a].size(): Die Wurzelbäume b und c besitzen 
                                                    //                mindestens so viele Knoten wie der
                                                    //                Wurzelbauma     
                                                    // h:             Es werden mindestens zwei Wurzelbäume der
                                                    //                Höhe h zu einem Alkan zusammengesetzt
        for(b=a;b<laenge;b++)                       // b ist ein Wurzelbaum der Höhe h-1 
        {
          if(1+(int)L[a].size()+2*((int)L[b].size())+h<=n)      // 1:             die neue Wurzel
          {                                         // L[a].size():   Anzahl der Knoten von Wurzelbaum a
                                                    // 2*L[b].size(): Der Wurzelbaum c besitzt mindestens 
                                                    //                so viele Knoten wie Wurzelbaum b  
                                                    // h:             Es werden mindestens zwei Wurzelbäume der
                                                    //                Höhe h zu einem Alkan zusammengesetzt
            for(c=b;c<laenge;c++)                   // c ist ein Wurzelbaum der Höhe h-1 
              if(1+(int)L[a].size()+(int)L[b].size()+(int)L[c].size()+h<=n) // 1:           die neue Wurzel
              {                                              // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                             // L[b].size(): Anzahl der Knoten von Wurzelbaum b 
                                                             // L[c].size(): Anzahl der Knoten von Wurzelbaum b 
                                                             // h:           Es werden mindestens zwei Wurzelbäume der
                                                             //              Höhe h zu einem Alkan zusammengesetzt
                Fuege_an(L,a,b,c);                  // Füge die Wurzelbäume a,b und c zusammen und hänge sie an           
              }                                     // die Liste L an
              else                                  // Der entehende Wurzelbaum kann nicht zur Konstruktion eines 
                c=laenge;                           // Alkans mit maximal n C-Atomen verwendet werden, also gehe mit
          }                                         // b eins weiter und starte mit c wieder bei Position b
          else                                      // Die entehenden Wurzelbäume können nicht zur Konstruktion eines
          {                                         // Alkans mit maximal n C-Atomen verwendet werden, also gehe mit 
            b=laenge;                               // a eins weiter und starte mit b wieder bei Position a    
          }     
        }
      }
      else                                          // Die entehenden Wurzelbäume können nicht zur Konstruktion eines
      {                                             // Alkans mit maximal n C-Atomen verwendet werden, also beende die  
        a=laenge;                                   // Schleife 
      } 
    }
    if(ausgeben==false)
      cout << endl;
    
     
    sortiere_Wurzelbaum_Liste(L,laenge,n);          // Stelle die "richtige" Sortierung der Liste L wieder her, so
                                                    // wie sie für diesen Algorithmus gebraucht wird
    p=laenge;                                       // Start für Wurzelbäume der Länge h

    // Konstruiere Alkane mit längster gerader Kette der Länge 2h und 2h+1 aus den bereits erzeugten Wurzelbäumen
    konstruiere_Alkane(Alkane,L,p,h,n,ausgeben,ausgabe_bei_anzahl_c_atomen);             

  }
}

void konstruiere_Alkane(vector<Alkan>& Alkane,const vector< vector <unsigned char> >& L,
unsigned int p,unsigned int h,unsigned int n,bool ausgeben,int ausgabe_bei_anzahl_c_atomen)
{
  // Alkane:  eine Liste von Alkanen
  // L:       eine Liste von Wurzelbäumen
  // p:       Position in L, ab der die Wurzelbäume der Höhe h stehen
  // h:       Höhe der Wurzelbäume, die aktuell für die Konstruktion verwendet werden sollen 
  // n:       maximale Anzahl der C-Atome, die die Alkane besitzen sollen 
  //
  // Funktion:
  // ---------
  // Erweitert eine Liste von Alkanen mit denen solchen die eine längste gerade
  // Kette der Länge 2h oder 2h+1 und maximal n Knoten haben. Die dafür notwendigen
  // Wurzelbäume der Höhe h stehen in der Liste L ab Position p
  
  if(ausgeben==false)
  {
    cout << "Konstruiere zugehörige Alkane      ";// Fortschrittsanzeige
    if(n>=20) cout << " ";
    cout << ": " << flush;
  }

  unsigned int a,b,c,d;                           // Nummer der bis zu 4 Wurzelbäume, die aus der Liste L ausgewählt werden

  if(2*h>n) return;                               // Es können keine Alkane mit hinreichend wenig C-Atomen 
                                                  // konstruiert werden

  // In der Liste L stehen an den Positionen 0 bis p-1 Wurzelbäume der Höhe kleiner h 
  // geordnet nach der Anzahl der Knoten. An den Positionen p bis laenge-1 stehen 
  // Wurzelbäume der Höhe h, ebenfalls nach der Anzahl der Knoten geordnet.
  // Für die Kombination zu einem Alkan mit längster gerader Kette der Länge 2h gibt 
  // es nun 1 Möglichkeit:
  // (i)   zwei Wurzelbäume der Höhe h, deren Wurzeln mit einer Kante verbunden werden
  // Die Positionen der Wurzelbäume in L werden mit a und b bezeichnet. Um die Konstruktion 
  // zu beschleunigen, wird bei jedem Schritt überprüft, ob überhaupt ein Alkan mit höchstens 
  // n C-Atomen entstehen kann.


  for(a=p;a<L.size();a++)                         // a ist ein Wurzelbaum der Höhe h
  {
    if(ausgeben==false)
      cout << "." << flush;                       // Fortschrittsanzeige
    if(2*L[a].size()<=n)                          // der Wurzelbaum b besitzt mindestens so viele Knoten wie a
    {
      for(b=a;b<L.size();b++)                     // b ist ein Wurzelbaum der Höhe h
        if(L[a].size()+L[b].size()<=n)            // L[a].size(): Anzahl der Knoten vom Wurzelbaum a
        {                                         // L[b].size(): Anzahl der Knoten vom Wurzelbaum b 
          Fuege_an(Alkane,L,a,b,ausgeben,ausgabe_bei_anzahl_c_atomen); // Füge die Wurzelbäume a und b zusammen und  
        }                                         // hänge sie an die Liste Alkane an  
        else
          b=L.size();                             // Es kann kein Alkan mehr mit höchstens n C-Atomen konstruiert  
    }                                             // werden. Erhöhe a um eins und setze b zurück  
    else                                          // Es kann kein Alkan mehr mit höchstens n C-Atomen konstruiert
      a=L.size();                                 // werden. Beende die Schleife.  
  }


  // Für die Kombination zu einem Alkan mit längster gerader Kette der Länge 2h+1 gibt 
  // es nun 3 Möglichkeiten:
  // (i)   zwei Wurzelbäume der Höhe kleiner h und 2 Wurzelbäume der Höhe h
  // (ii)  ein Wurzelbaum der Höhe kleiner h und 3 Wurzelbäume der Höhe h 
  // (iii) vier Wurzelbäume der Höhe h
  // Die Positionen der Wurzelbäume in L werden mit a,b,c und d bezeichnet. Um die Konstruktion 
  // zu beschleunigen, wird bei jedem Schritt überprüft, ob überhaupt ein Alkan mit höchstens 
  // n C-Atomen entstehen kann.

  if(2*h+1<=n)                                    // Falls es überhaupt möglich ist, dass ein Alkan mit
  {                                               // höchstens n C-Atomen entstehen kann
    // Fall (i)             
    for(a=0;a<p;a++)                              // a ist ein Wurzelbaum der Höhe kleiner h
    {
      if(ausgeben==false)
        cout << "." << flush;                     // Fortschrittsanzeige
      if(1+2*L[a].size()+2*h<=n)                  // 1:             zusätzlicher Knoten 
      {                                           // 2*L[a].size(): der Wurzelbaum b besitzt mindestens so 
                                                  //                viele Knoten wie der Wurzelbaum a    
                                                  // 2*h:           die Wurzelbäume c und d besitzen die Höhe 
                                                  //                h, bestehen also aus mindestens h Knoten   
        for(b=a;b<p;b++)                          // b ist ein Wurzelbaum der Höhe kleiner h
          if(1+L[a].size()+L[b].size()+2*h<=n)    // 1:           zusätzlicher Knoten  
          {                                       // L[a].size(): Anzahl der Knoten von Wurzelbaum a 
                                                  // L[b].size(): Anzahl der Knoten von Wurzelbaum b 
                                                  // 2*h:           die Wurzelbäume c und d besitzen die Höhe 
                                                  //                h, bestehen also aus mindestens h Knoten
            for(c=p;c<L.size();c++)               // c ist ein Wurzelbaum der Höhe h 
              if(1+L[a].size()+L[b].size()+2*L[c].size()<=n)
              {                                   // 1:             zusätzlicher Knoten
                                                  // L[a].size():   Anzahl der Knoten von Wurzelbaum a 
                                                  // L[b].size():   Anzahl der Knoten von Wurzelbaum b 
                                                  // 2*L[c].size(): der Wurzelbaum d besitzt mindestens so 
                                                  //                viele Knoten wie der Wurzelbaum c
                for(d=c;d<L.size();d++)           // d ist ein Wurzelbaum der Höhe h 
                  if(1+L[a].size()+L[b].size()+L[c].size()+L[d].size()<=n)
                  {                               // 1:           zusätzlicher Knoten  
                                                  // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                  // L[b].size(): Anzahl der Knoten von Wurzelbaum b  
                                                  // L[c].size(): Anzahl der Knoten von Wurzelbaum c
                                                  // L[d].size(): Anzahl der Knoten von Wurzelbaum d
                    Fuege_an(Alkane,L,a,b,c,d,ausgeben,ausgabe_bei_anzahl_c_atomen);
                                                  // Füge die Wurzelbäume a, b, c und d mit einem zusätzlichen Knoten
                  }                               // zusammen und hänge das entstehende Alkan an die Liste an
                  else                            // Es kann kein Alkan mehr mit höchstens n C-Atomen konstruiert
                    d=L.size();                   // werden. Erhöhe c um eins und setze d zurück
              }
              else                                // Es kann kein Alkan mehr mit höchstens n C-Atomen konstruiert
                c=L.size();                       // werden. Erhöhe b um eins und setze c zurück 
          }
          else                                    // Es kann kein Alkan mehr mit höchstens n C-Atomen konstruiert
            b=p;                                  // werden. Erhöhe a um eins und setze b zurück 
      }
      else                                        // Es kann kein Alkan mehr mit höchstens n C-Atomen konstruiert
        a=p;                                      // werden. Beende die Schleife.  
    }

    // Fall (ii)
    for(a=0;a<p;a++)                              // a ist ein Wurzelbaum der Höhe kleiner h
    {
      if(ausgeben==false)
        cout << "." << flush;                     // Fortschrittsanzeige
      if(1+L[a].size()+3*h<=n)                    // 1:             zusätzlicher Knoten
      {                                           // L[a].size():   Anzahl der Knoten von Wurzelbaum a
                                                  // 3*h:           die Wurzelbäume b, c und d besitzen die Höhe 
                                                  //                h, bestehen also aus mindestens h Knoten  
        for(b=p;b<L.size();b++)                   // b ist ein Wurzelbaum der Höhe h
          if(1+L[a].size()+3*L[b].size()<=n)      // 1:             zusätzlicher Knoten
          {                                       // L[a].size():   Anzahl der Knoten von Wurzelbaum a
                                                  // 3*L[b].size(): die Wurzelbäume c und d besitzten mindestens  
                                                  //                so viele Knoten wie der Wurzelbaum b
            for(c=b;c<L.size();c++)               // c ist ein Wurzelbaum der Höhe h
              if(1+L[a].size()+L[b].size()+2*L[c].size()<=n)
              {                                   // 1:             zusätzlicher Knoten
                                                  // L[a].size():   Anzahl der Knoten von Wurzelbaum a 
                                                  // L[b].size():   Anzahl der Knoten von Wurzelbaum b 
                                                  // 2*L[c].size(): der Wurzelbaum d besitzt mindestens so 
                                                  //                viele Knoten wie der Wurzelbaum c  
                for(d=c;d<L.size();d++)           // d ist ein Wurzelbaum der Höhe h  
                  if(1+L[a].size()+L[b].size()+L[c].size()+L[d].size()<=n)
                  {                               // 1:           zusätzlicher Knoten  
                                                  // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                  // L[b].size(): Anzahl der Knoten von Wurzelbaum b  
                                                  // L[c].size(): Anzahl der Knoten von Wurzelbaum c
                                                  // L[d].size(): Anzahl der Knoten von Wurzelbaum d  
                    Fuege_an(Alkane,L,a,b,c,d,ausgeben,ausgabe_bei_anzahl_c_atomen);
                                                  // Füge die Wurzelbäume a, b, c und d mit einem zusätzlichen Knoten
                  }                               // zusammen und hänge das entstehende Alkan an die Liste an
                  else                            // Es kann kein Alkan mehr mit höchstens n C-Atomen konstruiert
                    d=L.size();                   // werden. Erhöhe c um eins und setze d zurück
              }
              else                                // Es kann kein Alkan mehr mit höchstens n C-Atomen konstruiert 
                c=L.size();                       // werden. Erhöhe b um eins und setze c zurück 
          }
          else                                    // Es kann kein Alkan mehr mit höchstens n C-Atomen konstruiert
            b=L.size();                           // werden. Erhöhe a um eins und setze b zurück
      }
      else                                        // Es kann kein Alkan mehr mit höchstens n C-Atomen konstruiert 
        a=p;                                      // werden. Beende die Schleife. 
    }
    
    // Fall (iii)   
    for(a=p;a<L.size();a++)                       // a ist ein Wurzelbaum der Höhe h 
    {
      if(ausgeben==false)
        cout << "." << flush;                     // Fortschrittsanzeige
      if(1+L[a].size()+3*h<=n)                    // 1:             zusätzlicher Knoten
      {                                           // L[a].size():   Anzahl der Knoten von Wurzelbaum a
                                                  // 3*h:           die Wurzelbäume b, c und d besitzen die Höhe 
                                                  //                h, bestehen also aus mindestens h Knoten 
        for(b=a;b<L.size();b++)                   // b ist ein Wurzelbaum der Höhe h
          if(1+L[a].size()+3*L[b].size()<=n)      // 1:             zusätzlicher Knoten
          {                                       // L[a].size():   Anzahl der Knoten von Wurzelbaum a
                                                  // 3*L[b].size(): die Wurzelbäume c und d besitzten mindestens  
                                                  //                so viele Knoten wie der Wurzelbaum b 
            for(c=b;c<L.size();c++)               // c ist ein Wurzelbaum der Höhe h  
              if(1+L[a].size()+L[b].size()+2*L[c].size()<=n)
              {                                   // 1:             zusätzlicher Knoten
                                                  // L[a].size():   Anzahl der Knoten von Wurzelbaum a 
                                                  // L[b].size():   Anzahl der Knoten von Wurzelbaum b 
                                                  // 2*L[c].size(): der Wurzelbaum d besitzt mindestens so 
                                                  //                viele Knoten wie der Wurzelbaum c  
                for(d=c;d<L.size();d++)           // d ist ein Wurzelbaum der Höhe h 
                  if(1+L[a].size()+L[b].size()+L[c].size()+L[d].size()<=n)
                  {                               // 1:           zusätzlicher Knoten  
                                                  // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                  // L[b].size(): Anzahl der Knoten von Wurzelbaum b  
                                                  // L[c].size(): Anzahl der Knoten von Wurzelbaum c
                                                  // L[d].size(): Anzahl der Knoten von Wurzelbaum d 
                    Fuege_an(Alkane,L,a,b,c,d,ausgeben,ausgabe_bei_anzahl_c_atomen);  
                                                  // Füge die Wurzelbäume a, b, c und d mit einem zusätzlichen Knoten
                  }                               // zusammen und hänge das entstehende Alkan an die Liste an 
                  else                            // Es kann kein Alkan mehr mit höchstens n C-Atomen konstruiert
                    d=L.size();                   // werden. Erhöhe c um eins und setze d zurück
              }
              else                                // Es kann kein Alkan mehr mit höchstens n C-Atomen konstruiert
                c=L.size();                       // werden. Erhöhe b um eins und setze c zurück  
          }
          else                                    // Es kann kein Alkan mehr mit höchstens n C-Atomen konstruiert
            b=L.size();                           // werden. Erhöhe a um eins und setze b zurück
      }
      else                                        // Es kann kein Alkan mehr mit höchstens n C-Atomen konstruiert
        a=L.size();                               // werden. Beende die Schleife.  
    }
  }
  if(ausgeben==false) 
    cout << endl;                                 // Fortschrittsanzeiger
}


// ********************************************************************
// *                       Hauptprogramm                              *
// ******************************************************************** 

int main(void)
{
  unsigned int n;                           // maximale Anzahl der Knoten bzw C-Atome 
  bool ausgeben;                            // sollen die konstruierten Alkane auf dem Bildschirm ausgegeben werden
  int eingabe;                              // Variable zum Einlesen von Eingaben
  int ausgabe_bei_anzahl_c_atomen;          // geben nur ALkane mit dieser Anzahl an C-Atomen auf dem Bildschirm aus

  cout << "Konstruktion von Isomeren der Alkane (Summenformel: C_nH_2n+2)" << endl;
  cout << "==============================================================" << endl;
  cout << endl;
  cout << "Bitte gebe die maximale Anzahl der C-Atome ein: ";
  cin  >> n;
  cout << "Sollen die konstruierten Alkane auf dem Bildschirm ausgegeben werden? (Nein=0, Ja=1) ";
  cin  >> eingabe;
  if(eingabe==1)
    ausgeben=true;
  else
    ausgeben=false;
  if(ausgeben==true)
  {
    cout << "Gebe bitte die Anzahl der C-Atome an, bei denen die konstruierten Alkane ausgegeben werden sollen. ";
    cout << "Möchtest du alle Alkane auf dem Bildschirm ausgegeben haben, so gebe bitte -1 ein. ";
    cin  >> ausgabe_bei_anzahl_c_atomen;
  }

  vector<Alkan> Alkane;                     // Liste von Alkanen 
  vector< vector <unsigned char> > L;       // Liste von Wurzelbäumen

  // Konstruiere eine vollständige Liste von Wurzelbäumen und damit alle ALkane aus bis zu n C-Atomen
  konstruiere_Wurzelbaeume(Alkane,L,n,ausgeben,ausgabe_bei_anzahl_c_atomen);     
  

  // Zur Kontrolle gebe die Anzahl der Alkane aus
  cout << endl;
  vector<long> anz;
  unsigned int i;
  unsigned int sp;
  anz.resize(n+1);
  for(i=0;i<=n;i++)
    anz[i]=0;
  for(i=0;i<Alkane.size();i++)
  {
    sp=Alkane[i].Wurzelbaum_1.size()+Alkane[i].Wurzelbaum_2.size();
    if(Alkane[i].laengste_gerade_Kette_ist_gerade==true)
      anz[sp]++;
    else
      anz[sp+Alkane[i].Wurzelbaum_3.size()+Alkane[i].Wurzelbaum_4.size()+1]++;
  }
  cout << "Anzahl der Alkane mit n C-Atomen" << endl;
  cout << "--------------------------------" << endl;
  for(i=0;i<=n;i++)
  {
    cout << i;
    if(i<10 && n>=10) cout << " ";
    cout << " : " << anz[i] << endl;
  }
  return 0;
};

// ****************************************************************************
// *              Subroutinen zur Ausgabe eines Alkans                        *
// ****************************************************************************

void umnummerieren(vector<unsigned char>& b,vector<unsigned char>& wurzel_baum); // Nummeriert einen Wurzelbaum um
// Platziere ein Alkan aus 2 Wurzelbäumen in "Ascii-Art"
void ascii_platzierung_alkan_aus_2_wurzelbaeumen(ascii_platzierung& p1,ascii_platzierung& p2,ascii_platzierung& p); 
// Platziere ein Alkan aus 4 Wurzelbäumen in "Ascii-Art"
void ascii_platzierung_alkan_aus_4_wurzelbaeumen(ascii_platzierung& p1,ascii_platzierung& p2,
     ascii_platzierung& p3,ascii_platzierung& p4,ascii_platzierung& p);
// Platziere einen Wurzelbaum in "Ascii-Art"
void bestimme_ascii_platzierung(vector<unsigned char>& b,ascii_platzierung& p);
inline void swap(unsigned int& a,unsigned int& b);                               // Vertauscht zwei Werte
// Fügt Teilplatzierungen eines Wurzelbaums zu einer größeren Teilplatzierung zusammen
void bestimme_platzierung(const int& i,vector<unsigned int>& sohn,vector<ascii_platzierung>& platzierungen);
// konvertiert ein Alkan, so dass es auf dem Bildschirm ausgegeben werden kann
int konvertiere_alkan_fuer_ausgabe(Alkan& Alk);

void umnummerieren(vector<unsigned char>& b,vector<unsigned char>& wurzel_baum)
{
  // b:            Wurzelbaum
  // wurzel_baum:  Adresse für einen Wurzelbaum
  //
  // Funktion:
  // ---------
  // Der Wurzelbaum b soll umnummeriert und in wurzel_baum gespeichert werden. Die
  // Umnummerierung soll folgende Bedingungn erfüllen:
  // (i)   die Wurzel bekommt die Nummer 0
  // (ii)  bei zwei Knoten liegt derjenige mit der kleineren Nummer entweder in der
  //       selben oder einer niedrigeren Ebene
  // (iii) alle Knoten, die einen Knoten v als Vater haben besitzen die Nummerierungen
  //       n,n+1,...,n+i, folgen also direkt aufeinander 

  unsigned int i,j,k;                      // Schleifenvariablen
  unsigned int n;                          // aktuell zu vergebene Nummer
  unsigned int anzahl_ebenen;              // Anzahl der Ebenen 
  vector<unsigned char> ebene;             // Vektor für die Ebene der Knoten 
  vector<unsigned char> perm;              // Vektor zur Speicherung der Umnummerierung
  vector<unsigned char> inv_perm;          // Vektor zur Speicherung der Inverse der Umnummerierung      
  wurzel_baum.resize(b.size());            // der umnummerierte Wurzelbaum enthält natürlich genauso viele 
                                           // Knoten wie der alte Wurzelbaum
  ebene.resize(b.size());                  // Jeder Knoten besitzt eine Ebene
  perm.resize(b.size()); 
  inv_perm.resize(b.size());
  for(i=0;i<ebene.size();i++)
    ebene[i]=0;
  for(i=0;i<b.size();i++)                  // Bestimme die Ebene der Knoten
    for(j=0;j<b.size();j++)
      if(b[j]!=j)
        ebene[j]=ebene[b[j]]+1;
   anzahl_ebenen=0;
   
  for(i=0;i<ebene.size();i++)              // Bestimme die Anzahl der Ebenen 
    if(ebene[i]>anzahl_ebenen)
      anzahl_ebenen=ebene[i];   
  if(b.size()>0) anzahl_ebenen++;

  for(i=0;i<ebene.size();i++)               // gebe der Wurzel die Nummer 0
    if(ebene[i]==0) 
    {
      perm[i]=0;
      break;
    }

  n=1;                                      // aktuell noch nicht veregebene Knotennummer  
  for(i=0;i+1<anzahl_ebenen;i++)            // durchlaufe die Ebenen von unten nach oben         
    for(j=0;j<ebene.size();j++)             // durchlaufe alle Knoten
    {
      if(ebene[j]==i)                       // Falls der Knoten in der gerade betrachteten Ebene liegt
        for(k=0;k<b.size();k++)             // durchlaufe alle Knoten
          if(b[k]==j && k!=j)               // falls k ein Sohn von j ist 
          {
            perm[k]=n;                      // der Knoten k heißt jetzt n
            n++;                            // der Name n wurde eben vergeben, ist also nicht mehr frei
          } 
    }
  for(i=0;i<perm.size();i++)               // Bestimme die inverse Permutation
    inv_perm[perm[i]]=i;

  for(i=0;i<wurzel_baum.size();i++)        // nummeriere den Wurzelbaum tatsächlich um
    wurzel_baum[i]=perm[b[inv_perm[i]]];
} 

int konvertiere_alkan_fuer_ausgabe(Alkan& Alk)
{
  // Alk: ein Alkan
  //
  // Funktion
  // --------
  // Die Funktion ausgabe(Alk) setzt vorraus, das Alk.Wurzelbaum_3 und Alk.Wurzelbaum_4 keine leeren 
  // Wurzelbäume sind. Diese Funktion kopiert u.U. Wurzelbäume um,so dass diese Bedingung erfüllt ist.

  if(Alk.laengste_gerade_Kette_ist_gerade==true) return 0;     // Alkan braucht nicht modifiziert werden

  vector<unsigned char> tmp;   // temporäres Array zum Zwischenspeichern eines Wurzelbaums
  unsigned int i;              // Schleifenvariable

  if(Alk.Wurzelbaum_3.size()>0 && Alk.Wurzelbaum_4.size()>0)
    return 0;                                                  // Es ist bereits alles in Ordnung
  if(Alk.Wurzelbaum_3.size()==0 && Alk.Wurzelbaum_4.size()==0) // beide Wurzelbäume müssen umkopiert werden
  {
    if(Alk.Wurzelbaum_1.size()>0 && Alk.Wurzelbaum_2.size()>0)
    {
      // Vertausche Wurzelbaum 1 und Wurzelbaum 3
      tmp.resize(Alk.Wurzelbaum_1.size());
      for(i=0;i<Alk.Wurzelbaum_1.size();i++)
        tmp[i]=Alk.Wurzelbaum_1[i];
      Alk.Wurzelbaum_1.resize(Alk.Wurzelbaum_3.size());
      for(i=0;i<Alk.Wurzelbaum_1.size();i++)
        Alk.Wurzelbaum_1[i]=Alk.Wurzelbaum_3[i];
      Alk.Wurzelbaum_3.resize(tmp.size());
      for(i=0;i<Alk.Wurzelbaum_3.size();i++)
        Alk.Wurzelbaum_3[i]=tmp[i];
      // Vertausche Wurzelbaum 2 und Wurzelbaum 4
      tmp.resize(Alk.Wurzelbaum_2.size());
      for(i=0;i<Alk.Wurzelbaum_2.size();i++)
        tmp[i]=Alk.Wurzelbaum_2[i];
      Alk.Wurzelbaum_2.resize(Alk.Wurzelbaum_4.size());
      for(i=0;i<Alk.Wurzelbaum_2.size();i++)
        Alk.Wurzelbaum_2[i]=Alk.Wurzelbaum_4[i];
      Alk.Wurzelbaum_4.resize(tmp.size());
      for(i=0;i<Alk.Wurzelbaum_4.size();i++)
        Alk.Wurzelbaum_4[i]=tmp[i];
      return 0;   // die Modifikation des Alkans war erfolgreich      
    }
    else          // Es gibt keine 2 nicht leeren Wurzelbäume im Alkan
    {
      return -1;  // Modifikation des Alkans ist nicht möglich
    }
  }
  if(Alk.Wurzelbaum_3.size()==0)
  {
    if(Alk.Wurzelbaum_1.size()>0)
    {
      // Vertausche Wurzelbaum 1 mit Wurzelbaum 3
      tmp.resize(Alk.Wurzelbaum_1.size());
      for(i=0;i<Alk.Wurzelbaum_1.size();i++)
        tmp[i]=Alk.Wurzelbaum_1[i];
      Alk.Wurzelbaum_1.resize(Alk.Wurzelbaum_3.size());
      for(i=0;i<Alk.Wurzelbaum_1.size();i++)
        Alk.Wurzelbaum_1[i]=Alk.Wurzelbaum_3[i];
      Alk.Wurzelbaum_3.resize(tmp.size());
      for(i=0;i<Alk.Wurzelbaum_3.size();i++)
        Alk.Wurzelbaum_3[i]=tmp[i];
      return 0;   // die Modifikation des Alkans war erfolgreich
    }
    if(Alk.Wurzelbaum_2.size()>0)
    {
      // Vertausche Wurzelbaum 2 mit Wurzelbaum 3
      tmp.resize(Alk.Wurzelbaum_2.size());
      for(i=0;i<Alk.Wurzelbaum_2.size();i++)
        tmp[i]=Alk.Wurzelbaum_2[i];
      Alk.Wurzelbaum_2.resize(Alk.Wurzelbaum_3.size());
      for(i=0;i<Alk.Wurzelbaum_2.size();i++)
        Alk.Wurzelbaum_2[i]=Alk.Wurzelbaum_3[i];
      Alk.Wurzelbaum_3.resize(tmp.size());
      for(i=0;i<Alk.Wurzelbaum_3.size();i++)
        Alk.Wurzelbaum_3[i]=tmp[i]; 
      return 0;   // die Modifikation des Alkans war erfolgreich  
    }
    return -1;    // die Modifikation des Alkans ist nicht möglich 
  }
  if(Alk.Wurzelbaum_4.size()==0)
  {
    if(Alk.Wurzelbaum_1.size()>0)
    {
      // Vertausche Wurzelbaum 1 mit Wurzelbaum 4
      tmp.resize(Alk.Wurzelbaum_1.size());
      for(i=0;i<Alk.Wurzelbaum_1.size();i++)
        tmp[i]=Alk.Wurzelbaum_1[i];
      Alk.Wurzelbaum_1.resize(Alk.Wurzelbaum_4.size());
      for(i=0;i<Alk.Wurzelbaum_1.size();i++)
        Alk.Wurzelbaum_1[i]=Alk.Wurzelbaum_4[i];
      Alk.Wurzelbaum_4.resize(tmp.size());
      for(i=0;i<Alk.Wurzelbaum_4.size();i++)
        Alk.Wurzelbaum_4[i]=tmp[i];
      return 0;   // die Modifikation des Alkans war erfolgreich
    }
    if(Alk.Wurzelbaum_2.size()>0)
    {
      // Vertausche Wurzelbaum 2 mit Wurzelbaum 4
      tmp.resize(Alk.Wurzelbaum_2.size());
      for(i=0;i<Alk.Wurzelbaum_2.size();i++)
        tmp[i]=Alk.Wurzelbaum_2[i];
      Alk.Wurzelbaum_2.resize(Alk.Wurzelbaum_4.size());
      for(i=0;i<Alk.Wurzelbaum_2.size();i++)
        Alk.Wurzelbaum_2[i]=Alk.Wurzelbaum_4[i];
      Alk.Wurzelbaum_4.resize(tmp.size());
      for(i=0;i<Alk.Wurzelbaum_4.size();i++)
        Alk.Wurzelbaum_4[i]=tmp[i]; 
      return 0;   // die Modifikation des Alkans war erfolgreich  
    }
    return -1;    // die Modifikation des Alkans ist nicht möglich 
  }
  return -1;      // dieser Fall kann sowieso nicht vorkommen
}

int ausgabe(Alkan& A)
{
  // A: ein Alkan
  //
  // Funktion:
  // ---------
  // Gibt eine zweidimensionale Platzierung des Alkanmoleküls A in "Ascii-Art" 
  // auf dem Bildschirm aus, sofern es nicht zu breit ist (-> wird -1 zurückgegeben 

  ascii_platzierung p1;                    // Speicherplatz für eine Platzierung des ersten Wurzelbaums in "Ascii-Art"
  ascii_platzierung p2;                    // Speicherplatz für eine Platzierung des zweiten Wurzelbaums in "Ascii-Art"
  ascii_platzierung p3;                    // Speicherplatz für eine Platzierung des dritten Wurzelbaums in "Ascii-Art"
  ascii_platzierung p4;                    // Speicherplatz für eine Platzierung des vierten Wurzelbaums in "Ascii-Art" 
  ascii_platzierung p;                     // Speicherplatz für eine Platzierung des gesamten Alkans in "Ascii-Art"  

  vector<unsigned char> wurzel_baum;       // Speicherplatz für einen umnummerierten Wurzelbaum
  unsigned int i,j;                        // Schleifenvariablen
  
  konvertiere_alkan_fuer_ausgabe(A);              // modifiziere evtl. die Anordnung der Wurzelbäume

  umnummerieren(A.Wurzelbaum_1,wurzel_baum);      // nummeriere den ersten Wurzelbaum um
  bestimme_ascii_platzierung(wurzel_baum,p1);     // platziere den ersten Wurzelbaum in "Ascii-Art" 
  umnummerieren(A.Wurzelbaum_2,wurzel_baum);      // nummeriere den zweiten Wurzelbaum um
  bestimme_ascii_platzierung(wurzel_baum,p2);     // platziere den zweiten Wurzelbaum in "Ascii-Art" 
  if(A.laengste_gerade_Kette_ist_gerade==false)   // falls das Alkan aus 4 Wurzelbäumen besteht
  {
    umnummerieren(A.Wurzelbaum_3,wurzel_baum);    // nummeriere den dritten Wurzelbaum um
    bestimme_ascii_platzierung(wurzel_baum,p3);   // platziere den dritten Wurzelbaum in "Ascii-Art" 
    umnummerieren(A.Wurzelbaum_4,wurzel_baum);    // nummeriere den vierten Wurzelbaum um  
    bestimme_ascii_platzierung(wurzel_baum,p4);   // platziere den vierten Wurzelbaum in "Ascii-Art"
  }
  
  if(A.laengste_gerade_Kette_ist_gerade==true)    // das Alkan besteht aus 2 Wurzelbäumen
  {
    ascii_platzierung_alkan_aus_2_wurzelbaeumen(p1,p2,p);  // platziere das gesamte Alkan
    if(p.anzahl_spalten>BILDSCHIRMBREITE)         // Zu breit für eine Ausgabe auf dem Bildschirm
    {
      cout << "Das Alkan ist leider zu breit für eine Ausgabe auf dem Bildschirm." << endl << endl;
      return -1;
    } 
    for(j=0;j<p.anzahl_zeilen;j++)                // durchlaufe die Zeilen der Platzierung 
    {
      for(i=0;i<p.anzahl_spalten;i++)             // durchlaufe die Spalten der Platzierung
        cout << p.Matrix[i][j];                   // gebe das berechnete Symbol auf dem Bildschirm aus 
      cout << endl;
    }
    cout << endl; 
  }
  else                                            // das Alkan besteht aus 4 Wurzelbäumen
  {
    ascii_platzierung_alkan_aus_4_wurzelbaeumen(p1,p2,p3,p4,p); // platziere das gesamte Alkan
    if(p.anzahl_spalten>BILDSCHIRMBREITE)         // Zu breit für eine Ausgabe auf dem Bildschirm
    {
      cout << "Das Alkan ist leider zu breit für eine Ausgabe auf dem Bildschirm." << endl << endl;
      return -1;
    } 
    for(j=0;j<p.anzahl_zeilen;j++)                // durchlaufe die Zeilen der Platzierung
    {
      for(i=0;i<p.anzahl_spalten;i++)             // durchlaufe die Spalten der Platzierung
        cout << p.Matrix[i][j];                   // gebe das berechnete Symbol auf dem Bildschirm aus 
      cout << endl;
    }
    cout << endl;
  }

  //char taste;
  //cout << "Bitte beliebige Taste drücken: ";
  //cin >> taste;
  
  return 0;
}

void ascii_platzierung_alkan_aus_2_wurzelbaeumen(ascii_platzierung& p1,ascii_platzierung& p2,ascii_platzierung& p)
{
  // p1: Platzierung eines Wurzelbaums in "Ascii-Art"
  // p2: Platzierung eines Wurzelbaums in "Ascii-Art"
  // p:  Speicherplatz für Platzierung eines Alkans in "Ascii-Art"
  //
  // Funktion:
  // ---------
  // Kombiniert die zweidimensionalen Platzierungen p1 und p2 zweier Wurzelbäume zu einer 
  // Platzierung des Alkans aus diesen beiden Wurzelbäumen in "Ascii-Art"
  
  unsigned int i,j;   // Schleifenvariablen
  unsigned int m;     // Position der neuen Wurzel

  m=max(p1.position_wurzel,p2.position_wurzel);

  p.anzahl_zeilen=p1.anzahl_zeilen+1+p2.anzahl_zeilen; // das Alkan entsteht durch das aneinander setzen 
                                                       // dieser 2 Wurzelbäume
  p.anzahl_spalten=m+1;                                // benötigter Platz links des Zentrums + Platz für
                                                       // das Zentrum 
  p.anzahl_spalten+=max(p1.anzahl_spalten-1-p1.position_wurzel,p2.anzahl_spalten-1-p2.position_wurzel); // Platz rechts 
                                                       // des Zentrums
  p.position_wurzel=max(p1.position_wurzel,p2.position_wurzel); // Position der neuen "Wurzel"
  p.Matrix.resize(p.anzahl_spalten);                   // Reserviere Speicherplatz
  for(i=0;i<p.anzahl_spalten;i++)                      // durchlaufe die Spalten der Platzierung  
  {
    p.Matrix[i].resize(p.anzahl_zeilen);               // Reserviere Speicherplatz
    for(j=0;j<p.anzahl_zeilen;j++)                     // durchlaufe die Zeilen der Platzierung  
      p.Matrix[i][j]=' ';                              // beginne zunächst mit einem leeren Bildschirm
  }
  for(i=p1.anzahl_zeilen;i>0;i--)                      // durchlaufe die Zeilen des ersten Wurzelbaums
    for(j=0;j<p1.anzahl_spalten;j++)                   // durchlaufe die Spalten des ersten Wurzelbaums  
      p.Matrix[j+m-p1.position_wurzel][p1.anzahl_zeilen-i]=p1.Matrix[j][i-1]; // übernehme die Teilplatzierung
  p.Matrix[p.position_wurzel][p1.anzahl_zeilen]='|';   // gebe Symbol für das Zentrum aus
  for(i=0;i<p2.anzahl_zeilen;i++)                      // durchlaufe die Zeilen des zweiten Wurzelbaums
    for(j=0;j<p2.anzahl_spalten;j++)                   // durchlaufe die Spalten des zweiten Wurzelbaums
      switch(p2.Matrix[j][i])                          // übersetze die Symbole der Teilplatzierung  
      {
        case (char)92: p.Matrix[j+m-p2.position_wurzel][p1.anzahl_zeilen+1+i]='/';      // Orientierung ändert sich 
                       break;
        case '/':      p.Matrix[j+m-p2.position_wurzel][p1.anzahl_zeilen+1+i]=(char)92; // Orientierung ändert sich
                       break; 
        default:       p.Matrix[j+m-p2.position_wurzel][p1.anzahl_zeilen+1+i]=p2.Matrix[j][i];
      }
}

void ascii_platzierung_alkan_aus_4_wurzelbaeumen(ascii_platzierung& p1,ascii_platzierung& p2,
     ascii_platzierung& p3,ascii_platzierung& p4,ascii_platzierung& p)
{
  // p1: Platzierung eines Wurzelbaums in "Ascii-Art"
  // p2: Platzierung eines Wurzelbaums in "Ascii-Art"
  // p3: Platzierung eines Wurzelbaums in "Ascii-Art"
  // p4: Platzierung eines Wurzelbaums in "Ascii-Art"
  // p:  Speicherplatz für Platzierung eines Alkans in "Ascii-Art"
  //
  // Funktion:
  // ---------
  // Kombiniert die zweidimensionalen Platzierungen p1, p2, p3 und p4 von Wurzelbäume zu einer 
  // Platzierung des Alkans aus diesen Wurzelbäumen in "Ascii-Art"
  //
  //        p3
  //         |
  //      p1-o-p2
  //         |
  //        p4

  unsigned int i,j;                    // Schleifenvariablen
  unsigned int w1,w2,w3,w4;            // Abkürzungen für die Positionen der 4 Wurzeln
  unsigned int s1,s2,s3,s4;            // Abkürzungen für die 4 Anzahlen der Zeilen 
  unsigned int z1,z2,z3,z4;            // Abkürzungen für die 4 Anzahlen der Zeilen  
  w1=p1.position_wurzel;               // Position der Wurzel der Platzierung des ersten Wurzelbaums 
  w2=p2.position_wurzel;               // Position der Wurzel der Platzierung des zweiten Wurzelbaums
  w3=p3.position_wurzel;               // Position der Wurzel der Platzierung des dritten Wurzelbaums
  w4=p4.position_wurzel;               // Position der Wurzel der Platzierung des vierten Wurzelbaums
  s1=p1.anzahl_spalten;                // Anzahl der Spalten der Platzierung des ersten Wurzelbaums
  s2=p2.anzahl_spalten;                // Anzahl der Spalten der Platzierung des zweiten Wurzelbaums 
  s3=p3.anzahl_spalten;                // Anzahl der Spalten der Platzierung des dritten Wurzelbaums
  s4=p4.anzahl_spalten;                // Anzahl der Spalten der Platzierung des vierten Wurzelbaums
  z1=p1.anzahl_zeilen;                 // Anzahl der Zeilen der Platzierung des ersten Wurzelbaums
  z2=p2.anzahl_zeilen;                 // Anzahl der Zeilen der Platzierung des zweiten Wurzelbaums
  z3=p3.anzahl_zeilen;                 // Anzahl der Zeilen der Platzierung des dritten Wurzelbaums
  z4=p4.anzahl_zeilen;                 // Anzahl der Zeilen der Platzierung des vierten Wurzelbaums

  unsigned int links,rechts,oben,unten;         // Platzbedarf links,rechts,oberhalb und unterhalb des zusätzlichen Knotens

  oben=z3+1;                                    // Die Platzierung des dritten Wurzelbaums muß über 
                                                // den zusätzlichen Knoten passen, inklusive "|"
  if(s1-w1-1>oben && s1+w1>0) oben=s1-w1-1;     // ein Teil der Platzierung des ersten Wurzelbaums reicht nach oben
  if(s2-w2-1>oben && s2+w2>0) oben=s2-w2-1;     // ein Teil der Platzierung des zweiten Wurzelbaums reicht nach oben
  unten=z4+1;                                   // Die Platzierung des dritten Wurzelbaums muß über 
                                                // den zusätzlichen Knoten passen, inklusive "|" 
  if(w1>unten) unten=w1;                        // ein Teil der Platzierung des ersten Wurzelbaums reicht nach unten
  if(w2>unten) unten=w2;                        // ein Teil der Platzierung des zweiten Wurzelbaums reicht nach unten
  links=0;
  if(z1>0) links=z1+1;                          // falls der ersten Wurzelbaum aus mindestens einem Knoten besteht, 
                                                // benötigt er Platz + "-"
  if(w3>links) links=w3;                        // ein Teil der Platzierung des dritten Wurzelbaums reicht nach links
  if(w4>links) links=w4;                        // ein Teil der Platzierung des vierten Wurzelbaums reicht nach links 
  rechts=0;
  if(z2>0) rechts=z2+1;                         // falls der zweite Wurzelbaum aus mindestens einem Knoten besteht, 
                                                // benötigt er Platz + "-" 
  if(s3-w3-1>rechts) rechts=s3-w3-1;            // ein Teil der Platzierung des dritten Wurzelbaums reicht nach rechts
  if(s4-w4-1>rechts) rechts=s4-w4-1;            // ein Teil der Platzierung des vierten Wurzelbaums reicht nach rechts

  p.anzahl_zeilen=unten+1+oben;                 // Teil oberhalb des zusätzlichen Knoten + zusätzlicher Knoten + Teil 
                                                // unterhalb des zusätzlichen Knotens  
  p.anzahl_spalten=links+1+rechts;              // Teil links vom zusätzlichen Knoten + zusätzlicher Knoten + Teil   
                                                // rechts vom zusätzlichen Knoten
  p.position_wurzel=links;

  p.Matrix.resize(p.anzahl_spalten);            // Reserviere Speicherplatz 
  for(i=0;i<p.anzahl_spalten;i++)               // durchlaufe die Spalten der Platzierung   
  { 
    p.Matrix[i].resize(p.anzahl_zeilen);        // Reserviere Speicherplatz  
    for(j=0;j<p.anzahl_zeilen;j++)              // durchlaufe die Zeilen der Platzierung  
      p.Matrix[i][j]=' ';                       // starte mit einem leeren Bildschirm
  }
  for(i=z3;i>0;i--)                             // durchlaufe die Zeilen der Platzierung des dritten Wurzelbaums
    for(j=0;j<s3;j++)                           // durchlaufe die Spalten der Platzierung des dritten Wurzelbaums 
      p.Matrix[links-w3+j][(oben-z3-1)+(z3-i)]=p3.Matrix[j][i-1];
  p.Matrix[p.position_wurzel][oben-1]='|';
  p.Matrix[p.position_wurzel][oben]='o';
  p.Matrix[p.position_wurzel][oben+1]='|';
  for(i=0;i<z4;i++)                             // durchlaufe die Zeilen der Platzierung des vierten Wurzelbaums
    for(j=0;j<s4;j++)                           // durchlaufe die Spalten der Platzierung des vierten Wurzelbaums  
      switch(p4.Matrix[j][i])                   // übersetze die Symbole der Teilplatzierung
      {
        case '|':      p.Matrix[links-w4+j][oben+2+i]='|';       // immer noch hochkant
                       break;
        case '/':      p.Matrix[links-w4+j][oben+2+i]=(char)92;  // Orientierung ändert sich
                       break; 
        case (char)92: p.Matrix[links-w4+j][oben+2+i]='/';       // Orientierung ändert sich 
                       break; 
        case 'o':      p.Matrix[links-w4+j][oben+2+i]='o';       // ein Knoten bleibt ein Knoten
                       break;
      } 
  if(p1.anzahl_zeilen>0)
  {
    p.Matrix[p.position_wurzel-1][oben]='-';
    for(i=0;i<z1;i++)                           // durchlaufe die Zeilen der Platzierung des ersten Wurzelbaums
      for(j=0;j<s1;j++)                         // durchlaufe die Spalten der Platzierung des ersten Wurzelbaums
        switch(p1.Matrix[j][i])                 // übersetze die Symbole der Teilplatzierung
        {
          case '|':      p.Matrix[(links-z1-1)+(z1-i-1)][oben+w1+(-j)]='-';       // Orientierung ändert sich
                         break;
          case 'o':      p.Matrix[(links-z1-1)+(z1-i-1)][oben+w1+(-j)]='o';       // ein Knoten bleibt ein Knoten  
                         break;
          case '/':      p.Matrix[(links-z1-1)+(z1-i-1)][oben+w1+(-j)]=(char)92;  // Orientierung ändert sich
                         break;
          case (char)92: p.Matrix[(links-z1-1)+(z1-i-1)][oben+w1+(-j)]='/';       // Orientierung ändert sich
                         break;  
        } 
  }
  if(p2.anzahl_zeilen>0)
  {
    p.Matrix[p.position_wurzel+1][oben]='-';
    for(i=0;i<z2;i++)                           // durchlaufe die Zeilen der Platzierung des zweiten Wurzelbaums
      for(j=0;j<s2;j++)                         // durchlaufe die Spalten der Platzierung des zweiten Wurzelbaums
        switch(p2.Matrix[j][i])                 // übersetze die Symbole der Teilplatzierung 
        {
          case '|':      p.Matrix[p.position_wurzel+2+i][oben+w2+(-j)]='-';         // Orientierung ändert sich
                         break;
          case 'o':      p.Matrix[p.position_wurzel+2+i][oben+w2+(-j)]='o';         // ein Knoten bleibt ein Knoten
                         break;
          case '/':      p.Matrix[p.position_wurzel+2+i][oben+w2+(-j)]='/';
                         break;
          case (char)92: p.Matrix[p.position_wurzel+2+i][oben+w2+(-j)]=(char)92;
                         break;  
        }
  }
     
}

void bestimme_ascii_platzierung(vector<unsigned char>& b,ascii_platzierung& p)
{
  // b: ein Wurzelbaum, der bereits mit der Funktion "umnummerieren" bearbeitet wurde
  // p: Speicherplatz für eine Platzierung eines Wurzelbaums in "Ascii-Art"
  //
  // Funktion:
  // ---------
  // Platziert einen Wurzelbaum in "Ascii-Art" und speichert das Ergebnis in p.

  unsigned int i,j;                        // Schleifenvariablen
  unsigned int pos;                        // Position des zu bearbeitenden Knotens
  vector<unsigned char> ebene;             // Vektor für die Ebene der Knoten 
  ebene.resize(b.size());                  // Jeder Knoten besitzt eine Ebene
  vector<ascii_platzierung> platzierungen; // eine Liste von einzelnen Platzierungen, man kann jeden 
  platzierungen.resize(b.size());          // Knoten einzeln platzieren und die einzelnen Platzierungen
                                           // dann Stück für Stück zu größeren Platzierungen zusammenfassen
  vector<unsigned int> sohn;               // Liste von Söhnen eines Knotens

  for(i=0;i<ebene.size();i++)
    ebene[i]=0;
  for(i=0;i<b.size();i++)                  // Bestimme die Ebene der Knoten
    for(j=0;j<b.size();j++)
      if(b[j]!=j)
        ebene[j]=ebene[b[j]]+1;

  pos=0;
  for(i=b.size();i>0;i--)                    // durchlaufe die Knoten der obersten Ebene
    if(ebene[i-1]==ebene[b.size()-1])        // falls der Knoten noch zur obersten Ebene gehört
    {
      platzierungen[i-1].anzahl_zeilen=1;    // belege Matrix der Platzierung mit dem Symbol 
      platzierungen[i-1].anzahl_spalten=1;   // für einen Knoten: "o"
      platzierungen[i-1].position_wurzel=0;
      platzierungen[i-1].Matrix.resize(1);
      platzierungen[i-1].Matrix[0].resize(1);
      platzierungen[i-1].Matrix[0][0]='o';
    } 
    else
    {
      pos=i-1;                             // merke den Start der neuen Ebene  
      break;                               // oberste Ebene ist durchlaufen
    }
  
  if(b.size()>0) pos++;
  if(b.size()==1) pos=0;
  while(pos>0)                             // solange es noch Knoten zu bearbeiten gibt
  {
    for(i=pos;i>0;i--)                     // durchlaufe die Knoten der aktuell zu bearbeitenden Ebene
    { 
      if(ebene[i-1]==ebene[pos-1])         // falls der Knoten noch zu dieser Ebene gehört
      {
        sohn.resize(0);                    // bestimme die Liste der Söhne des Knoten i
        for(j=pos;j<b.size();j++)
        {
          if(ebene[j]>ebene[pos])          // es können keine Söhne mehr kommen, da die Knoten der Ebene  
          {                                // nach ansteigend sortiert sind
            break;
          }
          else
          {
           if((unsigned int)b[j]==i-1)
            {
              sohn.resize(sohn.size()+1);  // es gibt einen Sohn mehr
              sohn[sohn.size()-1]=j;       // speichere die Nummer des Sohns 
            }               
          }
        }
        bestimme_platzierung(i-1,sohn,platzierungen);  // Füge die Platzierungen der Söhne zu einer Platzierung vom
                                                       // Knoten i-1 zusammen
        if(i==1) pos=0;                                // Abbruchbedingung
      }
      else
      {
        pos=i;                             // merke den Start der neuen Ebene    
        break;                             // aktuelle Ebene ist bereits fertig durchlaufen  
      }   
    }
  }
  
  if(b.size()>0)                           // kopiere die Platzierung der Wurzel
  {
    p.anzahl_zeilen=platzierungen[0].anzahl_zeilen;
    p.anzahl_spalten=platzierungen[0].anzahl_spalten;
    p.position_wurzel=platzierungen[0].position_wurzel;
    p.Matrix.resize(p.anzahl_spalten);
    for(i=0;i<p.anzahl_spalten;i++)
    {
      p.Matrix[i].resize(p.anzahl_zeilen);
      for(j=0;j<p.anzahl_zeilen;j++)
        p.Matrix[i][j]=platzierungen[0].Matrix[i][j];
    } 
  }
  else                      // leere Platzierung
  {
    p.anzahl_zeilen=0;
    p.anzahl_spalten=0;
    p.position_wurzel=0;
    p.Matrix.resize(0);
  }
 
}

inline void swap(unsigned int& a,unsigned int& b)
{
  // Vertauscht zwei Werte
  unsigned int c=a;
  a=b;
  b=c;
}

void bestimme_platzierung(const int& i,vector<unsigned int>& sohn,vector<ascii_platzierung>& platzierungen)
{
  // i:             Nummer eines Knotens
  // sohn:          Liste der Nummern der Söhne von i
  // platzierungen: Liste von Platzierungen der Wurzelbäume oberhalb eines Knotens
  //
  // Funktion:
  // ---------
  // Es wird die Platzierung des Wurzelbaums oberhalb eines Knotens i bestimmt. Hierzu werden die 
  // bereits vorhanden derartigen Platzierungen der Söhne von i benutzt.

  unsigned int h,j;                  // Schleifenvariablen
  unsigned int links,mitte,rechts;   // Welcher der 3 Teilwurzelbäume kommt in die Mitte, nach links oder nach rechts?     
  unsigned int s1,s2,s3;             // Abkürzung für die Anzahl der Spalten der Platzierung der 3 Teilwurzelbäume 
  unsigned int w1,w2,w3;             // Abkürzung für die Position der Wurzel der Platzierung der 3 Teilwurzelbäume  
  unsigned int z1,z2,z3;             // Abkürzung für die Anzahl der Zeilen der Platzierung der 3 Teilwurzelbäume 
  unsigned int a;

  switch(sohn.size())                // Wieviele Teilwurzelbäume müssen hängen amKnoten i
  {
    // Es hängt kein Teilwurzelbaum am Knoten i, also platziere nur den Knoten i
    case 0: platzierungen[i].anzahl_zeilen=1;    // belege Matrix der Platzierung mit dem Symbol 
            platzierungen[i].anzahl_spalten=1;   // für einen Knoten: "o"
            platzierungen[i].position_wurzel=0;
            platzierungen[i].Matrix.resize(1);
            platzierungen[i].Matrix[0].resize(1);
            platzierungen[i].Matrix[0][0]='o';
            break;
    // Es hängt ein Teilwurzelbaum am Knoten i, also hänge die Platzierung an "o-" an
    case 1: platzierungen[i].anzahl_zeilen=platzierungen[sohn[0]].anzahl_zeilen+2;   // 2 höher, wegen "o-" 
            platzierungen[i].anzahl_spalten=platzierungen[sohn[0]].anzahl_spalten;   // genauso breit
            platzierungen[i].position_wurzel=platzierungen[sohn[0]].position_wurzel; // selbe Position der Wurzel
            platzierungen[i].Matrix.resize(platzierungen[i].anzahl_spalten);         // Reserviere Speicherplatz  
            for(j=0;j<platzierungen[i].anzahl_spalten;j++)                             
              platzierungen[i].Matrix[j].resize(platzierungen[i].anzahl_zeilen);     // Reserviere Speicherplatz
            for(j=0;j<platzierungen[sohn[0]].anzahl_spalten;j++)                     // Übernehme die Platzierung 
              for(h=0;h<platzierungen[sohn[0]].anzahl_zeilen;h++)                    // vomTeilwurzelbaum
              {
                platzierungen[i].Matrix[j][h+2]=platzierungen[sohn[0]].Matrix[j][h];
              }
            for(j=0;j<platzierungen[sohn[0]].anzahl_spalten;j++)                     // Zwei Leerzeilen
            {
              platzierungen[i].Matrix[j][0]=' ';
              platzierungen[i].Matrix[j][1]=' '; 
            }
            platzierungen[i].Matrix[platzierungen[sohn[0]].position_wurzel][0]='o';  // Knoten i
            platzierungen[i].Matrix[platzierungen[sohn[0]].position_wurzel][1]='|';  
            // Verbindung zur Wurzel vom Teilwurzelbaum
            break;
    // Es hängen zwei Teilwurzelbäume am Knoten i
    case 2: s1=platzierungen[sohn[0]].anzahl_spalten;        // Anzahl der Spalten der Platzierung des ersten Wurzelbaums 
            s2=platzierungen[sohn[1]].anzahl_spalten;        // Anzahl der Spalten der Platzierung des zweiten Wurzelbaums
            z1=platzierungen[sohn[0]].anzahl_zeilen;         // Anzahl der Zeilen der Platzierung des ersten Wurzelbaums
            z2=platzierungen[sohn[1]].anzahl_zeilen;         // Anzahl der Zeilen der Platzierung des zweiten Wurzelbaums
            w1=platzierungen[sohn[0]].position_wurzel;       // Position der Wurzel der Platzierung des ersten Wurzelbaums
            w2=platzierungen[sohn[1]].position_wurzel;       // Position der Wurzel der Platzierung des zweiten Wurzelbaums

            // Bestimme die platzsparendere Variante 
            if(max(s1-w1-1,w2)<=max(s2-w2-1,w1))
            {
              links=0;
              rechts=1;
              a=max(s1-w1-1,w2); 
            }
            else
            {
              links=1;
              rechts=0;
              swap(s1,s2);
              swap(z1,z2);
              swap(w1,w2);
              a=max(s2-w2-1,w1); 
            } 
            if(a<1) a=1;
            platzierungen[i].anzahl_zeilen=max(z1,z2)+a+1;    
            platzierungen[i].anzahl_spalten=w1+1+a+1+a+(s2-w2);   
            platzierungen[i].position_wurzel=w1+1+a;
            platzierungen[i].Matrix.resize(platzierungen[i].anzahl_spalten);
            for(j=0;j<platzierungen[i].anzahl_spalten;j++)
              platzierungen[i].Matrix[j].resize(platzierungen[i].anzahl_zeilen);
            for(j=0;j<platzierungen[i].anzahl_spalten;j++)
              for(h=0;h<platzierungen[i].anzahl_zeilen;h++)
                platzierungen[i].Matrix[j][h]=' ';
            for(j=0;j<s1;j++)
              for(h=0;h<z1;h++)
                platzierungen[i].Matrix [j][h+a+1]=platzierungen[sohn[links]].Matrix[j][h];
            for(j=0;j<s2;j++)
              for(h=0;h<z2;h++)
                platzierungen[i].Matrix [j+w1+1+a+1+a-w2][h+a+1]=platzierungen[sohn[rechts]].Matrix[j][h];  
            platzierungen[i].Matrix [platzierungen[i].position_wurzel][0]='o';
            for(j=1;j<=a;j++)
            { 
              platzierungen[i].Matrix [platzierungen[i].position_wurzel-j][j]=(char)(92);
              platzierungen[i].Matrix [platzierungen[i].position_wurzel+j][j]='/';  
            }  
            break;
    // Es hängen drei Teilwurzelbäume am Knoten i
    case 3: s1=platzierungen[sohn[0]].anzahl_spalten;  // Anzahl der Spalten der Platzierung des ersten Wurzelbaums
            s2=platzierungen[sohn[1]].anzahl_spalten;  // Anzahl der Spalten der Platzierung des zweiten Wurzelbaums
            s3=platzierungen[sohn[2]].anzahl_spalten;  // Anzahl der Spalten der Platzierung des dritten Wurzelbaums
            z1=platzierungen[sohn[0]].anzahl_zeilen;   // Anzahl der Zeilen der Platzierung des ersten Wurzelbaums
            z2=platzierungen[sohn[1]].anzahl_zeilen;   // Anzahl der Zeilen der Platzierung des zweiten Wurzelbaums     
            z3=platzierungen[sohn[2]].anzahl_zeilen;   // Anzahl der Zeilen der Platzierung des dritten Wurzelbaums   
            w1=platzierungen[sohn[0]].position_wurzel; // Position der Wurzel der Platzierung des ersten Wurzelbaums
            w2=platzierungen[sohn[1]].position_wurzel; // Position der Wurzel der Platzierung des zweiten Wurzelbaums
            w3=platzierungen[sohn[2]].position_wurzel; // Position der Wurzel der Platzierung des dritten Wurzelbaums 

            // bestimme die platzsparendere Variante der 6 verschiedenen Anordnungen der 3 Wurzelbäume
            links=0;
            mitte=1;
            rechts=2;
            a=max((s1-w1-1)+1+w2,(s2-w2-1)+1+w3);
            if(a>max((s1-w1-1)+1+w3,(s3-w3-1)+1+w2))
            {
              links=0;
              mitte=2;
              rechts=1; 
              a=max((s1-w1-1)+1+w3,(s3-w3-1)+1+w2);
            } 
            if(a>max((s2-w2-1)+1+w1,(s1-w1-1)+1+w3))
            {
              links=1;
              mitte=0;
              rechts=2; 
              a=max((s2-w2-1)+1+w1,(s1-w1-1)+1+w3);
            }
            if(a>max((s2-w2-1)+1+w3,(s3-w3-1)+1+w1))
            {
              links=1;
              mitte=2;
              rechts=0; 
              a=max((s2-w2-1)+1+w3,(s3-w3-1)+1+w1);
            }
            if(a>max((s3-w3-1)+1+w1,(s1-w1-1)+1+w2))
            {
              links=2;
              mitte=0;
              rechts=1; 
              a=max((s3-w3-1)+1+w1,(s1-w1-1)+1+w2);
            }
            if(a>max((s3-w3-1)+1+w2,(s2-w2-1)+1+w1))
            {
              links=2;
              mitte=1;
              rechts=0; 
              a=max((s3-w3-1)+1+w2,(s2-w2-1)+1+w1);
            }  

            s1=platzierungen[sohn[links]].anzahl_spalten;
            s2=platzierungen[sohn[mitte]].anzahl_spalten;
            s3=platzierungen[sohn[rechts]].anzahl_spalten;
            z1=platzierungen[sohn[links]].anzahl_zeilen;
            z2=platzierungen[sohn[mitte]].anzahl_zeilen;       
            z3=platzierungen[sohn[rechts]].anzahl_zeilen;  
            w1=platzierungen[sohn[links]].position_wurzel;
            w2=platzierungen[sohn[mitte]].position_wurzel;
            w3=platzierungen[sohn[rechts]].position_wurzel;  

            platzierungen[i].anzahl_zeilen=max(max(z1,z2),z3)+a+1;    
            platzierungen[i].anzahl_spalten=w1+1+a+1+a+1+(s3-w3-1);   
            platzierungen[i].position_wurzel=w1+1+a;

            platzierungen[i].Matrix.resize(platzierungen[i].anzahl_spalten);
            for(j=0;j<platzierungen[i].anzahl_spalten;j++)
              platzierungen[i].Matrix[j].resize(platzierungen[i].anzahl_zeilen);
            for(j=0;j<platzierungen[i].anzahl_spalten;j++)
              for(h=0;h<platzierungen[i].anzahl_zeilen;h++)
                platzierungen[i].Matrix[j][h]=' ';   
            for(j=0;j<s1;j++)
              for(h=0;h<z1;h++)
                platzierungen[i].Matrix [j][h+a+1]=platzierungen[sohn[links]].Matrix[j][h];
            for(j=0;j<s2;j++)
              for(h=0;h<z2;h++)
                platzierungen[i].Matrix [j+w1+1+a-w2][h+a+1]=platzierungen[sohn[mitte]].Matrix[j][h];             
            for(j=0;j<s3;j++)
              for(h=0;h<z3;h++)
                platzierungen[i].Matrix [j+w1+1+a+1+a-w3][h+a+1]=platzierungen[sohn[rechts]].Matrix[j][h]; 
            platzierungen[i].Matrix [platzierungen[i].position_wurzel][0]='o';
            for(j=1;j<=a;j++)
            { 
              platzierungen[i].Matrix [platzierungen[i].position_wurzel-j][j]=(char)(92);
              platzierungen[i].Matrix [platzierungen[i].position_wurzel][j]='|';
              platzierungen[i].Matrix [platzierungen[i].position_wurzel+j][j]='/';  
            }
            break; 
  }
}

