// *********************************************************************
// *                                                                   *
// *       Konstruktion von Isomeren der Alkene (C_nH_2n)              *
// *       ================================================            *
// *                                                                   *
// *       Autor:        Sascha Kurz                                   *
// *       modifiert im Rahmen eines Softwarepraktikums durch:         *
// *                     Sabine Boehm und Florian Haeberlein           * 
// *       Email:        sascha.kurz@uni-bayreuth.de                   *
// *       Homepage:     www.mathe2.uni-bayreuth.de/sascha             *
// *                                                                   *
// *       Filename:           konstruiere_alkene.cpp                  *
// *       zuletzt geaendert:  11.01.2005                              *
// *       Betriebssystem:     Suse Linux 9.1                          *
// *       Compiler:           gcc3.3.3                                *
// *       Compiler-Optionen:  g++ -O2 -Wall -pedantic -ansi           *
// *                           konstruiere_alkene.cpp                  *
// *                           -o konstruiere_alkene                   *
// *                                                                   *
// *       Algorithmus                                                 *
// *       -----------                                                 *
// *       Basierend auf dem Artikel "Anzahl von Strukturisomeren      *
// *       der Alkane". Es werden rekursiv Wurzelbaeume zusammenge-    *
// *       setzt, die wiederum zum Zusammensetzen von Alkenen ver-     *
// *       wendet werden.                                              *
// *                                                                   *
// *       + Wurzelbaeume                                              *
// *       -------------                                               *
// *       Ebene 3        o o o o o      Der Knoten in Ebene 1         *
// *                       \|  \|/       heisst Wurzel. Jeder Kno-     *
// *       Ebene 2          o o o        ten, ausser der Wurzel, hat   *
// *                         \|/         genau eine Kante zu einem     *
// *       Ebene 1            o          Knoten auf einer um 1 nie-    *
// *                                     drigeren Ebene.               *
// *                                                                   *
// *       Abgespeichert werden Wurzelbaeume als Array. Hierzu num-    *
// *       meriert man die Knoten, und speichert zu jedem Knoten       *
// *       die Nummer des Knotens auf der um eines niedrigeren Ebene   *
// *       zu dem er eine Kante besitzt. Gibt man der Wurzel die       *
// *       Nummer 0, so kann man als Konvention im Array an der        *
// *       Position 0 ebenfalls die 0 speichern.                       *
// *                                                                   *
// *       Ebene 3        4 5 6 7 8      Nummerierte Version des       *
// *                       \|  \|/       obigen Wurzelbaums. Das       *
// *       Ebene 2          1 2 3        entsprechende Array ist       *
// *                         \|/         in diesem Fall:               *
// *       Ebene 1            0          [0,0,0,0,1,1,3,3,3]           *
// *                                                                   *
// *       + Zusammensetzen von Wurzelbaeumen                          *
// *       ---------------------------------                           *
// *       Die Anzahl der Ebenen eines Wurzelbaums bezeichnet man      *
// *       als seine Hoehe. Wurzelbaeume der Hoehe h kann man folgt    *
// *       aus Wurzelbaeumen kleinerer Hoehe zusammensetzen:           *
// *                                                                   *
// *       4 5 6 7 8   2 3 4              5 6  7 8 9 12 13 14          *
// *        \|  \|/     \|/                \|   \|/    \|/             *
// *         1 2 3       1     1   --->     2 3 4      11 16           *
// *          \|/        |     |             \|/        | |            *
// *           0         0     0              1        10 15           *
// *                                           \        |/             *
// *                                            --------0              *
// *                                                                   *
// *       Das Zusammensetzen und Umnummerieren von Wurzelbaeumen,     *
// *       wird von der Funktion "Fuege_an()" erledigt. Diese fuegt    *
// *       den neu konstruierten Wurzelbaum gleichzeitig noch an       *
// *       eine Liste L von Wurzelbaeumen an. Die Wurzelbaeume         *
// *       kleinerer Hoehe werden als Positionen in der Liste L        *
// *       uebergeben. Da in unserer Anwendung jeder Knoten nur        *
// *       hoechstens 4 Nachbarn hat, brauchen nur 3 int-Werte a,b,c   *
// *       uebergeben werden.                                          *
// *                                                                   *
// *       + Erzeugen von Wurzelbaeumen                                *
// *       ---------------------------                                 *
// *       Im vorhergehenden Abschnitt haben wir erklaert, wie         *
// *       Wurzelbaeume mit kleinerer Hoehe zu Wurzelbaeumen mit       *
// *       groesserer Hoehe zusammengesetzt werden. Hierzu startet     *
// *       man mit einer Liste von Wurzelbaeumen der Hoehe kleiner-    *
// *       gleich h-1 und konstruieren daraus eine Liste von Wurzel-   *
// *       baeumen der Hoehe kleinergleich h. Fuer den Start initiali- *
// *       siert man die Liste mit dem leeren Wurzelbaum und dem       *
// *       Wurzelbaum der Hoehe 1.Nun waehlt man 3 Wurzelbaeume aus der*
// *       Liste der Wurzelbaeume der Hoehe kleinergleich h-1 aus,     *
// *       wobei mindestens einer davon exakt die Hoehe h-1 besitzen   *
// *       muss, und wendet die Funktion "Fuege_an()" an, um aus den   *
// *       3 Wurzelbaeumen einen neuen Wurzelbaum der Hoehe h zu       *
// *       konstruieren. Erledigt wird diese Arbeit von der Funktion   *
// *       "konstruiere_Wurzelbaeume()". Aus Geschwindigkeitsgruenden  *
// *       wird nur mit einer Liste von Wurzelbaeumen gearbeitet, und  *
// *       die Auswahl der 3 Wurzelbaeume wird vorausschauend durchge- *
// *       fuehrt, so dass keine Wurzelbaeume mit "zu vielen" Knoten   *
// *       konstruiert werden.                                         *
// *                                                                   *
// *       + Zusammensetzen der Wurzelbaeume zu Alkenen                *
// *       -------------------------------------------                 *
// *                                                                   *
// *       Um Wurzelbaeume zu Alkenen zusammenzusetzen, sucht man sich *
// *       den aktuellen Wurzelbaum mit der groessten Hoehe. Falls es  *
// *       davon mehrere gibt, nimmt man denjenigen mit der maximalen  *
// *       Anzahl an C-Atomen. Dieser wird als Wurzelbaum a definiert, *
// *       damit ist seine Position festgelegt. Fuer die restlichen    *
// *       drei Wurzelbaeume gibt es nun verschiedene Moeglichkeiten   *
// *       die alle bei der Funktion konstruiere_Alkene erklaert       *
// *       werden.                                                     *
// *                                                                   *
// *       + Ausgeben von Alkenen auf dem Bildschirm                   *
// *       -----------------------------------------                   *
// *       Die Alkene koennen im XML-Format ausgegeben werden, und     *
// *       dann z.B. mit dem Graphviewer betrachtet werden. Die        *
// *       angegebenen Koordinaten haben wenig mit der Realitaet       *
// *       zu tun und stellen auch nicht unbedingt eine schoene        *
// *       Zeichnung des Graphens dar, doch sind sie immerhin so       *
// *       berechnet, dass die Kanten sich nicht ueberschneiden, und   *
// *       man ein ungefaehres Bild von dem Molekuel bekommen kann.    *
// *                                                                   *
// *       Schlussbemerkung                                            *
// *       ===============                                             *
// *       Bei Fragen, Anmerkungen, Kritik oder sonstigen Kommentaren  *
// *       wende man sich bitte per email an den oben genannten Autor  *
// *       dieses Programms. Der Quellcode ist fuer nicht-kommerzielle *
// *       Anwendungen frei verfuegbar und darf beliebig veraendert    *
// *       werden. Bei Veraenderungen ist geeignet darauf hinzuweisen, *
// *       von wem sie vorgenommen wurden. Es wuerde mich freuen ueber *
// *       Verwendungen dieses Quellcodes eine kurze Nachricht per     *
// *       email zu erhalten.                                          *
// *                                                                   *
// *       Sascha Kurz                                                 *
// *                                                                   *
// *********************************************************************



#include<vector>              // Headerfile fuer die Klasse Vector
#include<iostream>            // Headerfile fuer die Ein- und Ausgabe
#include<fstream>             // Headerfile fuer die Ein- und Ausgabe in Dateien

using namespace std;

struct Alken                               // Datenstruktur fuer ein Alken
{
  vector <unsigned char> Wurzelbaum_1;     // Vektor fuer den ersten Wurzelbaum
  vector <unsigned char> Wurzelbaum_2;     // Vektor fuer den zweiten Wurzelbaum
  vector <unsigned char> Wurzelbaum_3;     // Vektor fuer den dritten Wurzelbaum
  vector <unsigned char> Wurzelbaum_4;     // Vektor fuer den vierten Wurzelbaum
};

// *********************************************************************
// *                  Prototypdeklarationen                            *
// *********************************************************************

void Fuege_an(vector< vector <unsigned char> >& L,int a,int b,int c);
// Konstruiert aus 3 Wurzelbaeumen einen neuen Wurzelbaum mit groesserer
// Hoehe und fuegt ihn der Liste L an

void Fuege_an(vector<Alken>&,const vector< vector <unsigned char> >& L,
unsigned int a,unsigned int b,unsigned int c,unsigned d,bool ausgeben,int ausgabe_bei_anzahl_c_atomen);
// Konstruiert aus 4 Wurzelbaeumen der Hoehe kleinergleich h ein Alken

void sortiere_Wurzelbaum_Liste(vector< vector <unsigned char> >& L,int p);
// Sortiert eine Liste L von Wurzelbaeumen, so dass die ersten p Wurzelbaeumen
// und die restlichen Wurzelbaeumen jeweils nach der Anzahl der Knoten
// aufsteigend angeordnet sind

void konstruiere_Wurzelbaeume(vector<Alken>& Alkene,vector< vector <unsigned char> >& L,int n,bool ausgeben,int ausgabe_bei_anzahl_c_atomen);
// Konstuiert eine "vollstaendige" Liste von Wurzelbaeumen, so dass damit
// eine vollstaendige Liste aller Alkene mit bis zu n C-Atomen konstruiert
// werden kann.

void konstruiere_Alkene(vector<Alken>& Alkene,const vector< vector <unsigned char> >& L,
unsigned int p,unsigned int h,unsigned int n,bool ausgeben,int ausgabe_bei_anzahl_c_atomen);
// Erweitert eine Liste von Alkenen mit denen solchen die einen laengsten Wurzelbaum der Hoehe h haben

int ausgabe(Alken& A);
// Gibt einen Graphen des Alkens im Graph xml-Format in der Datei graph aus




// ********************************************************************
// *         Implementierungen der benoetigten Funktionen              *
// ********************************************************************

void Fuege_an(vector< vector <unsigned char> >& L,int a,int b,int c)
{
  // L:   Liste von Wurzelbaeumen
  // a:   Position eines Wurzelbaums in der Liste L
  // b:   Position eines Wurzelbaums in der Liste L
  // c:   Position eines Wurzelbaums in der Liste L
  //
  // Funktion:
  // ---------
  // Konstruiert einen neuen Wurzelbaum W, indem die Wurzelbaeume Nr. a,b
  // und c vereinigt werden, und die Wurzeln von a,b und c mit einer
  // neuen Wurzel verbunden werden.
  //
  //      a b c
  //  W=   \|/
  //        o
  //
  // Der neu konstruierte Wurzelbaum W wird an die Liste L angehaengt

  int p=L.size();               // die alte Laenge der Liste der Wurzelbaeume
  unsigned int i;               // Schleifenvariable
  int a_s=L[a].size();          // aus wie vielen Knoten besteht Wurzelbaum Nr. a
  int b_s=L[b].size();          // aus wie vielen Knoten besteht Wurzelbaum Nr. b
  int c_s=L[c].size();          // aus wie vielen Knoten besteht Wurzelbaum Nr. c
  L.resize(p+1);                // Verlaengere die Liste um ein Element
  L[p].resize(a_s+b_s+c_s+1);   // Der neue Wurzelbaum setzt sich aus Wurzelbaum Nr. a,
                                // Wurzelbaum Nr. b, Wurzelbaum Nr. c und einem zusaetzlichen
                                // Knoten, der neuen Wurzel, zusammen
  L[p][0]=0;                    // neue Wurzel
  L[p][1]=0;                    // alte Wurzel von Wurzelbaum Nr. a zeigt auf neue Wurzel
  for(i=1;i<L[a].size();i++)    // durchlaufe die restlichen Knoten von Wurzelbaum Nr. a
    L[p][i+1]=L[a][i]+1;        // verschiebe die alte Nummerierung
  L[p][1+a_s]=0;                // alte Wurzel von Wurzelbaum Nr. b zeigt auf neue Wurzel
  for(i=1;i<L[b].size();i++)    // durchlaufe die restlichen Knoten von Wurzelbaum Nr. a
    L[p][i+1+a_s]=L[b][i]+1+a_s;// verschiebe die alte Nummerierung
  L[p][1+a_s+b_s]=0;            // alte Wurzel von Wurzelbaum Nr. c zeigt auf neue Wurzel
  for(i=1;i<L[c].size();i++)    // durchlaufe die restlichen Knoten von Wurzelbaum Nr. a
    L[p][i+1+a_s+b_s]=L[c][i]+1+a_s+b_s; // verschiebe die alte Nummerierung
}


void Fuege_an(vector<Alken>& Alkene,const vector< vector <unsigned char> >& L,unsigned int a,
unsigned int b,unsigned int c,unsigned int d,bool ausgeben,int ausgabe_bei_anzahl_c_atomen)
{
  // Alkene : Liste von Alkenen
  // L:       Liste von Wurzelbaeumen
  // a:       Position eines Wurzelbaums in der Liste L
  // b:       Position eines Wurzelbaums in der Liste L
  // c:       Position eines Wurzelbaums in der Liste L
  // d:       Position eines Wurzelbaums in der Liste L
  //
  // Funktion:
  // ---------
  // Konstruiert aus vier Wurzelbaeumen W_1, W_2, W_3 und W_4 der Hoehe kleinergleich h ein Alken A
  // Das neu konstruierte Alken A wird an die Liste Alkene angehaengt

  unsigned int i;               // Schleifenvariable;
  int p=Alkene.size();          // die alte Laenge der Liste der Alkene
  Alkene.resize(p+1);           // verlaengere die Liste um ein Element

  Alkene[p].Wurzelbaum_1.resize(L[a].size());   // Reserviere Speicherplatz fuer den Wurzelbaum Nr. a
  for(i=0;i<L[a].size();i++)                    // Kopiere Wurzelbaum Nr. a
    Alkene[p].Wurzelbaum_1[i]=L[a][i];
  Alkene[p].Wurzelbaum_2.resize(L[b].size());   // Reserviere Speicherplatz fuer den Wurzelbaum Nr. b
  for(i=0;i<L[b].size();i++)                    // Kopiere Wurzelbaum Nr. b
    Alkene[p].Wurzelbaum_2[i]=L[b][i];
  Alkene[p].Wurzelbaum_3.resize(L[c].size());   // Reserviere Speicherplatz fuer den Wurzelbaum Nr. c
  for(i=0;i<L[c].size();i++)                    // Kopiere Wurzelbaum Nr. c
    Alkene[p].Wurzelbaum_3[i]=L[c][i];
  Alkene[p].Wurzelbaum_4.resize(L[d].size());   // Reserviere Speicherplatz fuer den Wurzelbaum Nr. d
  for(i=0;i<L[d].size();i++)                    // Kopiere Wurzelbaum Nr. d
    Alkene[p].Wurzelbaum_4[i]=L[d][i];

  if(ausgeben==true)
  {
    if(ausgabe_bei_anzahl_c_atomen==-1|| (int)(Alkene[p].Wurzelbaum_1.size()+Alkene[p].Wurzelbaum_2.size()+Alkene[p].Wurzelbaum_3.size()+Alkene[p].Wurzelbaum_4.size()+2)==ausgabe_bei_anzahl_c_atomen)
    ausgabe(Alkene[p]);
  }

}


void sortiere_Wurzelbaum_Liste(vector< vector <unsigned char> >& L,int laenge,int n)
{
  // L:       Liste von Wurzelbaeumen
  // laenge:  Position in Liste L, siehe "Funktion"
  // n:       maximale Anzahl der Knoten
  //
  // Funktion:
  // ---------
  // Sortiert die Positionen 0 bis laenge-1 der Liste L so um, dass die Anzahl der
  // Knoten der Wurzelbaeume aufsteigend ist. Die Positionen laenge bis L.size()-1
  // werden entsprechend sortiert.
  // Da die maximale Anzahl der Knoten (n) gegenueber der Laenge der Liste eine sehr
  // kleine Zahl ist, werden anstatt einem Sortieralgorithmus wie Mergesort oder
  // Quicksort einfach n+1 Listen angelegt, und jeder Wurzelbaum wird in die Liste
  // mit der Nummer entsprechend der Anzahl seiner Knoten gelegt. Anschliessend
  // werden diese Listen wieder zu einer zusammengefuegt. Der Aufwand betraegt somit
  // O(n+L.size())

  int i,j;            // Hilfsvariablen
  unsigned int k,a,b; // Hilfsvariablen

  vector< vector< vector<unsigned char> > > sort;     // Hilfsvektor zum Sortieren

  sort.resize(n+1);                    // Anlegen der n+1 Hilfslisten
  for(i=0;i<=n;i++)                    // durchlaufe die Hilfslisten
    sort[i].resize(0);                 // Jede Liste ist am Anfang leer

  // Trage die Wurzelbaeume, der Positionen 0,...,laenge-1, in die Hilfslisten ein
  for(i=0;i<laenge;i++)                // durchlaufe die Positionen 0 bis laenge-1 von L
  {
    k=L[i].size();                     // Wie viele Knoten besitzt der gerade betrachtete Wurzelbaum?
    a=sort[k].size();                  // Wie lang ist die entsprechende Hilfsliste
    sort[k].resize(a+1);               // Verlaengere die Hilfsliste um ein Element
    sort[k][a].resize(k);              // Der neue Wurzelbaum besteht aus k Knoten
    for(b=0;b<k;b++)                   // Fuege den betrachteten Wurzelbaum in die
      sort[k][a][b]=L[i][b];           // Hilfsliste ein
  }

  // Trage die Wurzelbaeume aus den Hilfslisten wieder zurueck in L ein
  j=0;                                 // Aktuelle Position in L
  for(i=0;i<=n;i++)                    // durchlaufe die unterschiedlichen Hilfslisten
  {
    for(a=0;a<sort[i].size();a++)      // durchlaufe Hilfsliste i
    {
      L[j].resize(i);                  // Speichere an Position j in L einen Wurzelbaum bestehend aus i Knoten
      for(b=0;b<(unsigned int)i;b++)
        L[j][b]=sort[i][a][b];
      j++;                             // Verschiebe die aktuelle Position in L um eins nach rechts
    }
  }

  sort.resize(n+1);                    // "Loesche" die Hilfslisten wieder
  for(i=0;i<=n;i++)
  {
    for(a=0;a<sort[i].size();a++)
      sort[i][a].resize(0);
    sort[i].resize(0);
  }

  // Trage die Wurzelbaeume, der Positionen laenge,...,L.size()-1, in die Hilfslisten ein
  for(i=laenge;i<(int)L.size();i++)         // durchlaufe die Positionen laenge bis L.size()-1 von L
  {
    k=L[i].size();                     // Wie viele Knoten besitzt der gerade betrachtete Wurzelbaum?
    a=sort[k].size();                  // Wie lang ist die entsprechende Hilfsliste
    sort[k].resize(a+1);               // Verlaengere die Hilfsliste um ein Element
    sort[k][a].resize(k);              // Der neue Wurzelbaum besteht aus k Knoten
    for(b=0;b<k;b++)                   // Fuege den betrachteten Wurzelbaum in die
      sort[k][a][b]=L[i][b];           // Hilfsliste ein
  }

  // Trage die Wurzelbaeume aus den Hilfslisten wieder zurueck in L ein
  for(i=0;i<=n;i++)                    // durchlaufe die unterschiedlichen Hilfslisten
  {
    for(a=0;a<sort[i].size();a++)      // durchlaufe Hilfsliste i
    {
      L[j].resize(i);                  // Speichere an Position j in L einen Wurzelbaum bestehend aus i Knoten
      for(b=0;b<(unsigned int)i;b++)
        L[j][b]=sort[i][a][b];
      j++;                             // Verschiebe die aktuelle Position in L um eins nach rechts
    }
  }

  sort.resize(n+1);                    // "Loesche" die Hilfslisten wieder
  for(i=0;i<=n;i++)
  {
    for(a=0;a<sort[i].size();a++)
      sort[i][a].resize(0);
    sort[i].resize(0);
  }
}


void konstruiere_Wurzelbaeume(vector<Alken>& Alkene,vector< vector <unsigned char> >& L,int n,bool ausgeben,int ausgabe_bei_anzahl_c_atomen)
{
  // L: Liste zum Speichern von Wurzelbaeumen
  // n: maximale Anzahl von C-Atomen in den spaeter zu konstruierenden Alkenen
  //
  // Funktion:
  // ---------
  // Ziel ist es eine vollstaendige Liste aller Wurzelbaeume der Hoehe h mit maximal n-h Knoten
  // zu konstruieren. Hierbei laeuft h von 0 bis [n/2].
  // Dabei werden Wurzelbaeume der Hoehe h aus Wurzelbaeumen der Hoehe kleinergleich h-1 zusammen-
  // gesetzt. Fuer weitere Details lese den Artikel:"Anzahl von Strukturisomeren der Alkane"



  // Startbelegung fuer die Liste der Wurzelbaeume
  L.resize(2);                              // Am Anfang sind nur 2 Wurzelbaeume in der Liste eingetragen
  L[0].resize(0);                           // Leerer Wurzelbaum aus 0 Knoten
  L[1].resize(1);
  L[1][0]=0;                                // Wurzelbaum aus einem Knoten
  int p=1;                                  // Position des Wurzelbaums der Hoehe 1
                                            // Allgemein: Position des ersten Wurzelbaums der Hoehe h in der
                                            // Liste L

  // Startbelegung fuer die Liste der Alkene
  Alkene.resize(1);
  Alkene[0].Wurzelbaum_1.resize(0);   //Ethen
  Alkene[0].Wurzelbaum_2.resize(0);
  Alkene[0].Wurzelbaum_3.resize(0);
  Alkene[0].Wurzelbaum_4.resize(0);

  cout << endl;
  if(ausgeben==true)
    if(ausgabe_bei_anzahl_c_atomen==-1 || ausgabe_bei_anzahl_c_atomen==2)
      ausgabe(Alkene[0]);

  if(ausgeben==false)
  {
    cout << endl;
    cout << "Konstruiere Wurzelbaeume der Hoehe 1 ";
    if(n>=20) cout << " ";
    cout << ": ." << endl;
  }

  konstruiere_Alkene(Alkene,L,p,1,n,ausgeben,ausgabe_bei_anzahl_c_atomen);


  int a,b,c;                                // Position der drei kleineren Wurzelbaeume in der Liste L
  int laenge;                               // Hilfsgroesse, um "p" bzw. die Laenge der Liste auch bei
                                            // Änderungen an der Liste verfuegbar zu haben

  for(int h=2;h<=int(n-2);h++)              // Durchlaufe die moeglichen Hoehen der Wurzelbaeume. Die Wurzelbaeume
  {                                         // der Hoehe 0 und 1 wurden mit der Startbelegung behandelt.
    // Forstschrittsausgabe, da Berechnung lange dauern kann
    if(ausgeben==false)
    {
      cout << "Konstruiere Wurzelbaeume der Hoehe " << h << " ";
      if(n>=20 && h<10) cout << " ";
      cout << ": ";
    }

    laenge=L.size();                        // Wie  lang ist die Liste L bisher

    // In der Liste L stehen an den Positionen 0 bis p-1 Wurzelbaeume der Hoehe kleiner h-1
    // geordnet nach der Anzahl der Knoten. An den Positionen p bis laenge-1 stehen
    // Wurzelbaeume der Hoehe h-1, ebenfalls nach der Anzahl der Knoten geordnet.
    // Fuer die Kombination zu einem Wurzelbaum der Hoehe h gibt es nun 3 Moeglichkeiten:
    // (i)   zwei Wurzelbaeume der Hoehe kleiner h-1 und ein Wurzelbaum der Hoehe h-1
    // (ii)  ein Wurzelbaum der Hoehe kleiner h-1 und zwei Wurzelbaeume der Hoehe h-1
    // (iii) drei Wurzelbaeume der Hoehe h-1
    //
    // Die Positionen der Wurzelbaeume in L werden mit a,b und c bezeichnet. Um die Konstruktion
    // zu beschleunigen, wird bei jedem Schritt ueberprueft, ob ueberhaupt ein Alken mit hoechstens
    // n C-Atomen entstehen kann.


    // Fall (i)
    for(a=0;a<p;a++)                                // a ist ein Wurzelbaum der Hoehe kleiner h-1
    {
      if(ausgeben==false)
        cout << "." << flush;                       // Forstschrittsausgabe, da Berechnung lange dauern kann
      if(1+(int)L[a].size()*2+(h-1)<=n-2)           // 1:               die neue Wurzel
      {                                             // 2*L[a].size()*2: b besitzt mindestens genau so viele
                                                    //                  Knoten wie a, also fuer a und b zusammen
                                                    //                  2*L[a].size()*2
                                                    // (h-1):           Wurzelbaum c besitzt Hoehe h-1, also mindestens
                                                    //                  h-1 Knoten
        for(b=a;b<p;b++)                            // b ist ein Wurzelbaum der Hoehe kleiner h-1
        {
          if(1+(int)L[a].size()+(int)L[b].size()+(h-1)<=n-2)  // 1:           die neue Wurzel
          {                                         // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                    // L[b].size(): Anzahl der Knoten von Wurzelbaum b
                                                    // (h-1):       Wurzelbaum c besitzt Hoehe h-1, also mindestens
                                                    //              h-1 Knoten
            for(c=p;c<laenge;c++)                   // c ist ein Wurzelbaum der Hoehe h-1
            {
              if(1+(int)L[a].size()+(int)L[b].size()+(int)L[c].size()<=n-2) // 1:           die neue Wurzel
              {                                              // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                             // L[b].size(): Anzahl der Knoten von Wurzelbaum b
                                                             // L[c].size(): Anzahl der Knoten von Wurzelbaum b
                Fuege_an(L,a,b,c);                  // Fuege die Wurzelbaeume a,b und c zusammen und haenge sie an
              }                                     // die Liste L an
              else                                  // Der entehende Wurzelbaum kann nicht zur Konstruktion eines
                c=laenge;                           // Alkens mit maximal n C-Atomen verwendet werden, also gehe mit
            }                                       // b eins weiter und starte mit c wieder bei Position p
          }
          else                                      // Die entehenden Wurzelbaeume koennen nicht zur Konstruktion eines
          {                                         // Alkens mit maximal n C-Atomen verwendet werden, also gehe mit
            b=p;                                    // a eins weiter und starte mit b wieder bei Position a
          }
        }
      }
      else                                          // Die entehenden Wurzelbaeume koennen nicht zur Konstruktion eines
      {                                             // Alkens mit maximal n C-Atomen verwendet werden, also beende die
        a=p;                                        // Schleife
      }
    }


    // Fall (ii)
    for(a=0;a<p;a++)                                // a ist ein Wurzelbaum der Hoehe kleiner h-1
    {
      if(ausgeben==false)
        cout << "." << flush;                       // Forstschrittsausgabe, da Berechnung lange dauern kann
      if(1+(int)L[a].size()+2*(h-1)<=n-2)           // 1:           die neue Wurzel
      {                                             // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                    // 2*(h-1):     Die Wurzelbaeume b und c besitzen die Hoehe
                                                    //              h-1, bestehen also aus mindestens h-1 Knoten
        for(b=p;b<laenge;b++)                       // b ist ein Wurzelbaum der Hoehe h-1
        {
          if(1+(int)L[a].size()+2*((int)L[b].size())<=n-2)      // 1:           die neue Wurzel
          {                                         // L[a].size(): Anzahl der Knoten von Wurzelbaum a

                                                    // h:           Es werden mindestens zwei Wurzelbaeume der
                                                    //              Hoehe h zu einem Alken zusammengesetzt
            for(c=b;c<laenge;c++)                   // c ist ein Wurzelbaum der Hoehe h-1
              if(1+(int)L[a].size()+(int)L[b].size()+(int)L[c].size()<=n-2) // 1:           die neue Wurzel
              {                                              // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                             // L[b].size(): Anzahl der Knoten von Wurzelbaum b
                                                             // L[c].size(): Anzahl der Knoten von Wurzelbaum b
                Fuege_an(L,a,b,c);                  // Fuege die Wurzelbaeume a,b und c zusammen und haenge sie an
              }                                     // die Liste L an
              else                                  // Der entehende Wurzelbaum kann nicht zur Konstruktion eines
                c=laenge;                           // Alkens mit maximal n C-Atomen verwendet werden, also gehe mit
          }                                         // b eins weiter und starte mit c wieder bei Position b
          else                                      // Die entehenden Wurzelbaeume koennen nicht zur Konstruktion eines
          {                                         // Alkens mit maximal n C-Atomen verwendet werden, also gehe mit
            b=laenge;                               // a eins weiter und starte mit b wieder bei Position p
          }
        }
      }
      else                                          // Die entehenden Wurzelbaeume koennen nicht zur Konstruktion eines
      {                                             // Alkens mit maximal n C-Atomen verwendet werden, also beende die
        a=p;                                        // Schleife
      }
    }


    // Fall (iii)
    for(a=p;a<laenge;a++)                           // a ist ein Wurzelbaum der Hoehe h-1
    {
      if(ausgeben==false)
        cout << "." << flush;                       // Forstschrittsausgabe, da Berechnung lange dauern kann
      if(1+3*((int)L[a].size())<=n-2)               // 1:             die neue Wurzel
      {                                             // 3*L[a].size(): Die Wurzelbaeume b und c besitzen
                                                    //                mindestens so viele Knoten wie der
                                                    //                Wurzelbauma
        for(b=a;b<laenge;b++)                       // b ist ein Wurzelbaum der Hoehe h-1
        {
          if(1+(int)L[a].size()+2*((int)L[b].size())<=n-2)      // 1:             die neue Wurzel
          {                                         // L[a].size():   Anzahl der Knoten von Wurzelbaum a
                                                    // 2*L[b].size(): Der Wurzelbaum c besitzt mindestens
                                                    //                so viele Knoten wie Wurzelbaum b
            for(c=b;c<laenge;c++)                   // c ist ein Wurzelbaum der Hoehe h-1
              if(1+(int)L[a].size()+(int)L[b].size()+(int)L[c].size()<=n-2) // 1:           die neue Wurzel
              {                                              // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                             // L[b].size(): Anzahl der Knoten von Wurzelbaum b
                                                             // L[c].size(): Anzahl der Knoten von Wurzelbaum b
                Fuege_an(L,a,b,c);                  // Fuege die Wurzelbaeume a,b und c zusammen und haenge sie an
              }                                     // die Liste L an
              else                                  // Der entehende Wurzelbaum kann nicht zur Konstruktion eines
                c=laenge;                           // Alkens mit maximal n C-Atomen verwendet werden, also gehe mit
          }                                         // b eins weiter und starte mit c wieder bei Position b
          else                                      // Die entehenden Wurzelbaeume koennen nicht zur Konstruktion eines
          {                                         // Alkens mit maximal n C-Atomen verwendet werden, also gehe mit
            b=laenge;                               // a eins weiter und starte mit b wieder bei Position a
          }
        }
      }
      else                                          // Die entehenden Wurzelbaeume koennen nicht zur Konstruktion eines
      {                                             // Alkens mit maximal n C-Atomen verwendet werden, also beende die
        a=laenge;                                   // Schleife
      }
    }
    if(ausgeben==false)
      cout << endl;


    sortiere_Wurzelbaum_Liste(L,laenge,n);          // Stelle die "richtige" Sortierung der Liste L wieder her, so
                                                    // wie sie fuer diesen Algorithmus gebraucht wird
    p=laenge;                                       // Start fuer Wurzelbaeume der Laenge h

    // Konstruiere Alkene aus den bereits erzeugten Wurzelbaeumen
    konstruiere_Alkene(Alkene,L,p,h,n,ausgeben,ausgabe_bei_anzahl_c_atomen);

  }
}

void konstruiere_Alkene(vector<Alken>& Alkene,const vector< vector <unsigned char> >& L,
unsigned int p,unsigned int h,unsigned int n,bool ausgeben,int ausgabe_bei_anzahl_c_atomen)
{
  // Alkene:  eine Liste von Alkenen
  // L:       eine Liste von Wurzelbaeumen
  // p:       Position in L, ab der die Wurzelbaeume der Hoehe h stehen
  // h:       Hoehe der Wurzelbaeume, die aktuell fuer die Konstruktion verwendet werden sollen
  // n:       maximale Anzahl der C-Atome, die die Alkene besitzen sollen
  //
  // Funktion:
  // ---------
  // Erweitert eine Liste von Alkenen mit denen solchen die mindestens einen Wurzelbaum der Hoehe h haben

  if(ausgeben==false)
  {
    cout << "Konstruiere zugehoerige Alkene      ";// Fortschrittsanzeige
    if(n>=20) cout << " ";
    cout << ": " << flush;
  }

  unsigned int a,b,c,d;                           // Nummer der bis zu 4 Wurzelbaeume, die aus der Liste L ausgewaehlt werden

  if(h>n-2) return;                               // Es koennen keine Alkene mit hinreichend wenig C-Atomen
                                                  // konstruiert werden

  // In der Liste L stehen an den Positionen 0 bis p-1 Wurzelbaeume der Hoehe kleiner h
  // geordnet nach der Anzahl der Knoten. An den Positionen p bis laenge-1 stehen
  // Wurzelbaeume der Hoehe h, ebenfalls nach der Anzahl der Knoten geordnet.
  // Um Alkene aus den Wurzelbaeumen zusammenzusetzen wird nun derjenige Baum,
  // der die Hoehe h und die maximalste Anzahl an C-Atomen als Wurzelbaum a ausgezeichnet.
  // Nun gibt es fuer die restlichen drei Wurzelbaeume verschiedene Moeglichkeiten

  // Fall i): Alle Wurzelbaeume sind ungleich mit a > b > c > d

  if(n>4)  // Dieser Fall tritt erst ab 5 C-Atomen auf
  {
    for(a=p;a<L.size();a++)  // Baum a hat Hoehe h
    {
      if(L[a].size()<=n-2)   // Baum a hat hoechstens n-2 Atome
      {
        for(b=2;b<a;b++)     // Baum d kann 0 sein, dann muss Baum c 1 sein und Baum b mindestens 2
        {
          if(L[a].size()+L[b].size()<=n-2)  // Haben Baum a und b zusammen hoechstens n-2 C-Atome?
            {
              for(c=1;c<b;c++)  // Baum c liegt zwischen 0 und b
              {
                if(L[a].size()+L[b].size()+L[c].size()<=n-2)  // Haben Baum a, b und c zusammen hoechstens n-2 C-Atome?
                {
                  // Falls d der leere Baum ist:
                  Fuege_an(Alkene,L,a,b,c,0,ausgeben,ausgabe_bei_anzahl_c_atomen);   // Moegliche Isomere fuer d=0 an die Liste der Alkene anfuegen
                  Fuege_an(Alkene,L,a,c,b,0,ausgeben,ausgabe_bei_anzahl_c_atomen);
                  Fuege_an(Alkene,L,a,0,c,b,ausgeben,ausgabe_bei_anzahl_c_atomen);
//                  Fuege_an(Alkene,L,a,0,b,c,ausgeben,ausgabe_bei_anzahl_c_atomen);
                  if(c>1)
                  {
                    for(d=1;d<c;d++)  // d ist nicht der leere Baum
                    {
                      if(L[a].size()+L[b].size()+L[c].size()+L[d].size()<=n-2)  // Haben alle vier Baeume hoechstens n-2 C-Atome
                        Fuege_an(Alkene,L,a,b,c,d,ausgeben,ausgabe_bei_anzahl_c_atomen);   // Fuege alle Isomere der Liste der Alkene an
//                        Fuege_an(Alkene,L,a,b,d,c,ausgeben,ausgabe_bei_anzahl_c_atomen);
                        Fuege_an(Alkene,L,a,c,b,d,ausgeben,ausgabe_bei_anzahl_c_atomen);
//                        Fuege_an(Alkene,L,a,c,d,b,ausgeben,ausgabe_bei_anzahl_c_atomen);
                        Fuege_an(Alkene,L,a,d,b,c,ausgeben,ausgabe_bei_anzahl_c_atomen);
//                        Fuege_an(Alkene,L,a,d,c,b,ausgeben,ausgabe_bei_anzahl_c_atomen);
                    }
                  }
                }
              }
            }
        }
      }
      else
        a=L.size();
    }
  }


  //Fall ii): Zwei Wurzelbaeume sind gleich mit a > b > c = d
  if(n>3) // Dieser Fall kann erst ab 4 C-Atomen auftreten
  {
    for(a=p;a<L.size();a++)  // Baum a hat Hoehe h
    {
      if(L[a].size()<=n-2)   // Baum a hat hoechstens n-2 C-Atome
      {
        for(b=1;b<a;b++)     // falls c=d=0 muss b mindestens 1 sein
        {
          if(L[a].size()+L[b].size()<=n-2) // Haben Baum a und b zusammen hoechsten n-2 C-Atome
            {
              Fuege_an(Alkene,L,a,b,0,0,ausgeben,ausgabe_bei_anzahl_c_atomen);  // Falls c=d=0, fuege alle Isomere der Liste der Alkene an
              Fuege_an(Alkene,L,a,0,0,b,ausgeben,ausgabe_bei_anzahl_c_atomen);
              if(b>1)
              {
                for(c=1;c<b;c++)  // Falls c=d nicht der leere Baum ist
                {
                  if(L[a].size()+L[b].size()+2*L[c].size()<=n-2)  // Haben alle vier Baeume zusammen hoechstens n-2 C-Atome
                  {
                    Fuege_an(Alkene,L,a,b,c,c,ausgeben,ausgabe_bei_anzahl_c_atomen);  //Fuege alle Alken-Isomere der Liste an
//                    Fuege_an(Alkene,L,a,c,b,c,ausgeben,ausgabe_bei_anzahl_c_atomen);
                    Fuege_an(Alkene,L,a,c,c,b,ausgeben,ausgabe_bei_anzahl_c_atomen);
                  }
                }
              }
            }
        }
      }
      else
        a=L.size();
    }
  }


  //Fall iii): Zwei Wurzelbaeume sind gleich mit a > b = c > d
  if(n>3)  // Dieser Fall kann fruehestens ab 4 C-Atome auftreten
  {
    for(a=p;a<L.size();a++)  // Baum a hat Hoehe h
    {
      if(L[a].size()<=n-2)   // Baum a hat hoechstens n-2 C-Atome
      {
        for(b=1;b<a;b++)     // Baum b liegt zwischen 0 und a, da d 0 sein koennte
        {
          if(L[a].size()+2*L[b].size()<=n-2)  // Haben Baeume a, b, c zusammen hoechstens n-2 C-Atome?
            {
              Fuege_an(Alkene,L,a,b,b,0,ausgeben,ausgabe_bei_anzahl_c_atomen);  // Falls d=0 fuege alle Alken-Isomere der Liste an
              Fuege_an(Alkene,L,a,0,b,b,ausgeben,ausgabe_bei_anzahl_c_atomen);
              if(b>1)
              {
                for(d=1;d<b;d++)  // Wenn d nicht der leere Baum ist
                {
                  if(L[a].size()+2*L[b].size()+L[d].size()<=n-2)  // Alle Baeume zusammen hoechstens n-2 C-Atome?
                  {
                    Fuege_an(Alkene,L,a,b,b,d,ausgeben,ausgabe_bei_anzahl_c_atomen);  // Fuege die Isomere der Liste der Alkene an
//                    Fuege_an(Alkene,L,a,b,d,b,ausgeben,ausgabe_bei_anzahl_c_atomen);
                    Fuege_an(Alkene,L,a,d,b,b,ausgeben,ausgabe_bei_anzahl_c_atomen);
                  }
                }
              }
            }
        }
      }
      else
        a=L.size();
    }
  }

  //Fall iv): Zwei Wurzelbaeume sind gleich mit a = b > c > d
  if(n>6)  // Dieser Fall tritt erst am 7 C-Atome auf
  {
    for(a=p;a<L.size();a++)
    {
      if(2*L[a].size()<=n-2)
      {
        for(c=1;c<a;c++)
        {
          if(2*L[a].size()+L[c].size()<=n-2)
            {
              Fuege_an(Alkene,L,a,a,c,0,ausgeben,ausgabe_bei_anzahl_c_atomen);
              Fuege_an(Alkene,L,a,c,a,0,ausgeben,ausgabe_bei_anzahl_c_atomen);
              if(c>1)
              {
                for(d=1;d<c;d++)
                {
                  if(2*L[a].size()+L[c].size()+L[d].size()<=n-2)
                  {
                    Fuege_an(Alkene,L,a,a,c,d,ausgeben,ausgabe_bei_anzahl_c_atomen);
//                    Fuege_an(Alkene,L,a,c,a,d,ausgeben,ausgabe_bei_anzahl_c_atomen);
                    Fuege_an(Alkene,L,a,c,d,a,ausgeben,ausgabe_bei_anzahl_c_atomen);
                  }
                }
              }
            }
         }
      }
      else
        a=L.size();
    }
  }


  //Fall v): Je zwei Wurzelbaeume sind gleich mit a = b > c = d

  if(n>2)  // Dieser Fall tritt erst am 3 C-Atomen auf
  {
    for(a=p;a<L.size();a++)
    {
      if(2*L[a].size()<=n-2)
      {
        // c und d sind der leere Baum
        Fuege_an(Alkene,L,a,a,0,0,ausgeben,ausgabe_bei_anzahl_c_atomen);
        Fuege_an(Alkene,L,a,0,a,0,ausgeben,ausgabe_bei_anzahl_c_atomen);
        // c und d sind nicht der leere Baum
        if(a>1)
        {
          for(c=1;c<a;c++)
          {
            if(2*L[a].size()+2*L[c].size()<=n-2)
            {
              Fuege_an(Alkene,L,a,a,c,c,ausgeben,ausgabe_bei_anzahl_c_atomen);
              Fuege_an(Alkene,L,a,c,a,c,ausgeben,ausgabe_bei_anzahl_c_atomen);
//              Fuege_an(Alkene,L,a,c,c,a,ausgeben,ausgabe_bei_anzahl_c_atomen);
            }
          }
        }
      }
      else
        a=L.size();
    }
  }



    //Fall vi): Drei Wurzelbaeume sind gleich a > b = c = d


  if(n>2)  // Dieser Fall tritt erst ab 3 C-Atomen auf
  {
    for(a=p;a<L.size();a++)
    {
      if(L[a].size()<=n-2)
      {
        for(b=0;b<a;b++)
        {
          if(L[a].size()+3*L[b].size()<=n-2)
            {
              Fuege_an(Alkene,L,a,b,b,b,ausgeben,ausgabe_bei_anzahl_c_atomen);
            }
        }
       }
      else
        a=L.size();
    }
  }


  //Fall vii): Drei Wurzelbaeume sind gleich a = b = c > d


  if(n>4)  // Dieser Fall tritt erst ab 5 C-Atomen auf
  {
    for(a=p;a<L.size();a++)
    {
      if(3*L[a].size()<=n-2)
      {
        for(b=0;b<a;b++)
        {
          if(3*L[a].size()+L[b].size()<=n-2)
            {
              Fuege_an(Alkene,L,a,a,a,b,ausgeben,ausgabe_bei_anzahl_c_atomen);
            }
        }
      }
      else
        a=L.size();
    }
  }


  //Fall viii): Alle vier Wurzelbaeume sind gleich gross a = b = c = d

  for(a=p;a<L.size();a++)
    {
    if(4*L[a].size()<=n-2)
    {
      Fuege_an(Alkene,L,a,a,a,a,ausgeben,ausgabe_bei_anzahl_c_atomen);
    }
    else
      a=L.size();
  }




  if(ausgeben==false)
    cout << endl;                                 // Fortschrittsanzeiger
}


// ********************************************************************
// *                       Hauptprogramm                              *
// ********************************************************************

int main(void)
{
  unsigned int n;                           // maximale Anzahl der Knoten bzw C-Atome
  bool ausgeben;                            // sollen die konstruierten Alkene auf dem Bildschirm ausgegeben werden
  int eingabe;                              // Variable zum Einlesen von Eingaben
  int ausgabe_bei_anzahl_c_atomen;          // geben nur Alkene mit dieser Anzahl an C-Atomen auf dem Bildschirm aus

  cout << "Konstruktion von Isomeren der Alkene (Summenformel: C_nH_2n+2)" << endl;
  cout << "==============================================================" << endl;
  cout << endl;
  cout << "Bitte gebe die maximale Anzahl der C-Atome ein: ";
  cin  >> n;
  cout << "Sollen die konstruierten Alkene ausgegeben werden? (Nein=0, Ja=1) ";
  cin  >> eingabe;
  if(eingabe==1)
    ausgeben=true;
  else
    ausgeben=false;
  if(ausgeben==true)
  {
    cout << "Gebe bitte die Anzahl der C-Atome an, bei denen die konstruierten Alkene ausgegeben werden sollen. ";
    cout << "Moechtest du alle Alkene ausgegeben haben, so gebe bitte -1 ein. ";
    cin  >> ausgabe_bei_anzahl_c_atomen;
  }

  vector<Alken> Alkene;                     // Liste von Alkenen
  vector< vector <unsigned char> > L;       // Liste von Wurzelbaeumen

  // Konstruiere eine vollstaendige Liste von Wurzelbaeumen und damit alle Alkene aus bis zu n C-Atomen
  konstruiere_Wurzelbaeume(Alkene,L,n,ausgeben,ausgabe_bei_anzahl_c_atomen);


  // Zur Kontrolle gebe die Anzahl der Alkene aus
  cout << endl;
  vector<long> anz;
  unsigned int i;
  unsigned int sp;
  anz.resize(n+1);
  for(i=0;i<=n;i++)
    anz[i]=0;
  for(i=0;i<Alkene.size();i++)
  {
    sp=Alkene[i].Wurzelbaum_1.size()+Alkene[i].Wurzelbaum_2.size()+Alkene[i].Wurzelbaum_3.size()+Alkene[i].Wurzelbaum_4.size()+2;
    anz[sp]++;
  }
  cout << "Anzahl der Alkene mit n C-Atomen" << endl;
  cout << "--------------------------------" << endl;
  for(i=0;i<=n;i++)
  {
    cout << i;
    if(i<10 && n>=10) cout << " ";
    cout << " : " << anz[i] << endl;
  }
  return 0;
};

// ****************************************************************************
// *              Subroutinen zur Ausgabe eines Alkens                        *
// ****************************************************************************

int ausgabe(Alken& A)
{
  vector<unsigned int> Ebene;     // in welcher Ebene liegt der Knoten im Wurzelbaum
  vector<unsigned int> Position;  // Teile die Knoten eines Wurzelbaums in einer Ebene in 3^Ebene Positionen auf
  vector<unsigned int> Nachfolgervon;
  Ebene.resize(A.Wurzelbaum_1.size()+A.Wurzelbaum_2.size()+A.Wurzelbaum_3.size()+A.Wurzelbaum_4.size()+2);
  Position.resize(A.Wurzelbaum_1.size()+A.Wurzelbaum_2.size()+A.Wurzelbaum_3.size()+A.Wurzelbaum_4.size()+2);
  Nachfolgervon.resize(A.Wurzelbaum_1.size()+A.Wurzelbaum_2.size()+A.Wurzelbaum_3.size()+A.Wurzelbaum_4.size()+2);
  for(unsigned int i = 0; i<A.Wurzelbaum_1.size()+A.Wurzelbaum_2.size()+A.Wurzelbaum_3.size()+A.Wurzelbaum_4.size()+2; i++)
  {
    Ebene[i]=0;
    Position[i]=0;
    Nachfolgervon[i]=0;  //Anzahl der Nachfolger von Knoten i
  }




  // Berechne nun die Anzahl der Nachfolger vom Knoten i und speicher diese in Nachfolgervon[i]
  // Hat nun ein Knoten 3 Nachfolger, so muss kein weiteres H-Atom angehaengt werden, hat er 2 Nachfolger so fehlt ein H-Atom, hat er einen Nachfolger so fehlen 2 H-Atome, hat er keinen Nachfolger, so fehlen 3 H-Atome

  // Betrachte zuerst die beiden Knoten 0 und 1 die mit einer Zweifachbindung zusammenhaengen
  if(A.Wurzelbaum_1.size() > 0 && A.Wurzelbaum_2.size() > 0)
  {
    Nachfolgervon[0]=3;  // man braucht kein weiteres H-Atom
    Ebene[2]=0;
    Position[2]=0;
    Ebene[2+A.Wurzelbaum_1.size()]=0;
    Position[2+A.Wurzelbaum_1.size()]=0;
  }
  if((A.Wurzelbaum_1.size()==0 && A.Wurzelbaum_2.size() > 0) || (A.Wurzelbaum_1.size()>0 && A.Wurzelbaum_2.size()==0))
  {
    Nachfolgervon[0]=2;  // es muss ein H-Atom angehaengt werden (beachte die Zweifachbindung zwischen Knoten 0 und 1)
    Ebene[2]=0;
    Position[2]=0;
  }
  if(A.Wurzelbaum_1.size()==0 && A.Wurzelbaum_2.size()==0)
  {
    Nachfolgervon[0]=1;   // es muessen zwei H-Atome angefuegt werden
  } 

  if(A.Wurzelbaum_3.size() > 0 && A.Wurzelbaum_4.size() > 0)
  {
    Nachfolgervon[1]=3;  // man braucht kein weiteres H-Atom
    Ebene[2+A.Wurzelbaum_1.size()+A.Wurzelbaum_2.size()]=0;
    Position[2+A.Wurzelbaum_1.size()+A.Wurzelbaum_2.size()]=0;
    Ebene[2+A.Wurzelbaum_1.size()+A.Wurzelbaum_2.size()+A.Wurzelbaum_3.size()]=0;
    Position[2+A.Wurzelbaum_1.size()+A.Wurzelbaum_2.size()+A.Wurzelbaum_3.size()]=0;
  }
  if((A.Wurzelbaum_3.size()==0 && A.Wurzelbaum_4.size() > 0) || (A.Wurzelbaum_3.size()>0 && A.Wurzelbaum_4.size()==0))
  {
    Nachfolgervon[1]=2;  // es muss ein H-Atom angehaengt werden (beachte die Zweifachbindung zwischen Knoten 0 und 1)
    Ebene[2+A.Wurzelbaum_1.size()+A.Wurzelbaum_2.size()]=0;
    Position[2+A.Wurzelbaum_1.size()+A.Wurzelbaum_2.size()]=0;
  }
  if(A.Wurzelbaum_3.size()==0 && A.Wurzelbaum_4.size()==0)
  { 
    Nachfolgervon[1]=1;   // es muessen zwei H-Atome angefuegt werden
  }

  //  Betrachte nun die vier Wurzelbaeume
  unsigned int w1=A.Wurzelbaum_1.size();
  unsigned int w2=A.Wurzelbaum_2.size();
  unsigned int w3=A.Wurzelbaum_3.size();
  unsigned int w4=A.Wurzelbaum_4.size();
  for(unsigned int i = 1; i<w1; i++)
  {
    Ebene[2+i]=Ebene[2+A.Wurzelbaum_1[i]]+1;
    Position[2+i]=3*Position[2+A.Wurzelbaum_1[i]]+Nachfolgervon[2+A.Wurzelbaum_1[i]];
    Nachfolgervon[2+A.Wurzelbaum_1[i]]++;
  }
  for(unsigned int i = 1; i<w2; i++)
  {
    Ebene[2+w1+i]=Ebene[2+w1+A.Wurzelbaum_2[i]]+1;
    Position[2+w1+i]=3*Position[2+w1+A.Wurzelbaum_2[i]]+Nachfolgervon[2+w1+A.Wurzelbaum_2[i]];
    Nachfolgervon[2+w1+A.Wurzelbaum_2[i]]++;
  }
  for(unsigned int i = 1; i<w3; i++)
  {
    Ebene[2+w1+w2+i]=Ebene[2+w1+w2+A.Wurzelbaum_3[i]]+1;
    Position[2+w1+w2+i]=3*Position[2+w1+w2+A.Wurzelbaum_3[i]]+Nachfolgervon[2+w1+w2+A.Wurzelbaum_3[i]];
    Nachfolgervon[2+w1+w2+A.Wurzelbaum_3[i]]++;
  }
  for(unsigned int i = 1; i<w4; i++)
  {
    Ebene[2+w1+w2+w3+i]=Ebene[2+w1+w2+w3+A.Wurzelbaum_4[i]]+1;
    Position[2+w1+w2+w3+i]=3*Position[2+w1+w2+w3+A.Wurzelbaum_4[i]]+Nachfolgervon[2+w1+w2+w3+A.Wurzelbaum_4[i]];
    Nachfolgervon[2+w1+w2+w3+A.Wurzelbaum_4[i]]++;
  }


  ofstream datei("graph", ios_base::app);

  datei << "<GRAPH NAME=\"C" << w1+w2+w3+w4+2 << "H" << 2*(w1+w2+w3+w4+2) << "\" NUMBER_OF_VERTICES=\"";
  datei << w1+w2+w3+w4+2+2*(w1+w2+w3+w4+2) << "\" NUMBER_OF_EDGES=\"" << w1+w2+w3+w4+2+2*(w1+w2+w3+w4+2);
  datei << "\" MULTIGRAPH=\"1\"" << " MOLECULAR=\"1\">" << endl;
  datei << "<VERTEXLABELS>" << endl;
  for(unsigned int i=0; i< w1+w2+w3+w4+2; i++)
  {
    datei << "\"C\"" << endl;
  }
  for(unsigned int i=0; i<2*(w1+w2+w3+w4+2); i++)
  {
    datei << "\"H\"" << endl;
  }
  datei << "</VERTEXLABELS>" << endl << "<EDGELIST>" << endl << "0 1" << endl << "0 1" << endl;

  if(w1 != 0)
  {
    datei << "2 0" << endl;
    for(unsigned int i=1; i<w1; i++)
    {
      datei << i+2 << " " << A.Wurzelbaum_1[i]+2 << endl;
    }
  }

  if(w2 != 0)
  {
    datei << 2+w1 << " 0" << endl;
    for(unsigned int i=1; i<w2; i++)
    {
      datei << i+2+w1 << " " << A.Wurzelbaum_2[i]+2+w1 << endl;
    }
  }

  if(w3 != 0)
  {
    datei << 2+w1+w2 << " 1" << endl;
    for(unsigned int i=1; i<w3; i++)
    {
      datei << i+2+w1+w2 << " " << A.Wurzelbaum_3[i]+2+w1+w2 << endl;
    }
  }

  if(w4 != 0)
  {
    datei << 2+w1+w2+w3 << " 1" << endl;
    for(unsigned int i=1; i<w4; i++)
    {
      datei << i+2+w1+w2+w3 << " " << A.Wurzelbaum_4[i]+2+w1+w2+w3 << endl;
    }
  }


  unsigned int knoten;
  knoten = w1+w2+w3+w4+2;      // gibt die aktuelle Nummer des neu zu erstellenden Knotens an;
  for(unsigned int i=0; i<2+w1+w2+w3+w4; i++) // Ueberpruefe C-Atome auf fehlende H-Atome und fuege ggf. hinzu
  {
    if(Nachfolgervon[i]<3)                       // Fehlen H-Atome?
    {
      if(Nachfolgervon[i]==2)                    // Haenge ein weiteres H-Atom an den i-ten Knoten
      {
        datei << knoten << " " << i << endl;     // Fuege die Kante zwischen dem H-Atom und dem C-Atom ein
        knoten ++;                               // Erhoehe die aktuelle Anzahl der Knoten
      }
      if(Nachfolgervon[i]==1)                    // Haenge zwei weitere H-Atome an den i-ten Knoten
      {
        datei << knoten << " " << i << endl;
        knoten ++;
        datei << knoten << " " << i << endl;
        knoten ++;
      }
      if(Nachfolgervon[i]==0)                    // Haenge drei weitere H-Atome an den i-ten Knoten
      {
        datei << knoten << " " << i << endl;
        knoten ++;
        datei << knoten << " " << i << endl;
        knoten ++;
        datei << knoten << " " << i << endl;
        knoten ++;
      }
    }
  }
  datei << "</EDGELIST>" << endl;
  long double x,y,z;
  unsigned int eb,pos;
  datei << "<COORDS2D_FLOAT>" << endl;
    datei << "0 0.5" << endl;
    datei << "0 -0.5" << endl;
    for(unsigned int i=0;i<w1;i++)
    {
      if(Ebene[2+i]==0)
        z=0;
      else
        z=Position[2+i]*1.*Ebene[2+i]/(pow((long double)3,(int)Ebene[2+i])-1);
      x=-((int)Ebene[2+i])-1+z;
      y=1.5+z;
      datei << x << " " << y << endl;
    }
    for(unsigned int i=0;i<w2;i++)
    {
      if(Ebene[2+w1+i]==0)
        z=0;
      else
        z=Position[2+w1+i]*1.*Ebene[2+w1+i]/(pow((long double)3,(int)Ebene[2+w1+i])-1);
      x=((int)Ebene[2+w1+i])+1-z;
      y=1.5+z;
      datei << x << " " << y << endl;
    }
    for(unsigned int i=0;i<w3;i++)
    {
      if(Ebene[2+w1+w2+i]==0)
        z=0;
      else
        z=Position[2+w1+w2+i]*1.*Ebene[2+w1+w2+i]/(pow((long double)3,(int)Ebene[2+w1+w2+i])-1);
      x=-((int)Ebene[2+w1+w2+i])-1+z;
      y=-1.5-z;
      datei << x << " " << y << endl;
    }
    for(unsigned int i=0;i<w4;i++)
    {
      if(Ebene[2+w1+w2+w3+i]==0)
        z=0;
      else
        z=Position[2+w1+w2+w3+i]*1.*Ebene[2+w1+w2+w3+i]/(pow((long double)3,(int)Ebene[2+w1+w2+w3+i])-1);
      x=((int)Ebene[2+w1+w2+w3+i])+1-z;
      y=-1.5-z;
      datei << x << " " << y << endl;
    }
    if(w1==0) datei << "-1 1.5" << endl;
    if(w2==0) datei << "1 1.5" << endl;
    if(w3==0) datei << "-1 -1.5" << endl;
    if(w4==0) datei << "1 -1.5" << endl;
    for(unsigned int i=0;i<w1;i++)
      for(unsigned j=0;j<3-Nachfolgervon[2+i];j++)
      {
        pos=3*Position[2+i]+2-j;         
        eb=Ebene[2+i]+1;
        z=pos*1.*eb/(pow((long double)3,(int)eb)-1);
        x=-((int)eb)-1+z;
        y=1.5+z;
        datei << x << " " << y << endl;
      }
    for(unsigned int i=0;i<w2;i++)
      for(unsigned j=0;j<3-Nachfolgervon[2+w1+i];j++)
      {
        pos=3*Position[2+w1+i]+2-j;         
        eb=Ebene[2+w1+i]+1;
        z=pos*1.*eb/(pow((long double)3,(int)eb)-1);
        x=((int)eb)+1-z;
        y=1.5+z;
        datei << x << " " << y << endl;
      }
    for(unsigned int i=0;i<w3;i++)
      for(unsigned j=0;j<3-Nachfolgervon[2+w1+w2+i];j++)
      {
        pos=3*Position[2+w1+w2+i]+2-j;         
        eb=Ebene[2+w1+w2+i]+1;
        z=pos*1.*eb/(pow((long double)3,(int)eb)-1);
        x=-((int)eb)-1+z;
        y=-1.5-z;
        datei << x << " " << y << endl;
      }
    for(unsigned int i=0;i<w4;i++)
      for(unsigned j=0;j<3-Nachfolgervon[2+w1+w2+w3+i];j++)
      {
        pos=3*Position[2+w1+w2+w3+i]+2-j;         
        eb=Ebene[2+w1+w2+w3+i]+1;
        z=pos*1.*eb/(pow((long double)3,(int)eb)-1);
        x=((int)eb)+1-z;
        y=-1.5-z;
        datei << x << " " << y << endl;
      }
  datei << "</COORDS2D_FLOAT>" << endl; 
  datei << "</GRAPH>" << endl << endl;
  datei.close();
  return 0;
}
