// *********************************************************************
// *                                                                   *
// *       Konstruktion von Isomeren der Alkine (C_nH_2n-2)            *
// *       ================================================            *
// *                                                                   *
// *       Autor:        Sascha Kurz                                   *
// *       modifiert im Rahmen eines Softwarepraktikums durch:         *
// *                     Sabine Boehm und Florian Haeberlein           * 
// *       Email:        sascha.kurz@uni-bayreuth.de                   *
// *       Homepage:     www.mathe2.uni-bayreuth.de/sascha             *
// *                                                                   *
// *       Filename:           konstruiere_alkine.cpp                  *
// *       zuletzt geaendert:  11.01.2005                              *
// *       Betriebssystem:     Suse Linux 9.1                          *
// *       Compiler:           gcc3.3.3                                *
// *       Compiler-Optionen:  g++ -O2 -Wall -pedantic -ansi           *
// *                           konstruiere_alkine.cpp                  *
// *                           -o konstruktion                         *
// *                                                                   *
// *       Algorithmus                                                 *
// *       -----------                                                 *
// *       Basierend auf dem Artikel "Anzahl von Strukturisomeren      *
// *       der Alkane". Es werden rekursiv Wurzelbaeume zusammenge-    *
// *       setzt, die wiederum zum Zusammensetzen von Alkinen ver-     *
// *       wendet werden.                                              *
// *                                                                   *
// *       + Wurzelbaeume                                              *
// *       -------------                                               *
// *       Ebene 3        o o o o o      Der Knoten in Ebene 1         *
// *                       \|  \|/       heisst Wurzel. Jeder Kno-     *
// *       Ebene 2          o o o        ten, ausser der Wurzel, hat   *
// *                         \|/         genau eine Kante zu einem     *
// *       Ebene 1            o          Knoten auf einer um 1 nie-    *
// *                                     drigeren Ebene.               *
// *                                                                   *
// *       Abgespeichert werden Wurzelbaeume als Array. Hierzu num-    *
// *       meriert man die Knoten, und speichert zu jedem Knoten       *
// *       die Nummer des Knotens auf der um eines niedrigeren Ebene   *
// *       zu dem er eine Kante besitzt. Gibt man der Wurzel die       *
// *       Nummer 0, so kann man als Konvention im Array an der        *
// *       Position 0 ebenfalls die 0 speichern.                       *
// *                                                                   *
// *       Ebene 3        4 5 6 7 8      Nummerierte Version des       *
// *                       \|  \|/       obigen Wurzelbaums. Das       *
// *       Ebene 2          1 2 3        entsprechende Array ist       *
// *                         \|/         in diesem Fall:               *
// *       Ebene 1            0          [0,0,0,0,1,1,3,3,3]           *
// *                                                                   *
// *       + Zusammensetzen von Wurzelbaeumen                          *
// *       ---------------------------------                           *
// *       Die Anzahl der Ebenen eines Wurzelbaums bezeichnet man      *
// *       als seine Hoehe. Wurzelbaeume der Hoehe h kann man folgt    *
// *       aus Wurzelbaeumen kleinerer Hoehe zusammensetzen:           *
// *                                                                   *
// *       4 5 6 7 8   2 3 4              5 6  7 8 9 12 13 14          *
// *        \|  \|/     \|/                \|   \|/    \|/             *
// *         1 2 3       1     1   --->     2 3 4      11 16           *
// *          \|/        |     |             \|/        | |            *
// *           0         0     0              1        10 15           *
// *                                           \        |/             *
// *                                            --------0              *
// *                                                                   *
// *       Das Zusammensetzen und Umnummerieren von Wurzelbaeumen,     *
// *       wird von der Funktion "Fuege_an()" erledigt. Diese fuegt    *
// *       den neu konstruierten Wurzelbaum gleichzeitig noch an       *
// *       eine Liste L von Wurzelbaeumen an. Die Wurzelbaeume         *
// *       kleinerer Hoehe werden als Positionen in der Liste L        *
// *       uebergeben. Da in unserer Anwendung jeder Knoten nur        *
// *       hoechstens 4 Nachbarn hat, brauchen nur 3 int-Werte a,b,c   *
// *       uebergeben werden.                                          *
// *                                                                   *
// *       + Erzeugen von Wurzelbaeumen                                *
// *       ---------------------------                                 *
// *       Im vorhergehenden Abschnitt haben wir erklaert, wie         *
// *       Wurzelbaeume mit kleinerer Hoehe zu Wurzelbaeumen mit       *
// *       groesserer Hoehe zusammengesetzt werden. Hierzu startet     *
// *       man mit einer Liste von Wurzelbaeumen der Hoehe kleiner-    *
// *       gleich h-1 und konstruieren daraus eine Liste von Wurzel-   *
// *       baeumen der Hoehe kleinergleich h. Fuer den Start initiali- *
// *       siert man die Liste mit dem leeren Wurzelbaum und dem       *
// *       Wurzelbaum der Hoehe 1.Nun waehlt man 3 Wurzelbaeume aus der*
// *       Liste der Wurzelbaeume der Hoehe kleinergleich h-1 aus,     *
// *       wobei mindestens einer davon exakt die Hoehe h-1 besitzen   *
// *       muss, und wendet die Funktion "Fuege_an()" an, um aus den   *
// *       3 Wurzelbaeumen einen neuen Wurzelbaum der Hoehe h zu       *
// *       konstruieren. Erledigt wird diese Arbeit von der Funktion   *
// *       "konstruiere_Wurzelbaeume()". Aus Geschwindigkeitsgruenden  *
// *       wird nur mit einer Liste von Wurzelbaeumen gearbeitet, und  *
// *       die Auswahl der 3 Wurzelbaeume wird vorausschauend durchge- *
// *       fuehrt, so dass keine Wurzelbaeume mit "zu vielen" Knoten   *
// *       konstruiert werden.                                         *
// *                                                                   *
// *       + Zusammensetzen von Wurzelbaeumen zu Alkinen               *
// *       --------------------------------------------                *
// *       Wir koennen ein Alkin speichern, als eine Menge von         *
// *       2 Wurzelbaeumen.                                            *
// *       Die Konstruktion der Alkine aus Wurzelbaeumen uebernimmt die*
// *       Funktion "konstruiere_Alkine()". Das eigentlich zusammen-   *
// *       setzen zu einem Alkin wird von der Funktion "Fuege_an()"    *
// *       erledigt, die das Alkin auch gleichzeitig an eine Liste     *
// *       aus Alkinen anhaengt.                                       *
// *                                                                   *
// *       + Ausgeben von Alkinen auf dem Bildschirm                   *
// *       -----------------------------------------                   *
// *       Die Alkine koennen im XML-Format ausgegeben werden, und     *
// *       dann z.B. mit dem Graphviewer betrachtet werden. Die        *
// *       angegebenen Koordinaten haben wenig mit der Realitaet       *
// *       zu tun und stellen auch nicht unbedingt eine schoene        *
// *       Zeichnung des Graphens dar, doch sind sie immerhin so       *
// *       berechnet, dass die Kanten sich nicht ueberschneiden, und   *
// *       man ein ungefaehres Bild von dem Molekuel bekommen kann.    *   
// *                                                                   *
// *                                                                   *
// *       Schlussbemerkung                                            *
// *       ===============                                             *
// *       Bei Fragen, Anmerkungen, Kritik oder sonstigen Kommentaren  *
// *       wende man sich bitte per email an den oben genannten Autor  *
// *       dieses Programms. Der Quellcode ist fuer nicht-kommerzielle *
// *       Anwendungen frei verfuegbar und darf beliebig veraendert    *
// *       werden. Bei Veraenderungen ist geeignet darauf hinzuweisen, *
// *       von wem sie vorgenommen wurden. Es wuerde mich freuen ueber *
// *       Verwendungen dieses Quellcodes eine kurze Nachricht per     *
// *       email zu erhalten.                                          *
// *                                                                   *
// *       Sascha Kurz                                                 *
// *                                                                   *
// *********************************************************************



#include<vector>              // Headerfile fuer die Klasse Vector
#include<iostream>            // Headerfile fuer die Ein- und Ausgabe
#include<fstream>             // Headerfile fuer die Ausgabe in eine Datei



using namespace std;

struct Alkin                               // Datenstruktur fuer ein Alkin
{
  vector <unsigned char> Wurzelbaum_1;     // Vektor fuer den ersten Wurzelbaum
  vector <unsigned char> Wurzelbaum_2;     // Vektor fuer den zweiten Wurzelbaum
};

// *********************************************************************
// *                  Prototypdeklarationen                            *
// *********************************************************************

void Fuege_an(vector< vector <unsigned char> >& L,int a,int b,int c);
// Konstruiert aus 3 Wurzelbaeumen einen neuen Wurzelbaum mit groesserer
// Hoehe und fuegt ihn der Liste L an

void Fuege_an(vector<Alkin>&,const vector< vector <unsigned char> >& L,
unsigned int a,unsigned int b,bool ausgeben,int ausgabe_bei_anzahl_c_atomen);
// Konstruiert aus 2 Wurzelbaeumen der Hoehe kleinergleich h ein Alkin
// und fuegt es der Liste Alkine an

void sortiere_Wurzelbaum_Liste(vector< vector <unsigned char> >& L,int p);
// Sortiert eine Liste L von Wurzelbaeumen, so dass die ersten p Wurzelbaeumen
// und die restlichen Wurzelbaeumen jeweils nach der Anzahl der Knoten
// aufsteigend angeordnet sind

void konstruiere_Wurzelbaeume(vector<Alkin>& Alkine,vector< vector <unsigned char> >& L,int n,bool ausgeben,int ausgabe_bei_anzahl_c_atomen);
// Konstuiert eine "vollstaendige" Liste von Wurzelbaeumen, so dass damit
// eine vollstaendige Liste aller Alkine mit bis zu n C-Atomen konstruiert
// werden kann.

void konstruiere_Alkine(vector<Alkin>& Alkine,const vector< vector <unsigned char> >& L,
unsigned int p,unsigned int h,unsigned int n,bool ausgeben,int ausgabe_bei_anzahl_c_atomen);
// Erweitert eine Liste von Alkinen mit denen solchen die eine laengste gerade
// Kette der Laenge 2h+2 und maximal n Knoten haben. Die dafuer notwendigen
// Wurzelbaeume der Hoehe h stehen in der Liste L ab Position p

int ausgabe(Alkin& A);
// Gibt einen Graphen des Alkins im Graph xml-Format in der Datei graph aus

// ********************************************************************
// *         Implementierungen der benoetigten Funktionen              *
// ********************************************************************

void Fuege_an(vector< vector <unsigned char> >& L,int a,int b,int c)
{
  // L:   Liste von Wurzelbaeumen
  // a:   Position eines Wurzelbaums in der Liste L
  // b:   Position eines Wurzelbaums in der Liste L
  // c:   Position eines Wurzelbaums in der Liste L
  //
  // Funktion:
  // ---------
  // Konstruiert einen neuen Wurzelbaum W, indem die Wurzelbaeume Nr. a,b
  // und c vereinigt werden, und die Wurzeln von a,b und c mit einer
  // neuen Wurzel verbunden werden.
  //
  //      a b c
  //  W=   \|/
  //        o
  //
  // Der neu konstruierte Wurzelbaum W wird an die Liste L angehaengt

  int p=L.size();               // die alte Laenge der Liste der Wurzelbaeume
  unsigned int i;               // Schleifenvariable
  int a_s=L[a].size();          // aus wie vielen Knoten besteht Wurzelbaum Nr. a
  int b_s=L[b].size();          // aus wie vielen Knoten besteht Wurzelbaum Nr. b
  int c_s=L[c].size();          // aus wie vielen Knoten besteht Wurzelbaum Nr. c
  L.resize(p+1);                // Verlaengere die Liste um ein Element
  L[p].resize(a_s+b_s+c_s+1);   // Der neue Wurzelbaum setzt sich aus Wurzelbaum Nr. a,
                                // Wurzelbaum Nr. b, Wurzelbaum Nr. c und einem zusaetzlichen
                                // Knoten, der neuen Wurzel, zusammen
  L[p][0]=0;                    // neue Wurzel
  L[p][1]=0;                    // alte Wurzel von Wurzelbaum Nr. a zeigt auf neue Wurzel
  for(i=1;i<L[a].size();i++)    // durchlaufe die restlichen Knoten von Wurzelbaum Nr. a
    L[p][i+1]=L[a][i]+1;        // verschiebe die alte Nummerierung
  L[p][1+a_s]=0;                // alte Wurzel von Wurzelbaum Nr. b zeigt auf neue Wurzel
  for(i=1;i<L[b].size();i++)    // durchlaufe die restlichen Knoten von Wurzelbaum Nr. a
    L[p][i+1+a_s]=L[b][i]+1+a_s;// verschiebe die alte Nummerierung
  L[p][1+a_s+b_s]=0;            // alte Wurzel von Wurzelbaum Nr. c zeigt auf neue Wurzel
  for(i=1;i<L[c].size();i++)    // durchlaufe die restlichen Knoten von Wurzelbaum Nr. a
    L[p][i+1+a_s+b_s]=L[c][i]+1+a_s+b_s; // verschiebe die alte Nummerierung
}

void Fuege_an(vector<Alkin>& Alkine,const vector< vector <unsigned char> >& L,unsigned int a,
unsigned int b,bool ausgeben,int ausgabe_bei_anzahl_c_atomen)
{
  // Alkine : Liste von Alkinen
  // L:       Liste von Wurzelbaeumen
  // a:       Position eines Wurzelbaums in der Liste L
  // b:       Position eines Wurzelbaums in der Liste L
  //
  // Funktion:
  // ---------
  // Konstruiert aus zwei Wurzelbaeumen W_1 und W_2 der  Hoehe h ein Alkin A mit laengster gerader Kette 2h+2:
  //
  //  A=  W_1-C---C-W_2 (die Wurzeln von W_1 und W_2 werden mit einem Ethin-Molekuel verbunden)
  //
  // Das neu konstruierte Alkin A wird an die Liste Alkine angehaengt

  unsigned int i;               // Schleifenvariable;
  int p=Alkine.size();          // die alte Laenge der Liste der Alkine
  Alkine.resize(p+1);           // verlaengere die Liste um ein Element

  Alkine[p].Wurzelbaum_1.resize(L[a].size());   // Reserviere Speicherplatz fuer den Wurzelbaum Nr. a
  for(i=0;i<L[a].size();i++)                    // Kopiere Wurzelbaum Nr. a
    Alkine[p].Wurzelbaum_1[i]=L[a][i];
  Alkine[p].Wurzelbaum_2.resize(L[b].size());   // Reserviere Speicherplatz fuer den Wurzelbaum Nr. b
  for(i=0;i<L[b].size();i++)                    // Kopiere Wurzelbaum Nr. b
    Alkine[p].Wurzelbaum_2[i]=L[b][i];


  if(ausgeben==true)
  {
    if(ausgabe_bei_anzahl_c_atomen==-1|| (int)(Alkine[p].Wurzelbaum_1.size()+Alkine[p].Wurzelbaum_2.size())+2==ausgabe_bei_anzahl_c_atomen)
    ausgabe(Alkine[p]);
  }

}

void sortiere_Wurzelbaum_Liste(vector< vector <unsigned char> >& L,int laenge,int n)
{
  // L:       Liste von Wurzelbaeumen
  // laenge:  Position in Liste L, siehe "Funktion"
  // n:       maximale Anzahl der Knoten
  //
  // Funktion:
  // ---------
  // Sortiert die Positionen 0 bis laenge-1 der Liste L so um, dass die Anzahl der
  // Knoten der Wurzelbaeume aufsteigend ist. Die Positionen laenge bis L.size()-1
  // werden entsprechend sortiert.
  // Da die maximale Anzahl der Knoten (n) gegenueber der Laenge der Liste eine sehr
  // kleine Zahl ist, werden anstatt einem Sortieralgorithmus wie Mergesort oder
  // Quicksort einfach n+1 Listen angelegt, und jeder Wurzelbaum wird in die Liste
  // mit der Nummer entsprechend der Anzahl seiner Knoten gelegt. Anschliessend
  // werden diese Listen wieder zu einer zusammengefuegt. Der Aufwand betraegt somit
  // O(n+L.size())

  int i,j;            // Hilfsvariablen
  unsigned int k,a,b; // Hilfsvariablen

  vector< vector< vector<unsigned char> > > sort;     // Hilfsvektor zum Sortieren

  sort.resize(n+1);                    // Anlegen der n+1 Hilfslisten
  for(i=0;i<=n;i++)                    // durchlaufe die Hilfslisten
    sort[i].resize(0);                 // Jede Liste ist am Anfang leer

  // Trage die Wurzelbaeume, der Positionen 0,...,laenge-1, in die Hilfslisten ein
  for(i=0;i<laenge;i++)                // durchlaufe die Positionen 0 bis laenge-1 von L
  {
    k=L[i].size();                     // Wie viele Knoten besitzt der gerade betrachtete Wurzelbaum?
    a=sort[k].size();                  // Wie lang ist die entsprechende Hilfsliste
    sort[k].resize(a+1);               // Verlaengere die Hilfsliste um ein Element
    sort[k][a].resize(k);              // Der neue Wurzelbaum besteht aus k Knoten
    for(b=0;b<k;b++)                   // Fuege den betrachteten Wurzelbaum in die
      sort[k][a][b]=L[i][b];           // Hilfsliste ein
  }

  // Trage die Wurzelbaeume aus den Hilfslisten wieder zurueck in L ein
  j=0;                                 // Aktuelle Position in L
  for(i=0;i<=n;i++)                    // durchlaufe die unterschiedlichen Hilfslisten
  {
    for(a=0;a<sort[i].size();a++)      // durchlaufe Hilfsliste i
    {
      L[j].resize(i);                  // Speichere an Position j in L einen Wurzelbaum bestehend aus i Knoten
      for(b=0;b<(unsigned int)i;b++)
        L[j][b]=sort[i][a][b];
      j++;                             // Verschiebe die aktuelle Position in L um eins nach rechts
    }
  }

  sort.resize(n+1);                    // "Loesche" die Hilfslisten wieder
  for(i=0;i<=n;i++)
  {
    for(a=0;a<sort[i].size();a++)
      sort[i][a].resize(0);
    sort[i].resize(0);
  }

  // Trage die Wurzelbaeume, der Positionen laenge,...,L.size()-1, in die Hilfslisten ein
  for(i=laenge;i<(int)L.size();i++)         // durchlaufe die Positionen laenge bis L.size()-1 von L
  {
    k=L[i].size();                     // Wie viele Knoten besitzt der gerade betrachtete Wurzelbaum?
    a=sort[k].size();                  // Wie lang ist die entsprechende Hilfsliste
    sort[k].resize(a+1);               // Verlaengere die Hilfsliste um ein Element
    sort[k][a].resize(k);              // Der neue Wurzelbaum besteht aus k Knoten
    for(b=0;b<k;b++)                   // Fuege den betrachteten Wurzelbaum in die
      sort[k][a][b]=L[i][b];           // Hilfsliste ein
  }

  // Trage die Wurzelbaeume aus den Hilfslisten wieder zurueck in L ein
  for(i=0;i<=n;i++)                    // durchlaufe die unterschiedlichen Hilfslisten
  {
    for(a=0;a<sort[i].size();a++)      // durchlaufe Hilfsliste i
    {
      L[j].resize(i);                  // Speichere an Position j in L einen Wurzelbaum bestehend aus i Knoten
      for(b=0;b<(unsigned int)i;b++)
        L[j][b]=sort[i][a][b];
      j++;                             // Verschiebe die aktuelle Position in L um eins nach rechts
    }
  }

  sort.resize(n+1);                    // "Loesche" die Hilfslisten wieder
  for(i=0;i<=n;i++)
  {
    for(a=0;a<sort[i].size();a++)
      sort[i][a].resize(0);
    sort[i].resize(0);
  }
}


void konstruiere_Wurzelbaeume(vector<Alkin>& Alkine,vector< vector <unsigned char> >& L,int n,bool ausgeben,int ausgabe_bei_anzahl_c_atomen)
{
  // L: Liste zum Speichern von Wurzelbaeumen
  // n: maximale Anzahl von C-Atomen in den spaeter zu konstruierenden Alkinen
  //
  // Funktion:
  // ---------
  // Ziel ist es eine vollstaendige Liste aller Wurzelbaeume der Hoehe h mit maximal n-2 Knoten
  // zu konstruieren. Hierbei laeuft h von 0 bis n-2.
  // Dabei werden Wurzelbaeume der Hoehe h aus Wurzelbaeumen der Hoehe kleinergleich h-1 zusammen-
  // gesetzt. Fuer weitere Details lese den Artikel:"Anzahl von Strukturisomeren der Alkane"



  // Startbelegung fuer die Liste der Wurzelbaeume
  L.resize(2);                              // Am Anfang sind nur 2 Wurzelbaeume in der Liste eingetragen
  L[0].resize(0);                           // Leerer Wurzelbaum aus 0 Knoten
  L[1].resize(1);
  L[1][0]=0;                                // Wurzelbaum aus einem Knoten
  int p=1;                                  // Position des Wurzelbaums der Hoehe 1
                                            // Allgemein: Position des ersten Wurzelbaums der Hoehe h in der
                                            // Liste L

  // Startbelegung fuer die Liste der Alkine
  Alkine.resize(1);
  // Ethin
  Alkine[0].Wurzelbaum_1.resize(0);
  Alkine[0].Wurzelbaum_2.resize(0);



  cout << endl;
  if(ausgeben==true)
    if(ausgabe_bei_anzahl_c_atomen==-1 || ausgabe_bei_anzahl_c_atomen==2)
      ausgabe(Alkine[0]);

  if(ausgeben==false)
  {
    cout << endl;
    cout << "Konstruiere Wurzelbaeume der Hoehe 1 ";
    if(n>=20) cout << " ";
    cout << ": ." << endl;
  }

  konstruiere_Alkine(Alkine,L,p,1,n,ausgeben,ausgabe_bei_anzahl_c_atomen);


  int a,b,c;                                // Position der drei kleineren Wurzelbaeume in der Liste L
  int laenge;                               // Hilfsgroesse, um "p" bzw. die Laenge der Liste auch bei
                                            // Aenderungen an der Liste verfuegbar zu haben

  for(int h=2;h<=int(n-2);h++)              // Durchlaufe die moeglichen Hoehen der Wurzelbaeume. Die Wurzelbaeume
  {                                         // der Hoehe 0 und 1 wurden mit der Startbelegung behandelt.
    // Forstschrittsausgabe, da Berechnung lange dauern kann
    if(ausgeben==false)
    {
      cout << "Konstruiere Wurzelbaeume der Hoehe " << h << " ";
      if(n>=20 && h<10) cout << " ";
      cout << ": ";
    }


    laenge=L.size();                        // Wie  lang ist die Liste L bisher

    // In der Liste L stehen an den Positionen 0 bis p-1 Wurzelbaeume der Hoehe kleiner h-1
    // geordnet nach der Anzahl der Knoten. An den Positionen p bis laenge-1 stehen
    // Wurzelbaeume der Hoehe h-1, ebenfalls nach der Anzahl der Knoten geordnet.
    // Fuer die Kombination zu einem Wurzelbaum der Hoehe h gibt es nun 3 Moeglichkeiten:
    // (i)   zwei Wurzelbaeume der Hoehe kleiner h-1 und ein Wurzelbaum der Hoehe h-1
    // (ii)  ein Wurzelbaum der Hoehe kleiner h-1 und zwei Wurzelbaeume der Hoehe h-1
    // (iii) drei Wurzelbaeume der Hoehe h-1
    //
    // Die Positionen der Wurzelbaeume in L werden mit a,b und c bezeichnet. Um die Konstruktion
    // zu beschleunigen, wird bei jedem Schritt ueberprueft, ob ueberhaupt ein Alkin mit hoechstens
    // n C-Atomen entstehen kann.


    // Fall (i)
    for(a=0;a<p;a++)                                // a ist ein Wurzelbaum der Hoehe kleiner h-1
    {
      if(ausgeben==false)
        cout << "." << flush;                       // Forstschrittsausgabe, da Berechnung lange dauern kann
      if(1+(int)L[a].size()*2+(h-1)<=n-2)         // 1:               die neue Wurzel
      {                                             // L[a].size()*2: b besitzt mindestens genau so viele
                                                    //                  Knoten wie a, also fuer a und b zusammen
                                                    //                  L[a].size()*2
                                                    // (h-1):           Wurzelbaum c besitzt Hoehe h-1, also mindestens
                                                    //                  h-1 Knoten
        for(b=a;b<p;b++)                            // b ist ein Wurzelbaum der Hoehe kleiner h-1
        {
          if(1+(int)L[a].size()+(int)L[b].size()+(h-1)<=n-2)  // 1:           die neue Wurzel
          {                                         // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                    // L[b].size(): Anzahl der Knoten von Wurzelbaum b
                                                    // (h-1):       Wurzelbaum c besitzt Hoehe h-1, also mindestens
                                                    //              h-1 Knoten
            for(c=p;c<laenge;c++)                   // c ist ein Wurzelbaum der Hoehe h-1
            {
              if(1+(int)L[a].size()+(int)L[b].size()+(int)L[c].size()<=n-2) // 1:           die neue Wurzel
              {                                              // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                             // L[b].size(): Anzahl der Knoten von Wurzelbaum b
                                                             // L[c].size(): Anzahl der Knoten von Wurzelbaum b
                Fuege_an(L,a,b,c);                  // Fuege die Wurzelbaeume a,b und c zusammen und haenge sie an
              }                                     // die Liste L an
              else                                  // Der entehende Wurzelbaum kann nicht zur Konstruktion eines
                c=laenge;                           // Alkins mit maximal n C-Atomen verwendet werden, also gehe mit
            }                                       // b eins weiter und starte mit c wieder bei Position p
          }
          else                                      // Die entehenden Wurzelbaeume koennen nicht zur Konstruktion eines
          {                                         // Alkins mit maximal n C-Atomen verwendet werden, also gehe mit
            b=p;                                    // a eins weiter und starte mit b wieder bei Position a
          }
        }
      }
      else                                          // Die entehenden Wurzelbaeume koennen nicht zur Konstruktion eines
      {                                             // Alkins mit maximal n C-Atomen verwendet werden, also beende die
        a=p;                                        // Schleife
      }
    }


    // Fall (ii)
    for(a=0;a<p;a++)                                // a ist ein Wurzelbaum der Hoehe kleiner h-1
    {
      if(ausgeben==false)
        cout << "." << flush;                       // Forstschrittsausgabe, da Berechnung lange dauern kann
      if(1+(int)L[a].size()+2*(h-1)<=n-2)           // 1:           die neue Wurzel
      {                                             // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                    // 2*(h-1):     Die Wurzelbaeume b und c besitzen die Hoehe
                                                    //              h-1, bestehen also aus mindestens h-1 Knoten
        for(b=p;b<laenge;b++)                       // b ist ein Wurzelbaum der Hoehe h-1
        {
          if(1+(int)L[a].size()+2*((int)L[b].size())<=n-2)      // 1:           die neue Wurzel
          {                                         // L[a].size(): Anzahl der Knoten von Wurzelbaum a

            for(c=b;c<laenge;c++)                   // c ist ein Wurzelbaum der Hoehe h-1
              if(1+(int)L[a].size()+(int)L[b].size()+(int)L[c].size()<=n-2) // 1:           die neue Wurzel
              {                                              // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                             // L[b].size(): Anzahl der Knoten von Wurzelbaum b
                                                             // L[c].size(): Anzahl der Knoten von Wurzelbaum b
                Fuege_an(L,a,b,c);                  // Fuege die Wurzelbaeume a,b und c zusammen und haenge sie an
              }                                     // die Liste L an
              else                                  // Der entehende Wurzelbaum kann nicht zur Konstruktion eines
                c=laenge;                           // Alkins mit maximal n C-Atomen verwendet werden, also gehe mit
          }                                         // b eins weiter und starte mit c wieder bei Position b
          else                                      // Die entehenden Wurzelbaeume koennen nicht zur Konstruktion eines
          {                                         // Alkins mit maximal n C-Atomen verwendet werden, also gehe mit
            b=laenge;                               // a eins weiter und starte mit b wieder bei Position p
          }
        }
      }
      else                                          // Die entehenden Wurzelbaeume koennen nicht zur Konstruktion eines
      {                                             // Alkins mit maximal n C-Atomen verwendet werden, also beende die
        a=p;                                        // Schleife
      }
    }


    // Fall (iii)
    for(a=p;a<laenge;a++)                           // a ist ein Wurzelbaum der Hoehe h-1
    {
      if(ausgeben==false)
        cout << "." << flush;                       // Forstschrittsausgabe, da Berechnung lange dauern kann
      if(1+3*((int)L[a].size())<=n-2)               // 1:             die neue Wurzel
      {                                             // 3*L[a].size(): Die Wurzelbaeume b und c besitzen
                                                    //                mindestens so viele Knoten wie der
                                                    //                Wurzelbauma
        for(b=a;b<laenge;b++)                       // b ist ein Wurzelbaum der Hoehe h-1
        {
          if(1+(int)L[a].size()+2*((int)L[b].size())<=n-2)      // 1:             die neue Wurzel
          {                                         // L[a].size():   Anzahl der Knoten von Wurzelbaum a
                                                    // 2*L[b].size(): Der Wurzelbaum c besitzt mindestens
                                                    //                so viele Knoten wie Wurzelbaum b
            for(c=b;c<laenge;c++)                   // c ist ein Wurzelbaum der Hoehe h-1
              if(1+(int)L[a].size()+(int)L[b].size()+(int)L[c].size()<=n-2) // 1:           die neue Wurzel
              {                                              // L[a].size(): Anzahl der Knoten von Wurzelbaum a
                                                             // L[b].size(): Anzahl der Knoten von Wurzelbaum b
                                                             // L[c].size(): Anzahl der Knoten von Wurzelbaum b
                                                             // h:           Es werden mindestens zwei Wurzelbaeume der
                                                             //              Hoehe h zu einem Alkin zusammengesetzt
                Fuege_an(L,a,b,c);                  // Fuege die Wurzelbaeume a,b und c zusammen und haenge sie an
              }                                     // die Liste L an
              else                                  // Der entehende Wurzelbaum kann nicht zur Konstruktion eines
                c=laenge;                           // Alkins mit maximal n C-Atomen verwendet werden, also gehe mit
          }                                         // b eins weiter und starte mit c wieder bei Position b
          else                                      // Die entehenden Wurzelbaeume koennen nicht zur Konstruktion eines
          {                                         // Alkins mit maximal n C-Atomen verwendet werden, also gehe mit
            b=laenge;                               // a eins weiter und starte mit b wieder bei Position a
          }
        }
      }
      else                                          // Die entehenden Wurzelbaeume koennen nicht zur Konstruktion eines
      {                                             // Alkins mit maximal n C-Atomen verwendet werden, also beende die
        a=laenge;                                   // Schleife
      }
    }
    if(ausgeben==false)
      cout << endl;


    sortiere_Wurzelbaum_Liste(L,laenge,n);          // Stelle die "richtige" Sortierung der Liste L wieder her, so
                                                    // wie sie fuer diesen Algorithmus gebraucht wird
    p=laenge;                                       // Start fuer Wurzelbaeume der Laenge h

    // Konstruiere Alkine aus den bereits erzeugten Wurzelbaeumen
    konstruiere_Alkine(Alkine,L,p,h,n,ausgeben,ausgabe_bei_anzahl_c_atomen);
  }
}

void konstruiere_Alkine(vector<Alkin>& Alkine,const vector< vector <unsigned char> >& L,
unsigned int p,unsigned int h,unsigned int n,bool ausgeben,int ausgabe_bei_anzahl_c_atomen)
{
  // Alkine:  eine Liste von Alkinen
  // L:       eine Liste von Wurzelbaeumen
  // p:       Position in L, ab der die Wurzelbaeume der Hoehe h stehen
  // h:       Hoehe der Wurzelbaeume, die aktuell fuer die Konstruktion verwendet werden sollen
  // n:       maximale Anzahl der C-Atome, die die Alkine besitzen sollen
  //
  // Funktion:
  // ---------
  // Erweitert eine Liste von Alkinen mit denen solchen die
  // maximal n Knoten haben. Die dafuer notwendigen
  // Wurzelbaeume der Hoehe h stehen in der Liste L ab Position p

  if(ausgeben==false)
  {
    cout << "Konstruiere zugehoerige Alkine      ";// Fortschrittsanzeige
    if(n>=20) cout << " ";
    cout << ": " << flush;
  }

  unsigned int a,b;                           // Nummern der bis zu 2 Wurzelbaeume, die aus der Liste L ausgewaehlt werden

  if(h>n-2) return;                               // Es koennen keine Alkine mit hinreichend wenig C-Atomen
                                                  // konstruiert werden

  // In der Liste L stehen an den Positionen 0 bis p-1 Wurzelbaeume der Hoehe kleiner h
  // geordnet nach der Anzahl der Knoten. An den Positionen p bis laenge-1 stehen
  // Wurzelbaeume der Hoehe h, ebenfalls nach der Anzahl der Knoten geordnet.
  // Fuer die Kombination zu einem Alkin gibt
  // es nun nur 1 Moeglichkeit
  // zwei Wurzelbaeume der Hoehe <=h , deren Wurzeln mit einem Ethinmolekuel verbunden werden
  // Die Positionen der Wurzelbaeume in L werden mit a und b bezeichnet. Um die Konstruktion
  // zu beschleunigen, wird bei jedem Schritt ueberprueft, ob ueberhaupt ein Alkin mit hoechstens
  // n C-Atomen entstehen kann.


  for(a=p;a<L.size();a++)          // a ist ein Wurzelbaum der Hoehe h
  {
   if((L[a].size()<=n-2))          // Wurzelbaum a hat hoechstens n-2 Knoten
   {
     for(b=0;(b<=a);b++)           // b ist ein Wurzelbaum mit maximaler Hoehe h und hat weniger oder gleich viel Knoten wie a
      {
        if(L[a].size()+L[b].size()<=n-2)  //Wurzelbaeume a und b haben zusamme hoechstens n-2 Knoten
          Fuege_an(Alkine,L,a,b,ausgeben,ausgabe_bei_anzahl_c_atomen);
      }
   }
   else
     a=L.size();
  }



  if(ausgeben==false)
    cout << endl;                                 // Fortschrittsanzeiger
}


// ********************************************************************
// *                       Hauptprogramm                              *
// ********************************************************************

int main(void)
{
  unsigned int n;                           // maximale Anzahl der Knoten bzw C-Atome
  bool ausgeben;                            // sollen die konstruierten Alkine auf dem Bildschirm ausgegeben werden
  int eingabe;                              // Variable zum Einlesen von Eingaben
  int ausgabe_bei_anzahl_c_atomen;          // geben nur Alkine mit dieser Anzahl an C-Atomen auf dem Bildschirm aus

  cout << "Konstruktion von Isomeren der Alkine (Summenformel: C_nH_2n-2)" << endl;
  cout << "==============================================================" << endl;
  cout << endl;
  cout << "Bitte gebe die maximale Anzahl der C-Atome ein: ";
  cin  >> n;
  cout << "Sollen die konstruierten Alkine ausgegeben werden? (Nein=0, Ja=1) ";
  cin  >> eingabe;
  if(eingabe==1)
    ausgeben=true;
  else
    ausgeben=false;
  if(ausgeben==true)
  {
    cout << "Gebe bitte die Anzahl der C-Atome an, bei denen die konstruierten Alkine ausgegeben werden sollen. ";
    cout << "Moechtest du alle Alkine ausgegeben haben, so gebe bitte -1 ein. ";
    cin  >> ausgabe_bei_anzahl_c_atomen;
  }

  vector<Alkin> Alkine;                     // Liste von Alkinen
  vector< vector <unsigned char> > L;       // Liste von Wurzelbaeumen

  // Konstruiere eine vollstaendige Liste von Wurzelbaeumen und damit alle Alkine aus bis zu n C-Atomen
  konstruiere_Wurzelbaeume(Alkine,L,n,ausgeben,ausgabe_bei_anzahl_c_atomen);


  // Zur Kontrolle gebe die Anzahl der Alkine aus
  cout << endl;
  vector<long> anz;
  unsigned int i;
  unsigned int sp;
  anz.resize(n+1);
  for(i=0;i<=n;i++)
    anz[i]=0;
  for(i=0;i<Alkine.size();i++)
  {
    sp=Alkine[i].Wurzelbaum_1.size()+Alkine[i].Wurzelbaum_2.size()+2;
    anz[sp]++;
  }
  cout << "Anzahl der Alkine mit n C-Atomen" << endl;
  cout << "--------------------------------" << endl;
  for(i=0;i<=n;i++)
  {
    cout << i;
    if(i<10 && n>=10) cout << " ";
    cout << " : " << anz[i] << endl;
  }
  return 0;
};

// ****************************************************************************
// *              Subroutinen zur Ausgabe eines Alkins                        *
// ****************************************************************************

int ausgabe(Alkin& A)
{
  vector<unsigned int> Ebene;     // in welcher Ebene liegt der Knoten im Wurzelbaum
  vector<unsigned int> Position;  // Teile die Knoten eines Wurzelbaums in einer Ebene in 3^Ebene Positionen auf
  vector<unsigned int> Nachfolgervon;
  Ebene.resize(A.Wurzelbaum_1.size()+A.Wurzelbaum_2.size()+2);
  Position.resize(A.Wurzelbaum_1.size()+A.Wurzelbaum_2.size()+2);
  Nachfolgervon.resize(A.Wurzelbaum_1.size()+A.Wurzelbaum_2.size()+2);
  for(unsigned int i = 0; i<A.Wurzelbaum_1.size()+A.Wurzelbaum_2.size()+2; i++)
  {
    Nachfolgervon[i]=0;  //Anzahl der Nachfolger von Knoten i
    Ebene[i]=0;
    Position[i]=0;
  }

  unsigned int w1=A.Wurzelbaum_1.size();
  unsigned int w2=A.Wurzelbaum_2.size();

  // Berechne nun die Anzahl der Nachfolger vom Knoten i und speicher diese in Nachfolgervon[i]
  // Hat nun ein Knoten 3 Nachfolger, so muss kein weiteres H-Atom angehaengt werden, hat er 2 Nachfolger so fehlt ein H-Atom, hat er einen Nachfolger so fehlen 2 H-Atome, hat er keinen Nachfolger, so fehlen 3 H-Atome

  // Betrachte zuerst die beiden Knoten 0 und 1 die mit einer Dreifachbindung zusammenhaengen
  if(w1 > 0)
  {
    Nachfolgervon[0]=3;  // man braucht kein weiteres H-Atom
    Ebene[2]=0;
    Position[2]=0;
  }
  else
  {
    Nachfolgervon[0]=2;  // es muss ein H-Atom angehaengt werden (beachte die Dreifachbindung zwischen Knoten 0 und 1)
  }
  if(w2 > 0)
  {
    Nachfolgervon[1]=3;  // man braucht kein weiteres H-Atom
    Ebene[2+A.Wurzelbaum_1.size()]=0;
    Position[2+A.Wurzelbaum_1.size()]=0;
  }
  else
  {
    Nachfolgervon[1]=2;  // es muss ein H-Atom angehaengt werden (Dreifachbindung!!!)
  }

  //  Betrachte nun die beiden Wurzelbaeume
  for(unsigned int i = 1; i<A.Wurzelbaum_1.size(); i++)
  {
    Ebene[2+i]=Ebene[2+A.Wurzelbaum_1[i]]+1;
    Position[2+i]=3*Position[2+A.Wurzelbaum_1[i]]+Nachfolgervon[2+A.Wurzelbaum_1[i]];		
    Nachfolgervon[2+A.Wurzelbaum_1[i]]++;
  }
  for(unsigned int i = 1; i<A.Wurzelbaum_2.size(); i++)
  {
    Ebene[2+w1+i]=Ebene[2+w1+A.Wurzelbaum_2[i]]+1;
    Position[2+w1+i]=3*Position[2+w1+A.Wurzelbaum_2[i]]+Nachfolgervon[2+w1+A.Wurzelbaum_2[i]];
    Nachfolgervon[2+w1+A.Wurzelbaum_2[i]]++;
  }

  // geben nun die Daten fuer XML-File aus
  //ofstream&
  ofstream datei("graph", ios_base::app);

  datei << "<GRAPH NAME=\"C" << w1+w2+2 << "H" << 2*(w1+w2+2)-2 << "\" NUMBER_OF_VERTICES=\"" << w1+w2+2+2*(w1+w2+2)-2;
  datei << "\" NUMBER_OF_EDGES=\"" << w1+w2+3+2*(w1+w2+2)-2 << "\" MULTIGRAPH=\"1\"" << " MOLECULAR=\"1\">" << endl;
  datei << "<VERTEXLABELS>" << endl;
  for(unsigned int i=0; i< w1+w2+2; i++)
  {
    datei << "\"C\"" << endl;
  }
  for(unsigned int i=0; i< 2*(w1+w2+2)-2; i++)
  {
    datei << "\"H\"" << endl;
  }

  datei << "</VERTEXLABELS>" << endl << "<EDGELIST>" << endl;
  //Erstellen einer Edgelist fuer die verschiedenen Wurzelbaeume
  datei << "0 1" << endl << "0 1" << endl << "0 1" << endl;              // Erstellen der Dreifachbindung
  //Erster Wurzelbaum an der einen Seite der "Dreifachbindung"
  if(w1 != 0)
  {
    datei << "2 0" << endl;                                             // Anhaengen eines C-Atoms an die Dreifachbindung
    for(unsigned int i=1; i<w1; i++)                 // Schleife der konstruierten Wurzelbaeume
    {
      datei << i+2 << " " << A.Wurzelbaum_1[i]+2 << endl;               // Anhaengen der verschieden konstruierten Wurzelbaeume (aus Liste Wurzelbaum_1) an oben genanntes C-Atom
    }
  }
  //Zweiter Wurzelbaum an der anderen Seite der "Dreifachbindung"
  if(w2 != 0)
  {
    datei << 2+A.Wurzelbaum_1.size() << " 1" << endl;                   // Anhaengen eines C-Atoms an die andere Seite der Dreifachbindung
    for(unsigned int i=1; i<w2; i++)
    {
      datei << i+2+w1 << " " << A.Wurzelbaum_2[i]+2+w1 << endl;   // Anhaengen der verschieden konstruierten Wurzelbaeume (aus Liste Wurzelbaum_2) an oben genanntes C-Atom
    }
  }

  unsigned int knoten;
  knoten = w1+w2+2;                     // gibt die aktuelle Nummer des neu zu erstellenden Knotens an;
  for(unsigned int i=0;i<2+w1+w2;i++) // Ueberpruefe C-Atome auf fehlende H-Atome und fuege ggf hinzu
  {
    if(Nachfolgervon[i]<3)                       // Fehlen H-Atome?
    {
      if(Nachfolgervon[i]==2)                    // Haenge ein weiteres H-Atom an den i-ten Knoten
      {
        datei << knoten << " " << i << endl;     // Fuege die Kante zwischen dem H-Atom und dem C-Atom ein
        knoten ++;                               // Erhoehe die aktuelle Anzahl der Knoten
      }
      if(Nachfolgervon[i]==1)                    // Haenge zwei weitere H-Atome an den i-ten Knoten
      {
        datei << knoten << " " << i << endl;
        knoten ++;
        datei << knoten << " " << i << endl;
        knoten ++;
      }
      if(Nachfolgervon[i]==0)                    // Haenge drei weitere H-Atome an den i-ten Knoten
      {
        datei << knoten << " " << i << endl;
        knoten ++;
        datei << knoten << " " << i << endl;
        knoten ++;
        datei << knoten << " " << i << endl;
        knoten ++;
      }
    }
  }


  datei << "</EDGELIST>" << endl; 
  unsigned int pos;
  unsigned int eb;
  datei << "<COORDS2D_FLOAT>" << endl;
    datei << "0 0.5" << endl;
    datei << "0 -0.5" << endl;
    for(unsigned int i=0;i<w1;i++)
    {
      if(Ebene[2+i]==0)
        datei << "0";
      else
      {
        datei << -((long double)Ebene[2+i])+Position[2+i]*2.*Ebene[2+i]/(pow((long double)3,(int)Ebene[2+i])-1);
      }
      datei << " " << Ebene[2+i]+1.5  << endl;
    }
    for(unsigned int i=0;i<w2;i++)
    {
      if(Ebene[2+w1+i]==0)
        datei << "0";
      else
        datei << -((long double)Ebene[2+w1+i])+Position[2+w1+i]*2.*Ebene[2+w1+i]/(pow((long double)3,(int)Ebene[2+w1+i])-1);
      datei << " " << -((int)Ebene[2+w1+i])-1.5 << endl;		
    }
    if(Nachfolgervon[0]==2) datei << "0 1.5" << endl;
    if(Nachfolgervon[1]==2) datei << "0 -1.5" << endl; 
    for(unsigned int i=2;i<2+w1+w2;i++)
      for(unsigned j=0;j<3-Nachfolgervon[i];j++)
      {
        pos=3*Position[i]+2-j;         
        eb=Ebene[i]+1;
        datei << -((long double)eb)+pos*2.*eb/(pow((long double)3,(int)eb)-1) << " "; 
        if(i<2+w1)
          datei << eb+1.5;
        else
          datei << -((int)eb)-1.5;
        datei << endl;
      }  
  datei << "</COORDS2D_FLOAT>" << endl; 
  datei << "</GRAPH>" << endl << endl;
  datei.close();
  return 0;
}
