// *********************************************************************
// *                                                                   *
// *       Abzaehlen von Isomeren der Alkane (C_nH_2n+2)               *
// *       =============================================               *
// *                                                                   *
// *       Autor:        Sascha Kurz                                   *
// *       Email:        sascha.kurz@uni-bayreuth.de                   *
// *       modifiert im Rahmen eines Softwarepraktikums durch:         *
// *                     Sabine Boehm und Florian Haeberlein           * 
// *       Homepage:     www.mathe2.uni-bayreuth.de/sascha             *
// *                                                                   *
// *       Filename:           zaehle_alkane.cpp                       *
// *       zuletzt geaedert:   11.01.2005                              *
// *       Betriebssystem:     Suse Linux 9.1                          *
// *       Compiler:           gcc3.3.3                                *
// *       Compiler-Optionen:  g++ -O2 -Wall -pedantic -ansi           *
// *                           zaehle_alkane.cpp -lcln -o zaehle_alkane*
// *                                                                   *
// *       Algorithmus                                                 *
// *       -----------                                                 *
// *       Basierend auf dem Artikel "Anzahl von Strukturisomeren      *
// *       der Alkane". Es werden rekursiv Wurzelbaeme gezaelt, die    *
// *       wiederum zum Abzaelen von verwendet werden. Details sind    *
// *       bitte dem zitierten Artikel zu entnehmen. Eine genauere     *
// *       Dokumentation der Konstruktion von Alkanen findet sich      *
// *       in "konstruiere_alkane.cpp".                                *
// *                                                                   *
// *       Schlussemerkung                                             *
// *       ===============                                             *
// *       Bei Fragen, Anmerkungen, Kritik oder sonstigen Kommentaren  *
// *       wende man sich bitte per email an den oben genannten Autor  *
// *       dieses Programms. Der Quellcode ist fae nicht-kommerzielle  *
// *       Anwendungen frei verfaebar und darf beliebig veraedert      *
// *       werden. Bei Veraederungen ist geeignet darauf hinzuweisen,  *
// *       von wem sie vorgenommen wurden. Es waede mich freuen aeer   *
// *       Verwendungen dieses Quellcodes eine kurze Nachricht per     *
// *       email zu erhalten.                                          *
// *                                                                   *
// *       Sascha Kurz                                                 *
// *                                                                   *
// *********************************************************************





#include <iostream>                      // Headerfile fuer die Standard Ein- und Asugabe
#include <iomanip>                       // Headerfile fuer die Manipulation von Ausgaben
#include <fstream>                       // Headerfile fuer Dateiein- und Ausgaben
#include <cmath>			 // Headerfile fuer die mathematischen Funktionen

using namespace std;





// Die Anzahl der Isomere der Alkane kann sehr schnell ziemlich gross// werden. Deswegen verwenden wir die Langzahlbibliothek CLN, erkennbar
// an der Compileroption -lcln. Die hierfae notwendigen Headerfiles sind:
#include <cln/integer.h>                 // Langzahlarithmetik
#include <cln/integer_io.h>              // Ausgabe von langen Zahlen
// Fae eine ordententliche Ausgabe der Anzahlen ist es noch notwendig
// dem Programm zu sagen, ob man wirklich den Datentyp aus der CLN verwenden möchte
const int datentyp=1;                    // 1 -> cln::cl_I bzw. long long
                                         // 2 -> long double


int base;				// Anzahl der Stellen der Basis	in der die Zahlen gelesen werden 




// *********************************************************************
// *                  Prototypdeklarationen                            *
// *********************************************************************

// Berechnet die Anzahl der Stellen der Basis fuer einen Koeffizienten eines Polynoms
void determine_base(int length);

// Multipliziert zwei Polynome, repraesentiert durch Zahlen. Das Ergebnis ist ein Polyom
void multiply(cln::cl_I & factor1, cln::cl_I & factor2, cln::cl_I & result, cln::cl_I & gesamtschema);

// Addiert zwei Polynome, repraesentiert durch Zahlen. Das Ergebnis ist ein Polyom
void add(cln::cl_I & summand1, cln::cl_I & summand2, cln::cl_I & result);

// Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 2 auf [poly] an und
// speichert das Ergebnis als ein Polynom mit Maximalgrad [length] in [result]
bool CycleIndex_2(cln::cl_I & poly, cln::cl_I & result, unsigned int length, cln::cl_I & schema, cln::cl_I & gesamtschema);

// Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 3 auf [poly] an und
// speichert das Ergebnis als ein Polynom mit Maximalgrad [length] in [result]
bool CycleIndex_3(cln::cl_I & poly, cln::cl_I & result, unsigned int length, cln::cl_I & schema, cln::cl_I & gesamtschema);

// Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 4 auf [poly] an und
// speichert das Ergebnis als ein Polynom mit Maximalgrad [length] in [result]
bool CycleIndex_4(cln::cl_I & poly, cln::cl_I & result, unsigned int length, cln::cl_I & schema, cln::cl_I & gesamtschema);

inline long double exquo(long double a,long double b)
{
  // Gibt a/b zuruek, Funktion gibt es nur aus Kompatibilitaesgraeden zur CLN
  return a/b;
}

inline unsigned long exquo(unsigned long a,unsigned long b)
{
  // Gibt a/b zuruek, Funktion gibt es nur aus Kompatibilitaesgraeden zur CLN
  return a/b;
}




// ********************************************************************
// *         Implementierungen der benoetigten Funktionen             *
// ********************************************************************

void determine_base(int length)
{
  double d=24.*pow(2.83,length);
  if(length>=50) d=d/34775.;
  if(length>=100) d=d/2.95;
  //d=d*1000000000000000000000000.;
  base=(int)(ceil(log(d)/log(2.))+1);
  //base=2*length+15;
}

inline void multiply(cln::cl_I & factor1, cln::cl_I & factor2, cln::cl_I & result, cln::cl_I & gesamtschema)
{
  // Funktion multipliziert das durch [factor1] repraesentierte Polynom mit dem
  // durch [factor2] repraesentierten Polynom und speichert das Ergebnis als
  // Polyonom in [result].
  result = (factor1*factor2) & gesamtschema;
}

inline void add(cln::cl_I & summand1, cln::cl_I & summand2, cln::cl_I & result)
{
  // Funktion addiert das durch [summand1] repraesentierte Polynom zu dem
  // durch [summand2] repraesentierten Polynom und speichert das Ergebnis als
  // Polyonom in [result].
  result = summand1+summand2;
}

bool CycleIndex_2(cln::cl_I & poly, cln::cl_I & result, unsigned int length, cln::cl_I & schema, cln::cl_I & gesamtschema)
{
  // Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 2 auf [poly] an und
  // speichert das Ergebnis als ein Polynom in [result]

  cln::cl_I l2 = 0;           //L(x^2)
  cln::cl_I temp = 0;         //temporaere Zahl
  cln::cl_I l2invers = 0;     //enthaelt das Polynom l2 jedoch sind die Koeffizienten in umgekehrter Reihenfolge abgespeichert

  temp = poly;  //Berechnung von L(x^2);
  for(unsigned int i=0; i<=length; i++)
  {
    l2invers = l2invers | (temp & schema);
    temp = temp >> base;
    l2invers = l2invers << base;
  }
  for(unsigned int i=0; i<=length; i++)
  {
    l2invers = l2invers >> base;
    l2 = l2 << 2*base;                        //verschiebe um 2*base -->    L(x^2) = (an)(0)(an-1)(0) . . . (2)(0)(1)
    l2 = l2 | (l2invers & schema);
  }

  //Berechnung des gesamten Bruchs
  result = ((poly * poly) & gesamtschema) + l2;              
  //Teile nun noch jeden Koeffizienten durch 2
  temp = 0;
  for(unsigned int i=0; i<=length; i++)
  {
    temp = temp | (result & schema);
    temp = temp << base;
    result = result >> base;
  }
  for(unsigned int i=0;i<=length; i++)
  {
    result = result << base;
    temp = temp >> base;
    result = result | (exquo((temp & schema),2));
  }
  result = result&gesamtschema; 

  return true;
}

bool CycleIndex_3(cln::cl_I & poly, cln::cl_I & result, unsigned int length, cln::cl_I & schema, cln::cl_I & gesamtschema)
{
  // Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 3 auf [poly] an und
  // speichert das Ergebnis als ein Polynom mit Maximalgrad [length] in [result]
  
  //Verfahren mit Links- und Rechtsshifts wie bei CycleIndex_2

  cln::cl_I l2 = 0;  //3*L(x^2)
  cln::cl_I l3 = 0;  //2*L(x^3)
  cln::cl_I temp = 0;
  cln::cl_I linvers = 0;
  int g;


  temp = poly;  //Berechnung von 3*L(x^2);
  g=(length+1)/2;
  for(int i=0; i<=g; i++)
  {
    linvers = linvers | (temp & schema);
    //cout << "i: " << i << " , " << (temp & schema) << endl;
    temp = temp >> base;
    linvers = linvers << base;
  }
  for(int i=0; i<=g; i++)
  {
    l2 = l2 << 2*base;
    linvers = linvers >> base;
    l2 = l2 | (3*(linvers & schema));
  }

  linvers = 0;

  g=(length+2)/3;
  temp = poly;  //Berechnung von 2*L(x^3);
  for(int i=0; i<=g; i++)
  {
    linvers = linvers | (temp & schema);
    temp = temp >> base;
    linvers = linvers << base;
  }
  for(int i=0; i<=g; i++)
  {
    l3 = l3 << 3*base;
    linvers = linvers >> base;
    l3 = l3 | (2*(linvers & schema));
  }

  //Berechnung des gesamten Bruchs
  result = (((poly * poly)&gesamtschema)*poly + l2*poly + l3) & gesamtschema;
  temp = 0;
  //Teile nun noch durch 6
  for(unsigned int i=0; i<=length; i++)
  {
    temp = temp | (result & schema);
    temp = temp << base;
    result = result >> base;
  }
  for(unsigned int i=0;i<=length; i++)
  {
    temp = temp >> base;
    result = result << base;
    result = result | (exquo((temp & schema),6));
  }
  result = result&gesamtschema;
  return true;
}

bool CycleIndex_4(cln::cl_I & poly, cln::cl_I & result, unsigned int length, cln::cl_I & schema, cln::cl_I & gesamtschema)
{
  // Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 4 auf [poly] an und
  // speichert das Ergebnis als ein Polynom mit Maximalgrad [length] in [result]
  
  //Verfahren mit Links- und Rechtsshifts wie bei CycleIndex_2

  int g;

  cln::cl_I l62 = 0;  //6*L(x^2)
  cln::cl_I l322 = 0; //3*L(x^2)^2
  cln::cl_I l2 = 0;   //L(x^2);
  cln::cl_I l83 = 0;  //8*L(x^3)
  cln::cl_I l64 = 0;  //6*L(x^4)
  cln::cl_I temp = 0;
  cln::cl_I linvers = 0;

  temp = poly;  //Berechnung von 6*L(x^2);
  for(unsigned int i=0; i<=length; i++)
  {
    linvers = linvers | (temp & schema);
    temp = temp >> base;
    linvers = linvers << base;
  }
  for(unsigned int i=0; i<=length; i++)
  {
    linvers = linvers >> base;
    l62 = l62 << 2*base;
    l62 = l62 | (6*(linvers & schema));
  }

  linvers = 0;
  temp = poly; 
  
  cln::cl_I sq=(poly*poly)&gesamtschema;

  // Berechnung von L(x^2);
  g=(length+1)/2;                      
  for(int i=0; i<=g; i++)     
  {
    linvers = linvers | (temp & schema);
    temp = temp >> base;
    linvers = linvers << base;
  }
  for(int i=0; i<=g; i++)    
  {
    linvers = linvers >> base;
    l2 = l2 << 2*base;
    l2 = l2 | (linvers & schema);
  }

  linvers = 0;

  //Berechnung von 3*(l2*l2) = 3*temp = 3*L(x^2)^2;
  temp=sq;
  for(unsigned int i=0; i<=length; i++)
  {
    linvers = linvers | (temp & schema);
    temp = temp >> base;
    linvers = linvers << base;
  }
  for(unsigned int i=0; i<=length; i++)
  {
    linvers = linvers >> base;
    l322 = l322 << 2*base;        
    l322 = l322 | (3*(linvers & schema));
  }

  linvers = 0;
  temp = poly;
  
  g=(length+2)/3; 
  //Berechnung von 8*L(x^3);
  for(int i=0; i<=g; i++)
  {
    linvers = linvers | (temp & schema);
    temp = temp >> base;
    linvers = linvers << base;
  }
  for(int i=0; i<=g; i++)
  {
    linvers = linvers >> base;
    l83 = l83 << 3*base;
    l83 = l83 | (8*(linvers & schema));
    l64 = l64 << 4*base;
    l64 = l64 | (6*(linvers & schema));
  }
  
  //Berechnung des gesamten Bruchs
  result = ((sq*sq)&gesamtschema) + ((((l62*sq)&gesamtschema)&gesamtschema)) + l322 + ((l83*poly)&gesamtschema) + l64;
  temp = 0;
  //Teile nun noch durch 24
  for(unsigned int i=0; i<=length; i++)
  {
    temp = temp | (result & schema);
    temp = temp << base;
    result = result >> base;
  }
  for(unsigned int i=0;i<=length; i++)
  {
    result = result << base;
    temp = temp >> base;
    result = result | (exquo((temp & schema),24));
  }
  
  result = result&gesamtschema;

  return true;
}


// ********************************************************************
// *                       Hauptprogramm                              *
// ********************************************************************


int main(void)
{
  int n;          	              // Anzahl der C-Atome
  cout << "Anzahl der Strukturisomere der Alkane [ C(n)H(2n+2) ]" << endl;
  cout << "=====================================================" << endl;
  cout << endl;
  cout << "Bitte geben Sie die Anzahl der C-Atome ein : ";
  cin  >> n;

  determine_base(n);                   // Bestimme die Basis       

  int i;       			       // Schleifenvariable
  unsigned int d=(n+1)/2;              // 2d bzw. 2d+1 ist die Laenge der laengsten geraden Kette von C-Atomen

  cln::cl_I schema;                    // Schema zum Umwandeln von Zahlen in Vektoren: besteht aus (base) Bit 1-ern
  schema = 1;
  for(int i=1;i<base;i++)
    {
    schema = schema << 1;
    schema = schema | 1;
    }
  cln::cl_I* gesamtschema;
  gesamtschema=new cln::cl_I[n+1];
  
  gesamtschema[0] = schema;
  for(int i=1;i<=n;i++)
  {
    gesamtschema[i] = (gesamtschema[i-1] << base) | schema;
  }


  cln::cl_I sum;                          // Polynom, das die kleineren (laengste gerade Kette) Teilstrukturen abzaehlt
  cln::cl_I longest;                      // Polynom, das die groesseren (laengste gerade Kette) Teilstrukturen beschreibt
  cln::cl_I result;                       // Ergebnispolynom
  cln::cl_I t1, t2, t3, ts;               // temporaere Zahlen zum Speichern von Zwischenergebnissen
  
  //initialisiere die Polynome
  longest = 1;
  sum = 0;
  t1 = 0;
  t2 = 0;
  t3 = 0;
  result = 1;
  result = result << base;

  for(i=1;i<=(int)d;i++)             // laufe durch die moeglichen Laengen der laengsten geraden Ketten der Wurzelbaeume
  {
    cout << "." << flush;
    // Berechne t1 = CycleIndex_3(longest)+CycleIndex_2(longest)*sum+longest*CycleIndex_2(sum)
    
    if(n-3*(i-1)-i>0)
    {
      ts=(longest >> ((i-1)*base)) & gesamtschema[n-3*(i-1)-i];
      CycleIndex_3(ts,t1,n-3*(i-1)-i,schema,gesamtschema[n-3*(i-1)-i]);
      t1=t1 << (3*(i-1)*base);
    }
    else 
      t1=0;

    if(n-2*(i-1)-i>0)
    {
      ts=(longest >> ((i-1)*base)) & gesamtschema[n-2*(i-1)-i];
      CycleIndex_2(ts,t2,n-2*(i-1)-i,schema,gesamtschema[n-2*(i-1)-i]);
      t2=t2 << (2*(i-1)*base); 
    }
    else
      t2=0;  
    multiply(t2,sum,t3,gesamtschema[n-i]);
    add(t1,t3,t2);

    CycleIndex_2(sum,t1,n-i,schema,gesamtschema[n-i]);
    multiply(t1,longest,t3,gesamtschema[n-i]);
    add(t2,t3,t1);

    // Aktualisiertes [sum] = sum + longest
    add(sum,longest,sum);
    // Aktualisiertes [longest]= t1 * X
    longest=t1<<base;

     // Addiere Alkane mit laengster gerader Kette 2i zum Ergebnisvektor hinzu

    if(n-2*i>=0)
    {
      ts=(longest >> (i*base)) & gesamtschema[n-2*i];
      CycleIndex_2(ts,t2,n-2*i,schema,gesamtschema[n-2*i]);
      t2=t2 << (2*i*base);  
    }
    else
      t2=0;

    result=result+t2;

    // Berechne Anzahl der Alkane mit laengster gerader Kette 2i+1
    CycleIndex_2(sum,t1,n,schema,gesamtschema[n]);
    multiply(t1,t2,t3,gesamtschema[n]);

    if(n-3*i>0)
    {
      ts=(longest >> (i*base)) & gesamtschema[n-3*i];
      CycleIndex_3(ts,t1,n-3*i,schema,gesamtschema[n-3*i]);
      t1=t1 << (3*i*base);
      multiply(t1,sum,t2,gesamtschema[n]);
    }
    else
      t2=0;

    add(t3,t2,t1);
    if(n-4*i>0)
    {
      ts=(longest >> (i*base)) & gesamtschema[n-4*i];
      CycleIndex_4(ts,t2,n-4*i,schema,gesamtschema[n-4*i]);
      t2=t2 << (4*i*base); 
    }
    else
      t2=0;

    // Addiere zum Ergebnisvektor hinzu
    t1 = t1 << base;
    t2 = t2 << base;
    result = (result + t1 + t2);  
  }



  cout << "\n\n\n" << endl;



  // Ausgabe der Ergebnisse
  ofstream ausgabe;
  ausgabe.open("alkane.txt");
  cout << "Anzahl der C-Atome |    Anzahl der Isomere" << endl;
  cout << "------------------------------------------" << endl;
  if(datentyp==2)
    for(i=0;i<=n;i++)
    {
      cout << "      " << setw(5) << i << "        | " << setw(35) << setprecision(35) << (result&schema) << endl;
      result = result >> base;
    }
  else
    for(i=0;i<=n;i++)
    {
      cout << "      " << setw(5) << i << "        | " << (result&schema) << endl;
      ausgabe << i << "\t" << (result&schema) << endl;
      result = result >> base;
    }
  cout << endl;
  ausgabe << endl;
  ausgabe.close();

  return 0;
}
