// *********************************************************************
// *                                                                   *
// *       Abzaehlen von Isomeren der Alkane (C_nH_2n+2)               *
// *       =============================================               *
// *                                                                   *
// *       Autor:        Sascha Kurz                                   *
// *       Email:        sascha.kurz@uni-bayreuth.de                   *
// *       Homepage:     www.mathe2.uni-bayreuth.de/sascha             *
// *                                                                   *
// *       Filename:           zaehle_alkane.cpp                       *
// *       zuletzt geändert:   25.10.2004                              *
// *       Betriebssystem:     Suse Linux 9.1                          *
// *       Compiler:           gcc3.3.3                                *
// *       Compiler-Optionen:  g++ -O2 -Wall -pedantic -ansi           *
// *                           zaehle_alkane.cpp -lcln                 *
// *                           -o abzaehlen                            *
// *                                                                   *
// *       Algorithmus                                                 *
// *       -----------                                                 *
// *       Basierend auf dem Artikel "Anzahl von Strukturisomeren      *
// *       der Alkane". Es werden rekursiv Wurzelbäume gezählt, die    *
// *       wiederum zum Abzählen von verwendet werden. Details sind    *
// *       bitte dem zitierten Artikel zu entnehmen. Eine genauere     *
// *       Dokumentation der Konstruktion von Alkanen findet sich      *
// *       in "konstruiere_alkane.cpp".                                *  
// *                                                                   *
// *       Schlußbemerkung                                             *
// *       ===============                                             *
// *       Bei Fragen, Anmerkungen, Kritik oder sonstigen Kommentaren  *
// *       wende man sich bitte per email an den oben genannten Autor  *
// *       dieses Programms. Der Quellcode ist für nicht-kommerzielle  *
// *       Anwendungen frei verfügbar und darf beliebig verändert      *
// *       werden. Bei Veränderungen ist geeignet darauf hinzuweisen,  *
// *       von wem sie vorgenommen wurden. Es würde mich freuen über   *
// *       Verwendungen dieses Quellcodes eine kurze Nachricht per     *
// *       email zu erhalten.                                          *
// *                                                                   *
// *       Sascha Kurz                                                 *    
// *                                                                   *
// *********************************************************************


#include <iostream>                      // Headerfile für die Standard Ein- und Asugabe 
#include <iomanip>                       // Headerfile für die Manipulation von Ausgaben
#include <vector>                        // Headerfile für die Klasse Vektor
#include <fstream>                       // Headerfile für Dateiein- und ausgaben 

using namespace std;


// Die Anzahl der Isomere der Alkane kann sehr schnell ziemlich groß 
// werden. Deswegen verwenden wir die Langzahlbibliothek CLN, erkennbar 
// an der Compileroption -lcln. Die hierfür notwendigen Headerfiles sind:
#include <cln/integer.h>                 // Langzahlarithmetik
#include <cln/integer_io.h>              // Ausgabe von langen Zahlen
// Für eine ordententliche Ausgabe der Anzahlen ist es noch notwendig 
// dem Programm zu sagen, ob man wirklich den Datentyp aus der CLN verwenden möchte
const int datentyp=1;                    // 1 -> cln::cl_I bzw. long long
                                         // 2 -> long double
// Kann oder möchte man die CLN nicht verwenden, so sind im Hauptprogramm 
// die Stellen der Definition der 6 Vektoren umzukommentieren


// Im Hauptprogramm muss noch entsprechend entkommentiert werden




// *********************************************************************
// *                  Prototypdeklarationen                            *
// *********************************************************************

// Multipliziert zwei Polynome, repraesentiert durch Vektoren. Das Ergebnis ist ein Polyom vom Maximalgrad [length]
template <class T>
bool multiply(vector<T>& factor_1,vector<T>& factor_2,vector<T>& result,unsigned int length);

// Addiert zwei Polynome, repraesentiert durch Vektoren. Das Ergebnis ist ein Polyom vom Maximalgrad [length]
template <class T>
bool add(vector<T>& summand_1,vector<T>& summand_2,vector<T>& result,unsigned int length);

// Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 2 auf [poly] an und
// speichert das Ergebnis als ein Polynom mit Maximalgrad [length] in [result]
template <class T>
bool CycleIndex_2(vector<T> & poly,vector<T>& result,unsigned int length);

// Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 3 auf [poly] an und
// speichert das Ergebnis als ein Polynom mit Maximalgrad [length] in [result]
template <class T>
bool CycleIndex_3(vector<T> & poly,vector<T>& result,unsigned int length);

// Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 4 auf [poly] an und
// speichert das Ergebnis als ein Polynom mit Maximalgrad [length] in [result]
template <class T>
bool CycleIndex_4(vector<T> & poly,vector<T>& result,unsigned int length);

inline long double exquo(long double a,long double b)
{
  // Gibt a/b zurück, Funktion gibt es nur aus Kompatibilitätsgründen zur CLN
  return a/b;
}

inline unsigned long exquo(unsigned long a,unsigned long b)
{
  // Gibt a/b zurück, Funktion gibt es nur aus Kompatibilitätsgründen zur CLN
  return a/b;
}




// ********************************************************************
// *         Implementierungen der benötigten Funktionen              *
// ********************************************************************

template <class T>
bool multiply(vector<T>& factor_1,vector<T>& factor_2,vector<T>& result,unsigned int length)
{
  // Funktion multipliziert das durch [factor_1] repraesentierte Polynom mit dem
  // durch [factor_2] repraesentierten Polynom und speichert das Ergebnis als
  // Polyonom hoechstens [length]-ten Grades in [result].

  unsigned int i,j;                      // Schleifenvariablen
  unsigned int m1,m2;                    // Schranken

  result.resize(length+1);               // Bringe Ergebnisvektor auf die richtige Laenge
  for(i=0;i<=length;i++)                 // Initialisiere den Ergebnisvektor mit Null
    result[i]=0;

  // Einfache Polyonommultiplikation mit Gradbeschraenkung
  m1=min(factor_1.size(),length+1);      // Bestimme die obere Schranke fuer die aeussere Schleife
  for(i=0;i<m1;i++)
  {
    m2=min(factor_2.size(),length+1-i);  // Bestimme die obere Schranke fuer die innere Schleife
    for(j=0;j<m2;j++)
      result[i+j]=result[i+j]+factor_1[i]*factor_2[j];
  }

  return true;
}

template <class T>
bool add(vector<T>& summand_1,vector<T>& summand_2,vector<T>& result,unsigned int length)
{
  // Funktion addiert das durch [summand_1] repraesentierte Polynom zu dem
  // durch [summand_2] repraesentierten Polynom und speichert das Ergebnis als
  // Polyonom hoechstens [length]-ten Grades in [result].

  unsigned int i;                        // Schleifenvariable

  if(min(summand_1.size(),summand_2.size())>=length+1)  // Falls beide Polynome ausreichend lang sind
  {
    for(i=0;i<=length;i++)
      result[i]=summand_1[i]+summand_2[i];              // addiere einfach 
    return true;
  }

  if(summand_1.size()<=summand_2.size())                // das erste Polynom ist nicht ausreichend lang 
  {
    for(i=0;i<summand_1.size();i++)
      result[i]=summand_1[i]+summand_2[i];
    if(summand_2.size()>=length+1)                      // das zweite Polyonom ist ausreichend lang 
    {
      for(i=summand_1.size();i<=length;i++)
        result[i]=summand_2[i];
    }
    else                                                // das zweite Polynom ist nicht ausreichend lang
    {
      for(i=summand_1.size();i<summand_2.size();i++)
        result[i]=summand_2[i];
      for(i=summand_2.size();i<=length;i++)
        result[i]=0;
    }
    return true;
  }


  // summand_1.size()>summand_2.size()
  for(i=0;i<summand_2.size();i++)
    result[i]=summand_1[i]+summand_2[i];
  if(summand_1.size()>=length+1)                       // das erste Polynom ist ausreichend lang
  {
    for(i=summand_2.size();i<=length;i++)
      result[i]=summand_1[i];
  }
  else                                                 // das erste Polynomist nicht ausreichend lang
  {
    for(i=summand_2.size();i<summand_1.size();i++)
      result[i]=summand_1[i];
    for(i=summand_1.size();i<=length;i++)
      result[i]=0;
  }

  return true;
}

template <class T>
bool CycleIndex_2(vector<T> & poly,vector<T>& result,unsigned int length)
{
  // Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 2 auf [poly] an und
  // speichert das Ergebnis als ein Polynom mit Maximalgrad [length] in [result]

  unsigned int i;                        // Schleifenvariable
  unsigned int l,m;                      // Schranken

  // CycleIndex(S_2) = (X^2+X_2)/2
  multiply(poly,poly,result,length);     // Berechne "X^2"

  // Berechne zuerst die Koeffizienten der geraden X-Potenzen
  l=(unsigned int)(length/2);
  m=min(l,poly.size()-1);
  for(i=0;i<=m;i++)
    result[2*i]=exquo(result[2*i]+poly[i],2);
  for(i=poly.size();i<=l;i++)
    result[2*i]=exquo(result[2*i],2);

  // Berechne nun die Koeffizienten der ungeraden X-Potenzen
  l=(unsigned int)((length-1)/2);
  for(i=0;i<=l;i++)
    result[2*i+1]=exquo(result[2*i+1],2);

  return true;
}

template <class T>
bool CycleIndex_3(vector<T> & poly,vector<T>& result,unsigned int length)
{
  // Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 3 auf [poly] an und
  // speichert das Ergebnis als ein Polynom mit Maximalgrad [length] in [result]

  unsigned int i,j;                      // Schleifenvariablen
  unsigned int l,m;                      // Schranken
  vector<T> temp;                        // Vektor fuer Zwischenrechnungen

  // CycleIndex(S_3) = (X^3+3X*X_2+2X_3)/6
  multiply(poly,poly,temp,length);       // Berechne "X * X = X^2"
  multiply(poly,temp,result,length);     // Berechne "X * X^2 = X^3

  // Berechne "X^3 + 3*X*X_2"
  m=min(length+1,(unsigned int)poly.size());
  for(i=0;i<m;i++)
  {
    l=min((unsigned int)((length-i)/2),poly.size()-1);
    for(j=0;j<=l;j++)
      result[i+2*j]=result[i+2*j]+3*poly[i]*poly[j];
  }

  // Berechne "X^3+3*X*X_2 + 2*X_3"
  l=min((unsigned int)(length/3),poly.size()-1);
  for(i=0;i<=l;i++)
    result[3*i]=result[3*i]+2*poly[i];

  // Teile nun nur noch durch 6
  for(i=0;i<=length;i++)
    result[i]=exquo(result[i],6);

  return true;
}

template <class T>
bool CycleIndex_4(vector<T> & poly,vector<T>& result,unsigned int length)
{
  // Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 4 auf [poly] an und
  // speichert das Ergebnis als ein Polynom mit Maximalgrad [length] in [result]

  unsigned int i,j;                      // Schleifenvariablen
  unsigned int l,m;                      // Schranken
  vector<T> temp;                        // Vektor fuer Zwischenrechnungen

  // CycleIndex(S_4) = (X^4+6X_2*X^2+8X_3*X+3(X_2)^2+6X_4)/24

  multiply(poly,poly,temp,length);       // Berechne "X * X = X^2"
  multiply(temp,temp,result,length);     // Berechne "X^2 * X^2 = X^4"

  // Berechne "X^4 + 8*X*X_3"
  m=min(length+1,poly.size());
  for(i=0;i<m;i++)
  {
    l=min((unsigned int)((length-i)/3),poly.size()-1);
    for(j=0;j<=l;j++)
      result[i+3*j]=result[i+3*j]+8*poly[i]*poly[j];
  }

  // Berechne "X^4+8*X*X_3 + 6X^2*X_2"
  for(i=0;i<=length;i++)
  {
    l=min((unsigned int)((length-i)/2),poly.size()-1);
    for(j=0;j<=l;j++)
      result[i+2*j]=result[i+2*j]+6*temp[i]*poly[j];
  }

  // Berechne "X^4+8*X*X_3+6X^2*X_2 + 6X_4"
  l=min((unsigned int)(length/4),poly.size()-1);
  for(i=0;i<=l;i++)
    result[4*i]=result[4*i]+6*poly[i];

  // Berechne "X^4+8*X*X_3+6X^2*X_2+6X_4 + 3X^2_2"
  l=(unsigned int)(length/2);
  for(i=0;i<=l;i++)
    result[2*i]=result[2*i]+3*temp[i];

  // Teile nun nur noch durch 24
  for(i=0;i<=length;i++)
    result[i]=exquo(result[i],24);

  return 0;
}




// ********************************************************************
// *                       Hauptprogramm                              *
// ******************************************************************** 


int main(void)
{
  unsigned int n;                        // Anzahl der C-Atome
  cout << "Anzahl der Strukturisomere der Alkane [ C(n)H(2n+2) ]" << endl;
  cout << "=====================================================" << endl;
  cout << endl;
  cout << "Bitte geben Sie die Anzahl der C-Atome ein : ";
  cin  >> n;

  unsigned int i,j;                      // Schleifenvariablen
  unsigned int d=(n+1)/2;                // 2d bzw. 2d+1 ist die Laenge der laengsten geraden Kette von C-Atomen

  vector<cln::cl_I> sum;                 // Vektor, der die kleineren (laengste gerade Kette) Teilstrukturen abzaehlt
  vector<cln::cl_I> longest;             // Vektor, der die groesseren (laengste gerade Kette) Teilstrukturen beschreibt
  vector<cln::cl_I> result;              // Ergebnisvektor
  vector<cln::cl_I> t1,t2,t3;            // temporaere Vektoren zum Speichern von Zwischenergebnissen

  /*vector<long double> sum;             // Vektor, der die kleineren (laengste gerade Kette) Teilstrukturen abzaehlt
  vector<long double> longest;           // Vektor, der die groesseren (laengste gerade Kette) Teilstrukturen beschreibt
  vector<long double> result;            // Ergebnisvektor
  vector<long double> t1,t2,t3;          // temporaere Vektoren zum Speichern von Zwischenergebnissen
  */
  /*
  vector<unsigned long long> sum;               // Vektor, der die kleineren (laengste gerade Kette) Teilstrukturen abzaehlt
  vector<unsigned long long> longest;           // Vektor, der die groesseren (laengste gerade Kette) Teilstrukturen beschreibt
  vector<unsigned long long> result;            // Ergebnisvektor
  vector<unsigned long long> t1,t2,t3;          // temporaere Vektoren zum Speichern von Zwischenergebnissen
  */

  // Reserviere Speicherplatz
  sum.resize(n+1);
  longest.resize(n+1);
  result.resize(n+1);
  t1.resize(n+1);
  t2.resize(n+1);
  t3.resize(n+1);

  // Startbelegung der Polynome
  for(i=0;i<=n;i++)
  {
    sum[i]=0;
    longest[i]=0;
    result[i]=0;
    t1[i]=0;
    t2[i]=0;
    t3[i]=0;
  }
  longest[0]=1;
  result[1]=1;

  for(i=1;i<=d;i++)             // laufe durch die möglichen Längen der längsten geraden Ketten der Wurzelbäume
  {
    cout << "." << flush;
    // Berechne t1 = CycleIndex_3(longest)+CycleIndex_2(longest)*sum+longest*CycleIndex_2(sum)
    CycleIndex_3(longest,t1,n);
    CycleIndex_2(longest,t2,n);
    multiply(t2,sum,t3,n);
    add(t1,t3,t2,n);
    CycleIndex_2(sum,t1,n);
    multiply(t1,longest,t3,n);
    add(t2,t3,t1,n);

    // Aktualisiertes [sum] = sum + longest
    add(sum,longest,t2,n);
    for(j=0;j<=n;j++)
      sum[j]=t2[j];

    // Aktualisiertes [longest]= t1 * X
    for(j=0;j<n;j++)
      longest[j+1]=t1[j];
    longest[0]=0;

     // Addiere Alkane mit laengster gerader Kette 2i zum Ergebnisvektor hinzu
    CycleIndex_2(longest,t2,n);
    for(j=0;j<=n;j++)
      result[j]=result[j]+t2[j];

    // Berechne Anzahl der Alkane mit laengster gerader Kette 2i+1
    n--;                                 // Dieser Teil wird anschliessend sowieso mit X multipliziert
    CycleIndex_2(sum,t1,n);
    multiply(t1,t2,t3,n);
    CycleIndex_3(longest,t1,n);
    multiply(t1,sum,t2,n);
    add(t3,t2,t1,n);
    CycleIndex_4(longest,t2,n);
    n++;

    // Addiere zum Ergebnisvektor hinzu
    for(j=0;j<n;j++)
      result[j+1]=result[j+1]+t1[j]+t2[j];
  }
  cout << "\n\n\n" << endl;

  // Ausgabe der Ergebnisse
  ofstream ausgabe;
  ausgabe.open("alkane.txt");
  cout << "Anzahl der C-Atome |    Anzahl der Isomere" << endl;
  cout << "------------------------------------------" << endl;
  if(datentyp==2)
    for(i=0;i<=n;i++)
      cout << "      " << setw(5) << i << "        | " << setw(35) << setprecision(35) << result[i] << endl;
  else
    for(i=0;i<=n;i++)
    {
      cout << "      " << setw(5) << i << "        | " << result[i] << endl;
      ausgabe << i << "\t" << result[i] << endl;
    }
  cout << endl;
  ausgabe << endl;
  ausgabe.close();

  return 0;
}

