// *********************************************************************
// *                                                                   *
// *       Abzaehlen von Isomeren der Alkene (C_nH_2n)                 *
// *       ===========================================                 *
// *                                                                   *
// *       Autor:        Sascha Kurz                                   *
// *       modifiert im Rahmen eines Softwarepraktikums durch:         *
// *                     Sabine Boehm und Florian Haeberlein           * 
// *       Email:        sascha.kurz@uni-bayreuth.de                   *
// *       Homepage:     www.mathe2.uni-bayreuth.de/sascha             *
// *                                                                   *
// *       Filename:           zaehle_alkene.cpp                       *
// *       zuletzt geaendert:  11.01.2005                              *
// *       Betriebssystem:     Suse Linux 9.1                          *
// *       Compiler:           gcc3.3.3                                *
// *       Compiler-Optionen:  g++ -O2 -Wall -pedantic -ansi           *
// *                           zaehle_alkene.cpp -lcln -o zaehle_alkene*
// *                                                                   *
// *       Algorithmus                                                 *
// *       -----------                                                 *
// *       Basierend auf dem Artikel "Anzahl von Strukturisomeren      *
// *       der Alkane". Es werden rekursiv Wurzelbaeume gezaehlt, die  *
// *       wiederum zum Abzaehlen von verwendet werden. Details sind   *
// *       bitte dem zitierten Artikel zu entnehmen. Eine genauere     *
// *       Dokumentation der Konstruktion von Alkenen findet sich      *
// *       in "konstruiere_alkene.cpp".                                *
// *                                                                   *
// *       Schlussbemerkung                                            *
// *       ===============                                             *
// *       Bei Fragen, Anmerkungen, Kritik oder sonstigen Kommentaren  *
// *       wende man sich bitte per email an den oben genannten Autor  *
// *       dieses Programms. Der Quellcode ist fuer nicht-kommerzielle *
// *       Anwendungen frei verfuegbar und darf beliebig veruendert    *
// *       werden. Bei Veraenderungen ist geeignet darauf hinzuweisen, *
// *       von wem sie vorgenommen wurden. Es wuerde mich freuen Ueber *
// *       Verwendungen dieses Quellcodes eine kurze Nachricht per     *
// *       email zu erhalten.                                          *
// *                                                                   *
// *       Sascha Kurz                                                 *
// *                                                                   *
// *********************************************************************





#include <iostream>                      // Headerfile fuer die Standard Ein- und Asugabe
#include <iomanip>                       // Headerfile fuer die Manipulation von Ausgaben
#include <fstream>                       // Headerfile fuer Dateiein- und ausgaben
#include <time.h>

using namespace std;





// Die Anzahl der Isomere der Alkine kann sehr schnell ziemlich gross
// werden. Deswegen verwenden wir die Langzahlbibliothek CLN, erkennbar
// an der Compileroption -lcln. Die hierfuer notwendigen Headerfiles sind:
#include <cln/integer.h>                 // Langzahlarithmetik
#include <cln/integer_io.h>              // Ausgabe von langen Zahlen
// Fuer eine ordententliche Ausgabe der Anzahlen ist es noch notwendig
// dem Programm zu sagen, ob man wirklich den Datentyp aus der CLN verwenden moechte
const int datentyp=1;                    // 1 -> cln::cl_I bzw. long long
                                         // 2 -> long double



#define base ((2*length)+5)   //Wird in den Funktionen benutzt um die Zahlen, die Polynome darstellen, um (2*length)+20 Stellen zu verschieben
#define base2 ((2*n)+5)       //Wird im Hauptprogramm benutzt um die Zahlen, die Polynome darstellen, um (2*n)+20 Stellen zu verschieben







// *********************************************************************
// *                  Prototypdeklarationen                            *
// *********************************************************************

// Multipliziert zwei Polynome, repraesentiert durch Zahlen. Das Ergebnis ist ein Polyom
bool multiply(cln::cl_I & factor1, cln::cl_I & factor2, cln::cl_I & result, cln::cl_I & gesamtschema);

// Addiert zwei Polynome, repraesentiert durch Zahlen. Das Ergebnis ist ein Polyom
bool add(cln::cl_I & summand1, cln::cl_I & summand2, cln::cl_I & result);

// Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 2 auf [poly] an und
// speichert das Ergebnis als ein Polynom mit Maximalgrad [length] in [result]
bool CycleIndex_2(cln::cl_I & poly, cln::cl_I & result, unsigned int length, cln::cl_I & schema, cln::cl_I & gesamtschema);

// Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 3 auf [poly] an und
// speichert das Ergebnis als ein Polynom mit Maximalgrad [length] in [result]
bool CycleIndex_3(cln::cl_I & poly, cln::cl_I & result, unsigned int length, cln::cl_I & schema, cln::cl_I & gesamtschema);

inline long double exquo(long double a,long double b)
{
  // Gibt a/b zurueck, Funktion gibt es nur aus Kompatibilitaetsgruenden zur CLN
  return a/b;
}

inline unsigned long exquo(unsigned long a,unsigned long b)
{
  // Gibt a/b zurueck, Funktion gibt es nur aus Kompatibilitaetsgruenden zur CLN
  return a/b;
}




// ********************************************************************
// *         Implementierungen der benoetigten Funktionen             *
// ********************************************************************

bool multiply(cln::cl_I & factor1, cln::cl_I & factor2, cln::cl_I & result, cln::cl_I & gesamtschema)
{
  // Funktion multipliziert das durch [factor1] repraesentierte Polynom mit dem
  // durch [factor2] repraesentierten Polynom und speichert das Ergebnis als
  // Polyonom in [result].

  result = factor1*factor2;
  result = result & gesamtschema;

  return true;
}

bool add(cln::cl_I & summand1, cln::cl_I & summand2, cln::cl_I & result)
{
  // Funktion addiert das durch [summand1] repraesentierte Polynom zu dem
  // durch [summand2] repraesentierten Polynom und speichert das Ergebnis als
  // Polyonom in [result].

  result = summand1+summand2;

  return true;
}

bool CycleIndex_2(cln::cl_I & poly, cln::cl_I & result, unsigned int length, cln::cl_I & schema, cln::cl_I & gesamtschema)
{
  // Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 2 auf [poly] an und
  // speichert das Ergebnis als ein Polynom in [result]

  cln::cl_I l2 = 0;           //L(x^2)
  cln::cl_I temp = 0;         //temporaere Zahl
  cln::cl_I l2invers = 0;     //enthaelt das Polynom l2 jedoch sind die Koeffizienten in umgekehrter Reihenfolge abgespeichert

  temp = poly;  //Berechnung von L(x^2);
  for(unsigned int i=0; i<=length; i++)
  {
    l2invers = l2invers | (temp & schema);
    temp = temp >> base;
    l2invers = l2invers << base;
  }
  for(unsigned int i=0; i<=length; i++)
  {
    l2invers = l2invers >> base;
    l2 = l2 << 2*base;                        //verschiebe um 2*base -->    L(x^2) = (an)(0)(an-1)(0) . . . (2)(0)(1)
    l2 = l2 | (l2invers & schema);
  }

  //Berechnung des gesamten Bruchs
  result = ((poly * poly)&gesamtschema) + l2;
  //Teile nun noch jeden Koeffizienten durch 2
  temp = 0;
  for(unsigned int i=0; i<=length; i++)
  {
    temp = temp | (result & schema);
    temp = temp << base;
    result = result >> base;
  }
  for(unsigned int i=0;i<=length; i++)
  {
    result = result << base;
    temp = temp >> base;
    result = result | (exquo((temp & schema),2));
  }
  result = result&gesamtschema;

  return true;
}

bool CycleIndex_3(cln::cl_I & poly, cln::cl_I & result, unsigned int length, cln::cl_I & schema, cln::cl_I & gesamtschema)
{
  // Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 3 auf [poly] an und
  // speichert das Ergebnis als ein Polynom mit Maximalgrad [length] in [result]

  //Verfahren mit Links- und Rechtsshifts wie bei CycleIndex_2

  cln::cl_I l2 = 0;  //3*L(x^2)
  cln::cl_I l3 = 0;  //2*L(x^3)
  cln::cl_I temp = 0;
  cln::cl_I linvers = 0;

  temp = poly;  //Berechnung von 3*L(x^2);
  for(unsigned int i=0; i<=length; i++)
  {
    linvers = linvers | (temp & schema);
    temp = temp >> base;
    linvers = linvers << base;
  }
  for(unsigned int i=0; i<=length; i++)
  {
    l2 = l2 << 2*base;
    linvers = linvers >> base;
    l2 = l2 | (3*(linvers & schema));
  }

  linvers = 0;

  temp = poly;  //Berechnung von 2*L(x^3);
  for(unsigned int i=0; i<=length; i++)
  {
    linvers = linvers | (temp & schema);
    temp = temp >> base;
    linvers = linvers << base;
  }
  for(unsigned int i=0; i<=length; i++)
  {
    l3 = l3 << 3*base;
    linvers = linvers >> base;
    l3 = l3 | (2*(linvers & schema));
  }

  //Berechnung des gesamten Bruchs
  result = ((((poly * poly)&gesamtschema)*poly)&gesamtschema) + ((l2*poly)&gesamtschema) + l3;
  temp = 0;
  //Teile nun noch durch 6
  for(unsigned int i=0; i<=length; i++)
  {
    temp = temp | (result & schema);
    temp = temp << base;
    result = result >> base;
  }
  for(unsigned int i=0;i<=length; i++)
  {
    temp = temp >> base;
    result = result << base;
    result = result | (exquo((temp & schema),6));
  }
  result = result&gesamtschema;
  return true;
}




// ********************************************************************
// *                       Hauptprogramm                              *
// ********************************************************************


int main(void)
{
  unsigned int n;                        // Anzahl der C-Atome
  cout << "Anzahl der Strukturisomere der Alkene [ C(n)H(2n+2) ]" << endl;
  cout << "=====================================================" << endl;
  cout << endl;
  cout << "Bitte geben Sie die Anzahl der C-Atome ein : ";
  cin  >> n;


  unsigned int i;                      // Schleifenvariable
  unsigned int d=n-2;                // n-2 ist die maximale Laenge eines Teilbaumes

  cln::cl_I schema;                  // Schema zum Umwandeln von Zahlen in Vektoren: besteht aus (base) Bit 1-ern
  schema = 1;
  for(unsigned int i=1;i<base2;i++)
    {
    schema = schema << 1;
    schema = schema | 1;
    }
  cln::cl_I gesamtschema;       // Schema zum Abschneiden der ueberfluessigen Bits beim Multiplizieren: besteht aus (n*base) Bit 1-ern
  gesamtschema = schema;
  for(unsigned int i=0;i<n;i++)
    {
    gesamtschema = gesamtschema << base2;
    gesamtschema = gesamtschema | schema;
    }

  cln::cl_I sum;                          // Polynom, das die kleineren (laengste gerade Kette) Teilstrukturen abzaehlt
  cln::cl_I longest;                      // Polynom, das die groesseren (laengste gerade Kette) Teilstrukturen beschreibt
  cln::cl_I result;                       // Ergebnispolynom
  cln::cl_I t1, t2, t3, t4;                   // temporaere Zahlen zum Speichern von Zwischenergebnissen

  //initialisiere die Polynome
  longest = 1;
  sum = 0;
  t1 = 0;
  t2 = 0;
  t3 = 0;
  t4 = 0;
  result = 1;
  result = result << 2*base2;

  for(i=1;i<=d;i++)             // laufe durch die moeglichen Laengen der laengsten geraden Ketten der Wurzelbaeume
  {
    cout << "." << flush;
    // Berechne t1 = CycleIndex_3(longest)+CycleIndex_2(longest)*sum+longest*CycleIndex_2(sum)
    CycleIndex_3(longest,t1,n,schema,gesamtschema);
    CycleIndex_2(longest,t2,n,schema,gesamtschema);
    multiply(t2,sum,t3,gesamtschema);
    add(t1,t3,t2);
    CycleIndex_2(sum,t1,n,schema,gesamtschema);
    multiply(t1,longest,t3,gesamtschema);
    add(t2,t3,t1);

    // Aktualisiertes [sum] = sum + longest
    add(sum,longest,t2);
    sum=t2;

    // Aktualisiertes [longest]= t1 * X
    longest=t1;
    longest=longest<<base2;
    longest=longest&gesamtschema;
     
  }
  
  add(sum,longest,t2);
  CycleIndex_2(t2,t3,n,schema,gesamtschema);
  CycleIndex_2(t3,result,n,schema,gesamtschema);
  result=(result << 2*base2) & gesamtschema;

  cout << "\n\n\n" << endl;



  // Ausgabe der Ergebnisse
  ofstream ausgabe;
  ausgabe.open("alkene.txt");
  cout << "Anzahl der C-Atome |    Anzahl der Isomere" << endl;
  cout << "------------------------------------------" << endl;
  if(datentyp==2)
    for(i=0;i<=n;i++)
    {
      cout << "      " << setw(5) << i << "        | " << setw(35) << setprecision(35) << (result&schema) << endl;
      result = result >> base2;
    }
  else
    for(i=0;i<=n;i++)
    {
      cout << "      " << setw(5) << i << "        | " << (result&schema) << endl;
      ausgabe << i << "\t" << (result&schema) << endl;
      result = result >> base2;
    }


  cout << endl << endl << endl;
  ausgabe << endl;
  ausgabe.close();

  return 0;
}
