// *********************************************************************
// *                                                                   *
// *       Zufaelliges Erzeugen von Isomeren der Alkane (C_nH_2n+2)    *
// *       ========================================================    *
// *                                                                   *
// *       Autor:        Sascha Kurz                                   *
// *       Email:        sascha.kurz@uni-bayreuth.de                   *
// *       Homepage:     www.mathe2.uni-bayreuth.de/sascha             *
// *                                                                   *
// *       Filename:           zufalls_alkane.cpp                      *
// *       zuletzt geaendert:  11.01.2005                              *
// *       Betriebssystem:     Suse Linux 9.1                          *
// *       Compiler:           gcc3.3.3                                *
// *       Compiler-Optionen:  g++ -O2 -Wall -pedantic -ansi           *
// *                           konstruktion_alkane_2.cpp -lcln         *
// *                           -o zufall                               *
// *                                                                   *
// *       Algorithmus                                                 *
// *       -----------                                                 *
// *       Basierend auf dem Artikel "Anzahl von Strukturisomeren      *
// *       der Alkane". Es werden 4-valente Wurzelbaeume nach Hoehe    *
// *       und Anzahl der Knoten abgezaehlt. Mit diesen Anzahlen ist   *
// *       es moeglich einem Alkan bijektiv eine Zahl zuzuordnen, so   *
// *       dass die Alkane aus n C-Atomen die Nummern 0,1,2,...,a      *
// *       besitzen, wobei a+1 die Anzahl der Isomere der Alkane aus   *
// *       n C-Atomen ist. Details sind bitte dem zitierten Artikel    *
// *       zu entnehmen. Eine genauere Dokumentation der Konstruktion  *
// *       von Alkanen findet sich in "konstruiere_alkane.cpp". Zum    *  
// *       reinen Abzaehlen von Isomeren der Alkane kann man das       *
// *       Programm "zaehle_alkane.cpp" verwenden.                     * 
// *                                                                   *
// *       Schlussbemerkung                                            *
// *       ================                                            *
// *       Bei Fragen, Anmerkungen, Kritik oder sonstigen Kommentaren  *
// *       wende man sich bitte per email an den oben genannten Autor  *
// *       dieses Programms. Der Quellcode ist fuer nicht-kommerzielle *
// *       Anwendungen frei verfuegbar und darf beliebig veraendert    *
// *       werden. Bei Veraenderungen ist geeignet darauf hinzuweisen, *
// *       von wem sie vorgenommen wurden. Es wuerde mich freuen ueber *
// *       Verwendungen dieses Quellcodes eine kurze Nachricht per     *
// *       email zu erhalten.                                          *
// *                                                                   *
// *       Sascha Kurz                                                 *    
// *                                                                   *
// *********************************************************************

#include<iostream>                       // Headerfile fuer die Aus- und Eingabe
#include<iomanip>                        // Headerfile zu Manipulation der Ausgabe
#include<fstream>                        // Headerfile fuer Datei-Funktionen
#include<vector>                         // Headerfile fuer die Klasse Vektor
#include<cln/real.h>                     // Dezimalzahlen 
#include<cln/integer.h>                  // Langzahlarithmetik
#include<cln/integer_io.h>               // Ausgabe von langen Zahlen
#include<cln/random.h>                   // Headerfile fuer Zufallszahlen

#define DEBUG false                      // Sollen zusaetzliche Ausgaben zum Aufspueren von Fehler gemacht werden?
#define BILDSCHIRMBREITE 122             // Breite des Bildschirms in Ascii-Zeichen

using namespace std;

struct Alkan                               // Datenstruktur fuer ein Alkan
{
  vector <unsigned char> Wurzelbaum_1;     // Vektor fuer den ersten Wurzelbaum
  vector <unsigned char> Wurzelbaum_2;     // Vektor fuer den zweiten Wurzelbaum
  vector <unsigned char> Wurzelbaum_3;     // Vektor fuer den dritten Wurzelbaum  
  vector <unsigned char> Wurzelbaum_4;     // Vektor fuer den vierten Wurzelbaum 
  bool laengste_gerade_Kette_ist_gerade;   // Ist die laengste gerade Kette des Alkans eine gerade Zahl?
};

struct ascii_platzierung                   // Datenstruktur fuer eine Platzierung eines Alkans 
{                                          // in "Ascii-Art" auf dem Bildschirm 
  vector< vector<char> > Matrix;           // Matrix der auf dem Bildschirm dartzstellenden Zeichen
  unsigned int anzahl_zeilen;              // Anzahl der Spalten der Matrix 
  unsigned int anzahl_spalten;             // Anzahl der Zeilen der Matrix  
  unsigned int position_wurzel;            // Spalten-Nr. der Wurzel in der untersten (0.) Zeile 
};

// *********************************************************************
// *                  Prototypdeklarationen                            *
// *********************************************************************



// bildet eine Zahl r bijektiv auf ein Alkan aus n C-Atomen ab
int unrank_alkane(cln::cl_I r,Alkan& Alk,unsigned int n,vector< vector<cln::cl_I> >& W,vector< vector<cln::cl_I> >& W2,vector< vector<cln::cl_I> >& A);    

// bildet eine Zahl r bijektiv auf einen Wurzelbaum der Hoehe kleiner hoehe bestehend aus anzahl_knoten Knoten ab
int unrank_wurzelbaum_kleiner(cln::cl_I r,unsigned int hoehe, unsigned int anzahl_knoten, vector<unsigned int>& wurzelbaum,vector< vector<cln::cl_I> >& W1,vector< vector<cln::cl_I> >& W2);

// bildet eine Zahl r bijektiv auf einen Wurzelbaum der Hoehe hoehe bestehend aus anzahl_knoten Knoten ab
int unrank_wurzelbaum(cln::cl_I r,unsigned int hoehe, unsigned int anzahl_knoten, vector<unsigned int>& wurzelbaum,vector< vector<cln::cl_I> >& W1,vector< vector<cln::cl_I> >& W2);

// berechnet aus einem Rang rest und der Gesamtzahl g der moeglichen Strukturen den Rang der Struktur und speichert
// ihn in u (technische Funktion, siehe hierzu ihre Anwendung in den unrank-Funktionen)
void choose_1(cln::cl_I& rest,cln::cl_I& g,cln::cl_I& u);

// berechnet aus einem Rang rest und der Gesamtzahl g der moeglichen Strukturen die Einzelraenge von einer 2-er 
// Menge derartiger Strukturen und speichert sie in u1 und 2 (technische Funktion, siehe hierzu ihre Anwendung
// in den unrank-Funktionen)
void choose_2(cln::cl_I& rest,cln::cl_I& g,cln::cl_I& u1,cln::cl_I& u2);

// berechnet aus einem Rang rest und der Gesamtzahl g der moeglichen Strukturen die Einzelraenge von einer 3-er 
// Menge derartiger Strukturen und speichert sie in u1, u2 und 3 (technische Funktion, siehe hierzu ihre Anwendung
// in den unrank-Funktionen)
void choose_3(cln::cl_I& rest,cln::cl_I& g,cln::cl_I& u1,cln::cl_I& u2,cln::cl_I& u3);

// berechnet aus einem Rang rest und der Gesamtzahl g der moeglichen Strukturen die Einzelraenge von einer 4-er 
// Menge derartiger Strukturen und speichert sie in u1, u2, u3 und 4 (technische Funktion, siehe hierzu ihre 
// Anwendung in den unrank-Funktionen)
void choose_4(cln::cl_I& rest,cln::cl_I& g,cln::cl_I& u1,cln::cl_I& u2,cln::cl_I& u3,cln::cl_I& u4);

// technische Funktion, bestimmt einen Endwert fuer eine Schleifenvariable, siehe Implementierung
unsigned int bestimme_ende(unsigned int h,unsigned int n);

// technische Funktion, bestimmt einen Startwert fuer eine Schleifenvariable, siehe Implementierung
unsigned int bestimme_start_1(unsigned int h,unsigned int n);

// technische Funktion, bestimmt einen Startwert fuer eine Schleifenvariable, siehe Implementierung
unsigned int bestimme_start_2(unsigned int h,unsigned int n);

// technische Funktion, bestimmt einen Startwert fuer eine Schleifenvariable, siehe Implementierung
unsigned int bestimme_start_3(unsigned int h,unsigned int n);

// technische Funktion, bestimmt einen Startwert fuer eine Schleifenvariable, siehe Implementierung
unsigned int bestimme_start_4(unsigned int i,unsigned int h,unsigned int n);

// technische Funktion, bestimmt einen Startwert fuer eine Schleifenvariable, siehe Implementierung
unsigned int bestimme_start_5(unsigned int h,unsigned int n);

// technische Funktion, bestimmt einen Startwert fuer eine Schleifenvariable, siehe Implementierung
unsigned int bestimme_start_6(unsigned int i,unsigned int h,unsigned int n);

// technische Funktion, bestimmt einen Startwert fuer eine Schleifenvariable, siehe Implementierung
unsigned int bestimme_start_7(unsigned int i,unsigned int h,unsigned int n);

// technische Funktion, bestimmt einen Startwert fuer eine Schleifenvariable, siehe Implementierung
unsigned int bestimme_start_8(unsigned int i,unsigned int h,unsigned int n);

// technische Funktion, bestimmt einen Startwert fuer eine Schleifenvariable, siehe Implementierung
unsigned int bestimme_start_9(unsigned int i,unsigned int h,unsigned int n);

// technische Funktion, bestimmt einen Startwert fuer eine Schleifenvariable, siehe Implementierung
unsigned int bestimme_start_10(unsigned int h,unsigned int n);

// technische Funktion, bestimmt einen Startwert fuer eine Schleifenvariable, siehe Implementierung
unsigned int bestimme_start_11(unsigned int h,unsigned int n);

// technische Funktion, bestimmt einen Startwert fuer eine Schleifenvariable, siehe Implementierung
unsigned int bestimme_start_12(unsigned int h,unsigned int n);

// technische Funktion, bestimmt einen Startwert fuer eine Schleifenvariable, siehe Implementierung
unsigned int bestimme_start_13(unsigned int h,unsigned int n);

// technische Funktion, bestimmt einen Startwert fuer eine Schleifenvariable, siehe Implementierung
unsigned int bestimme_start_14(unsigned int i,unsigned int h,unsigned int n);

// technische Funktion, bestimmt einen Startwert fuer eine Schleifenvariable, siehe Implementierung
unsigned int bestimme_start_15(unsigned int h,unsigned int n);

// technische Funktion, bestimmt einen Startwert fuer eine Schleifenvariable, siehe Implementierung
unsigned int bestimme_start_16(unsigned int i,unsigned int h,unsigned int n);

// gibt die maximale Anzahl an Knoten eines 4-valenten 
// Wurzelbaums der Hoehe h zurueck
int max_knoten(unsigned int h);

int ausgabe_ascii(Alkan& A);
// Gibt eine zweidimensionale Platzierung eines Alkanmolekuels in "Ascii-Art" 
// auf dem Bildschirm aus, sofern es nicht zu breit ist 

int ausgabe(Alkan& A);
// Schreibt ein Alkan im XML-Format in die Datei graph 

// Berechnet die Anzahl der Wurzelbaeume nach Hoehe und Anzahl der Knoten
void Berechne_Anzahl_Wurzelbaeume(vector< vector<cln::cl_I> >& Wurzelbaeume,vector< vector<cln::cl_I> >& Wurzelbaeume_2, unsigned int n);

// Berechnet die Anzahl der Alkane nach der Laenge der laengsten geraden Kette und nach Anzahl der Knoten
void Berechne_Anzahl_Alkane(vector< vector<cln::cl_I> >& Alkane, vector< vector<cln::cl_I> >& Wurzelbaeume,vector< vector<cln::cl_I> >& Wurzelbaeume_2, unsigned int n);

// Multipliziert zwei Polynome, repraesentiert durch Vektoren. Das Ergebnis ist ein Polyom vom Maximalgrad [length]
bool multiply(vector<cln::cl_I>& factor_1,vector<cln::cl_I>& factor_2,vector<cln::cl_I>& result,unsigned int length);

// Addiert zwei Polynome, repraesentiert durch Vektoren. Das Ergebnis ist ein Polyom vom Maximalgrad [length]
bool add(vector<cln::cl_I>& summand_1,vector<cln::cl_I>& summand_2,vector<cln::cl_I>& result,unsigned int length);

// Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 2 auf [poly] an und
// speichert das Ergebnis als ein Polynom mit Maximalgrad [length] in [result]
bool CycleIndex_2(vector<cln::cl_I>& poly,vector<cln::cl_I>& result,unsigned int length);

// Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 3 auf [poly] an und
// speichert das Ergebnis als ein Polynom mit Maximalgrad [length] in [result]
bool CycleIndex_3(vector<cln::cl_I> & poly,vector<cln::cl_I>& result,unsigned int length);

// Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 4 auf [poly] an und
// speichert das Ergebnis als ein Polynom mit Maximalgrad [length] in [result]
bool CycleIndex_4(vector<cln::cl_I> & poly,vector<cln::cl_I>& result,unsigned int length);

inline long double exquo(long double a,long double b)
{
  // Gibt a/b zurueck, Funktion gibt es nur aus Kompatibilitaetsgruenden zur CLN
  if(b!=0) return a/b;
  return -1;
}

inline unsigned long exquo(unsigned long a,unsigned long b)
{
  // Gibt a/b zurueck, Funktion gibt es nur aus Kompatibilitaetsgruenden zur CLN
  if(b!=0) return a/b;
  return 0;
}



// ********************************************************************
// *         Implementierungen der benoetigten Funktionen              *
// ********************************************************************

int max_knoten(unsigned int h)
{
  // h: Hoehe eines Wurzelbaums
  //
  // Funktion
  // --------
  // Gibt die maximale Anzahl an Knoten eines 4-valenten Wurzelbaums 
  // der Hoehe h zurueck. Falls diese Zahl allerdings zu gross waere, wird 
  // -1 zurueckgegeben.

  if(h<0) return -1;  // Wurzelbaeume mit negativer Hoehe gibt es nicht
  if(h==0) return 0;  // ein Wurzelbaum der Hoehe 0 besteht aus 0 Knoten
  if(h>16) return -1; // die Anzahl der moeglichen Knoten passt nicht mehr 
                      // in den Datentyp int 

  unsigned int i;
  int a=1;
  int s=1;
  for(i=1;i<h;i++)
  {
    a=a*3;
    s=s+a;
  }
  return s;
}

unsigned int bestimme_ende(unsigned int h,unsigned int n)
{
  // h: maximale Hoehe des Wurzelbaums
  // n: bisherige obere Schranke
  // 
  // Funktion
  // --------
  // Wie viele Knoten koennen Wurzelbaeume der Hoehe n hoechstens haben? 
  // Verwende diese und die uebergegebene obere Schranke n zur Konstruktion
  // einer oberen Schranke.
  int r2=max_knoten(h);  // Wieviele Knoten kann ein Wurzelbaum der 
                         // Hoehe h maximal haben
  if(r2<0) return n;
  return min(n,(unsigned int) r2); // gebe bessere obere Schranke zurueck   
}

unsigned int bestimme_start_10(unsigned int h,unsigned int n)
{
  // technische Funktion
  // konstruiert werden sollen Alkane aus zwei Wurzelbaeumen der Hoehe h.
  // Insgesamt soll das Alkan aus n C-Atomen bestehen. Die beiden Wurzelbaeume
  // sollen jeweils aus i1 bzw. i2 Knoten bestehen. Der erste Wurzelbaum hat die 
  // Hoehe h und hoechstens so viele Knoten wie der zweite Wurzelbaum.
  // Ziel dieser Funktion ist die Bestimmung der unteren Grenze fuer 
  // eine Schleife ueber i1.
  unsigned int r;        // Rueckgabewert 
  r=h;                   // um die Hoehe h zu besitzen muss der Wurzelbaum aus 
                         // mindestens h Knoten bestehen
  int rest;              // Knoten die noch zu verteilen waeren
  int r2;                // zur Speicherung eines Funktionswertes
  rest=n;                // Es stehen noch n Knoten zur Verfuegung          
  r2=max_knoten(h);      // Wieviele Knoten kann ein Wurzelbaum der 
                         // Hoehe h maximal besitzen?
  if(r2<0 || rest<r2)    // Falls der zweite Wurzelbaum die restlichen
    return r;            // Knoten alleine aufbrauchen koennte, so laesst
                         // sich keine bessere untere Schranke fuer i1 bestimmen
  rest=rest-r2;          // fuer den ersten Wurzelbaum sind noch mindestens 
                         // rest Knoten verfuegbar 
  r=max(r,(unsigned int) rest);
  return r;              // gebe untere Schranke zurueck
}

unsigned int bestimme_start_11(unsigned int h,unsigned int n)
{
  // technische Funktion
  // konstruiert werden sollen Wurzelbaeume der Hoehe h+1 aus einem Wurzelbaum der 
  // Hoehe h und zwei Wurzelbaeumen der Hoehe kleiner h. Die 3 Teilwurzelbaeume 
  // duerfen insgesamt n Knoten verbrauchen. Der erste Wurzelbaum soll aus i1, der
  // zweite aus i2 und der dritte aus i3 Knoten bestehen.
  // Ziel dieser Funktion ist die Bestimmung der unteren Grenze fuer 
  // eine Schleife ueber i1.
  unsigned int r;        // Rueckgabewert 
  r=h;                   // um die Hoehe h zu besitzen muss der Wurzelbaum aus 
                         // mindestens h Knoten bestehen
  int rest;              // Knoten die noch zu verteilen waeren
  int r2;                // zur Speicherung eines Funktionswertes
  rest=n;                // Es stehen noch n Knoten zur Verfuegung          
  r2=max_knoten(h-1);    // Wieviele Knoten kann ein Wurzelbaum der 
                         // Hoehe kleiner h maximal besitzen?
  if(r2<0 || rest<2*r2)  // Falls der zweite und der dritte Wurzelbaum die
    return r;            // restlichen Knoten alleine aufbrauchen koennte, so laesst
                         // sich keine bessere untere Schranke fuer i1 bestimmen
  rest=rest-2*r2;        // fuer den ersten Wurzelbaum sind noch mindestens 
                         // rest Knoten verfuegbar 
  r=max(r,(unsigned int) rest);
  return r;              // gebe untere Schranke zurueck
}

unsigned int bestimme_start_12(unsigned int h,unsigned int n)
{
  // technische Funktion
  // konstruiert werden sollen Wurzelbaeume der Hoehe h+1 aus einem Wurzelbaum der 
  // Hoehe h und zwei Wurzelbaeumen der Hoehe kleiner h. Die letzen beiden Teilwurzelbaeume 
  // duerfen insgesamt n Knoten verbrauchen. Der erste Wurzelbaum soll aus i1, der
  // zweite aus i2 und der dritte aus i3 Knoten bestehen.
  // Ziel dieser Funktion ist die Bestimmung der unteren Grenze fuer 
  // eine Schleife ueber i2.
  unsigned int r;        // Rueckgabewert 
  r=0;                   // triviale untere Schranke
  int rest;              // Knoten die noch zu verteilen waeren
  int r2;                // zur Speicherung eines Funktionswertes
  rest=n;                // Es stehen noch n Knoten zur Verfuegung          
  r2=max_knoten(h-1);    // Wieviele Knoten kann ein Wurzelbaum der 
                         // Hoehe kleiner h maximal besitzen?
  if(r2<0 || rest<r2)    // Falls der dritte Wurzelbaum die restlichen
    return r;            // Knoten alleine aufbrauchen koennte, so laesst sich
                         // keine bessere untere Schranke fuer i2 bestimmen
  rest=rest-r2;          // fuer den zweiten Wurzelbaum sind noch mindestens 
                         // rest Knoten verfuegbar 
  r=max(r,(unsigned int) rest);
  return r;              // gebe untere Schranke zurueck
}

unsigned int bestimme_start_13(unsigned int h,unsigned int n)
{
  // technische Funktion
  // konstruiert werden sollen Wurzelbaeume der Hoehe h+1 aus zwei Wurzelbaeumen der 
  // Hoehe h und einem Wurzelbaum der Hoehe kleiner h. Die 3 Teilwurzelbaeume 
  // duerfen insgesamt n Knoten verbrauchen. Der erste Wurzelbaum soll aus i1, der
  // zweite aus i2 und der dritte aus i3 Knoten bestehen.
  // Ziel dieser Funktion ist die Bestimmung der unteren Grenze fuer 
  // eine Schleife ueber i1.
  unsigned int r;        // Rueckgabewert 
  r=h;                   // um die Hoehe h zu besitzen muss der Wurzelbaum aus 
                         // mindestens h Knoten bestehen
  int rest;              // Knoten die noch zu verteilen waeren
  int r2;                // zur Speicherung eines Funktionswertes
  rest=n;                // Es stehen noch n Knoten zur Verfuegung          
  r2=max_knoten(h-1);    // Wieviele Knoten kann ein Wurzelbaum der 
                         // Hoehe kleiner h maximal besitzen?
  if(r2<0 || rest<r2)    // Falls der dritte Wurzelbaum die restlichen
    return r;            // Knoten alleine aufbrauchen koennte, so laesst
                         // sich keine bessere untere Schranke fuer i1 bestimmen
  rest=rest-r2;          // fuer den ersten und zweiten Wurzelbaum sind noch mindestens 
                         // rest Knoten verfuegbar 
  r2=max_knoten(h);      // Wieviele Knoten kann ein Wurzelbaum der 
                         // Hoehe h maximal besitzen?
  if(r2<0 || rest<r2)    // Falls der zweite Wurzelbaum die restlichen
    return r;            // Knoten alleine aufbrauchen koennte, so laesst
                         // sich keine bessere untere Schranke fuer i1 bestimmen
  rest=rest-r2;          // fuer den ersten Wurzelbaum sind noch mindestens 
                         // rest Knoten verfuegbar 
  r=max(r,(unsigned int) rest);
  return r;              // gebe untere Schranke zurueck
}

unsigned int bestimme_start_14(unsigned int i,unsigned int h,unsigned int n)
{
  // technische Funktion
  // konstruiert werden sollen Wurzelbaeume der Hoehe h+1 aus zwei Wurzelbaeumen der 
  // Hoehe h und einem Wurzelbaum der Hoehe kleiner h. Die beiden letzen Teilwurzelbaeume 
  // duerfen insgesamt n Knoten verbrauchen. Der erste Wurzelbaum soll aus i1 bzw. i, der
  // zweite aus i2 und der dritte aus i3 Knoten bestehen. Weiter soll i=i1<=i2 erfuellt sein.
  // Ziel dieser Funktion ist die Bestimmung der unteren Grenze fuer 
  // eine Schleife ueber i2.
  unsigned int r;        // Rueckgabewert 
  r=i;                   // i2>=i1
  int rest;              // Knoten die noch zu verteilen waeren
  int r2;                // zur Speicherung eines Funktionswertes
  rest=n;                // Es stehen noch n Knoten zur Verfuegung          
  r2=max_knoten(h-1);    // Wieviele Knoten kann ein Wurzelbaum der 
                         // Hoehe kleiner h maximal besitzen?
  if(r2<0 || rest<r2)    // Falls der dritte Wurzelbaum die restlichen
    return r;            // Knoten alleine aufbrauchen koennte, so laesst
                         // sich keine bessere untere Schranke fuer i2 bestimmen
  rest=rest-r2;          // fuer den zweiten Wurzelbaum sind noch mindestens 
                         // rest Knoten verfuegbar 
  r=max(r,(unsigned int) rest);
  return r;              // gebe untere Schranke zurueck
}

unsigned int bestimme_start_15(unsigned int h,unsigned int n)
{
  // technische Funktion
  // konstruiert werden sollen Wurzelbaeume der Hoehe h+1 aus drei Wurzelbaeumen der 
  // Hoehe h. Die 3 Teilwurzelbaeume duerfen insgesamt n Knoten verbrauchen. Der erste 
  // Wurzelbaum soll aus i1, der zweite aus i2 und der dritte aus i3 Knoten bestehen.
  // Weiter soll i1<=i2<=i3 gelten.
  // Ziel dieser Funktion ist die Bestimmung der unteren Grenze fuer 
  // eine Schleife ueber i1.
  unsigned int r;        // Rueckgabewert 
  r=h;                   // um die Hoehe h zu besitzen muss der Wurzelbaum aus 
                         // mindestens h Knoten bestehen
  int rest;              // Knoten die noch zu verteilen waeren
  int r2;                // zur Speicherung eines Funktionswertes
  rest=n;                // Es stehen noch n Knoten zur Verfuegung          
  r2=max_knoten(h);      // Wieviele Knoten kann ein Wurzelbaum der 
                         // Hoehe h maximal besitzen?
  if(r2<0 || rest<2*r2)  // Falls der zweite und der dritte Wurzelbaum die 
    return r;            // restlichen Knoten alleine aufbrauchen koennte, so laesst
                         // sich keine bessere untere Schranke fuer i1 bestimmen
  rest=rest-2*r2;        // fuer den ersten Wurzelbaum sind noch mindestens 
                         // rest Knoten verfuegbar 
  r=max(r,(unsigned int) rest);
  return r;              // gebe untere Schranke zurueck
}

unsigned int bestimme_start_16(unsigned int i,unsigned int h,unsigned int n)
{
  // technische Funktion
  // konstruiert werden sollen Wurzelbaeume der Hoehe h+1 aus drei Wurzelbaeumen der 
  // Hoehe h. Die beiden letzen Teilwurzelbaeume duerfen insgesamt n Knoten verbrauchen. 
  // Der erste Wurzelbaum soll aus i1 bzw. i, der zweite aus i2 und der dritte aus i3
  // Knoten bestehen. Weiter soll i=i1<=i2<=i3 erfuellt sein.
  // Ziel dieser Funktion ist die Bestimmung der unteren Grenze fuer 
  // eine Schleife ueber i2.
  unsigned int r;        // Rueckgabewert 
  r=i;                   // i2>=i1
  int rest;              // Knoten die noch zu verteilen waeren
  int r2;                // zur Speicherung eines Funktionswertes
  rest=n;                // Es stehen noch n Knoten zur Verfuegung          
  r2=max_knoten(h);      // Wieviele Knoten kann ein Wurzelbaum der 
                         // Hoehe h maximal besitzen?
  if(r2<0 || rest<r2)    // Falls der dritte Wurzelbaum die restlichen
    return r;            // Knoten alleine aufbrauchen koennte, so laesst
                         // sich keine bessere untere Schranke fuer i2 bestimmen
  rest=rest-r2;          // fuer den zweiten Wurzelbaum sind noch mindestens 
                         // rest Knoten verfuegbar 
  r=max(r,(unsigned int) rest);
  return r;              // gebe untere Schranke zurueck
}

unsigned int bestimme_start_1(unsigned int h,unsigned int n)
{
  // technische Funktion
  // konstruiert werden sollen Alkane aus zwei Wurzelbaeumen der Hoehe h
  // und zwei Wurzelbaeumen der Hoehe kleiner h. Insgesamt soll das Alkan 
  // aus n C-Atomen bestehen. Die einzelnen Wurzelbaeume sollen jeweils 
  // aus i1,i2,i3 bzw. i4 Knoten bestehen. Der erste Wurzelbaum hat die 
  // Hoehe h und hoechstens so viele Knoten wie der zweite Wurzelbaum.
  // Ziel dieser Funktion ist die Bestimmung der unteren Grenze fuer 
  // eine Schleife ueber i1.
  unsigned int r;        // Rueckgabewert 
  r=h;                   // um die Hoehe h zu besitzen muss der Wurzelbaum aus 
                         // mindestens h Knoten bestehen
  int rest;              // Knoten die noch zu verteilen waeren
  int r2;                // zur Speicherung eines Funktionswertes
  rest=n-1;              // Es stehen abzueglich des zentralen Knotens
                         // noch n-1 Knoten zur Verfuegung          
  r2=max_knoten(h-1);    // Wieviele Knoten kann ein Wurzelbaum der 
                         // Hoehe kleiner h maximal besitzen?
  if(r2<0 || rest<2*r2)  // Falls die zwei Wurzelbaeume der Hoehe kleiner h
    return r;            // die restlichen Knoten alleine aufbrauchen
                         // koennten, so laesst sich keine bessere untere
                         // Schranke fuer i1 bestimmen
  rest=rest-2*r2;        // fuer die ersten zwei Wurzelbaeume sind noch
                         // mindestens rest Knoten verfuegbar
  r2=max_knoten(h);      // Wieviele Knoten besitzt ein Wurzelbaum der Hoehe
                         // h maximal?
  if(r2<0 || rest<r2)    // Falls der zweite Wurzelbaum, der Hoehe h, die  
    return r;            // restlichen Knoten alleine aufbrauchen koennte,
                         // so  laesst sich keine bessere untere Schranke fuer
                         // i1 bestimmen 
  rest=rest-r2;          // fuer den ersten Wurzelbaum sind noch mindestens 
                         // rest Knoten verfuegbar 
  r=max(r,(unsigned int) rest);
  return r;              // gebe untere Schranke zurueck
}

unsigned int bestimme_start_2(unsigned int h,unsigned int n)
{
  // technische Funktion
  // konstruiert werden sollen Alkane aus zwei Wurzelbaeumen der Hoehe h+1
  // h+1 und zwei Wurzelbaeumen der Hoehe kleiner h+1. Die Anzahl der Knoten 
  // i1 und i2 fuer den ersten und den zweiten Wurzelbaum sind bereits
  // festgelegt. Die restlichen n Knoten sollen auf die zwei uebrigen
  // Wurzelbaeume der Hoehe kleiner h verteilt werden, so dass der dritte 
  // Wurzelbaum aus i3 und der vierte Wurzelbaum aus i4 Knoten besteht.
  // Es soll i3<=i4 gelten.
  // Ziel dieser Funktion ist die Bestimmung der unteren Grenze fuer 
  // eine Schleife ueber i3.
  unsigned int r;        // Rueckgabewert
  r=0;                   // 0 ist eine natuerliche untere Schranke
  int r2;                // zur Speicherung eines Funktionswertes
  r2=max_knoten(h);      // Wieviele Knoten kann ein Wurzelbaum der 
                         // Hoehe kleiner h+1 maximal besitzen? 
  if(r2<0 || ((int)n)<r2)// Falls der vierte Wurzelbaum die 
    return r;            // restlichen Knoten alleine aufbrauchen koennte, so 
                         // laesst sich keine bessere untere Schranke fuer i3
                         // bestimmen
  return n-(unsigned int)r2; // gebe bessere untere Schranke zurueck
}


unsigned int bestimme_start_3(unsigned int h,unsigned int n)
{
  // technische Funktion
  // konstruiert werden sollen Alkane aus drei Wurzelbaeumen der Hoehe h
  // und einem Wurzelbaum der Hoehe kleiner h. Insgesamt soll das Alkan 
  // aus n C-Atomen bestehen. Die einzelnen Wurzelbaeume sollen jeweils 
  // aus i1,i2,i3 bzw. i4 Knoten bestehen. Der erste Wurzelbaum hat die 
  // Hoehe h und hoechstens so viele Knoten wie der zweite Wurzelbaum oder
  // der dritte Wurzelbaum.
  // Ziel dieser Funktion ist die Bestimmung der unteren Grenze fuer 
  // eine Schleife ueber i1.
  unsigned int r;        // Rueckgabewert
  r=h;                   // um die Hoehe h zu besitzen muss der Wurzelbaum aus 
                         // mindestens h Knoten bestehen
  int rest;              // Knoten die noch zu verteilen waeren
  int r2;                // zur Speicherung eines Funktionswertes
  rest=n-1;              // Es stehen abzueglich des zentralen Knotens
                         // noch n-1 Knoten zur Verfuegung          
  r2=max_knoten(h-1);    // Wieviele Knoten kann ein Wurzelbaum der 
                         // Hoehe kleiner h maximal besitzen?
  if(r2<0 || rest<r2)    // Falls der vierte Wurzelbaum, der Hoehe kleiner 
    return r;            // h, die restlichen Knoten alleine aufbrauchen
                         // koennten, so laesst sich keine bessere untere
                         // Schranke fuer i1 bestimmen
  rest=rest-r2;          // fuer die ersten drei Wurzelbaeume sind noch
                         // mindestens rest Knoten verfuegbar
  r2=max_knoten(h);      // Wieviele Knoten besitzt ein Wurzelbaum der Hoehe
                         // h maximal?
  if(r2<0 || rest<2*r2)  // Falls der zweite und der dritte Wurzelbaum, der
    return r;            // Hoehe h, die restlichen Knoten alleine
                         // aufbrauchen koennte, so  laesst sich keine bessere
                         // untere Schranke fuer i1 bestimmen 
  rest=rest-2*r2;        // fuer den ersten Wurzelbaum sind noch mindestens 
                         // rest Knoten verfuegbar 
  r=max(r,(unsigned int) rest);
  return r;              // gebe untere Schranke zurueck
}


unsigned int bestimme_start_4(unsigned int i,unsigned int h,unsigned int n)
{
  // technische Funktion
  // konstruiert werden sollen Alkane aus drei Wurzelbaeumen der Hoehe h
  // und einem Wurzelbaum der Hoehe kleiner h. Die Anzahl der Knoten 
  // i1 fuer den ersten Wurzelbaum ist bereits festgelegt. Die restlichen
  // n Knoten sollen auf den zweiten und dritten Wurzelbaum, der Hoehe 
  // h, und den vierten Wurzelbaum der Hoehe kleiner h verteilt werden, so 
  // dass der zweite Wurzelbaum uas i2, der dritte Wurzelbaum aus i3 und
  // der vierte Wurzelbaum aus i4 Knoten besteht. Es soll i1<=i2<=i3
  // gelten. Der Wert i1 wird in Form der Variable i uebergeben.
  // Ziel dieser Funktion ist die Bestimmung der unteren Grenze fuer 
  // eine Schleife ueber i2.
  unsigned int r;        // Rueckgabewert
  r=i;                   // i2>=i1 liefert erste untere Schranke
  int rest;              // Knoten die noch zu verteilen waeren
  int r2;                // zur Speicherung eines Funktionswertes
  rest=n;                // Es stehen noch n Knoten zur Verfuegung
  r2=max_knoten(h-1);    // Wieviele Knoten kann ein Wurzelbaum der 
                         // Hoehe kleiner h maximal besitzen?
  if(r2<0 || rest<r2)    // Falls der vierte Wurzelbaum, der Hoehe kleiner 
    return r;            // h, die restlichen Knoten alleine aufbrauchen
                         // koennten, so laesst sich keine bessere untere
                         // Schranke fuer i2 bestimmen
  rest=rest-r2;          // fuer den zweiten und den dritten Wurzelbaum sind 
                         // mindestens noch rest Knoten verfuegbar
  r2=max_knoten(h);      // Wieviele Knoten kann ein Wurzelbaum der 
                         // Hoehe h maximal besitzen?
  if(r2<0 || rest<r2)    // Falls der dritte Wurzelbaum, der Hoehe  
    return r;            // h, die restlichen Knoten alleine aufbrauchen
                         // koennte, so laesst sich keine bessere untere
                         // Schranke fuer i2 bestimmen
  rest=rest-r2;          // fuer den zweiten Wurzelbaum sind mindestens noch
                         // rest knoten verfuegbar
  r=max(r,(unsigned int) rest);
  return r;              // gebe untere Schranke zurueck
}


unsigned int bestimme_start_5(unsigned int h,unsigned int n)
{
  // technische Funktion
  // konstruiert werden sollen Alkane aus vier Wurzelbaeumen der Hoehe h.
  // Insgesamt soll das Alkan aus n C-Atomen bestehen. Die einzelnen
  // Wurzelbaeume sollen jeweils aus i1,i2,i3 bzw. i4 Knoten bestehen. Dabei
  // soll i1<=i2<=i3<=i4 gelten.
  // Ziel dieser Funktion ist die Bestimmung der unteren Grenze fuer 
  // eine Schleife ueber i1.
  unsigned int r;        // Rueckgabewert 
  r=h;                   // um die Hoehe h zu besitzen muss der Wurzelbaum aus 
                         // mindestens h Knoten bestehen
  int rest;              // Knoten die noch zu verteilen waeren
  int r2;                // zur Speicherung eines Funktionswertes
  rest=n-1;              // Es stehen abzueglich des zentralen Knotens
                         // noch n-1 Knoten zur Verfuegung          
  r2=max_knoten(h);      // Wieviele Knoten kann ein Wurzelbaum der 
                         // Hoehe h maximal besitzen?
  if(r2<0 || rest<3*r2)  // Falls die letzen drei Wurzelbaeume die restlichen
    return r;            // Knoten alleine aufbrauchen koennten, so laesst
                         // sich keine bessere untere i1 bestimmen
  rest=rest-3*r2;        // fuer den ersten Wurzelbaum sind noch mindestens 
                         // rest Knoten verfuegbar
  r=max(r,(unsigned int) rest);
  return r;              // gebe untere Schranke zurueck
}


unsigned int bestimme_start_6(unsigned int i,unsigned int h,unsigned int n)
{
  // technische Funktion
  // konstruiert werden sollen Alkane aus vier Wurzelbaeumen der Hoehe h.
  // Die Anzahl der Knoten i1 fuer den ersten Wurzelbaum ist bereits
  // festgelegt. Die restlichen n Knoten sollen auf den zweiten, dritten
  // und vierten Wurzelbaum, der Hoehe h, verteilt werden, so dass der 
  // zweite Wurzelbaum uas i2, der dritte Wurzelbaum aus i3 und der vierte
  // Wurzelbaum aus i4 Knoten besteht. Es soll i1<=i2<=i3<=i4 
  // gelten. Der Wert i1 wird in Form der Variable i uebergeben.
  // Ziel dieser Funktion ist die Bestimmung der unteren Grenze fuer 
  // eine Schleife ueber i2.
  unsigned int r;        // Rueckgabewert
  r=i;                   // i2>=i1
  int rest;              // Knoten die noch zu verteilen waeren
  int r2;                // zur Speicherung eines Funktionswertes
  rest=n;                // Es stehen noch n Knoten zur Verfuegung
  r2=max_knoten(h);      // Wieviele Knoten kann ein Wurzelbaum der 
                         // Hoehe h maximal besitzen?
  if(r2<0 || rest<2*r2)  // Falls der dritte und vierte Wurzelbaum die
    return r;            // restlichen Knoten alleine aufbrauchen, so
                         // laesst sich keine bessere untere i2 bestimmen
  rest=rest-2*r2;        // fuer den zweiten Wurzelbaum sind noch mindestens 
                         // rest Knoten verfuegbar
  r=max(r,(unsigned int) rest);
  return r;              // gebe untere Schranke zurueck
}


unsigned int bestimme_start_7(unsigned int i,unsigned int h,unsigned int n)
{
  // technische Funktion
  // konstruiert werden sollen Alkane aus vier Wurzelbaeumen der Hoehe h.
  // Die Anzahl der Knoten i1 fuer den ersten Wurzelbaum bzw. die Anzahl 
  // der Knoten i2 fuer den zweiten Wurzelbaum ist bereits festgelegt. Die
  // restlichen n Knoten sollen auf den dritten und vierten Wurzelbaum, der
  // Hoehe h, verteilt werden, so dass der dritte Wurzelbaum aus i3 und der
  // vierte Wurzelbaum aus i4 Knoten besteht. Es soll i1<=i2<=i3<=i4 
  // gelten. Der Wert i2 wird in Form der Variable i uebergeben.
  // Ziel dieser Funktion ist die Bestimmung der unteren Grenze fuer 
  // eine Schleife ueber i3.
  unsigned int r;        // Rueckgabewert
  r=i;                   // i3>=i2
  int rest;              // Knoten die noch zu verteilen waeren
  int r2;                // zur Speicherung eines Funktionswertes
  rest=n;                // Es stehen noch n Knoten zur Verfuegung
  r2=max_knoten(h);      // Wieviele Knoten kann ein Wurzelbaum der 
                         // Hoehe h maximal besitzen?
  if(r2<0 || rest<r2)    // Falls der vierte Wurzelbaum die restlichen
    return r;            // Knoten alleine aufbrauchen koennte, so
                         // laesst sich keine bessere untere i3 bestimmen
  rest=rest-r2;          // fuer den dritten Wurzelbaum sind noch mindestens 
                         // rest Knoten verfuegbar
  r=max(r,(unsigned int) rest);
  return r;              // gebe untere Schranke zurueck
}

unsigned int bestimme_start_8(unsigned int i,unsigned int h,unsigned int n)
{
  // technische Funktion
  // konstruiert werden sollen Alkane aus drei Wurzelbaeumen der Hoehe h und 
  // einem Wurzelbaum der Hoehe kleiner h. Die Anzahl der Knoten i1 fuer den 
  // ersten Wurzelbaum bzw. die Anzahl der Knoten i2 fuer den zweiten Wurzelbaum
  // ist bereits festgelegt. Die restlichen n Knoten sollen auf den dritten und 
  // vierten Wurzelbaum verteilt werden, so dass der dritte Wurzelbaum aus i3  
  // und der vierte Wurzelbaum aus i4 Knoten besteht. Es soll i1<=i2<=i3 
  // gelten. Der Wert i2 wird in Form der Variable i uebergeben.
  // Ziel dieser Funktion ist die Bestimmung der unteren Grenze fuer 
  // eine Schleife ueber i3.
  unsigned int r;        // Rueckgabewert
  r=i;                   // i3>=i2
  int rest;              // Knoten die noch zu verteilen waeren
  int r2;                // zur Speicherung eines Funktionswertes
  rest=n;                // Es stehen noch n Knoten zur Verfuegung
  r2=max_knoten(h-1);    // Wieviele Knoten kann ein Wurzelbaum der 
                         // Hoehe kleiner h maximal besitzen?
  if(r2<0 || rest<r2)    // Falls der vierte Wurzelbaum die restlichen
    return r;            // Knoten alleine aufbrauchen koennte, so
                         // laesst sich keine bessere untere i3 bestimmen
  rest=rest-r2;          // fuer den dritten Wurzelbaum sind noch mindestens 
                         // rest Knoten verfuegbar
  r=max(r,(unsigned int) rest);
  return r;              // gebe untere Schranke zurueck
}

unsigned int bestimme_start_9(unsigned int i,unsigned int h,unsigned int n)
{
  // technische Funktion
  // konstruiert werden sollen Alkane aus zwei Wurzelbaeumen der Hoehe h und 
  // zwei Wurzelbaeumen der Hoehe kleiner h. Die Anzahl der Knoten i1 fuer den 
  // ersten Wurzelbaum ist bereits festgelegt. Die restlichen n Knoten sollen 
  // auf den zweiten, den dritten und den vierten Wurzelbaum verteilt werden, 
  // so dass der zweite Wurzelbaum aus i2 Knoten, dritte Wurzelbaum aus i3  
  // und der vierte Wurzelbaum aus i4 Knoten besteht. Es soll i1<=i2 
  // gelten. Der Wert i1 wird in Form der Variable i uebergeben.
  // Ziel dieser Funktion ist die Bestimmung der unteren Grenze fuer 
  // eine Schleife ueber i2.
  unsigned int r;        // Rueckgabewert
  r=i;                   // i2>=i1
  int rest;              // Knoten die noch zu verteilen waeren
  int r2;                // zur Speicherung eines Funktionswertes
  rest=n;                // Es stehen noch n Knoten zur Verfuegung
  r2=max_knoten(h-1);    // Wieviele Knoten kann ein Wurzelbaum der 
                         // Hoehe kleiner h maximal besitzen?
  if(r2<0 || rest<2*r2)  // Falls der dritte und der vierte Wurzelbaum die 
    return r;            // restlichen Knoten alleine aufbrauchen koennten, 
                         // so laesst sich keine bessere untere i2 bestimmen
  rest=rest-2*r2;        // fuer den zweiten Wurzelbaum sind noch mindestens 
                         // rest Knoten verfuegbar
  r=max(r,(unsigned int) rest);
  return r;              // gebe untere Schranke zurueck
}

void Berechne_Anzahl_Alkane(vector< vector<cln::cl_I> >& Alkane, vector< vector<cln::cl_I> >& Wurzelbaeume,vector< vector<cln::cl_I> >& Wurzelbaeume_2, unsigned int n)
{
  // Alkane:          Speicherplatz fuer Anzahl von Alkanen. Alkane[i][j] ist die Anzahl der Alkane 
  //                  mit laengster Kette der Laenge i bestehend aus j C-Atomen          
  // Wurzelbaeume:    Wurzelbaeume[i][j] ist Anzahl der Wurzelbaeume der Hoehe i bestehend j Knoten      
  // Wurzelbaeume_2:  Wurzelbaeume_2[i][j] ist Anzahl der Wurzelbaeume der Hoehe kleiner i bestehend j Knoten
  // n:               maximale Anzahl an C-Atomen             
  //
  // Funktion
  // --------
  // Berechnet die Anzahl Alkane[i][j] der Alkane mit laengster Kette der Laenge i bestehend aus j C-Atomen

  unsigned int i,j;       // Schleifenvariablen
  vector<cln::cl_I> t1;   // Vektor fuer Zwischenergebnisse
  vector<cln::cl_I> t2;   // Vektor fuer Zwischenergebnisse  
  vector<cln::cl_I> t3;   // Vektor fuer Zwischenergebnisse
  t1.resize(n+2);         // reserviere Speicherplatz
  t2.resize(n+2);         // reserviere Speicherplatz
  t3.resize(n+2);         // reserviere Speicherplatz
  Alkane.resize(n+1);     // reserviere Speicherplatz  
  for(i=0;i<=n;i++)       // reserviere Speicherplatz
    Alkane[i].resize(n+2);
  
  for(i=0;i<=n;i++)       // Startbelegung fuer die Alkane
  {
    Alkane[0][i]=0;
    Alkane[1][i]=0;
  } 
  Alkane[1][1]=1;         // Es gibt ein Alkan aus einem C-Atom, besitzt laengste Kette der Laenge 1
  for(j=2;j<=n;j++)       // durchlaufe die moeglichen Laengen der Ketten maximaler Laenge
  {
    if((j%2)==0)          // falls die Laenge der laengsten Kett gerade ist
    {
      CycleIndex_2(Wurzelbaeume[j/2],t2,n);  // Zwei Wurzelbaeume der Hoehe j/2 ergeben
      for(i=0;i<=n;i++)                      // ein Alkan mit laengster Kette der  
        Alkane[j][i]=t2[i];                  // Laenge j 
    }
    else
    {
      n--;                                   // Dieser Teil wird anschliessend sowieso mit X multipliziert
      CycleIndex_2(Wurzelbaeume[(j-1)/2],t2,n);   // Zwei Wurzelbaeume der Hoehe (j-1)/2
      CycleIndex_2(Wurzelbaeume_2[(j-1)/2],t1,n); // Zwei Wurzelbaeume der Hoehe kleiner (j-1)/2
      multiply(t1,t2,t3,n);
      CycleIndex_3(Wurzelbaeume[(j-1)/2],t1,n);   // Drei Wurzelbaeume der Hoehe (j-1)/2
      multiply(t1,Wurzelbaeume_2[(j-1)/2],t2,n);  // Ein Wuzelbaum der Hoehe kleiner (j-1)/2
      add(t3,t2,t1,n);
      CycleIndex_4(Wurzelbaeume[(j-1)/2],t2,n);   // Vier Wurzelbaeume der Hoehe (j-1)/2
      n++; 
      Alkane[j][0]=0;                           
      for(i=0;i<n;i++)
        Alkane[j][i+1]=t1[i]+t2[i];               // Zaehle die Zwischenergebnisse zusammen
    }
  }
}


void Berechne_Anzahl_Wurzelbaeume(vector< vector<cln::cl_I> >& Wurzelbaeume,vector< vector<cln::cl_I> >& Wurzelbaeume_2, unsigned int n)
{
  // Wurzelbaeume:    Speicherplatz fuer Anzahlen von Wurzelbaeumen. Wurzelbaeume[i][j] ist Anzahl der Wurzelbaeume der 
  //                  Hoehe i bestehend j Knoten      
  // Wurzelbaeume_2:  Speicherplatz fuer Anzahlen von Wurzelbaeumen. Wurzelbaeume_2[i][j] ist Anzahl der Wurzelbaeume 
  //                  der Hoehe kleiner i bestehend j Knoten
  // n:               maximale Anzahl an C-Atomen             
  //
  // Funktion
  // --------
  // Berechnet die entsprechenden Anzahlen von Wurzelbaeumen fuer Wurzelbaeume und Wurzelbaeume_2.

  unsigned int i,j;                             // Schleifenvariablen
  unsigned int max_hoehe=(unsigned int)(n/2);   // maximale Hoehe, die ein Wurzelbaum haben kann, damit beim Zusammen-
                                                // setzen zu einem Alkan maximal n C-Atome entstehen 

  vector<cln::cl_I> t1;                         // Vektor fuer Zwischenergebnisse 
  vector<cln::cl_I> t2;                         // Vektor fuer Zwischenergebnisse
  vector<cln::cl_I> t3;                         // Vektor fuer Zwischenergebnisse 
  t1.resize(n+1);                               // reserviere Speicherplatz 
  t2.resize(n+1);                               // reserviere Speicherplatz
  t3.resize(n+1);                               // reserviere Speicherplatz

  Wurzelbaeume.resize(max_hoehe+1);             // Reserviere Speicherplatz fuer die Anzahlvektoren
  Wurzelbaeume_2.resize(max_hoehe+1);
  for(i=0;i<=max_hoehe;i++)
  {
    Wurzelbaeume[i].resize(n+2);
    Wurzelbaeume_2[i].resize(n+2);
    for(j=0;j<=n+1;j++)
    {
      Wurzelbaeume[i][j]=0;                     // Belege die Vektoren mit Nullen vor 
      Wurzelbaeume_2[i][j]=0; 
    } 
  }

  // Startbelegung 
  for(i=0;i<=n;i++)
  {
    Wurzelbaeume[0][i]=0;
    Wurzelbaeume[1][i]=0;
    Wurzelbaeume_2[0][i]=0;
    Wurzelbaeume_2[1][i]=0; 
  } 
  Wurzelbaeume[0][0]=1;         // Es gibt genau einen Wurzelbaum der Hoehe 0 bestehend aus 0 Knoten
  Wurzelbaeume[1][1]=1;         // Es gibt genau einen Wurzelbaum der Hoehe 1 bestehend aus 1 Knoten 
  Wurzelbaeume_2[1][0]=1;       // Es gibt genau einen Wurzelbaum der Hoehe 0 bestehend aus 0 Knoten
  
  for(unsigned int h=2;h<=max_hoehe;h++)           // durchlaufe die moeglichen Hoehen von Wurzelbaeumen
  {
    CycleIndex_3(Wurzelbaeume[h-1],t1,n);          // 3 Wurzelbaeume der Hoehe h-1 
    CycleIndex_2(Wurzelbaeume[h-1],t2,n);          // 2 Wurzelbaeume der Hoehe h-1  
    multiply(t2,Wurzelbaeume_2[h-1],t3,n);         // 1 Wurzelbaum der Hoehe kleiner h-1 
    add(t1,t3,t2,n);                               // fasse Zwischenergebnisse zusammen 
    CycleIndex_2(Wurzelbaeume_2[h-1],t1,n);        // 2 Wurzelbaeume der Hoehe kleiner h-1 
    multiply(t1,Wurzelbaeume[h-1],t3,n);           // 1 Wurzelbaum der Hoehe h-1  
    add(t2,t3,t1,n);                               // fasse Zwischenergebnisse zusammen

    // Wurzelbaeume[h]:
    Wurzelbaeume[h][0]=0;
    for(j=0;j<n;j++)                               // kopiere die Werte, beachte dabei die neue Wurzel
      Wurzelbaeume[h][j+1]=t1[j];                  
    
    // Wurzelbaeume_2[h]: 
    for(j=0;j<=n;j++)                              // Wurzelbaeume der Hoehe kleiner h sind Wurzelbaeume der Hoehe h-1  
      Wurzelbaeume_2[h][j]=Wurzelbaeume_2[h-1][j]+Wurzelbaeume[h-1][j]; // bzw. Wurzelbaeume der Hoehe kleiner h-1
  }
}

bool multiply(vector<cln::cl_I>& factor_1,vector<cln::cl_I>& factor_2,vector<cln::cl_I>& result,unsigned int length)
{
  // Funktion multipliziert das durch [factor_1] repraesentierte Polynom mit dem
  // durch [factor_2] repraesentierten Polynom und speichert das Ergebnis als
  // Polyonom hoechstens [length]-ten Grades in [result].

  unsigned int i,j;                      // Schleifenvariablen
  unsigned int m1,m2;                    // Schranken

  result.resize(length+1);               // Bringe Ergebnisvektor auf die richtige Laenge
  for(i=0;i<=length;i++)                 // Initialisiere den Ergebnisvektor mit Null
    result[i]=0;

  // Einfache Polyonommultiplikation mit Gradbeschraenkung
  m1=min(factor_1.size(),length+1);      // Bestimme die obere Schranke fuer die aeussere Schleife
  for(i=0;i<m1;i++)
  {
    m2=min(factor_2.size(),length+1-i);  // Bestimme die obere Schranke fuer die innere Schleife
    for(j=0;j<m2;j++)
    {
      result[i+j]=result[i+j]+factor_1[i]*factor_2[j];
    }
  }

  return true;
}

bool add(vector<cln::cl_I>& summand_1,vector<cln::cl_I>& summand_2,vector<cln::cl_I>& result,unsigned int length)
{
  // Funktion addiert das durch [summand_1] repraesentierte Polynom zu dem
  // durch [summand_2] repraesentierten Polynom und speichert das Ergebnis als
  // Polyonom hoechstens [length]-ten Grades in [result].

  unsigned int i;                        // Schleifenvariable

  if(min(summand_1.size(),summand_2.size())>=length+1)  // Falls beide Polynome ausreichend lang sind
  {
    for(i=0;i<=length;i++)
      result[i]=summand_1[i]+summand_2[i];              // addiere einfach 
    return true;
  }

  if(summand_1.size()<=summand_2.size())                // das erste Polynom ist nicht ausreichend lang 
  {
    for(i=0;i<summand_1.size();i++)
      result[i]=summand_1[i]+summand_2[i];
    if(summand_2.size()>=length+1)                      // das zweite Polyonom ist ausreichend lang 
    {
      for(i=summand_1.size();i<=length;i++)
        result[i]=summand_2[i];
    }
    else                                                // das zweite Polynom ist nicht ausreichend lang
    {
      for(i=summand_1.size();i<summand_2.size();i++)
        result[i]=summand_2[i];
      for(i=summand_2.size();i<=length;i++)
        result[i]=0;
    }
    return true;
  }


  // summand_1.size()>summand_2.size()
  for(i=0;i<summand_2.size();i++)
    result[i]=summand_1[i]+summand_2[i];
  if(summand_1.size()>=length+1)                       // das erste Polynom ist ausreichend lang
  {
    for(i=summand_2.size();i<=length;i++)
      result[i]=summand_1[i];
  }
  else                                                 // das erste Polynomist nicht ausreichend lang
  {
    for(i=summand_2.size();i<summand_1.size();i++)
      result[i]=summand_1[i];
    for(i=summand_1.size();i<=length;i++)
      result[i]=0;
  }

  return true;
}

bool CycleIndex_2(vector<cln::cl_I> & poly,vector<cln::cl_I>& result,unsigned int length)
{
  // Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 2 auf [poly] an und
  // speichert das Ergebnis als ein Polynom mit Maximalgrad [length] in [result]

  unsigned int i;                        // Schleifenvariable
  unsigned int l,m;                      // Schranken

  // CycleIndex(S_2) = (X^2+X_2)/2
  multiply(poly,poly,result,length);     // Berechne "X^2"

  // Berechne zuerst die Koeffizienten der geraden X-Potenzen
  l=(unsigned int)(length/2);
  m=min(l,poly.size()-1);
  for(i=0;i<=m;i++)
    result[2*i]=exquo(result[2*i]+poly[i],2);
  for(i=poly.size();i<=l;i++)
    result[2*i]=exquo(result[2*i],2);

  // Berechne nun die Koeffizienten der ungeraden X-Potenzen
  l=(unsigned int)((length-1)/2);
  for(i=0;i<=l;i++)
    result[2*i+1]=exquo(result[2*i+1],2);

  return true;
}

bool CycleIndex_3(vector<cln::cl_I> & poly,vector<cln::cl_I>& result,unsigned int length)
{
  // Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 3 auf [poly] an und
  // speichert das Ergebnis als ein Polynom mit Maximalgrad [length] in [result]

  unsigned int i,j;                      // Schleifenvariablen
  unsigned int l,m;                      // Schranken
  vector<cln::cl_I> temp;                // Vektor fuer Zwischenrechnungen
  temp.resize(length+2);

  // CycleIndex(S_3) = (X^3+3X*X_2+2X_3)/6
  multiply(poly,poly,temp,length);       // Berechne "X * X = X^2"
  multiply(poly,temp,result,length);     // Berechne "X * X^2 = X^3

  // Berechne "X^3 + 3*X*X_2"
  m=min(length+1,(unsigned int)poly.size());
  for(i=0;i<m;i++)
  {
    l=min((unsigned int)((length-i)/2),poly.size()-1);
    for(j=0;j<=l;j++)
      result[i+2*j]=result[i+2*j]+3*poly[i]*poly[j];
  }

  // Berechne "X^3+3*X*X_2 + 2*X_3"
  l=min((unsigned int)(length/3),poly.size()-1);
  for(i=0;i<=l;i++)
    result[3*i]=result[3*i]+2*poly[i];

  // Teile nun nur noch durch 6
  for(i=0;i<=length;i++)
    result[i]=exquo(result[i],6);

  return true;
}

bool CycleIndex_4(vector<cln::cl_I> & poly,vector<cln::cl_I>& result,unsigned int length)
{
  // Wendet den Cycle-Index der Symmetrischen Gruppe der Ordnung 4 auf [poly] an und
  // speichert das Ergebnis als ein Polynom mit Maximalgrad [length] in [result]

  unsigned int i,j;                      // Schleifenvariablen
  unsigned int l,m;                      // Schranken
  vector<cln::cl_I> temp;                // Vektor fuer Zwischenrechnungen

  // CycleIndex(S_4) = (X^4+6X_2*X^2+8X_3*X+3(X_2)^2+6X_4)/24

  multiply(poly,poly,temp,length);       // Berechne "X * X = X^2"
  multiply(temp,temp,result,length);     // Berechne "X^2 * X^2 = X^4"

  // Berechne "X^4 + 8*X*X_3"
  m=min(length+1,poly.size());
  for(i=0;i<m;i++)
  {
    l=min((unsigned int)((length-i)/3),poly.size()-1);
    for(j=0;j<=l;j++)
      result[i+3*j]=result[i+3*j]+8*poly[i]*poly[j];
  }

  // Berechne "X^4+8*X*X_3 + 6X^2*X_2"
  for(i=0;i<=length;i++)
  {
    l=min((unsigned int)((length-i)/2),poly.size()-1);
    for(j=0;j<=l;j++)
      result[i+2*j]=result[i+2*j]+6*temp[i]*poly[j];
  }

  // Berechne "X^4+8*X*X_3+6X^2*X_2 + 6X_4"
  l=min((unsigned int)(length/4),poly.size()-1);
  for(i=0;i<=l;i++)
    result[4*i]=result[4*i]+6*poly[i];

  // Berechne "X^4+8*X*X_3+6X^2*X_2+6X_4 + 3X^2_2"
  l=(unsigned int)(length/2);
  for(i=0;i<=l;i++)
    result[2*i]=result[2*i]+3*temp[i];

  // Teile nun nur noch durch 24
  for(i=0;i<=length;i++)
    result[i]=exquo(result[i],24);

  return 0;
}

void choose_1(cln::cl_I& rest,cln::cl_I& g,cln::cl_I& u)
{
  // Berechnet einen Rang u
  // Hierbei wird ein Gesamtrang rest und die Gesamtzahl g an einzelstrukturen uebergeben
  // rest wird anschliessend modifiziert
  u=mod(rest,g);
  rest=exquo(rest-u,g);
} 

void choose_2(cln::cl_I& rest,cln::cl_I& g,cln::cl_I& u1,cln::cl_I& u2)
{
  // "Waehle zwei Elemente u1,u2 (mit Wiederholung) aus einer Menge aus g Elemente, so dass der Rang zu rest passt"
  // Berechne den Rang von u1,u2 bei gegebenen rest und g, und modifiziere g  
  cln::cl_I r=rest;  
  rest=mod(rest,exquo(g*(g+1),2)); 
  u1=floor1((sqrt((cln::cl_R)(8*(rest)+1))-1)/(cln::cl_R)(2));
  if(exquo(u1*(u1+1),2)>rest) u1=u1-1;
  if(rest-exquo(u1*(u1+1),2)>u1) u1=u1+1;
  u2=rest-exquo(u1*(u1+1),2);
  if(u2<0 || u2>u1) cout << "Fehler" << endl; 
  rest=exquo(r-exquo(u1*(u1+1),2)-u2,exquo(g*(g+1),2));
}

void choose_3(cln::cl_I& rest,cln::cl_I& g,cln::cl_I& u1,cln::cl_I& u2,cln::cl_I& u3)
{
  // "Waehle drei Elemente u1,u2,u3 (mit Wiederholung) aus einer Menge aus g Elemente, so dass der Rang zu rest passt"
  // Berechne den Rang von u1,u2,u3 bei gegebenen rest und g, und modifiziere g
  cln::cl_I r=rest;  
  rest=mod(rest,exquo(g*(g+1)*(g+2),6)); 

  u1=0;                                  // Dieser Teil ist noch sehr ineffizient, da rest u.U.
  while(exquo(u1*(u1+1)*(u1+2),6)<=rest) // sehr gross werden kann. Man sollte besser die dritte
    u1=u1+1;                             // Wurzel von rest ziehen, und dann den Wert korrigieren 
  u1=u1-1;

  rest=rest-exquo(u1*(u1+1)*(u1+2),6);
  cln::cl_I gg=u1+1;
  choose_2(rest,gg,u2,u3);
  rest=exquo(r-exquo(u1*(u1+1)*(u1+2),6)-exquo(u2*(u2+1),2)-u3,exquo(g*(g+1)*(g+2),6));
}

void choose_4(cln::cl_I& rest,cln::cl_I& g,cln::cl_I& u1,cln::cl_I& u2,cln::cl_I& u3,cln::cl_I& u4)
{
  // "Waehle vier Elemente u1,u2,u3,u4 (mit Wiederholung) aus einer Menge aus g Elemente, so dass der Rang zu rest passt"
  // Berechne den Rang von u1,u2,u3,u4 bei gegebenen rest und g, und modifiziere g
  cln::cl_I r=rest;  
  rest=mod(rest,exquo(g*(g+1)*(g+2)*(g+3),24)); 

  u1=0;                                         // Dieser Teil ist noch sehr ineffizient, da rest u.U.
  while(exquo(u1*(u1+1)*(u1+2)*(u1+3),24)<=rest)// sehr gross werden kann. Man sollte besser die vierte
    u1=u1+1;                                    // Wurzel von rest ziehen, und dann den Wert korrigieren 
  u1=u1-1;

  rest=rest-exquo(u1*(u1+1)*(u1+2)*(u1+3),24);
  cln::cl_I gg=u1+1;
  choose_3(rest,gg,u2,u3,u3);
  rest=exquo(r-exquo(u1*(u1+1)*(u1+2)*(u1+3),24)-exquo(u2*(u2+1)*(u2+2),6)-exquo(u3*(u3+1),2)-u4,exquo(g*(g+1)*(g+2)*(g+3),24));
}

int unrank_wurzelbaum(cln::cl_I r,unsigned int hoehe, unsigned int anzahl_knoten, vector<unsigned int>& wurzelbaum,vector< vector<cln::cl_I> >& W1,vector< vector<cln::cl_I> >& W2)
{
  // r:             Rang eine Wurzelbaums
  // hoehe:         Hoehe des gewuenschten Wurzelbaums
  // anzahl_knoten: Anzahl der Knoten des gewuenschten Wurzelbaums
  // wurzelbaum:    Datenstruktur zum Abspeichern eines Wurzelbaums
  // W1:            Anzahl von Wurzelbaeumen der Hoehe i bestehend aus j Knoten
  // W2:            Anzahl von Wurzelbaeumen der Hoehe kleiner i bestehend aus j Knoten 
  // 
  // Funktion
  // --------
  // Berechnet zu einem Rang r den entsprechenden Wurzelbaum der Hoehe hoehe bestehend aus anzahl_knoten Knoten

  if(anzahl_knoten<=1)         // die trivialen Faelle
  {
    if(anzahl_knoten==0)       // Leerer Wurzelbaum
    {
      wurzelbaum.resize(0);
      if(DEBUG)
        cout << "Erzeuge Wurzelbaum aus 0 Knoten" << endl;
    }
    if(anzahl_knoten==1)       // Wurzelbaum aus einem Knoten
    {
      wurzelbaum.resize(1);
      wurzelbaum[0]=0;
      if(DEBUG)
        cout << "Erzeuge Wurzelbaum aus 1 Knoten" << endl;
    } 
    return 0;
  } 
  unsigned int i;                       // Schleifenvariable
  vector<unsigned int> wb1;             // Datenstruktur zum Abspeichern eines Teilwurzelbaums
  vector<unsigned int> wb2;             // Datenstruktur zum Abspeichern eines Teilwurzelbaums  
  vector<unsigned int> wb3;             // Datenstruktur zum Abspeichern eines Teilwurzelbaums  
  unsigned int i1,i2,i3;                // Schleifenvariablen
  unsigned int g1,g2;                   // obere Schranken fuer Schleifenvariablen
  unsigned int h1,h2,h3;                // Hoehen der drei Teilwurzelbaeume
  bool br;                              // koennen die Schleifendurchlaeufe beendet werden?  
  int ret;                              // Rueckgabewert 
  cln::cl_I u1,u2,u3;                   // Rang der drei Teilwurzelbaeume
  cln::cl_I rest;                       // Rest vom Rang, der noch nicht behandelt wurde 
  cln::cl_I p=0;                        // Anzahlmaessiger Durchlauf durch Liste  
  h1=h2=h3=0;                           // damit der Compiler keine Warnung ausgibt 


  // Fall (i): Wurzelbaum besteht aus einem Wurzelbaum der Hoehe hoehe-1 und zwei Wurzelbaeumen der Hoehe kleiner hoehe-1 
  br=false;                                    // es muss noch durch die Schleifen durchgelaufen werden
  h1=2*anzahl_knoten+2;                        // zu grosser Wert, der nie angenommen werden kann
  g1=bestimme_ende(hoehe-1,anzahl_knoten-1);
  for(i1=bestimme_start_11(hoehe-1,anzahl_knoten-1);i1<=g1;i1++)
  {                                            // ein Wurzelbaum der Hoehe hoehe-1 besteht aus mindestens hoehe-1
                                               // Knoten, ein Knoten ist fuer die neue Wurzel reserviert
    if(br==true) break;                        
    g2=bestimme_ende(hoehe-2,(anzahl_knoten-1-i1)/2);
    for(i2=bestimme_start_12(hoehe-1,anzahl_knoten-1-i1);i2<=g2;i2++)  
    {                                          // der zweite Teilwurzelbaum besitzt hoechstens so  
                                               // viele Knoten wie der dritte Teilwurzelbaum
      i3=anzahl_knoten-1-i1-i2;                // So viele Knoten bleiben fuer den dritten Teilwurzelbaum uebrig 
      // Unterscheide 2 Faelle, ob Teilwurzelbaum 2 und Teilwurzelbaum 3 der selben Menge entnommen werden oder nicht
      if(i2==i3)                               // Es gibt entsprechend viele Wurzelbaeume der festgelegten Hoehen und   
        p=p+W1[hoehe-1][i1]*exquo(W2[hoehe-1][i2]*(W2[hoehe-1][i3]+1),2);  // Anzahl der Knoten 
      if(i2<i3)                                // Es gibt entsprechend viele Wurzelbaeume der festgelegten Hoehen und     
      {                                        // Anzahl der Knoten
        p=p+W1[hoehe-1][i1]*W2[hoehe-1][i2]*W2[hoehe-1][i3];
      }
      if(p>r)                                  // Falls der gewuenschte Rang ueberschritten wurde
      {
        h1=i1;                                 // speichere aktuelle Parameter
        h2=i2;
        h3=i3;
        // Mache das dazuaddieren von eben rueckgaengig  
        if(h2==h3)
          p=p-W1[hoehe-1][h1]*exquo(W2[hoehe-1][h2]*(W2[hoehe-1][h3]+1),2);
        if(h2<h3)
          p=p-W1[hoehe-1][h1]*W2[hoehe-1][h2]*W2[hoehe-1][h3];
        br=true;                               // Schleifen brauchen nicht weiter durchlaufen werden   
        break;  
      } 
    } 
  }
  if(h1<2*anzahl_knoten+2)  // Wurzelbaum besteht aus einem Wurzelbaum der Hoehe hoehe-1 und zwei
  {                         // Wurzelbaeumen der Hoehe kleiner hoehe-1
    // Unterscheide wieder zwei Faelle, ob Teilwurzelbaum 2 und Teilwurzelbaum 3 der selben Menge entnommen 
    // werden oder nicht.
    if(h2==h3)                                // Teilwurzelbaum 2 und 3 besitzen die selbe Anzahl an Knoten
    {
      rest=r-p;                               // Der Teil des Rangs, der fuer die 3 Teilwurzelbaeume verantwortlich ist
      choose_1(rest,W1[hoehe-1][h1],u1);      // bestimme den Rang des ersten Teilwurzelbaums
      choose_2(rest,W2[hoehe-1][h2],u2,u3);   // bestimme den Rang des zweiten und dritten Teilwurzelbaums
    }
    if(h2<h3)                                 // Teilwurzelbaum 2 und 3 besitzen unterschiedliche Anzahlen an Knoten
    {
      rest=r-p;                               // Der Teil des Rangs, der fuer die 3 Teilwurzelbaeume verantwortlich ist
      choose_1(rest,W1[hoehe-1][h1],u1);      // bestimme den Rang des ersten Teilwurzelbaums
      choose_1(rest,W2[hoehe-1][h2],u2);      // bestimme den Rang des zweiten Teilwurzelbaums   
      choose_1(rest,W2[hoehe-1][h3],u3);      // bestimme den Rang des dritten Teilwurzelbaums
    }  
    ret=0;
    if(DEBUG)
      cout << "unrank(" << u1 << ") -> Wurzelbaum der Hoehe " << hoehe-1 << " bestehend aus " << h1 << " Knoten" << endl;
    ret=ret+unrank_wurzelbaum(u1,hoehe-1,h1,wb1,W1,W2);          // konstruiere den ersten Teilwurzelbaum
    if(DEBUG)
      cout << "unrank(" << u2 << ") -> Wurzelbaum der Hoehe < " << hoehe-1 << " bestehend aus " << h2 << " Knoten" << endl;
    ret=ret+unrank_wurzelbaum_kleiner(u2,hoehe-1,h2,wb2,W1,W2);  // konstruiere den zweiten Teilwurzelbaum
    if(DEBUG)
      cout << "unrank(" << u3 << ") -> Wurzelbaum der Hoehe < " << hoehe-1 << " bestehend aus " << h3 << " Knoten" << endl;
    ret=ret+unrank_wurzelbaum_kleiner(u3,hoehe-1,h3,wb3,W1,W2);  // konstruiere den dritten Teilwurzelbaum
  }
  else                      // Vielleicht besteht der Wurzelbaum aus zwei Wurzelbaeumen der 
  {                         // Hoehe hoehe-1 und einem Wurzelbaum der Hoehe kleiner hoehe-1
    br=false;               // Schleifen muessen noch durchlaufen werden 
    h1=2*anzahl_knoten+2;   // grosser Wert, der nicht angenommen werden kann
    g1=bestimme_ende(hoehe-1,(anzahl_knoten-1)/2);
    for(i1=bestimme_start_13(hoehe-1,anzahl_knoten-1);i1<=g1;i1++)   
    {                                              // der erste Teilwurzelbaum besitzt die Hoehe hoehe-1 und besteht 
                                                   // somit mindestens aus hoehe-1 Knoten, weiter besitzt er hoechstens 
                                                   // so viele Knoten, wie der zweite Teilwurzelbaum (+neue Wurzel)
      if(br==true) break;                 
      g2=bestimme_ende(hoehe-1,anzahl_knoten-1-i1);
      for(i2=bestimme_start_14(i1,hoehe-1,anzahl_knoten-1-i1);i2<=g2;i2++)       
      {                                            // der zweite Teilwurzelbaum besitzt mindestens so  
                                                   // viele Knoten wie der erste Teilwurzelbaum
        i3=anzahl_knoten-1-i1-i2;                  // der dritte Teilwurzelbaum besteht aus den uebrigen Knoten
        // Unterscheide zwei Faelle   
        if(i1==i2)                                 // der erste Teilwurzelbaum enthaelt so viele Knoten wie der zweite
          p=p+exquo(W1[hoehe-1][i1]*(W1[hoehe-1][i2]+1),2)*W2[hoehe-1][i3];
        if(i1<i2)                                  // der erste Teilwurzelbaum enthaelt weniger Knoten wie der zweite  
          p=p+W1[hoehe-1][i1]*W1[hoehe-1][i2]*W2[hoehe-1][i3];
        if(p>r)            // falls der gesuchte Rang ueberschritten wurde
        {
          h1=i1;           // merke die Parameter
          h2=i2;
          h3=i3;
          // mache das aufaddieren von eben wieder rueckgaengig 
          if(h1==h2)
            p=p-exquo(W1[hoehe-1][h1]*(W1[hoehe-1][h2]+1),2)*W2[hoehe-1][h3];
          if(h1<h2)
            p=p-W1[hoehe-1][h1]*W1[hoehe-1][h2]*W2[hoehe-1][h3];  
          br=true;         // die Schleifen brauchen nicht weiter durchlaufen zu werden
          break;
        }    
      }    
    }
    if(h1<2*anzahl_knoten+2)   // Der Wurzelbaum besteht aus zwei Wurzelbaeumen der Hoehe hoehe-1 und einem 
    {                          // Wurzelbaum der Hoehe kleiner hoehe-1
      if(h1==h2)               // der erste Teilwurzelbaum besteht aus genau so vielen Knoten wie der zweite
      {
        rest=r-p;              // Der Teil des Rangs, der fuer die 3 Teilwurzelbaeume verantwortlich ist  
        choose_2(rest,W1[hoehe-1][h1],u1,u2); // bestimme den Rang des ersten und zweiten Teilwurzelbaums
        choose_1(rest,W2[hoehe-1][h3],u3);    // bestimme den Rang des dritten Teilwurzelbaums 
      }
      if(h1<h2)                // der erste Teilwurzelbaum besteht aus weniger Knoten wie der zweite 
      {
        rest=r-p;              // Der Teil des Rangs, der fuer die 3 Teilwurzelbaeume verantwortlich ist
        choose_1(rest,W1[hoehe-1][h1],u1);    // bestimme den Rang des ersten Teilwurzelbaums
        choose_1(rest,W1[hoehe-1][h2],u2);    // bestimme den Rang des zweiten Teilwurzelbaums
        choose_1(rest,W2[hoehe-1][h3],u3);    // bestimme den Rang des dritten Teilwurzelbaums
      }
      ret=0;
      if(DEBUG)
        cout << "unrank(" << u1 << ") -> Wurzelbaum der Hoehe " << hoehe-1 << " bestehend aus " << h1 << " Knoten" << endl;
      ret=ret+unrank_wurzelbaum(u1,hoehe-1,h1,wb1,W1,W2);            // konstruiere den ersten Teilwurzelbaum
      if(DEBUG)
        cout << "unrank(" << u2 << ") -> Wurzelbaum der Hoehe " << hoehe-1 << " bestehend aus " << h2 << " Knoten" << endl;
      ret=ret+unrank_wurzelbaum(u2,hoehe-1,h2,wb2,W1,W2);            // konstruiere den zweiten Teilwurzelbaum
      if(DEBUG)
        cout << "unrank(" << u3 << ") -> Wurzelbaum der Hoehe < " << hoehe-1 << " bestehend aus " << h3 << " Knoten" << endl;
      ret=ret+unrank_wurzelbaum_kleiner(u3,hoehe-1,h3,wb3,W1,W2);    // konstruiere den dritten Teilwurzelbaum
    }
    else  // Vielleicht besteht der Wurzelbaum aus drei Wurzelbaeumen der Hoehe hoehe-1
    {
      br=false;                // die Schleifen muessen noch durchlaufen werden
      h1=2*anzahl_knoten+2;    // grosser Wert, der nicht angenommen werden kann
      g1=bestimme_ende(hoehe-1,(anzahl_knoten-1)/3);
      for(i1=bestimme_start_15(hoehe-1,anzahl_knoten-1);i1<=g1;i1++) 
      {                                            // der erste Teilwurzelbaum besitzt die Hoehe hoehe-1 und besteht 
                                                   // somit mindestens aus hoehe-1 Knoten, weiterhin besitzt er 
                                                   // hoechstens so viele Knoten wie der zweite und dritte Teilwurzelbaum 
        if(br==true) break;
        g2=bestimme_ende(hoehe-1,(anzahl_knoten-1-i1)/2);
        for(i2=bestimme_start_16(i1,hoehe-1,anzahl_knoten-1-i1);i2<=g2;i2++) 
        {                                          // der zweite Teilwurzelbaum besitzt mindestens so viele Knoten wie 
                                                   // der erste Teilwurzelbaum und hoechstens so viele Knoten wie der 
                                                   // dritte Teilwurzelbaum  
          i3=anzahl_knoten-1-i1-i2;                // der dritte Teilwurzelbaum besteht aus den restlichen Knoten
          // Unterscheide 4 Faelle, ob zwei oder mehr der drei Teilwurzelbaeume die 
          // selbe Anzahl an Knoten besitzen oder nicht
          if(i1==i2 && i2==i3)
            p=p+exquo(W1[hoehe-1][i1]*(W1[hoehe-1][i2]+1)*(W1[hoehe-1][i3]+2),6);
          if(i1==i2 && i2<i3)
            p=p+exquo(W1[hoehe-1][i1]*(W1[hoehe-1][i2]+1),2)*W1[hoehe-1][i3];
          if(i1<i2 && i2==i3)
            p=p+W1[hoehe-1][i1]*exquo(W1[hoehe-1][i2]*(W1[hoehe-1][i3]+1),2);
          if(i1<i2 && i2<i3)
            p=p+W1[hoehe-1][i1]*W1[hoehe-1][i2]*W1[hoehe-1][i3];
          if(p>r)              // falls der gewuenschte Rang ueberschritten wurde
          {
            h1=i1;             // merke die Parameter
            h2=i2;
            h3=i3;
            // mache das dazuaddieren von eben wieder rueckgaengig
            if(h1==h2 && h2==h3)
              p=p-exquo(W1[hoehe-1][i1]*(W1[hoehe-1][i2]+1)*(W1[hoehe-1][i3]+2),6);
            if(h1==h2 && h2<h3)
              p=p-exquo(W1[hoehe-1][i1]*(W1[hoehe-1][i2]+1),2)*W1[hoehe-1][i3];
            if(h1<h2 && h2==h3)
              p=p-W1[hoehe-1][i1]*exquo(W1[hoehe-1][i2]*(W1[hoehe-1][i3]+1),2);
            if(h1<h2 && h2<h3)
              p=p-W1[hoehe-1][i1]*W1[hoehe-1][i2]*W1[hoehe-1][i3];
            br=true;           // es ist kein weiterer Schleifendurchlauf mehr noetig
            break; 
          }  
        }   
      }
      if(h1<2*anzahl_knoten+2) // Der Wurzelbaum besteht aus drei Wurzelbaeumen der Hoehe hoehe-1
      {
        if(h1==h2 && h2==h3)
        {
          rest=r-p;                               // Der Teil des Rangs, der fuer die 3 Teilwurzelbaeume verantwortlich ist  
          choose_3(rest,W1[hoehe-1][h1],u1,u2,u3);// bestimme die Raenge der 3 Wurzelbaeume
        }
        if(h1==h2 && h2<h3)
        {
          rest=r-p;                               // Der Teil des Rangs, der fuer die 3 Teilwurzelbaeume verantwortlich ist  
          choose_2(rest,W1[hoehe-1][h1],u1,u2);   // bestimme die Raenge der ersten beiden Teilwurzelbaeume
          choose_1(rest,W1[hoehe-1][h3],u3);      // bestimme den Rang des dritten Teilwurzelbaums  
        }
        if(h1<h2 && h2==h3)
        {
          rest=r-p;                               // Der Teil des Rangs, der fuer die 3 Teilwurzelbaeume verantwortlich ist  
          choose_1(rest,W1[hoehe-1][h1],u1);      // bestimme den Rang des ersten Teilwurzelbaums
          choose_2(rest,W1[hoehe-1][h2],u2,u3);   // bestimme die Raenge der beiden letzten Teilwurzelbaeume 
        }
        if(h1<h2 && h2<h3)
        {
           rest=r-p;                              // Der Teil des Rangs, der fuer die 3 Teilwurzelbaeume verantwortlich ist  
           choose_1(rest,W1[hoehe-1][h1],u1);     // bestimme den Rang des ersten Teilwurzelbaums
           choose_1(rest,W2[hoehe-1][h2],u2);     // bestimme den Rang des zweiten Teilwurzelbaums
           choose_1(rest,W2[hoehe-1][h3],u3);     // bestimme den Rang des dritten Teilwurzelbaums 
        }
        ret=0;
        if(DEBUG)
        {
          cout << "unrank(" << u1 << ") -> Wurzelbaum der Hoehe " << hoehe-1 << " bestehend aus ";
          cout << h1 << " Knoten" << endl;
        }
        ret=ret+unrank_wurzelbaum(u1,hoehe-1,h1,wb1,W1,W2);   // konstruiere den ersten Teilwurzelbaum
        if(DEBUG) 
        {
          cout << "unrank(" << u2 << ") -> Wurzelbaum der Hoehe " << hoehe-1 << " bestehend aus ";
          cout << h2 << " Knoten" << endl;
        }
        ret=ret+unrank_wurzelbaum(u2,hoehe-1,h2,wb2,W1,W2);   // konstruiere den zweiten Teilwurzelbaum   
        if(DEBUG)
        {
          cout << "unrank(" << u3 << ") -> Wurzelbaum der Hoehe " << hoehe-1 << " bestehend aus ";
          cout << h3 << " Knoten" << endl;
        }
        ret=ret+unrank_wurzelbaum(u3,hoehe-1,h3,wb3,W1,W2);   // konstruiere den dritten Teilwurzelbaum
      }
      else      // Es gibt keinen Wurzelbaum mit dieser Nummer
      {
        cout << "Den Wurzelbaum der Hoehe " << hoehe << " bestehend aus " << anzahl_knoten << " Knoten mit der Nr. ";
        cout << r << " gibt es nicht." << endl;
        return -1; 
      }
    } 
  }
  if(ret<0) return -1;                          // Wurzelbaum konnte nicht konstruiert werden 
  // Konstruktion des Wurzelbaums
  wurzelbaum.resize(anzahl_knoten);             // reserviere Speicherplatz
  wurzelbaum[0]=0;                              // neue Wurzel
  if(wb1.size()>0)                              // Wurzel von Teilwurzelbaum 1, falls sie existiert, zeigt auf neue Wurzel
    wurzelbaum[1]=0;
  for(i=1;i<wb1.size();i++)                     // kopiere Teilwurzelbaum 1 
    wurzelbaum[i+1]=wb1[i]+1;
  if(wb2.size()>0)                              // Wurzel von Teilwurzelbaum 2, falls sie existiert, zeigt auf neue Wurzel
    wurzelbaum[1+wb1.size()]=0;
  for(i=1;i<wb2.size();i++)                     // kopiere Teilwurzelbaum 2   
    wurzelbaum[1+wb1.size()+i]=wb2[i]+1+wb1.size();
  if(wb3.size()>0)                              // Wurzel von Teilwurzelbaum 3, falls sie existiert, zeigt auf neue Wurzel
    wurzelbaum[1+wb1.size()+wb2.size()]=0;
  for(i=1;i<wb3.size();i++)                     // kopiere Teilwurzelbaum 3 
    wurzelbaum[1+wb1.size()+wb2.size()+i]=wb3[i]+1+wb1.size()+wb2.size();
  return 0;                                     // unrank erfolgreich  
}

int unrank_wurzelbaum_kleiner(cln::cl_I r,unsigned int hoehe, unsigned int anzahl_knoten, vector<unsigned int>& wurzelbaum,vector< vector<cln::cl_I> >& W1,vector< vector<cln::cl_I> >& W2)
{
  // r:             Rang eine Wurzelbaums
  // hoehe:         Hoehe des gewuenschten Wurzelbaums
  // anzahl_knoten: Anzahl der Knoten des gewuenschten Wurzelbaums
  // wurzelbaum:    Datenstruktur zum Abspeichern eines Wurzelbaums
  // W1:            Anzahl von Wurzelbaeumen der Hoehe i bestehend aus j Knoten
  // W2:            Anzahl von Wurzelbaeumen der Hoehe kleiner i bestehend aus j Knoten 
  // 
  // Funktion
  // --------
  // Berechnet zu einem Rang r den entsprechenden Wurzelbaum der Hoehe kleiner hoehe bestehend aus anzahl_knoten Knoten
  
  cln::cl_I p=0;
  unsigned int j=2*hoehe+2;
  for(unsigned int i=0;i<hoehe;i++) // durchlaufe die moeglichen Hoehen
  {
    p=p+W1[i][anzahl_knoten];
    if(p>r)
    {
      j=i;
      p=p-W1[i][anzahl_knoten]; 
      break;   
    }
  }
  if(j==2*hoehe+2)
  {
    cout << "Den Wurzelbaum der Hoehe kleiner " << hoehe << " bestehend aus " << anzahl_knoten << " Knoten mit der Nr. ";
    cout << r << " gibt es nicht." << endl;
    return -1; 
  }
  if(DEBUG) 
    cout << "unrank(" << r-p << ") Wurzelbaum der Hoehe " << j << " bestehend aus " << anzahl_knoten << " Knoten" << endl;
  return unrank_wurzelbaum(r-p,j,anzahl_knoten,wurzelbaum,W1,W2); // konstruiere einen Wurzelbaum der Hoehe j
}

int unrank_alkane(cln::cl_I r,Alkan& Alk,unsigned int n,vector< vector<cln::cl_I> >& W1,vector< vector<cln::cl_I> >& W2,vector< vector<cln::cl_I> >& A)
{
  // r:             Rang eine Wurzelbaums
  // Alk:           Datenstruktur zum Abspeichern eines Alkans
  // n:             Anzahl der C-Atome des gewuenschten Alkans
  // W1:            Anzahl von Wurzelbaeumen der Hoehe i bestehend aus j Knoten
  // W2:            Anzahl von Wurzelbaeumen der Hoehe kleiner i bestehend aus j Knoten 
  // A:             Anzahl von Alkanen mit laengster Kette der Laenge i bestehend aus j Knoten
  // 
  // Funktion
  // --------
  // Berechnet zu einem Rang r das entsprechende Alkan bestehend aus n C-Atomen

  // behandele zunaechst die trivialen Faelle
  if(n<=1)
  {
    if(n==0 && r==0)                         // leeres Alkan
    {
      Alk.Wurzelbaum_1.resize(0);
      Alk.Wurzelbaum_2.resize(0);
      Alk.Wurzelbaum_3.resize(0);  
      Alk.Wurzelbaum_4.resize(0);
      Alk.laengste_gerade_Kette_ist_gerade=true;
      return 0;
    }
    if(n==1 && r==0)                         // Alkan aus einem C-Atom                
    {
      Alk.Wurzelbaum_1.resize(0);
      Alk.Wurzelbaum_2.resize(0);
      Alk.Wurzelbaum_3.resize(0);  
      Alk.Wurzelbaum_4.resize(0);
      Alk.laengste_gerade_Kette_ist_gerade=false;
      return 0;
    }
    cout << "Alkan Nr. " << r << " bestehend aus " << n << " C-Atomen gibt es nicht." << endl;
    return -1;
  } 

  cln::cl_I p=0;
 
  unsigned int i,j,h;              // Schleifenvariablen
  unsigned int i1,i2,i3,i4;        // Schleifenvariablen
  unsigned int g1,g2,g3;           // obere Schranken fuer Schleifenvariablen
  unsigned int h1,h2,h3,h4;        // Anzahl der Knoten der 4 zugehoerigen Wurzelbaeume 
  unsigned int j_2;                // maximale Hoehe der zugehoerigen Wurzelbaeume  
  bool br;                         // sollen die Schleifen weiter durchlaufen werden? 
  int ret;                         // Rueckgabewert
  cln::cl_I u1,u2,u3,u4;           // Raenge der Wurzelbaeume
  cln::cl_I rest;                  // ein Teil vom Rang 
  vector<unsigned int> wb1;        // Datenstruktur fuer den ersten Wurzelbaum
  vector<unsigned int> wb2;        // Datenstruktur fuer den zweiten Wurzelbaum  
  vector<unsigned int> wb3;        // Datenstruktur fuer den dritten Wurzelbaum 
  vector<unsigned int> wb4;        // Datenstruktur fuer den vierten Wurzelbaum 


  // bestimme die Laenge j der laengsten Kette des Alkans
  j=2*n+2;                         // grosse Zahl, di nicht erreicht werden kann
  for(i=0;i<=n;i++)                // durchlaufe die moeglichen Laengen der laengsten Ketten   
  {
    p=p+A[i][n];
    if(p>r)                        // falls der gewuenschte Rang ueberschritten wurde
    {
      j=i;
      p=p-A[i][n]; 
      break;
    }
  }
  if(j==2*n+2)
  {
    cout << "Alkan Nr. " << r << " bestehend aus " << n << " C-Atomen gibt es nicht." << endl;
    return -1;
  }
  if(DEBUG)
    cout << "Erzeuge ein Alkan mit laengster gerader Kette der Laenge " << j << endl;

  h=2*n+2;           // grosse Zahl, die nicht erreicht werden kann        
  if((j%2)==0)
  {
    j_2=j/2;         // maximale 
    g1=bestimme_ende(j_2,(unsigned int)(n/2));
    for(i=bestimme_start_10(j_2,n);i<=g1;i++) 
    {                                     // durchlaufe die Anzahl der Knoten des ersten Wurzelbaums, da er die Hoehe
                                          // j_2 besitzt, muss er mindestens so viele Knoten besitzen, weiter besitzt 
                                          // er hoechstens so viele Knoten wie der zweite Teilwurzelbaum  
      if(i<n-i)      // erster Wurzelbaum enthaelt weniger Knoten als der zweite
      {
        p=p+W1[j_2][i]*W1[j_2][n-i]; 
      }
      else           // beide Wurzelbaeume enthalten gleich viele Knoten 
      {
        p=p+exquo(W1[j_2][i]*(1+W1[j_2][i]),2); 
      }
      if(p>r)       // falls der gewuenschte Rang ueberschritten wurde
      { 
        h=i;        // merke die Anzahl der Knoten fuer den ersten Wurzelbaum
        // mache das dazuaddieren von eben wieder rueckgaengig  
        if(i<n-i)
          p=p-W1[j_2][i]*W1[j_2][n-i];
        else
          p=p-exquo(W1[j_2][i]*(1+W1[j_2][i]),2);
        break; 
      }  
    }
    if(h==2*n+2)
    {
      cout << "Alkan Nr. " << r << " bestehend aus " << n << " C-Atomen gibt es nicht." << endl;
      return -1;
    }
    if(h<n-h)                            // der erste Wurzelbaum besitzt weniger Knoten wie der zweite
    {
      rest=r-p;                          // der Teil, der fuer den Rang der zwei Wurzelbaeume verantwortlich ist
      choose_1(rest,W1[j_2][h],u1);      // bestimme den Rang des ersten Wurzelbaums
      choose_1(rest,W1[j_2][n-h],u2);    // bestimme den Rang des zweiten Wurzelbaums
    }
    else // h==n-h                       // der erste Wurzelbaum besitzt genauso viele Knoten wie der zweite
    {
      rest=r-p;                          // der Teil, der fuer den Rang der zwei Wurzelbaeume verantwortlich ist
      choose_2(rest,W1[j_2][h],u1,u2);   // bestimme den Rang des ersten und des zweiten Wurzelbaums 
    }
    ret=0;
    if(DEBUG)
      cout << "unrank(" << u1 << ") -> Wurzelbaum der Hoehe " << j_2 << " bestehend aus " << h << " Knoten" << endl;
    ret=unrank_wurzelbaum(u1,j_2,h,wb1,W1,W2);        // konstruiere den ersten Wurzelbaum  
    if(DEBUG) 
      cout << "unrank(" << u2 << ") -> Wurzelbaum der Hoehe " << j_2 << " bestehend aus " << n-h << " Knoten" << endl;
    ret=ret+unrank_wurzelbaum(u2,j_2,n-h,wb2,W1,W2);  // konstruiere den zweiten Wurzelbaum 
    if(ret<0) return -1;                          // Konstruktion ar nicht erfolgreich    
    // Fuege die zwei Wurzelbaeume zu einem Alkan zusammen
    Alk.laengste_gerade_Kette_ist_gerade=true;
    Alk.Wurzelbaum_1.resize(wb1.size());          // reserviere Speicherplatz
    for(i=0;i<wb1.size();i++)                     // kopiere den ersten Wurzelbaum 
      Alk.Wurzelbaum_1[i]=wb1[i];
    Alk.Wurzelbaum_2.resize(wb2.size());          // reserviere Speicherplatz
    for(i=0;i<wb2.size();i++)                     // kopiere den zweiten Wurzelbaum
      Alk.Wurzelbaum_2[i]=wb2[i];
    Alk.Wurzelbaum_3.resize(0);                   // es gibt keinen dritten Wurzelbaum
    Alk.Wurzelbaum_4.resize(0);                   // es gibt keinen vierten Wurzelbaum
    return 0;                                     // Konstruktion war erfolgreich 
  }
  else      // das Alkan besteht aus 4 Wurzelbaeumen
  {
    j_2=(j-1)/2;   // maximale Hoehe eines Wurzelbaums
    h1=2*n+2;      // nicht erreichbare Anzahl der Knoten des ersten Wurzelbaums
    h2=2*n+2;      // nicht erreichbare Anzahl der Knoten des zweiten Wurzelbaums
    h3=2*n+2;      // nicht erreichbare Anzahl der Knoten des dritten Wurzelbaums
    h4=2*n+2;      // nicht erreichbare Anzahl der Knoten des vierten Wurzelbaums  
    br=false;      // Schleifen muessen noch durchlaufen werden

    // Unterscheide 3 Faelle
    // (i)   das Alkan besitzt 2 Wurzelbaeume der Hoehe j_2 und 2 Wurzelbaeume der Hoehe kleiner j_2
    // (ii)  das Alkan besitzt 3 Wurzelbaeume der Hoehe j_2 und 1 Wurzelbaum der Hoehe kleiner j_2
    // (iii) das Alkan besitzt 4 Wurzelbaeume der Hoehe j_2 

    // Fall (i)
    g1=bestimme_ende(j_2,(n-1)/2);
    for(i1=bestimme_start_1(j_2,n);i1<=g1;i1++)
    {                              // durchlaufe die moeglichen Anzahlen an Knoten des ersten Wurzelbaums
                                   // o Hoehe=j_2 => mindestens j_2 Knoten
                                   // o hoechstens so viele Knoten wie Wurzelbaum 2  
      if(br==true) break;
      g2=bestimme_ende(j_2,n-1-i1);
      for(i2=bestimme_start_9(i1,j_2,n-1-i1);i2<=g2;i2++)   
      {                                  // durchlaufe die moeglichen Anzahlen an Knoten des zweiten Wurzelbaums   
        if(br==true) break; 
        g3=bestimme_ende(j_2-1,(n-1-i1-i2)/2);
        for(i3=bestimme_start_2(j_2-1,n-1-i1-i2);i3<=g3;i3++) 
        {                                // durchlaufe die moeglichen Anzahlen an Knoten des dritten Wurzelbaums
                                         // o hoechstens so viele Knoten wie der vierte Wurzelbaum 
          i4=n-1-i1-i2-i3;               // der vierte Wurzelbaum besteht aus den restlichen Knoten
          // Unterscheide 4 Faelle, ob der erste und der zweite oder der dritte und der vierte
          // Wurzelbaum genauso viele Knoten besitzen oder nicht
          if(i1<i2 && i3<i4) 
            p=p+W1[j_2][i1]*W1[j_2][i2]*W2[j_2][i3]*W2[j_2][i4];
          if(i1<i2 && i3==i4)   
            p=p+W1[j_2][i1]*W1[j_2][i2]*exquo(W2[j_2][i3]*(W2[j_2][i4]+1),2);
          if(i1==i2 && i3<i4)
            p=p+exquo(W1[j_2][i1]*(W1[j_2][i2]+1),2)*W2[j_2][i3]*W2[j_2][i4];
          if(i1==i2 && i3==i4)
            p=p+exquo(W1[j_2][i1]*(W1[j_2][i2]+1),2)*exquo(W2[j_2][i3]*(W2[j_2][i4]+1),2); 
          if(p>r)      // falls der gewuenschte Rang ueberschritten wurde
          {
            h1=i1;     // merke die Anzahl der Knoten des ersten Wurzelbaums
            h2=i2;     // merke die Anzahl der Knoten des zweiten Wurzelbaums 
            h3=i3;     // merke die Anzahl der Knoten des dritten Wurzelbaums
            h4=i4;     // merke die Anzahl der Knoten des vierten Wurzelbaums

            // mache das dazuaddieren von eben wieder rueckgaengig
            if(i1<i2 && i3<i4) 
              p=p-W1[j_2][i1]*W1[j_2][i2]*W2[j_2][i3]*W2[j_2][i4];
            if(i1<i2 && i3==i4)   
              p=p-W1[j_2][i1]*W1[j_2][i2]*exquo(W2[j_2][i3]*(W2[j_2][i4]+1),2);
            if(i1==i2 && i3<i4)
              p=p-exquo(W1[j_2][i1]*(W1[j_2][i2]+1),2)*W2[j_2][i3]*W2[j_2][i4];
            if(i1==i2 && i3==i4)
              p=p-exquo(W1[j_2][i1]*(W1[j_2][i2]+1),2)*exquo(W2[j_2][i3]*(W2[j_2][i4]+1),2);
            br=true;   // die Schleifen brauchen nicht mehr durchlaufen zu werden  
            break;
          }  
        } 
      }   
    }
    if(h1<2*n+2)         // Alkan besteht aus zwei Wurzelbaeumen der Hoehe j_2 und zwei Wurzelbaeumen der
    {                    // Hoehe kleiner j_2
      if(h1<h2 && h3<h4)
      {
        rest=r-p;                                // der Teil des Rangs, der fuer die 4 Wurzelbaeume verantwortlich ist
        choose_1(rest,W1[j_2][h1],u1);           // bestimme den Rang des ersten Wurzelbaums   
        choose_1(rest,W1[j_2][h2],u2);           // bestimme den Rang des zweiten Wurzelbaums
        choose_1(rest,W2[j_2][h3],u3);           // bestimme den Rang des dritten Wurzelbaums
        choose_1(rest,W2[j_2][h4],u4);           // bestimme den Rang des vierten Wurzelbaums
      }
      if(h1<h2 && h3==h4)
      {
        rest=r-p;                                // der Teil des Rangs, der fuer die 4 Wurzelbaeume verantwortlich ist
        choose_1(rest,W1[j_2][h1],u1);           // bestimme den Rang des ersten Wurzelbaums
        choose_1(rest,W1[j_2][h2],u2);           // bestimme den Rang des zweiten Wurzelbaums 
        choose_2(rest,W2[j_2][h3],u3,u4);        // bestimme den Rang des dritten und des vierten Wurzelbaums
     }
     if(h1==h2 && h3<h4)
     {
        rest=r-p;                                // der Teil des Rangs, der fuer die 4 Wurzelbaeume verantwortlich ist
        choose_2(rest,W1[j_2][h1],u1,u2);        // bestimme den Rang des ersten und des zweiten Wurzelbaums
        choose_1(rest,W2[j_2][h3],u3);           // bestimme den Rang des dritten Wurzelbaums 
        choose_1(rest,W2[j_2][h4],u4);           // bestimme den Rang des vierten Wurzelbaums
      }
      if(h1==h2 && h3==h4) 
      {
        rest=r-p;                                // der Teil des Rangs, der fuer die 4 Wurzelbaeume verantwortlich ist
        choose_2(rest,W1[j_2][h1],u1,u2);        // bestimme den Rang des ersten und des zweiten Wurzelbaums
        choose_2(rest,W2[j_2][h3],u3,u4);        // bestimme den Rang des dritten und des vierten Wurzelbaums
      }
      ret=0; 
      if(DEBUG)
        cout << "unrank(" << u1 << ") -> Wurzelbaum der Hoehe " << j_2 << " bestehend aus " << h1 << " Knoten" << endl;
      ret=ret+unrank_wurzelbaum(u1,j_2,h1,wb1,W1,W2);           // konstruiere den ersten Wurzelbaum
      if(DEBUG)
        cout << "unrank(" << u2 << ") -> Wurzelbaum der Hoehe " << j_2 << " bestehend aus " << h2 << " Knoten" << endl;
      ret=ret+unrank_wurzelbaum(u2,j_2,h2,wb2,W1,W2);           // konstruiere den zweiten Wurzelbaum  
      if(DEBUG)
        cout << "unrank(" << u3 << ") -> Wurzelbaum der Hoehe < " << j_2 << " bestehend aus " << h3 << " Knoten" << endl;
      ret=ret+unrank_wurzelbaum_kleiner(u3,j_2,h3,wb3,W1,W2);   // konstruiere den dritten Wurzelbaum 
      if(DEBUG)
        cout << "unrank(" << u4 << ") -> Wurzelbaum der Hoehe < " << j_2 << " bestehend aus " << h4 << " Knoten" << endl;
      ret=ret+unrank_wurzelbaum_kleiner(u4,j_2,h4,wb4,W1,W2);   // // konstruiere den vierten Wurzelbaum
      if(ret<0) return -1;                            // Konstruktion war nicht erfolgreich

      // Konstruiere das Alkan aus den 4 Wurzelbaeumen
      Alk.laengste_gerade_Kette_ist_gerade=false;

      // Bemerkung:
      // komische Reihenfolge beim kopieren, da Ausgabe vorraussetzt, das die 
      // Wurzelbaum_3 und Wurzelbaum_4 nicht leer sind
      Alk.Wurzelbaum_3.resize(wb1.size());            // reserviere Speicherplatz
      for(i=0;i<wb1.size();i++)                       // kopiere den ersten Wurzelbaum
        Alk.Wurzelbaum_3[i]=wb1[i];
      Alk.Wurzelbaum_4.resize(wb2.size());            // reserviere Speicherplatz
      for(i=0;i<wb2.size();i++)                       // kopiere den zweiten Wurzelbaum  
        Alk.Wurzelbaum_4[i]=wb2[i];
      Alk.Wurzelbaum_1.resize(wb3.size());            // reserviere Speicherplatz
      for(i=0;i<wb3.size();i++)                       // kopiere den dritten Wurzelbaum
        Alk.Wurzelbaum_1[i]=wb3[i];
      Alk.Wurzelbaum_2.resize(wb4.size());            // reserviere Speicherplatz
      for(i=0;i<wb4.size();i++)                       // kopiere den vierten Wurzelbaum
        Alk.Wurzelbaum_2[i]=wb4[i]; 
      return 0;                                       // Konstruktion war erfolgreich   
    }
    else   // besteht das Alkan vielleicht aus 3 Wurzelbaeumen der Hoehe j_2 und einem Wurzel-
    {      // baum der Hoehe kleiner j_2?  (Fall (ii))
      h1=2*n+2;  // setze Anzahl der Knoten des ersten Wurzelbaums auf eine nciht erreichbare Zahl
      h2=2*n+2;  // setze Anzahl der Knoten des zweiten Wurzelbaums auf eine nciht erreichbare Zahl
      h3=2*n+2;  // setze Anzahl der Knoten des dritten Wurzelbaums auf eine nciht erreichbare Zahl
      h4=2*n+2;  // setze Anzahl der Knoten des vierten Wurzelbaums auf eine nciht erreichbare Zahl 
      br=false;  // Schleifen muessen noch durchlaufen werden 
      g1=bestimme_ende(j_2,(n-1)/3);
      for(i1=bestimme_start_3(j_2,n);i1<=g1;i1++)       
      {                                  // durchlaufe die moeglichen Anzahlen an Knoten des ersten Wurzelbaums
                                         // o Hoehe=j_2 => mindestens j_2 Knoten
                                         // o hoechstens so viele Knoten wie der zweite und der dritte Wurzelbaum
        if(br==true) break;
        g2=bestimme_ende(j_2,(n-1-i1)/2);
        for(i2=bestimme_start_4(i1,j_2,n-1-i1);i2<=g2;i2++)   
        {                                // durchlaufe die moeglichen Anzahlen an Knoten des zweiten Wurzelbaums
                                         // o mindestens so viele Knoten wie der erste Wurzelbaum
                                         // o hoechstens so viele Knoten wie der dritte Wurzelbaum
          if(br==true) break; 
          g3=bestimme_ende(j_2,n-1-i1-i2);
          for(i3=bestimme_start_7(i2,j_2,n-1-i1-i2);i3<=g3;i3++)  
          {                              // durchlaufe die moeglichen Anzahlen an Knoten des dritten Wurzelbaums
                                         // o mindestens so viele Knoten wie der zweite Wurzelbaum 
            i4=n-1-i1-i2-i3;             // der vierte Wurzelbaum besteht aus den restlichen Knoten
            // Unterscheide 4 Faelle, ob zwei oder mehr der ersten drei Wurzelbaeume die selbe Anzhal an Knoten besitzen 
            if(i1==i2 && i2==i3)
              p=p+exquo(W1[j_2][i1]*(W1[j_2][i2]+1)*(W1[j_2][i3]+2),6)*W2[j_2][i4];
            if(i1==i2 && i2<i3)
              p=p+exquo(W1[j_2][i1]*(W1[j_2][i2]+1),2)*W1[j_2][i3]*W2[j_2][i4];
            if(i1<i2 && i2==i3)
              p=p+W1[j_2][i1]*exquo(W1[j_2][i2]*(W1[j_2][i3]+1),2)*W2[j_2][i4];
            if(i1<i2 && i2<i3)
              p=p+W1[j_2][i1]*W1[j_2][i2]*W1[j_2][i3]*W2[j_2][i4];
            if(p>r)      // falls der gewuenschte Rang ueberschritten wurde
            {
              h1=i1;     // merke die Anzahl der Knoten des ersten Wurzelbaums
              h2=i2;     // merke die Anzahl der Knoten des zweiten Wurzelbaums  
              h3=i3;     // merke die Anzahl der Knoten des dritten Wurzelbaums  
              h4=i4;     // merke die Anzahl der Knoten des vierten Wurzelbaums
              // mache das dazuaddieren von eben wieder rueckgaengig  
              if(h1==h2 && h2==h3)
                p=p-exquo(W1[j_2][h1]*(W1[j_2][h2]+1)*(W1[j_2][h3]+2),6)*W2[j_2][h4];
              if(h1==h2 && h2<h3)
                p=p-exquo(W1[j_2][h1]*(W1[j_2][h2]+1),2)*W1[j_2][h3]*W2[j_2][h4];
              if(h1<h2 && h2==h3)
                p=p-W1[j_2][h1]*exquo(W1[j_2][h2]*(W1[j_2][h3]+1),2)*W2[j_2][h4];
              if(h1<h2 && h2<h3)
                p=p-W1[j_2][h1]*W1[j_2][h2]*W1[j_2][h3]*W2[j_2][h4];
              br=true;   // die Schleifen brauchen nicht mehr weiter durchlaufen zu werden 
              break;  
            } 
          } 
        }
      }
      if(h1<2*n+2)         // Alkan besteht aus drei Wurzelbaeumen der Hoehe j_2 und einem Wurzelbaum der
      {                    // Hoehe kleiner j_2 
        if(h1==h2 && h2==h3)
        {
          rest=r-p;                            // der Teil, der fuer die Raenge der 4 Wurzelbaeume verantwortlich ist
          choose_3(rest,W1[j_2][h1],u1,u2,u3); // bestimme die Raenge der ersten 3 Wurzelbaeume
          choose_1(rest,W2[j_2][h4],u4);       // bestimme den Rang des vierten Wurzelbaums
        } 
        if(h1==h2 && h2<h3)
        {
          rest=r-p;                            // der Teil, der fuer die Raenge der 4 Wurzelbaeume verantwortlich ist
          choose_2(rest,W1[j_2][h1],u1,u2);    // bestimme den Rang des ersten und des zweiten Wurzelbaums
          choose_1(rest,W1[j_2][h3],u3);       // bestimme den Rang des dritten Wurzelbaums
          choose_1(rest,W2[j_2][h4],u4);       // bestimme den Rang des vierten Wurzelbaums
        }
        if(h1<h2 && h2==h3)
        {
          rest=r-p;                            // der Teil, der fuer die Raenge der 4 Wurzelbaeume verantwortlich ist 
          choose_1(rest,W1[j_2][h1],u1);       // bestimme den Rang des ersten Wurzelbaums
          choose_2(rest,W1[j_2][h2],u2,u3);    // bestimme den Rang des zweiten und des dritten Wurzelbaums
          choose_1(rest,W2[j_2][h4],u4);       // bestimme den Rang des vierten Wurzelbaums 
        }
        if(h1<h2 && h2<h3)
        {
          rest=r-p;                            // der Teil, der fuer die Raenge der 4 Wurzelbaeume verantwortlich ist
          choose_1(rest,W1[j_2][h1],u1);       // bestimme den Rang des ersten Wurzelbaums
          choose_1(rest,W1[j_2][h2],u2);       // bestimme den Rang des zweiten Wurzelbaums 
          choose_1(rest,W1[j_2][h3],u3);       // bestimme den Rang des dritten Wurzelbaums
          choose_1(rest,W2[j_2][h4],u4);       // bestimme den Rang des vierten Wurzelbaums 
        }    
        ret=0;
        if(DEBUG)
          cout << "unrank(" << u1 << ") -> Wurzelbaum der Hoehe " << j_2 << " bestehend aus " << h1 << " Knoten" << endl;
        ret=ret+unrank_wurzelbaum(u1,j_2,h1,wb1,W1,W2);          // konstruiere den ersten Wurzelbaum
        if(DEBUG)
          cout << "unrank(" << u2 << ") -> Wurzelbaum der Hoehe " << j_2 << " bestehend aus " << h2 << " Knoten" << endl;
        ret=ret+unrank_wurzelbaum(u2,j_2,h2,wb2,W1,W2);          // konstruiere den zweiten Wurzelbaum 
        if(DEBUG)  
          cout << "unrank(" << u3 << ") -> Wurzelbaum der Hoehe " << j_2 << " bestehend aus " << h3 << " Knoten" << endl;
        ret=ret+unrank_wurzelbaum(u3,j_2,h3,wb3,W1,W2);          // konstruiere den dritten Wurzelbaum
        if(DEBUG) 
          cout << "unrank(" << u4 << ") -> Wurzelbaum der Hoehe < " << j_2 << " bestehend aus " << h4 << " Knoten" << endl;
        ret=ret+unrank_wurzelbaum_kleiner(u4,j_2,h4,wb4,W1,W2);  // konstruiere den vierten Wurzelbaum
        if(ret<0) return -1;                                     // Konstruktion war nicht erfolgreich

        // Setze die 4 Wurzelbaeume zu einem Alkan zusammen
        // Bemerkung:
        // komische Reihenfolge beim kopieren, da Ausgabe vorraussetzt, 
        // das die Wurzelbaum_3 und Wurzelbaum_4 nicht leer sind
        Alk.laengste_gerade_Kette_ist_gerade=false;
        Alk.Wurzelbaum_3.resize(wb1.size());     // reserviere Speicherplatz
        for(i=0;i<wb1.size();i++)                // kopiere den ersten Wurzelbaum
          Alk.Wurzelbaum_3[i]=wb1[i];
        Alk.Wurzelbaum_4.resize(wb2.size());     // reserviere Speicherplatz 
        for(i=0;i<wb2.size();i++)                // kopiere den zweiten Wurzelbaum 
          Alk.Wurzelbaum_4[i]=wb2[i];
        Alk.Wurzelbaum_1.resize(wb3.size());     // reserviere Speicherplatz 
        for(i=0;i<wb3.size();i++)                // kopiere den dritten Wurzelbaum
         Alk.Wurzelbaum_1[i]=wb3[i];
        Alk.Wurzelbaum_2.resize(wb4.size());     // reserviere Speicherplatz 
        for(i=0;i<wb4.size();i++)                // kopiere den vierten Wurzelbaum 
          Alk.Wurzelbaum_2[i]=wb4[i]; 
        return 0;                                // Konstruktion war erfolgreich 
      }
      else                 // Besteht das Alkan vielleicht aus 4 Wurzelbaeumen der Hoehe j_2? (Fall (iii))
      {
        h1=2*n+2;          // setze die Anzahl der Knoten des ersten Wurzelbaums auf einen nicht erreichbaren Wert 
        h2=2*n+2;          // setze die Anzahl der Knoten des zweiten Wurzelbaums auf einen nicht erreichbaren Wert
        h3=2*n+2;          // setze die Anzahl der Knoten des dritten Wurzelbaums auf einen nicht erreichbaren Wert
        h4=2*n+2;          // setze die Anzahl der Knoten des vierten Wurzelbaums auf einen nicht erreichbaren Wert 
        br=false;          // Schleifen muessen noch durchlaufen werden
        g1=bestimme_ende(j_2,(n-1)/4);
        for(i1=bestimme_start_5(j_2,n);i1<=g1;i1++)   
        {                              // durchlaufe die moeglichen Anzahlen an Knoten des ersten Wurzelbaums
                                       // Hoehe=j_2 => mindestens j_2 Knoten
                                       // hoechstens so viele Knoten wie die restlichen 3 Wurzelbaeume     
          if(br==true) break;
          g2=bestimme_ende(j_2,(n-1-i1)/3); 
          for(i2=bestimme_start_6(i1,j_2,n-1-i1);i2<=g2;i2++)  
          {                               // durchlaufe die moeglichen Anzahlen an Knoten des zweiten Wurzelbaums
                                          // o mindestens so viele Knoten wie der erste Wurzelbaum
                                          // o hoechstens so viele Knoten wie der dritte und der vierte Wurzelbaum 
            if(br==true) break; 
            g3=bestimme_ende(j_2,(n-1-i1-i2)/2);
            for(i3=bestimme_start_7(i2,j_2,n-1-i1-i2);i3<=g3;i3++)  
            {                                  // durchlaufe die moeglichen Anzahlen an Knoten des dritten Wurzelbaums
                                               // o mindestens so viele Knoten wie der zweite Wurzelbaum
                                               // o hoechstens so viele Knoten wie der vierte Wurzelbaum
              i4=n-1-i1-i2-i3;                 // der vierte Wurzelbaum besteht aus den restlichen Knoten

              // Unterscheide 8 Faelle, Wurzelbaeume von diesen 4 Wurzelbaeumen die selbe Anzahl an Knoten besitzen 
              if(i1==i2 && i2==i3 && i2==i4)
                p=p+exquo(W1[j_2][i1]*(W1[j_2][i2]+1)*(W1[j_2][i3]+2)*(W1[j_2][i4]+3),24);
              if(i1==i2 && i2==i3 && i3<i4)
                p=p+exquo(W1[j_2][i1]*(W1[j_2][i2]+1)*(W1[j_2][i3]+2),6)*W1[j_2][i4];
              if(i1<i2 && i2==i3 && i3==i4)
                p=p+W1[j_2][i1]*exquo(W1[j_2][i2]*(W1[j_2][i3]+1)*(W1[j_2][i4]+2),6);
              if(i1==i2 && i2<i3 && i3==i4)
                p=p+exquo(W1[j_2][i1]*(W1[j_2][i2]+1),2)*exquo(W1[j_2][i3]*(W1[j_2][i4]+1),2);
              if(i1==i2 && i2<i3 && i3<i4)
                p=p+exquo(W1[j_2][i1]*(W1[j_2][i2]+1),2)*W1[j_2][i3]*W1[j_2][i4];
              if(i1<i2 && i2==i3 && i3<i4)
                p=p+W1[j_2][i1]*exquo(W1[j_2][i2]*(W1[j_2][i3]+1),2)*W1[j_2][i4];
              if(i1<i2 && i2<i3 && i3==i4)
                p=p+W1[j_2][i1]*W1[j_2][i2]*exquo(W1[j_2][i3]*(W1[j_2][i4]+1),2);
              if(i1<i2 && i2<i3 && i3<i4)
                p=p+W1[j_2][i1]*W1[j_2][i2]*W1[j_2][i3]*W1[j_2][i4];    
              if(p>r)       // falls der gewuenschte Rang ueberschritten wurde
              {
                h1=i1;      // merke die Anzahl der Knoten des ersten Wurzelbaums 
                h2=i2;      // merke die Anzahl der Knoten des zweiten Wurzelbaums
                h3=i3;      // merke die Anzahl der Knoten des dritten Wurzelbaums 
                h4=i4;      // merke die Anzahl der Knoten des vierten Wurzelbaums 
  
                // mache das dazuaddieren von eben wieder rueckgaengig
                if(h1==h2 && h2==h3 && h2==h4)
                  p=p-exquo(W1[j_2][h1]*(W1[j_2][h2]+1)*(W1[j_2][h3]+2)*(W1[j_2][h4]+3),24);
                if(h1==h2 && h2==h3 && h3<h4)
                  p=p-exquo(W1[j_2][h1]*(W1[j_2][h2]+1)*(W1[j_2][h3]+2),6)*W1[j_2][h4];
                if(h1<h2 && h2==h3 && h3==h4)
                  p=p-W1[j_2][h1]*exquo(W1[j_2][h2]*(W1[j_2][h3]+1)*(W1[j_2][h4]+2),6);
                if(h1==h2 && h2<h3 && h3==h4)
                  p=p-exquo(W1[j_2][h1]*(W1[j_2][h2]+1),2)*exquo(W1[j_2][h3]*(W1[j_2][h4]+1),2);
                if(h1==h2 && h2<h3 && h3<h4)
                  p=p-exquo(W1[j_2][h1]*(W1[j_2][h2]+1),2)*W1[j_2][h3]*W1[j_2][h4];
                if(h1<h2 && h2==h3 && h3<h4)
                  p=p-W1[j_2][h1]*exquo(W1[j_2][h2]*(W1[j_2][h3]+1),2)*W1[j_2][h4];
                if(h1<h2 && h2<h3 && h3==h4)
                  p=p-W1[j_2][h1]*W1[j_2][h2]*exquo(W1[j_2][h3]*(W1[j_2][h4]+1),2);
                if(h1<h2 && h2<h3 && h3<h4)
                  p=p-W1[j_2][h1]*W1[j_2][h2]*W1[j_2][h3]*W1[j_2][h4];
                br=true;    // die Schleifen brauchen nicht weiter durchlaufen zu werden
                break; 
              }
            }
          }      
        }
        if(h1<2*n+2) // Alkan besteht aus 4 Wurzelbaeumen der Hoehe j_2
        {
          if(h1==h2 && h2==h3 && h2==h4)
          {
            rest=r-p;                                // der fuer die 4 Wurzelbaeume verantwortliche Teil 
            choose_4(rest,W1[j_2][h1],u1,u2,u3,u4);  // bestimme die Raenge der 4 Wurzelbaeume
          }  
          if(h1==h2 && h2==h3 && h3<h4)
          {
            rest=r-p;                                // der fuer die 4 Wurzelbaeume verantwortliche Teil 
            choose_3(rest,W1[j_2][h1],u1,u2,u3);     // bestimme die Raenge der ersten 3 Wurzelbaeume
            choose_1(rest,W1[j_2][h4],u4);           // bestimme den Rang des vierten Wurzelbaums  
          }
          if(h1<h2 && h2==h3 && h3==h4)
          {
            rest=r-p;                                // der fuer die 4 Wurzelbaeume verantwortliche Teil
            choose_1(rest,W1[j_2][h1],u1);           // bestimme den Rang des ersten Wurzelbaums
            choose_3(rest,W1[j_2][h2],u2,u3,u4);     // bestimme die Raenge der letzten 3 Wurzelbaeume 
          } 
          if(h1==h2 && h2<h3 && h3==h4)
          {
            rest=r-p;                                // der fuer die 4 Wurzelbaeume verantwortliche Teil
            choose_2(rest,W1[j_2][h1],u1,u2);        // bestimme den Rang von Wurzelbaum 1 und 2 
            choose_2(rest,W1[j_2][h3],u3,u4);        // bestimme den Rang von Wurzelbaum 3 und 4
          }
          if(h1==h2 && h2<h3 && h3<h4)
          {
            rest=r-p;                                // der fuer die 4 Wurzelbaeume verantwortliche Teil 
            choose_2(rest,W1[j_2][h1],u1,u2);        // bestimme den Rang von Wurzelbaum 1 und 2 
            choose_1(rest,W1[j_2][h3],u3);           // bestimme den Rang des dritten Wurzelbaums   
            choose_1(rest,W1[j_2][h4],u4);           // bestimme den Rang des vierten Wurzelbaums   
          }  
          if(h1<h2 && h2==h3 && h3<h4)
          {
            rest=r-p;                                // der fuer die 4 Wurzelbaeume verantwortliche Teil
            choose_1(rest,W1[j_2][h1],u1);           // bestimme den Rang des ersten Wurzelbaums 
            choose_2(rest,W1[j_2][h2],u2,u3);        // bestimme den Rang des zweiten und des dritten Wurzelbaums 
            choose_1(rest,W1[j_2][h4],u4);           // bestimme den Rang des vierten Wurzelbaums 
          }
          if(h1<h2 && h2<h3 && h3==h4)
          {
            rest=r-p;                                // der fuer die 4 Wurzelbaeume verantwortliche Teil
            choose_1(rest,W1[j_2][h1],u1);           // bestimme den Rang des ersten Wurzelbaums
            choose_1(rest,W1[j_2][h2],u2);           // bestimme den Rang des zweiten Wurzelbaums
            choose_2(rest,W1[j_2][h3],u3,u4);        // bestimme den Rang des dritten und des vierten Wurzelbaums
          } 
          if(h1<h2 && h2<h3 && h3<h4)
          {
            rest=r-p;                                // der fuer die 4 Wurzelbaeume verantwortliche Teil
            choose_1(rest,W1[j_2][h1],u1);           // bestimme den Rang des ersten Wurzelbaums
            choose_1(rest,W1[j_2][h2],u2);           // bestimme den Rang des zweiten Wurzelbaums 
            choose_1(rest,W1[j_2][h3],u3);           // bestimme den Rang des dritten Wurzelbaums
            choose_1(rest,W1[j_2][h4],u4);           // bestimme den Rang des vierten Wurzelbaums
          }
          ret=0;
          if(DEBUG)
            cout << "unrank(" << u1 << ") -> Wurzelbaum der Hoehe " << j_2 << " bestehend aus " << h1 << " Knoten" << endl;
          ret=ret+unrank_wurzelbaum(u1,j_2,h1,wb1,W1,W2);    // konstruiere den ersten Wurzelbaum
          if(DEBUG)
            cout << "unrank(" << u2 << ") -> Wurzelbaum der Hoehe " << j_2 << " bestehend aus " << h2 << " Knoten" << endl;
          ret=ret+unrank_wurzelbaum(u2,j_2,h2,wb2,W1,W2);    // konstruiere den zweiten Wurzelbaum
          if(DEBUG) 
            cout << "unrank(" << u3 << ") -> Wurzelbaum der Hoehe " << j_2 << " bestehend aus " << h3 << " Knoten" << endl;
          ret=ret+unrank_wurzelbaum(u3,j_2,h3,wb3,W1,W2);    // konstruiere den dritten Wurzelbaum
          if(DEBUG)
            cout << "unrank(" << u4 << ") -> Wurzelbaum der Hoehe " << j_2 << " bestehend aus " << h4 << " Knoten" << endl;
          ret=ret+unrank_wurzelbaum(u4,j_2,h4,wb4,W1,W2);    // konstruiere den vierten Wurzelbaum  
          if(ret<0) return -1;                               // Konstruktion war nicht erfolgreich

          // Fuege die 4 Wurzelbaeume zu einem Alkan zusammen
          // Bemerkung:
          // komische Reihenfolge beim kopieren, da Ausgabe vorraussetzt, 
          // das die Wurzelbaum_3 und Wurzelbaum_4 nicht leer sind
          Alk.laengste_gerade_Kette_ist_gerade=false;
          Alk.Wurzelbaum_3.resize(wb1.size());    // reserviere Speicherplatz
          for(i=0;i<wb1.size();i++)               // kopiere den ersten Wurzelbaum 
            Alk.Wurzelbaum_3[i]=wb1[i];
          Alk.Wurzelbaum_4.resize(wb2.size());    // reserviere Speicherplatz  
          for(i=0;i<wb2.size();i++)               // kopiere den zweiten Wurzelbaum
             Alk.Wurzelbaum_4[i]=wb2[i];
          Alk.Wurzelbaum_1.resize(wb3.size());    // reserviere Speicherplatz
          for(i=0;i<wb3.size();i++)               // kopiere den dritten Wurzelbaum 
            Alk.Wurzelbaum_1[i]=wb3[i];
          Alk.Wurzelbaum_2.resize(wb4.size());    // reserviere Speicherplatz
          for(i=0;i<wb4.size();i++)               // kopiere den vierten Wurzelbaum 
            Alk.Wurzelbaum_2[i]=wb4[i]; 
          return 0;                               // Konstruktion war erfolgreich
        }
        else         // Es gibt kein Alkan mit dieser Nummer
        {
          cout << "Alkan Nr. " << r << " bestehend aus " << n << " C-Atomen gibt es nicht." << endl;
          return -1;
        }           
      }
    }
  }
  return 0;    
}


// ********************************************************************
// *                       Hauptprogramm                              *
// ******************************************************************** 

int main(void)
{
  unsigned int n;       // maximale Anzahl an C-Atomen
  unsigned int i,j;     // Schleifenvariablen
  int menu;             // Variable fuer eine Menueauswahl
  char dateiname[100];  // Array fuer einen Dateinamen
  
  menu=-1;
  while(menu<1 || menu>4)
  {
    cout << "Zufaelliges Erzeugen von Isomeren der Alkane (C_nH_2n+2)" << endl;
    cout << "=======================================================" << endl;
    cout << endl;
    cout << "Um Isomere der Alkane zufaellig zu erzeugen werden Anzahlen von Alkanen und ";
    cout << "Wurzelbaeumen benoetigt. Diese koennen entweder neu berechnet werden, oder sie ";
    cout << "koennen aus einer Datei eingelesen werden. Was moechten sie tun?" << endl;
    cout << endl;  
    cout << "(1) Anzahlen aus einer Datei einlesen" << endl;
    cout << "(2) Anzahlen neu berechnen" << endl;
    cout << "(3) Anzahlen neu berechnen und in einer Datei abspeichern" << endl;
    cout << "(4) das Programm beenden" << endl;
    cout << ": ";
    cin  >> menu; 
    cout << endl;
    cout << endl; 
  }
  if(menu==4)  // beende das Programm
  {
    return 0;
  }
  if(menu==2 || menu==3)
  {
    cout << "Bitte gebe die maximale Anzahl an C-Atomen ein: ";
    cin  >> n;
    cout << endl;
  }
  if(menu==1 || menu==3)
  {
    cout << "Bitte gebe den Namen der Datei ein: ";
    cin  >> dateiname;
    cout << endl;
  }

  // Datenstrukturen fuer die Anzahlen
  vector< vector<cln::cl_I> > Wurzelbaeume;   // Wurzelbaeume[i][j] enthaelt die Anzahl der Wurzelbaeume der 
                                              // Hoehe i bestehend aus j Knoten  
  vector< vector<cln::cl_I> > Wurzelbaeume_2; // Wurzelbaeume_2[i][j] enthaelt die Anzahl der Wurzelbaeume der
                                              // Hoehe kleiner i aus j Knoten
  vector< vector<cln::cl_I> > Alkane;         // Alkane[i][j] enthaelt die Anzahl der Alkane mit laengster gerader
                                              // Kette der Laenge i bestehend aus j Knoten 
  vector<cln::cl_I> Gesamt;                   // Gesamt[i] enthaelt die Anzahl der Isomere der Alkane bestehend     
                                              // aus i C-Atomen 

  if(menu==2 || menu==3)                      // Berechnet die Anzahl der Alkane
  {
    cout << "Berechne Wurzelbaeume" << endl;
    Berechne_Anzahl_Wurzelbaeume(Wurzelbaeume,Wurzelbaeume_2,n);
    cout << "Berechne Alkane" << endl;
    Berechne_Anzahl_Alkane(Alkane,Wurzelbaeume,Wurzelbaeume_2,n);
    cout << endl;
  }

  if(menu==1) // Lese Anzahlen aus einer Datei ein
  {
    ifstream eingabe;
    eingabe.open(dateiname);                // Öffne Datei 
    eingabe >> n;                           // Lese maximale Anzahl an C-Atomen ein
    Wurzelbaeume.resize(n/2+1);             // reserviere Speicherplatz
    Wurzelbaeume_2.resize(n/2+1);           // resreviere Speicherplatz  
    for(i=0;i<=n/2;i++)                     // durchlaufe die moeglichen Hoehen der Wurzelbaeume  
    {
      Wurzelbaeume[i].resize(n+1);          // reserviere Speicherplatz
      Wurzelbaeume_2[i].resize(n+1);        // reserviere Speicherplatz     
    }
    // Wurzelbaeume der Hoehe ??
    for(i=0;i<=n/2;i++)                     // durchlaufe die moeglichen Hoehen der Wurzelbaeume 
      for(j=0;j<=n;j++)                     // durchlaufe die moeglichen Anzahlen an Knoten  
        eingabe >> Wurzelbaeume[i][j];      // lese Anzahl ein 
    // Wurzelbaeume der Hoehe kleiner ??
    for(i=0;i<=n/2;i++)                     // durchlaufe die moeglichen Hoehen der Wurzelbaeume 
      for(j=0;j<=n;j++)                     // durchlaufe die moeglichen Anzahlen an Knoten  
        eingabe >> Wurzelbaeume_2[i][j];    // lese Anzahl ein
    // Alkane
    Alkane.resize(n+1);                     // reserviere Speicherplatz
    for(i=0;i<=n;i++)                       // durchlaufe die moeglichen Laengen der laengsten Kette
      Alkane[i].resize(n+1);                // reserviere Speicherplatz       
    for(i=0;i<=n;i++)                       // durchlaufe die moeglichen Laengen der laengsten Kette
      for(j=0;j<=n;j++)                     // durchlaufe die moeglichen Anzahlen an C-Atomen 
        eingabe >> Alkane[i][j];            // Lese Anzahl ein 
    eingabe.close();                        // schliesse Datei wieder
  }
  if(menu==3) // Speichere Anzahlen in einer Datei ab
  {
    ofstream ausgabe;
    ausgabe.open(dateiname);                // oeffne die Datei 
    ausgabe << n << endl;                   // schreibe die maximale Anzahl an C-Atomen in Datei 
    // Wurzelbaeume der Hoehe ??
    for(i=0;i<=n/2;i++)                     // durchlaufe die moeglichen Hoehen               
    {
      for(j=0;j<=n;j++)                     // durchlaufe die moeglichen Anzahlen an Knoten
        ausgabe << Wurzelbaeume[i][j] << " ";  // schreibe Anzahl getrennt durch Leerzeichen
      ausgabe << endl;                      // beende Zeile  
    }
    // Wurzelbaeume der Hoehe kleiner ??
    for(i=0;i<=n/2;i++)                     // durchlaufe die moeglichen Hoehen
    {
      for(j=0;j<=n;j++)                     // durchlaufe die moeglichen Anzahlen an Knoten  
        ausgabe << Wurzelbaeume_2[i][j] << " "; // schreibe Anzahl getrennt durch Leerzeichen
      ausgabe << endl;                      // beende Zeile 
    }
    // Alkane
    for(i=0;i<=n;i++)                       // durchlaufe moegliche Laengen der laensten Kette
    {
      for(j=0;j<=n;j++)                     // durchlaufe moegliche Anzahlen an Knoten 
        ausgabe << Alkane[i][j] << " ";     // schreibe Anzahl getrennt durch Leerzeichen
      ausgabe << endl;                      // beende Zeile
    }  
    ausgabe.close();                        // schliesse Datei 
  }   
 
  // Testausgabe
  Gesamt.resize(n+1);                         // Reserviere Speicherplatz
  for(i=0;i<=n;i++)                           // durchlaufe die moeglichen Anzahlen an C-Atomen
  {
    Gesamt[i]=0;
    for(j=0;j<=i;j++)                         // durchlaufe die moeglichen Laengen der laengsten Kette
      Gesamt[i]=Gesamt[i]+Alkane[j][i]; 
  }

  // kurze Tabelle ueber die Anzahl der Alkane mit n C-Atomen
  cout << "Anzahl der Alkane mit n C-Atomen" << endl;
  cout << "================================" << endl;
  for(i=0;i<=n;i++)
  {
    cout << setw(5) << i << "        |      " << setw(35) << setprecision(35) << Gesamt[i] << endl;
  }
  cout << endl;


  int a;            // Anzahl an C-Atomen des zufaelligen Alkans
  cln::cl_I r;      // Rang eines Alkans
  while(true)
  {
    cout << "Zufaelliges Erzeugen von Isomeren der Alkane" << endl;
    cout << "===========================================" << endl;
    cout << endl;
    cout << "Bitte gebe die Anzahl der C-Atome ein, die das Alkan besitzen soll (<=";
    cout << n << "). Zum Beenden gebe -1 ein: ";
    cin  >> a;   
    if(a<0) break;
    if(a>(int)n)
    {
      cout << "Du musst eine Zahl kleiner gleich " << n << " eingeben, oder das Programm ";
      cout << "erneut mit einem groesseren Wert von n starten." << endl;
      char taste;
      cin  >> taste;
      cout << endl;   
    }
    else
    {
      cout << "Bitte gebe eine Rang zwischen 0 und " << Gesamt[a]-1 << " ein. Fuer eine ";
      cout << "Zufallszahl gebe -1 ein. : ";
      cin  >> r;
      cout << endl;
      if(r==-1)
      {
        r=random_I(Gesamt[a]);  
        cout << "unrank_alkane(" << r << ")" << endl;
        cout << endl;
      }    
      if(r>Gesamt[a]-1 || r<0)
      {
        cout << "Du musst fuer den Rang eine Zahl zwischen 0 und " << Gesamt[a]-1 << " eingeben." << endl;
        char taste;
        cin  >> taste;
        cout << endl;
      }
      else
      {
        Alkan Alk;
        unrank_alkane(r,Alk,a,Wurzelbaeume,Wurzelbaeume_2,Alkane);
        ausgabe(Alk); 
      }
    }
  }

  return 0;
}

// ****************************************************************************
// *              Subroutinen zur Ausgabe eines Alkans                        *
// ****************************************************************************

void umnummerieren(vector<unsigned char>& b,vector<unsigned char>& wurzel_baum); // Nummeriert einen Wurzelbaum um
// Platziere ein Alkan aus 2 Wurzelbaeumen in "Ascii-Art"
void ascii_platzierung_alkan_aus_2_wurzelbaeumen(ascii_platzierung& p1,ascii_platzierung& p2,ascii_platzierung& p); 
// Platziere ein Alkan aus 4 Wurzelbaeumen in "Ascii-Art"
void ascii_platzierung_alkan_aus_4_wurzelbaeumen(ascii_platzierung& p1,ascii_platzierung& p2,
     ascii_platzierung& p3,ascii_platzierung& p4,ascii_platzierung& p);
// Platziere einen Wurzelbaum in "Ascii-Art"
void bestimme_ascii_platzierung(vector<unsigned char>& b,ascii_platzierung& p);
inline void swap(unsigned int& a,unsigned int& b);                               // Vertauscht zwei Werte
// Fuegt Teilplatzierungen eines Wurzelbaums zu einer groesseren Teilplatzierung zusammen
void bestimme_platzierung(const int& i,vector<unsigned int>& sohn,vector<ascii_platzierung>& platzierungen);
// konvertiert ein Alkan, so dass es auf dem Bildschirm ausgegeben werden kann
int konvertiere_alkan_fuer_ausgabe(Alkan& Alk);


int konvertiere_alkan_fuer_ausgabe(Alkan& Alk)
{
  // Alk: ein Alkan
  //
  // Funktion
  // --------
  // Die Funktion ausgabe(Alk) setzt vorraus, das Alk.Wurzelbaum_3 und Alk.Wurzelbaum_4 keine leeren 
  // Wurzelbaeume sind. Diese Funktion kopiert u.U. Wurzelbaeume um,so dass diese Bedingung erfuellt ist.

  if(Alk.laengste_gerade_Kette_ist_gerade==true) return 0;     // Alkan braucht nicht modifiziert werden

  vector<unsigned char> tmp;   // temporaeres Array zum Zwischenspeichern eines Wurzelbaums
  unsigned int i;              // Schleifenvariable

  if(Alk.Wurzelbaum_3.size()>0 && Alk.Wurzelbaum_4.size()>0)
    return 0;                                                  // Es ist bereits alles in Ordnung
  if(Alk.Wurzelbaum_3.size()==0 && Alk.Wurzelbaum_4.size()==0) // beide Wurzelbaeume muessen umkopiert werden
  {
    if(Alk.Wurzelbaum_1.size()>0 && Alk.Wurzelbaum_2.size()>0)
    {
      // Vertausche Wurzelbaum 1 und Wurzelbaum 3
      tmp.resize(Alk.Wurzelbaum_1.size());
      for(i=0;i<Alk.Wurzelbaum_1.size();i++)
        tmp[i]=Alk.Wurzelbaum_1[i];
      Alk.Wurzelbaum_1.resize(Alk.Wurzelbaum_3.size());
      for(i=0;i<Alk.Wurzelbaum_1.size();i++)
        Alk.Wurzelbaum_1[i]=Alk.Wurzelbaum_3[i];
      Alk.Wurzelbaum_3.resize(tmp.size());
      for(i=0;i<Alk.Wurzelbaum_3.size();i++)
        Alk.Wurzelbaum_3[i]=tmp[i];
      // Vertausche Wurzelbaum 2 und Wurzelbaum 4
      tmp.resize(Alk.Wurzelbaum_2.size());
      for(i=0;i<Alk.Wurzelbaum_2.size();i++)
        tmp[i]=Alk.Wurzelbaum_2[i];
      Alk.Wurzelbaum_2.resize(Alk.Wurzelbaum_4.size());
      for(i=0;i<Alk.Wurzelbaum_2.size();i++)
        Alk.Wurzelbaum_2[i]=Alk.Wurzelbaum_4[i];
      Alk.Wurzelbaum_4.resize(tmp.size());
      for(i=0;i<Alk.Wurzelbaum_4.size();i++)
        Alk.Wurzelbaum_4[i]=tmp[i];
      return 0;   // die Modifikation des Alkans war erfolgreich      
    }
    else          // Es gibt keine 2 nicht leeren Wurzelbaeume im Alkan
    {
      return -1;  // Modifikation des Alkans ist nicht moeglich
    }
  }
  if(Alk.Wurzelbaum_3.size()==0)
  {
    if(Alk.Wurzelbaum_1.size()>0)
    {
      // Vertausche Wurzelbaum 1 mit Wurzelbaum 3
      tmp.resize(Alk.Wurzelbaum_1.size());
      for(i=0;i<Alk.Wurzelbaum_1.size();i++)
        tmp[i]=Alk.Wurzelbaum_1[i];
      Alk.Wurzelbaum_1.resize(Alk.Wurzelbaum_3.size());
      for(i=0;i<Alk.Wurzelbaum_1.size();i++)
        Alk.Wurzelbaum_1[i]=Alk.Wurzelbaum_3[i];
      Alk.Wurzelbaum_3.resize(tmp.size());
      for(i=0;i<Alk.Wurzelbaum_3.size();i++)
        Alk.Wurzelbaum_3[i]=tmp[i];
      return 0;   // die Modifikation des Alkans war erfolgreich
    }
    if(Alk.Wurzelbaum_2.size()>0)
    {
      // Vertausche Wurzelbaum 2 mit Wurzelbaum 3
      tmp.resize(Alk.Wurzelbaum_2.size());
      for(i=0;i<Alk.Wurzelbaum_2.size();i++)
        tmp[i]=Alk.Wurzelbaum_2[i];
      Alk.Wurzelbaum_2.resize(Alk.Wurzelbaum_3.size());
      for(i=0;i<Alk.Wurzelbaum_2.size();i++)
        Alk.Wurzelbaum_2[i]=Alk.Wurzelbaum_3[i];
      Alk.Wurzelbaum_3.resize(tmp.size());
      for(i=0;i<Alk.Wurzelbaum_3.size();i++)
        Alk.Wurzelbaum_3[i]=tmp[i]; 
      return 0;   // die Modifikation des Alkans war erfolgreich  
    }
    return -1;    // die Modifikation des Alkans ist nicht moeglich 
  }
  if(Alk.Wurzelbaum_4.size()==0)
  {
    if(Alk.Wurzelbaum_1.size()>0)
    {
      // Vertausche Wurzelbaum 1 mit Wurzelbaum 4
      tmp.resize(Alk.Wurzelbaum_1.size());
      for(i=0;i<Alk.Wurzelbaum_1.size();i++)
        tmp[i]=Alk.Wurzelbaum_1[i];
      Alk.Wurzelbaum_1.resize(Alk.Wurzelbaum_4.size());
      for(i=0;i<Alk.Wurzelbaum_1.size();i++)
        Alk.Wurzelbaum_1[i]=Alk.Wurzelbaum_4[i];
      Alk.Wurzelbaum_4.resize(tmp.size());
      for(i=0;i<Alk.Wurzelbaum_4.size();i++)
        Alk.Wurzelbaum_4[i]=tmp[i];
      return 0;   // die Modifikation des Alkans war erfolgreich
    }
    if(Alk.Wurzelbaum_2.size()>0)
    {
      // Vertausche Wurzelbaum 2 mit Wurzelbaum 4
      tmp.resize(Alk.Wurzelbaum_2.size());
      for(i=0;i<Alk.Wurzelbaum_2.size();i++)
        tmp[i]=Alk.Wurzelbaum_2[i];
      Alk.Wurzelbaum_2.resize(Alk.Wurzelbaum_4.size());
      for(i=0;i<Alk.Wurzelbaum_2.size();i++)
        Alk.Wurzelbaum_2[i]=Alk.Wurzelbaum_4[i];
      Alk.Wurzelbaum_4.resize(tmp.size());
      for(i=0;i<Alk.Wurzelbaum_4.size();i++)
        Alk.Wurzelbaum_4[i]=tmp[i]; 
      return 0;   // die Modifikation des Alkans war erfolgreich  
    }
    return -1;    // die Modifikation des Alkans ist nicht moeglich 
  }
  return -1;      // dieser Fall kann sowieso nicht vorkommen
}

void umnummerieren(vector<unsigned char>& b,vector<unsigned char>& wurzel_baum)
{
  // b:            Wurzelbaum
  // wurzel_baum:  Adresse fuer einen Wurzelbaum
  //
  // Funktion:
  // ---------
  // Der Wurzelbaum b soll umnummeriert und in wurzel_baum gespeichert werden. Die
  // Umnummerierung soll folgende Bedingungn erfuellen:
  // (i)   die Wurzel bekommt die Nummer 0
  // (ii)  bei zwei Knoten liegt derjenige mit der kleineren Nummer entweder in der
  //       selben oder einer niedrigeren Ebene
  // (iii) alle Knoten, die einen Knoten v als Vater haben besitzen die Nummerierungen
  //       n,n+1,...,n+i, folgen also direkt aufeinander 

  unsigned int i,j,k;                      // Schleifenvariablen
  unsigned int n;                          // aktuell zu vergebene Nummer
  unsigned int anzahl_ebenen;              // Anzahl der Ebenen 
  vector<unsigned char> ebene;             // Vektor fuer die Ebene der Knoten 
  vector<unsigned char> perm;              // Vektor zur Speicherung der Umnummerierung
  vector<unsigned char> inv_perm;          // Vektor zur Speicherung der Inverse der Umnummerierung      
  wurzel_baum.resize(b.size());            // der umnummerierte Wurzelbaum enthaelt natuerlich genauso viele 
                                           // Knoten wie der alte Wurzelbaum
  ebene.resize(b.size());                  // Jeder Knoten besitzt eine Ebene
  perm.resize(b.size()); 
  inv_perm.resize(b.size());
  for(i=0;i<ebene.size();i++)
    ebene[i]=0;
  for(i=0;i<b.size();i++)                  // Bestimme die Ebene der Knoten
    for(j=0;j<b.size();j++)
      if(b[j]!=j)
        ebene[j]=ebene[b[j]]+1;
   anzahl_ebenen=0;
   
  for(i=0;i<ebene.size();i++)              // Bestimme die Anzahl der Ebenen 
    if(ebene[i]>anzahl_ebenen)
      anzahl_ebenen=ebene[i];   
  if(b.size()>0) anzahl_ebenen++;

  for(i=0;i<ebene.size();i++)               // gebe der Wurzel die Nummer 0
    if(ebene[i]==0) 
    {
      perm[i]=0;
      break;
    }

  n=1;                                      // aktuell noch nicht veregebene Knotennummer  
  for(i=0;i+1<anzahl_ebenen;i++)            // durchlaufe die Ebenen von unten nach oben         
    for(j=0;j<ebene.size();j++)             // durchlaufe alle Knoten
    {
      if(ebene[j]==i)                       // Falls der Knoten in der gerade betrachteten Ebene liegt
        for(k=0;k<b.size();k++)             // durchlaufe alle Knoten
          if(b[k]==j && k!=j)               // falls k ein Sohn von j ist 
          {
            perm[k]=n;                      // der Knoten k heisst jetzt n
            n++;                            // der Name n wurde eben vergeben, ist also nicht mehr frei
          } 
    }
  for(i=0;i<perm.size();i++)               // Bestimme die inverse Permutation
    inv_perm[perm[i]]=i;

  for(i=0;i<wurzel_baum.size();i++)        // nummeriere den Wurzelbaum tatsaechlich um
    wurzel_baum[i]=perm[b[inv_perm[i]]];
} 

int ausgabe_ascii(Alkan& A)
{
  // A: ein Alkan
  //
  // Funktion:
  // ---------
  // Gibt eine zweidimensionale Platzierung des Alkanmolekuels A in "Ascii-Art" 
  // auf dem Bildschirm aus, sofern es nicht zu breit ist (-> wird -1 zurueckgegeben 

  ascii_platzierung p1;                    // Speicherplatz fuer eine Platzierung des ersten Wurzelbaums in "Ascii-Art"
  ascii_platzierung p2;                    // Speicherplatz fuer eine Platzierung des zweiten Wurzelbaums in "Ascii-Art"
  ascii_platzierung p3;                    // Speicherplatz fuer eine Platzierung des dritten Wurzelbaums in "Ascii-Art"
  ascii_platzierung p4;                    // Speicherplatz fuer eine Platzierung des vierten Wurzelbaums in "Ascii-Art" 
  ascii_platzierung p;                     // Speicherplatz fuer eine Platzierung des gesamten Alkans in "Ascii-Art"  

  vector<unsigned char> wurzel_baum;       // Speicherplatz fuer einen umnummerierten Wurzelbaum
  unsigned int i,j;                        // Schleifenvariablen

  konvertiere_alkan_fuer_ausgabe(A);              // modifiziere evtl. die Anordnung der Wurzelbaeume 
  
  umnummerieren(A.Wurzelbaum_1,wurzel_baum);      // nummeriere den ersten Wurzelbaum um
  bestimme_ascii_platzierung(wurzel_baum,p1);     // platziere den ersten Wurzelbaum in "Ascii-Art" 
  umnummerieren(A.Wurzelbaum_2,wurzel_baum);      // nummeriere den zweiten Wurzelbaum um
  bestimme_ascii_platzierung(wurzel_baum,p2);     // platziere den zweiten Wurzelbaum in "Ascii-Art" 
  if(A.laengste_gerade_Kette_ist_gerade==false)   // falls das Alkan aus 4 Wurzelbaeumen besteht
  {
    umnummerieren(A.Wurzelbaum_3,wurzel_baum);    // nummeriere den dritten Wurzelbaum um
    bestimme_ascii_platzierung(wurzel_baum,p3);   // platziere den dritten Wurzelbaum in "Ascii-Art" 
    umnummerieren(A.Wurzelbaum_4,wurzel_baum);    // nummeriere den vierten Wurzelbaum um  
    bestimme_ascii_platzierung(wurzel_baum,p4);   // platziere den vierten Wurzelbaum in "Ascii-Art"
  }
  
  if(A.laengste_gerade_Kette_ist_gerade==true)    // das Alkan besteht aus 2 Wurzelbaeumen
  {
    ascii_platzierung_alkan_aus_2_wurzelbaeumen(p1,p2,p);  // platziere das gesamte Alkan
    if(p.anzahl_spalten>BILDSCHIRMBREITE)         // Zu breit fuer eine Ausgabe auf dem Bildschirm
    {
      cout << "Das Alkan ist leider zu breit fuer eine Ausgabe auf dem Bildschirm. (";
      cout << p.anzahl_spalten << ")" << endl << endl;
      return -1;
    } 
    for(j=0;j<p.anzahl_zeilen;j++)                // durchlaufe die Zeilen der Platzierung 
    {
      for(i=0;i<p.anzahl_spalten;i++)             // durchlaufe die Spalten der Platzierung
        cout << p.Matrix[i][j];                   // gebe das berechnete Symbol auf dem Bildschirm aus 
      cout << endl;
    }
    cout << endl; 
  }
  else                                            // das Alkan besteht aus 4 Wurzelbaeumen
  {
    ascii_platzierung_alkan_aus_4_wurzelbaeumen(p1,p2,p3,p4,p); // platziere das gesamte Alkan
    if(p.anzahl_spalten>BILDSCHIRMBREITE)         // Zu breit fuer eine Ausgabe auf dem Bildschirm
    {
      cout << "Das Alkan ist leider zu breit fuer eine Ausgabe auf dem Bildschirm. (";
      cout << p.anzahl_spalten << ")" << endl << endl;
      return -1;
    } 
    for(j=0;j<p.anzahl_zeilen;j++)                // durchlaufe die Zeilen der Platzierung
    {
      for(i=0;i<p.anzahl_spalten;i++)             // durchlaufe die Spalten der Platzierung
        cout << p.Matrix[i][j];                   // gebe das berechnete Symbol auf dem Bildschirm aus 
      cout << endl;
    }
    cout << endl;
  }

  //char taste;
  //cout << "Bitte beliebige Taste druecken: ";
  //cin >> taste;
  
  return 0;
}

void ascii_platzierung_alkan_aus_2_wurzelbaeumen(ascii_platzierung& p1,ascii_platzierung& p2,ascii_platzierung& p)
{
  // p1: Platzierung eines Wurzelbaums in "Ascii-Art"
  // p2: Platzierung eines Wurzelbaums in "Ascii-Art"
  // p:  Speicherplatz fuer Platzierung eines Alkans in "Ascii-Art"
  //
  // Funktion:
  // ---------
  // Kombiniert die zweidimensionalen Platzierungen p1 und p2 zweier Wurzelbaeume zu einer 
  // Platzierung des Alkans aus diesen beiden Wurzelbaeumen in "Ascii-Art"
  
  unsigned int i,j;   // Schleifenvariablen
  unsigned int m;     // Position der neuen Wurzel

  m=max(p1.position_wurzel,p2.position_wurzel);

  p.anzahl_zeilen=p1.anzahl_zeilen+1+p2.anzahl_zeilen; // das Alkan entsteht durch das aneinander setzen 
                                                       // dieser 2 Wurzelbaeume
  p.anzahl_spalten=m+1;                                // benoetigter Platz links des Zentrums + Platz fuer
                                                       // das Zentrum 
  p.anzahl_spalten+=max(p1.anzahl_spalten-1-p1.position_wurzel,p2.anzahl_spalten-1-p2.position_wurzel); // Platz rechts 
                                                       // des Zentrums
  p.position_wurzel=max(p1.position_wurzel,p2.position_wurzel); // Position der neuen "Wurzel"
  p.Matrix.resize(p.anzahl_spalten);                   // Reserviere Speicherplatz
  for(i=0;i<p.anzahl_spalten;i++)                      // durchlaufe die Spalten der Platzierung  
  {
    p.Matrix[i].resize(p.anzahl_zeilen);               // Reserviere Speicherplatz
    for(j=0;j<p.anzahl_zeilen;j++)                     // durchlaufe die Zeilen der Platzierung  
      p.Matrix[i][j]=' ';                              // beginne zunaechst mit einem leeren Bildschirm
  }
  for(i=p1.anzahl_zeilen;i>0;i--)                      // durchlaufe die Zeilen des ersten Wurzelbaums
    for(j=0;j<p1.anzahl_spalten;j++)                   // durchlaufe die Spalten des ersten Wurzelbaums  
      p.Matrix[j+m-p1.position_wurzel][p1.anzahl_zeilen-i]=p1.Matrix[j][i-1]; // uebernehme die Teilplatzierung
  p.Matrix[p.position_wurzel][p1.anzahl_zeilen]='|';   // gebe Symbol fuer das Zentrum aus
  for(i=0;i<p2.anzahl_zeilen;i++)                      // durchlaufe die Zeilen des zweiten Wurzelbaums
    for(j=0;j<p2.anzahl_spalten;j++)                   // durchlaufe die Spalten des zweiten Wurzelbaums
      switch(p2.Matrix[j][i])                          // uebersetze die Symbole der Teilplatzierung  
      {
        case (char)92: p.Matrix[j+m-p2.position_wurzel][p1.anzahl_zeilen+1+i]='/';      // Orientierung aendert sich 
                       break;
        case '/':      p.Matrix[j+m-p2.position_wurzel][p1.anzahl_zeilen+1+i]=(char)92; // Orientierung aendert sich
                       break; 
        default:       p.Matrix[j+m-p2.position_wurzel][p1.anzahl_zeilen+1+i]=p2.Matrix[j][i];
      }
}

void ascii_platzierung_alkan_aus_4_wurzelbaeumen(ascii_platzierung& p1,ascii_platzierung& p2,
     ascii_platzierung& p3,ascii_platzierung& p4,ascii_platzierung& p)
{
  // p1: Platzierung eines Wurzelbaums in "Ascii-Art"
  // p2: Platzierung eines Wurzelbaums in "Ascii-Art"
  // p3: Platzierung eines Wurzelbaums in "Ascii-Art"
  // p4: Platzierung eines Wurzelbaums in "Ascii-Art"
  // p:  Speicherplatz fuer Platzierung eines Alkans in "Ascii-Art"
  //
  // Funktion:
  // ---------
  // Kombiniert die zweidimensionalen Platzierungen p1, p2, p3 und p4 von Wurzelbaeume zu einer 
  // Platzierung des Alkans aus diesen Wurzelbaeumen in "Ascii-Art"
  //
  //        p3
  //         |
  //      p1-o-p2
  //         |
  //        p4

  unsigned int i,j;                    // Schleifenvariablen
  unsigned int w1,w2,w3,w4;            // Abkuerzungen fuer die Positionen der 4 Wurzeln
  unsigned int s1,s2,s3,s4;            // Abkuerzungen fuer die 4 Anzahlen der Zeilen 
  unsigned int z1,z2,z3,z4;            // Abkuerzungen fuer die 4 Anzahlen der Zeilen  
  w1=p1.position_wurzel;               // Position der Wurzel der Platzierung des ersten Wurzelbaums 
  w2=p2.position_wurzel;               // Position der Wurzel der Platzierung des zweiten Wurzelbaums
  w3=p3.position_wurzel;               // Position der Wurzel der Platzierung des dritten Wurzelbaums
  w4=p4.position_wurzel;               // Position der Wurzel der Platzierung des vierten Wurzelbaums
  s1=p1.anzahl_spalten;                // Anzahl der Spalten der Platzierung des ersten Wurzelbaums
  s2=p2.anzahl_spalten;                // Anzahl der Spalten der Platzierung des zweiten Wurzelbaums 
  s3=p3.anzahl_spalten;                // Anzahl der Spalten der Platzierung des dritten Wurzelbaums
  s4=p4.anzahl_spalten;                // Anzahl der Spalten der Platzierung des vierten Wurzelbaums
  z1=p1.anzahl_zeilen;                 // Anzahl der Zeilen der Platzierung des ersten Wurzelbaums
  z2=p2.anzahl_zeilen;                 // Anzahl der Zeilen der Platzierung des zweiten Wurzelbaums
  z3=p3.anzahl_zeilen;                 // Anzahl der Zeilen der Platzierung des dritten Wurzelbaums
  z4=p4.anzahl_zeilen;                 // Anzahl der Zeilen der Platzierung des vierten Wurzelbaums

  unsigned int links,rechts,oben,unten;         // Platzbedarf links,rechts,oberhalb und unterhalb des zusaetzlichen Knotens

  oben=z3+1;                                    // Die Platzierung des dritten Wurzelbaums muss ueber 
                                                // den zusaetzlichen Knoten passen, inklusive "|"
  if(s1-w1-1>oben && s1+w1>0) oben=s1-w1-1;     // ein Teil der Platzierung des ersten Wurzelbaums reicht nach oben
  if(s2-w2-1>oben && s2+w2>0) oben=s2-w2-1;     // ein Teil der Platzierung des zweiten Wurzelbaums reicht nach oben
  unten=z4+1;                                   // Die Platzierung des dritten Wurzelbaums muss ueber 
                                                // den zusaetzlichen Knoten passen, inklusive "|" 
  if(w1>unten) unten=w1;                        // ein Teil der Platzierung des ersten Wurzelbaums reicht nach unten
  if(w2>unten) unten=w2;                        // ein Teil der Platzierung des zweiten Wurzelbaums reicht nach unten
  links=0;
  if(z1>0) links=z1+1;                          // falls der ersten Wurzelbaum aus mindestens einem Knoten besteht, 
                                                // benoetigt er Platz + "-"
  if(w3>links) links=w3;                        // ein Teil der Platzierung des dritten Wurzelbaums reicht nach links
  if(w4>links) links=w4;                        // ein Teil der Platzierung des vierten Wurzelbaums reicht nach links 
  rechts=0;
  if(z2>0) rechts=z2+1;                         // falls der zweite Wurzelbaum aus mindestens einem Knoten besteht, 
                                                // benoetigt er Platz + "-" 
  if(s3-w3-1>rechts) rechts=s3-w3-1;            // ein Teil der Platzierung des dritten Wurzelbaums reicht nach rechts
  if(s4-w4-1>rechts) rechts=s4-w4-1;            // ein Teil der Platzierung des vierten Wurzelbaums reicht nach rechts

  p.anzahl_zeilen=unten+1+oben;                 // Teil oberhalb des zusaetzlichen Knoten + zusaetzlicher Knoten + Teil 
                                                // unterhalb des zusaetzlichen Knotens  
  p.anzahl_spalten=links+1+rechts;              // Teil links vom zusaetzlichen Knoten + zusaetzlicher Knoten + Teil   
                                                // rechts vom zusaetzlichen Knoten
  p.position_wurzel=links;

  p.Matrix.resize(p.anzahl_spalten);            // Reserviere Speicherplatz 
  for(i=0;i<p.anzahl_spalten;i++)               // durchlaufe die Spalten der Platzierung   
  { 
    p.Matrix[i].resize(p.anzahl_zeilen);        // Reserviere Speicherplatz  
    for(j=0;j<p.anzahl_zeilen;j++)              // durchlaufe die Zeilen der Platzierung  
      p.Matrix[i][j]=' ';                       // starte mit einem leeren Bildschirm
  }
  for(i=z3;i>0;i--)                             // durchlaufe die Zeilen der Platzierung des dritten Wurzelbaums
    for(j=0;j<s3;j++)                           // durchlaufe die Spalten der Platzierung des dritten Wurzelbaums 
      p.Matrix[links-w3+j][(oben-z3-1)+(z3-i)]=p3.Matrix[j][i-1];
  p.Matrix[p.position_wurzel][oben-1]='|';
  p.Matrix[p.position_wurzel][oben]='o';
  p.Matrix[p.position_wurzel][oben+1]='|';
  for(i=0;i<z4;i++)                             // durchlaufe die Zeilen der Platzierung des vierten Wurzelbaums
    for(j=0;j<s4;j++)                           // durchlaufe die Spalten der Platzierung des vierten Wurzelbaums  
      switch(p4.Matrix[j][i])                   // uebersetze die Symbole der Teilplatzierung
      {
        case '|':      p.Matrix[links-w4+j][oben+2+i]='|';       // immer noch hochkant
                       break;
        case '/':      p.Matrix[links-w4+j][oben+2+i]=(char)92;  // Orientierung aendert sich
                       break; 
        case (char)92: p.Matrix[links-w4+j][oben+2+i]='/';       // Orientierung aendert sich 
                       break; 
        case 'o':      p.Matrix[links-w4+j][oben+2+i]='o';       // ein Knoten bleibt ein Knoten
                       break;
      } 
  if(p1.anzahl_zeilen>0)
  {
    p.Matrix[p.position_wurzel-1][oben]='-';
    for(i=0;i<z1;i++)                           // durchlaufe die Zeilen der Platzierung des ersten Wurzelbaums
      for(j=0;j<s1;j++)                         // durchlaufe die Spalten der Platzierung des ersten Wurzelbaums
        switch(p1.Matrix[j][i])                 // uebersetze die Symbole der Teilplatzierung
        {
          case '|':      p.Matrix[(links-z1-1)+(z1-i-1)][oben+w1+(-j)]='-';       // Orientierung aendert sich
                         break;
          case 'o':      p.Matrix[(links-z1-1)+(z1-i-1)][oben+w1+(-j)]='o';       // ein Knoten bleibt ein Knoten  
                         break;
          case '/':      p.Matrix[(links-z1-1)+(z1-i-1)][oben+w1+(-j)]=(char)92;  // Orientierung aendert sich
                         break;
          case (char)92: p.Matrix[(links-z1-1)+(z1-i-1)][oben+w1+(-j)]='/';       // Orientierung aendert sich
                         break;  
        } 
  }
  if(p2.anzahl_zeilen>0)
  {
    p.Matrix[p.position_wurzel+1][oben]='-';
    for(i=0;i<z2;i++)                           // durchlaufe die Zeilen der Platzierung des zweiten Wurzelbaums
      for(j=0;j<s2;j++)                         // durchlaufe die Spalten der Platzierung des zweiten Wurzelbaums
        switch(p2.Matrix[j][i])                 // uebersetze die Symbole der Teilplatzierung 
        {
          case '|':      p.Matrix[p.position_wurzel+2+i][oben+w2+(-j)]='-';         // Orientierung aendert sich
                         break;
          case 'o':      p.Matrix[p.position_wurzel+2+i][oben+w2+(-j)]='o';         // ein Knoten bleibt ein Knoten
                         break;
          case '/':      p.Matrix[p.position_wurzel+2+i][oben+w2+(-j)]='/';
                         break;
          case (char)92: p.Matrix[p.position_wurzel+2+i][oben+w2+(-j)]=(char)92;
                         break;  
        }
  }
     
}

void bestimme_ascii_platzierung(vector<unsigned char>& b,ascii_platzierung& p)
{
  // b: ein Wurzelbaum, der bereits mit der Funktion "umnummerieren" bearbeitet wurde
  // p: Speicherplatz fuer eine Platzierung eines Wurzelbaums in "Ascii-Art"
  //
  // Funktion:
  // ---------
  // Platziert einen Wurzelbaum in "Ascii-Art" und speichert das Ergebnis in p.

  unsigned int i,j;                        // Schleifenvariablen
  unsigned int pos;                        // Position des zu bearbeitenden Knotens
  vector<unsigned char> ebene;             // Vektor fuer die Ebene der Knoten 
  ebene.resize(b.size());                  // Jeder Knoten besitzt eine Ebene
  vector<ascii_platzierung> platzierungen; // eine Liste von einzelnen Platzierungen, man kann jeden 
  platzierungen.resize(b.size());          // Knoten einzeln platzieren und die einzelnen Platzierungen
                                           // dann Stueck fuer Stueck zu groesseren Platzierungen zusammenfassen
  vector<unsigned int> sohn;               // Liste von Soehnen eines Knotens

  for(i=0;i<ebene.size();i++)
    ebene[i]=0;
  for(i=0;i<b.size();i++)                  // Bestimme die Ebene der Knoten
    for(j=0;j<b.size();j++)
      if(b[j]!=j)
        ebene[j]=ebene[b[j]]+1;

  pos=0;
  for(i=b.size();i>0;i--)                    // durchlaufe die Knoten der obersten Ebene
    if(ebene[i-1]==ebene[b.size()-1])        // falls der Knoten noch zur obersten Ebene gehoert
    {
      platzierungen[i-1].anzahl_zeilen=1;    // belege Matrix der Platzierung mit dem Symbol 
      platzierungen[i-1].anzahl_spalten=1;   // fuer einen Knoten: "o"
      platzierungen[i-1].position_wurzel=0;
      platzierungen[i-1].Matrix.resize(1);
      platzierungen[i-1].Matrix[0].resize(1);
      platzierungen[i-1].Matrix[0][0]='o';
    } 
    else
    {
      pos=i-1;                             // merke den Start der neuen Ebene  
      break;                               // oberste Ebene ist durchlaufen
    }
  
  if(b.size()>0) pos++;
  if(b.size()==1) pos=0;
  while(pos>0)                             // solange es noch Knoten zu bearbeiten gibt
  {
    for(i=pos;i>0;i--)                     // durchlaufe die Knoten der aktuell zu bearbeitenden Ebene
    { 
      if(ebene[i-1]==ebene[pos-1])         // falls der Knoten noch zu dieser Ebene gehoert
      {
        sohn.resize(0);                    // bestimme die Liste der Soehne des Knoten i
        for(j=pos;j<b.size();j++)
        {
          if(ebene[j]>ebene[pos])          // es koennen keine Soehne mehr kommen, da die Knoten der Ebene  
          {                                // nach ansteigend sortiert sind
            break;
          }
          else
          {
           if((unsigned int)b[j]==i-1)
            {
              sohn.resize(sohn.size()+1);  // es gibt einen Sohn mehr
              sohn[sohn.size()-1]=j;       // speichere die Nummer des Sohns 
            }               
          }
        }
        bestimme_platzierung(i-1,sohn,platzierungen);  // Fuege die Platzierungen der Soehne zu einer Platzierung vom
                                                       // Knoten i-1 zusammen
        if(i==1) pos=0;                                // Abbruchbedingung
      }
      else
      {
        pos=i;                             // merke den Start der neuen Ebene    
        break;                             // aktuelle Ebene ist bereits fertig durchlaufen  
      }   
    }
  }
  
  if(b.size()>0)                           // kopiere die Platzierung der Wurzel
  {
    p.anzahl_zeilen=platzierungen[0].anzahl_zeilen;
    p.anzahl_spalten=platzierungen[0].anzahl_spalten;
    p.position_wurzel=platzierungen[0].position_wurzel;
    p.Matrix.resize(p.anzahl_spalten);
    for(i=0;i<p.anzahl_spalten;i++)
    {
      p.Matrix[i].resize(p.anzahl_zeilen);
      for(j=0;j<p.anzahl_zeilen;j++)
        p.Matrix[i][j]=platzierungen[0].Matrix[i][j];
    } 
  }
  else                      // leere Platzierung
  {
    p.anzahl_zeilen=0;
    p.anzahl_spalten=0;
    p.position_wurzel=0;
    p.Matrix.resize(0);
  }
 
}

inline void swap(unsigned int& a,unsigned int& b)
{
  // Vertauscht zwei Werte
  unsigned int c=a;
  a=b;
  b=c;
}

void bestimme_platzierung(const int& i,vector<unsigned int>& sohn,vector<ascii_platzierung>& platzierungen)
{
  // i:             Nummer eines Knotens
  // sohn:          Liste der Nummern der Soehne von i
  // platzierungen: Liste von Platzierungen der Wurzelbaeume oberhalb eines Knotens
  //
  // Funktion:
  // ---------
  // Es wird die Platzierung des Wurzelbaums oberhalb eines Knotens i bestimmt. Hierzu werden die 
  // bereits vorhanden derartigen Platzierungen der Soehne von i benutzt.

  unsigned int h,j;                  // Schleifenvariablen
  unsigned int links,mitte,rechts;   // Welcher der 3 Teilwurzelbaeume kommt in die Mitte, nach links oder nach rechts?     
  unsigned int s1,s2,s3;             // Abkuerzung fuer die Anzahl der Spalten der Platzierung der 3 Teilwurzelbaeume 
  unsigned int w1,w2,w3;             // Abkuerzung fuer die Position der Wurzel der Platzierung der 3 Teilwurzelbaeume  
  unsigned int z1,z2,z3;             // Abkuerzung fuer die Anzahl der Zeilen der Platzierung der 3 Teilwurzelbaeume 
  unsigned int a;

  switch(sohn.size())                // Wieviele Teilwurzelbaeume muessen haengen amKnoten i
  {
    // Es haengt kein Teilwurzelbaum am Knoten i, also platziere nur den Knoten i
    case 0: platzierungen[i].anzahl_zeilen=1;    // belege Matrix der Platzierung mit dem Symbol 
            platzierungen[i].anzahl_spalten=1;   // fuer einen Knoten: "o"
            platzierungen[i].position_wurzel=0;
            platzierungen[i].Matrix.resize(1);
            platzierungen[i].Matrix[0].resize(1);
            platzierungen[i].Matrix[0][0]='o';
            break;
    // Es haengt ein Teilwurzelbaum am Knoten i, also haenge die Platzierung an "o-" an
    case 1: platzierungen[i].anzahl_zeilen=platzierungen[sohn[0]].anzahl_zeilen+2;   // 2 hoeher, wegen "o-" 
            platzierungen[i].anzahl_spalten=platzierungen[sohn[0]].anzahl_spalten;   // genauso breit
            platzierungen[i].position_wurzel=platzierungen[sohn[0]].position_wurzel; // selbe Position der Wurzel
            platzierungen[i].Matrix.resize(platzierungen[i].anzahl_spalten);         // Reserviere Speicherplatz  
            for(j=0;j<platzierungen[i].anzahl_spalten;j++)                             
              platzierungen[i].Matrix[j].resize(platzierungen[i].anzahl_zeilen);     // Reserviere Speicherplatz
            for(j=0;j<platzierungen[sohn[0]].anzahl_spalten;j++)                     // Übernehme die Platzierung 
              for(h=0;h<platzierungen[sohn[0]].anzahl_zeilen;h++)                    // vomTeilwurzelbaum
              {
                platzierungen[i].Matrix[j][h+2]=platzierungen[sohn[0]].Matrix[j][h];
              }
            for(j=0;j<platzierungen[sohn[0]].anzahl_spalten;j++)                     // Zwei Leerzeilen
            {
              platzierungen[i].Matrix[j][0]=' ';
              platzierungen[i].Matrix[j][1]=' '; 
            }
            platzierungen[i].Matrix[platzierungen[sohn[0]].position_wurzel][0]='o';  // Knoten i
            platzierungen[i].Matrix[platzierungen[sohn[0]].position_wurzel][1]='|';  
            // Verbindung zur Wurzel vom Teilwurzelbaum
            break;
    // Es haengen zwei Teilwurzelbaeume am Knoten i
    case 2: s1=platzierungen[sohn[0]].anzahl_spalten;        // Anzahl der Spalten der Platzierung des ersten Wurzelbaums 
            s2=platzierungen[sohn[1]].anzahl_spalten;        // Anzahl der Spalten der Platzierung des zweiten Wurzelbaums
            z1=platzierungen[sohn[0]].anzahl_zeilen;         // Anzahl der Zeilen der Platzierung des ersten Wurzelbaums
            z2=platzierungen[sohn[1]].anzahl_zeilen;         // Anzahl der Zeilen der Platzierung des zweiten Wurzelbaums
            w1=platzierungen[sohn[0]].position_wurzel;       // Position der Wurzel der Platzierung des ersten Wurzelbaums
            w2=platzierungen[sohn[1]].position_wurzel;       // Position der Wurzel der Platzierung des zweiten Wurzelbaums

            // Bestimme die platzsparendere Variante 
            if(max(s1-w1-1,w2)<=max(s2-w2-1,w1))
            {
              links=0;
              rechts=1;
              a=max(s1-w1-1,w2); 
            }
            else
            {
              links=1;
              rechts=0;
              swap(s1,s2);
              swap(z1,z2);
              swap(w1,w2);
              a=max(s2-w2-1,w1); 
            } 
            if(a<1) a=1;
            platzierungen[i].anzahl_zeilen=max(z1,z2)+a+1;    
            platzierungen[i].anzahl_spalten=w1+1+a+1+a+(s2-w2);   
            platzierungen[i].position_wurzel=w1+1+a;
            platzierungen[i].Matrix.resize(platzierungen[i].anzahl_spalten);
            for(j=0;j<platzierungen[i].anzahl_spalten;j++)
              platzierungen[i].Matrix[j].resize(platzierungen[i].anzahl_zeilen);
            for(j=0;j<platzierungen[i].anzahl_spalten;j++)
              for(h=0;h<platzierungen[i].anzahl_zeilen;h++)
                platzierungen[i].Matrix[j][h]=' ';
            for(j=0;j<s1;j++)
              for(h=0;h<z1;h++)
                platzierungen[i].Matrix [j][h+a+1]=platzierungen[sohn[links]].Matrix[j][h];
            for(j=0;j<s2;j++)
              for(h=0;h<z2;h++)
                platzierungen[i].Matrix [j+w1+1+a+1+a-w2][h+a+1]=platzierungen[sohn[rechts]].Matrix[j][h];  
            platzierungen[i].Matrix [platzierungen[i].position_wurzel][0]='o';
            for(j=1;j<=a;j++)
            { 
              platzierungen[i].Matrix [platzierungen[i].position_wurzel-j][j]=(char)(92);
              platzierungen[i].Matrix [platzierungen[i].position_wurzel+j][j]='/';  
            }  
            break;
    // Es haengen drei Teilwurzelbaeume am Knoten i
    case 3: s1=platzierungen[sohn[0]].anzahl_spalten;  // Anzahl der Spalten der Platzierung des ersten Wurzelbaums
            s2=platzierungen[sohn[1]].anzahl_spalten;  // Anzahl der Spalten der Platzierung des zweiten Wurzelbaums
            s3=platzierungen[sohn[2]].anzahl_spalten;  // Anzahl der Spalten der Platzierung des dritten Wurzelbaums
            z1=platzierungen[sohn[0]].anzahl_zeilen;   // Anzahl der Zeilen der Platzierung des ersten Wurzelbaums
            z2=platzierungen[sohn[1]].anzahl_zeilen;   // Anzahl der Zeilen der Platzierung des zweiten Wurzelbaums     
            z3=platzierungen[sohn[2]].anzahl_zeilen;   // Anzahl der Zeilen der Platzierung des dritten Wurzelbaums   
            w1=platzierungen[sohn[0]].position_wurzel; // Position der Wurzel der Platzierung des ersten Wurzelbaums
            w2=platzierungen[sohn[1]].position_wurzel; // Position der Wurzel der Platzierung des zweiten Wurzelbaums
            w3=platzierungen[sohn[2]].position_wurzel; // Position der Wurzel der Platzierung des dritten Wurzelbaums 

            // bestimme die platzsparendere Variante der 6 verschiedenen Anordnungen der 3 Wurzelbaeume
            links=0;
            mitte=1;
            rechts=2;
            a=max((s1-w1-1)+1+w2,(s2-w2-1)+1+w3);
            if(a>max((s1-w1-1)+1+w3,(s3-w3-1)+1+w2))
            {
              links=0;
              mitte=2;
              rechts=1; 
              a=max((s1-w1-1)+1+w3,(s3-w3-1)+1+w2);
            } 
            if(a>max((s2-w2-1)+1+w1,(s1-w1-1)+1+w3))
            {
              links=1;
              mitte=0;
              rechts=2; 
              a=max((s2-w2-1)+1+w1,(s1-w1-1)+1+w3);
            }
            if(a>max((s2-w2-1)+1+w3,(s3-w3-1)+1+w1))
            {
              links=1;
              mitte=2;
              rechts=0; 
              a=max((s2-w2-1)+1+w3,(s3-w3-1)+1+w1);
            }
            if(a>max((s3-w3-1)+1+w1,(s1-w1-1)+1+w2))
            {
              links=2;
              mitte=0;
              rechts=1; 
              a=max((s3-w3-1)+1+w1,(s1-w1-1)+1+w2);
            }
            if(a>max((s3-w3-1)+1+w2,(s2-w2-1)+1+w1))
            {
              links=2;
              mitte=1;
              rechts=0; 
              a=max((s3-w3-1)+1+w2,(s2-w2-1)+1+w1);
            }  

            s1=platzierungen[sohn[links]].anzahl_spalten;
            s2=platzierungen[sohn[mitte]].anzahl_spalten;
            s3=platzierungen[sohn[rechts]].anzahl_spalten;
            z1=platzierungen[sohn[links]].anzahl_zeilen;
            z2=platzierungen[sohn[mitte]].anzahl_zeilen;       
            z3=platzierungen[sohn[rechts]].anzahl_zeilen;  
            w1=platzierungen[sohn[links]].position_wurzel;
            w2=platzierungen[sohn[mitte]].position_wurzel;
            w3=platzierungen[sohn[rechts]].position_wurzel;  

            platzierungen[i].anzahl_zeilen=max(max(z1,z2),z3)+a+1;    
            platzierungen[i].anzahl_spalten=w1+1+a+1+a+1+(s3-w3-1);   
            platzierungen[i].position_wurzel=w1+1+a;

            platzierungen[i].Matrix.resize(platzierungen[i].anzahl_spalten);
            for(j=0;j<platzierungen[i].anzahl_spalten;j++)
              platzierungen[i].Matrix[j].resize(platzierungen[i].anzahl_zeilen);
            for(j=0;j<platzierungen[i].anzahl_spalten;j++)
              for(h=0;h<platzierungen[i].anzahl_zeilen;h++)
                platzierungen[i].Matrix[j][h]=' ';   
            for(j=0;j<s1;j++)
              for(h=0;h<z1;h++)
                platzierungen[i].Matrix [j][h+a+1]=platzierungen[sohn[links]].Matrix[j][h];
            for(j=0;j<s2;j++)
              for(h=0;h<z2;h++)
                platzierungen[i].Matrix [j+w1+1+a-w2][h+a+1]=platzierungen[sohn[mitte]].Matrix[j][h];             
            for(j=0;j<s3;j++)
              for(h=0;h<z3;h++)
                platzierungen[i].Matrix [j+w1+1+a+1+a-w3][h+a+1]=platzierungen[sohn[rechts]].Matrix[j][h]; 
            platzierungen[i].Matrix [platzierungen[i].position_wurzel][0]='o';
            for(j=1;j<=a;j++)
            { 
              platzierungen[i].Matrix [platzierungen[i].position_wurzel-j][j]=(char)(92);
              platzierungen[i].Matrix [platzierungen[i].position_wurzel][j]='|';
              platzierungen[i].Matrix [platzierungen[i].position_wurzel+j][j]='/';  
            }
            break; 
  }
}

int ausgabe(Alkan& A)
{
  // Gibt ein Alkan im XML-Format in der Datei graph aus
  unsigned int anzahl_c_atome;
  if(A.laengste_gerade_Kette_ist_gerade==true)
    anzahl_c_atome=A.Wurzelbaum_1.size()+A.Wurzelbaum_2.size();
  else
    anzahl_c_atome=1+A.Wurzelbaum_1.size()+A.Wurzelbaum_2.size()+A.Wurzelbaum_3.size()+A.Wurzelbaum_4.size();

  vector<unsigned int> Ebene;     // in welcher Ebene liegt der Knoten im Wurzelbaum
  vector<unsigned int> Position;  // Teile die Knoten eines Wurzelbaums in einer Ebene in 3^Ebene Positionen auf
  vector<unsigned int> Nachfolgervon;
  Ebene.resize(anzahl_c_atome);
  Position.resize(anzahl_c_atome);
  Nachfolgervon.resize(anzahl_c_atome);
  unsigned int w1,w2,w3,w4;
  for(unsigned int i = 0; i<anzahl_c_atome; i++)
  {
    Ebene[i]=0;
    Position[i]=0;
    Nachfolgervon[i]=0;  //Anzahl der Nachfolger von Knoten i
  }  
  unsigned int knoten;
  unsigned int eb;
  unsigned int pos;
  ofstream datei("graph", ios_base::app);
  w1=A.Wurzelbaum_1.size(); 
  w2=A.Wurzelbaum_2.size();
  datei << "<GRAPH NAME=\"C" << anzahl_c_atome << "H" << 2*(anzahl_c_atome)+2 << "\" NUMBER_OF_VERTICES=\"";
  datei << 3*(anzahl_c_atome)+2 << "\" NUMBER_OF_EDGES=\"" << 3*(anzahl_c_atome)+1 << "\" MOLECULAR=\"1\">" << endl;
  datei << "<VERTEXLABELS>" << endl;
    for(unsigned i=0;i<anzahl_c_atome;i++)
      datei << "\"C\"" << endl;
    for(unsigned i=0;i<2*(anzahl_c_atome)+2;i++)
      datei << "\"H\"" << endl;
  datei << "</VERTEXLABELS>" << endl;   
  if(A.laengste_gerade_Kette_ist_gerade==true)
  {
    // in der Mitte ist eine Kante
    for(unsigned int i = 1; i<w1; i++)
    {
      Ebene[i]=Ebene[A.Wurzelbaum_1[i]]+1;
      Position[i]=3*Position[A.Wurzelbaum_1[i]]+Nachfolgervon[A.Wurzelbaum_1[i]];
      Nachfolgervon[A.Wurzelbaum_1[i]]++;
    }
    for(unsigned int i = 1; i<w2; i++)
    {
      Ebene[w1+i]=Ebene[w1+A.Wurzelbaum_2[i]]+1;
      Position[w1+i]=3*Position[w1+A.Wurzelbaum_2[i]]+Nachfolgervon[w1+A.Wurzelbaum_2[i]];
      Nachfolgervon[w1+A.Wurzelbaum_2[i]]++;
    }
    datei << "<EDGELIST>" << endl;
      if(w1+w2>0) datei << "0 " << (int) w1 << endl;
      for(unsigned int i=1; i<w1; i++)
        datei << (int)i << " " << (int)A.Wurzelbaum_1[i] << endl;
      for(unsigned int i=1; i<w2; i++)
        datei << (int)(w1+i) << " " << (int)(w1+A.Wurzelbaum_2[i]) << endl;
      knoten=w1+w2;
      for(unsigned int i=0;i<w1+w2;i++) 
        for(unsigned j=0;j<3-Nachfolgervon[i];j++)
        {
          datei << (int)knoten << " " << (int)i << endl;
          knoten++;
        }
    datei << "</EDGELIST>" << endl;
    datei << "<COORDS2D_FLOAT>" << endl;
      for(unsigned int i=0;i<w1;i++)
      {
        if(Ebene[i]==0)
          datei << "0";
        else
        {
          datei << -((long double)Ebene[i])+Position[i]*2.*Ebene[i]/(pow((long double)3,(int)Ebene[i])-1);
        }
        datei << " " << Ebene[i]+0.5 << endl;
      }
      for(unsigned int i=0;i<w2;i++)
      {
        if(Ebene[w1+i]==0)
          datei << "0";
        else
          datei << -((long double)Ebene[w1+i])+Position[w1+i]*2.*Ebene[w1+i]/(pow((long double)3,(int)Ebene[w1+i])-1);
        datei << " " << -((int)Ebene[w1+i])-0.5 << endl;		
      }
      for(unsigned int i=0;i<w1+w2;i++)
        for(unsigned j=0;j<3-Nachfolgervon[i];j++) 
        {
          pos=3*Position[i]+2-j;         
          eb=Ebene[i]+1;
          datei << -((long double)eb)+pos*2.*eb/(pow((long double)3,(int)eb)-1) << " ";
          if(i<w1)
            datei << ((int)eb)+0.5;
          else
            datei << -((int)eb)-0.5;
          datei << endl; 		
        }    
    datei << "</COORDS2D_FLOAT>" << endl;
    datei << "</GRAPH>" << endl;
  }
  else
  {
    // in der Mitte ist ein zusaetzlicher Knoten
    w3=A.Wurzelbaum_3.size(); 
    w4=A.Wurzelbaum_4.size();
    Nachfolgervon[0]=0;
    if(w1>0) Nachfolgervon[0]++;  
    if(w2>0) Nachfolgervon[0]++;
    if(w3>0) Nachfolgervon[0]++;
    if(w4>0) Nachfolgervon[0]++;
    for(unsigned int i = 1; i<w1; i++)
    {
      Ebene[1+i]=Ebene[1+A.Wurzelbaum_1[i]]+1;
      Position[1+i]=3*Position[1+A.Wurzelbaum_1[i]]+Nachfolgervon[1+A.Wurzelbaum_1[i]];
      Nachfolgervon[1+A.Wurzelbaum_1[i]]++;
    }
    for(unsigned int i = 1; i<w2; i++)
    {
      Ebene[1+w1+i]=Ebene[1+w1+A.Wurzelbaum_2[i]]+1;
      Position[1+w1+i]=3*Position[1+w1+A.Wurzelbaum_2[i]]+Nachfolgervon[1+w1+A.Wurzelbaum_2[i]];
      Nachfolgervon[1+w1+A.Wurzelbaum_2[i]]++;
    } 
    for(unsigned int i = 1; i<w3; i++)
    {
      Ebene[1+w1+w2+i]=Ebene[1+w1+w2+A.Wurzelbaum_3[i]]+1;
      Position[1+w1+w2+i]=3*Position[1+w1+w2+A.Wurzelbaum_3[i]]+Nachfolgervon[1+w1+w2+A.Wurzelbaum_3[i]];
      Nachfolgervon[1+w1+w2+A.Wurzelbaum_3[i]]++;
    } 
    for(unsigned int i = 1; i<w4; i++)
    {
      Ebene[1+w1+w2+w3+i]=Ebene[1+w1+w2+w3+A.Wurzelbaum_4[i]]+1;
      Position[1+w1+w2+w3+i]=3*Position[1+w1+w2+w3+A.Wurzelbaum_4[i]]+Nachfolgervon[1+w1+w2+w3+A.Wurzelbaum_4[i]];
      Nachfolgervon[1+w1+w2+w3+A.Wurzelbaum_4[i]]++;
    }
    datei << "<EDGELIST>" << endl;
      if(w1>0) datei << "0 " << 1 << endl;
      if(w2>0) datei << "0 " << 1+w1 << endl;
      if(w3>0) datei << "0 " << 1+w1+w2 << endl;
      if(w4>0) datei << "0 " << 1+w1+w2+w3 << endl;  
      for(unsigned int i=1; i<w1; i++)
        datei << 1+i << " " << 1+A.Wurzelbaum_1[i] << endl;
      for(unsigned int i=1; i<w2; i++)
        datei << 1+w1+i << " " << 1+w1+A.Wurzelbaum_2[i] << endl;
      for(unsigned int i=1; i<w3; i++)
        datei << 1+w1+w2+i << " " << 1+w1+w2+A.Wurzelbaum_3[i] << endl;
      for(unsigned int i=1; i<w4; i++)
        datei << 1+w1+w2+w3+i << " " << 1+w1+w2+w3+A.Wurzelbaum_4[i] << endl;
      knoten=1+w1+w2+w3+w4;
      for(unsigned j=0;j<4-Nachfolgervon[0];j++)
      {
        datei << "0 " << (int)knoten << endl;
        knoten++;
      }
      for(unsigned int i=0;i<w1+w2+w3+w4;i++) 
        for(unsigned j=0;j<3-Nachfolgervon[1+i];j++)
        {
          datei << knoten << " " << 1+i << endl;
          knoten++;
        }
    datei << "</EDGELIST>" << endl;
    
    datei << "<COORDS2D_FLOAT>" << endl;
      datei << "0 0" << endl;
      for(unsigned int i=0;i<w1;i++)
      {
        if(Ebene[1+i]==0)
          datei << "0";
        else
        {
          datei << -((long double)Ebene[1+i])+Position[1+i]*2.*Ebene[1+i]/(pow((long double)3,(int)Ebene[1+i])-1);
        }
        datei << " " << Ebene[1+i]+1 << endl;
      }
      for(unsigned int i=0;i<w2;i++)
      {
        if(Ebene[1+w1+i]==0)
          datei << "0";
        else
          datei << -((long double)Ebene[1+w1+i])+Position[1+w1+i]*2.*Ebene[1+w1+i]/(pow((long double)3,(int)Ebene[1+w1+i])-1);
        datei << " " << -((int)Ebene[1+w1+i])-1 << endl;		
      }
      for(unsigned int i=0;i<w3;i++)
      {
        datei << Ebene[1+w1+w2+i]+1 << " ";
        if(Ebene[1+w1+w2+i]==0)
          datei << "0";
        else
          datei << -((long double)Ebene[1+w1+w2+i])+Position[1+w1+w2+i]*2.*Ebene[1+w1+w2+i]/(pow((long double)3,(int)Ebene[1+w1+w2+i])-1);
        datei << endl;   
      }
      for(unsigned int i=0;i<w4;i++)
      {
        datei << -((int)Ebene[1+w1+w2+w3+i])-1 << " ";
        if(Ebene[1+w1+w2+w3+i]==0)
          datei << "0";  
        else
          datei << -((long double)Ebene[1+w1+w2+w3+i])+Position[1+w1+w2+w3+i]*2.*Ebene[1+w1+w2+w3+i]/(pow((long double)3,(int)Ebene[1+w1+w2+w3+i])-1); 
        datei << endl;
      } 
      if(w1==0) datei << "0 1" << endl;
      if(w2==0) datei << "0 -1" << endl; 
      if(w3==0) datei << "1 0" << endl;
      if(w4==0) datei << "-1 0" << endl;
      for(unsigned i=0;i<w1;i++)
        for(unsigned j=0;j<3-Nachfolgervon[1+i];j++)
        {
          pos=3*Position[1+i]+2-j;         
          eb=Ebene[1+i]+1;   
          datei << -((long double)eb)+pos*2.*eb/(pow((long double)3,(int)eb)-1);
          datei << " " << eb+1 << endl;
        }
      for(unsigned i=0;i<w2;i++)
        for(unsigned j=0;j<3-Nachfolgervon[1+w1+i];j++)
        {
          pos=3*Position[1+w1+i]+2-j;         
          eb=Ebene[1+w1+i]+1;   
          datei << -((long double)eb)+pos*2.*eb/(pow((long double)3,(int)eb)-1);
          datei << " " << -((int)eb)-1  << endl; 
        }
      for(unsigned i=0;i<w3;i++)
        for(unsigned j=0;j<3-Nachfolgervon[1+w1+w2+i];j++)
        {
          pos=3*Position[1+w1+w2+i]+2-j;         
          eb=Ebene[1+w1+w2+i]+1;   
          datei << eb+1 << " ";
          datei << -((long double)eb)+pos*2.*eb/(pow((long double)3,(int)eb)-1) << endl;
        }
      for(unsigned i=0;i<w4;i++)
        for(unsigned j=0;j<3-Nachfolgervon[1+w1+w2+w3+i];j++)
        {
          pos=3*Position[1+w1+w2+w3+i]+2-j;         
          eb=Ebene[1+w1+w2+w3+i]+1;   
          datei << -((int)eb)-1 << " ";
          datei << -((long double)eb)+pos*2.*eb/(pow((long double)3,(int)eb)-1); 
          datei << endl;
        } 
    datei << "</COORDS2D_FLOAT>" << endl;
    datei << "</GRAPH>" << endl;
  }
  return 0;
}
